package cnpc;

import cnpc.engine.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;

import javax.xml.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;


public class CNPC_Controller
{
	private CN_Engine engine = null;

	private final CNPC_UiController uiController;
	private final CNPC_AiController aiController;

	private String[] playerNames = null;
	private int[] playerTypes = null;
	private String[] playerPersonalities = null;

	private String variantPath = null;

	public CNPC_Controller(CNPC_UiController uiController_in, CNPC_AiController aiController_in)
	{
		resetEngine();
		uiController = uiController_in;
		aiController = aiController_in;

		uiController.setLookAndFeel();
	}

	public void launch()
	{
		boolean automaticNewGame;

		while(true)
		{
			try
			{
				Frame someFrame = new Frame();
				someFrame.setVisible(false);
				new OpeningDialog(someFrame);
				someFrame.dispose();
				automaticNewGame = launchGame();

				if(automaticNewGame) continue;

				while(!uiController.getEndGameFinished())
				{
					try
					{
						Thread.sleep(500);
					}
					catch(Exception e){}
				}

				if(!uiController.newGameQuery())
					System.exit(0);
			}
			catch(CN_FatalException e)
			{
				handleFatalException(e);
				return;
			}
			catch(CNPC_UiKillingException e)
			{
				handleUiKillingException(e);
				return;
			}
		}

	}

	private void resetEngine()
	{
		engine = null;
		engine = new CN_Engine();
	}

	private boolean launchGame() throws CN_FatalException, CNPC_UiKillingException
	{
		boolean returnValue;

		CN_GameState initialState;

		CNPC_ProgressFrame.instance().resetProgress();
		CNPC_ProgressFrame.instance().setVisible(true);

		initialState = engine.newGame(playerNames.length, variantPath);
		uiController.initialize(initialState, variantPath, playerNames, playerPersonalities);

		if(aiController != null)
			aiController.initialize(initialState, variantPath, playerPersonalities);

		CNPC_ProgressFrame.instance().finish();

		returnValue = mainLoop(initialState);

		resetEngine();

		return returnValue;
	}

	//returns a value indicating whether we should skip prompting for a new game and just automatically do one
	private boolean mainLoop(CN_GameState initialState) throws CN_FatalException, CNPC_UiKillingException
	{
		CN_GameState currentState = initialState;
		CN_InputMessage currentInputMessage = null;

		boolean tryingAgain = false;
		Exception aiFatalException;
		int currentPlayer = initialState.currentPlayer;

		while(true)
		{
			try
			{
				Thread.sleep(100);
			}
			catch(Exception e){}


			if(uiController.getUserSetNewGame())
			{
				return true;
			}

			if(currentState != null)
				currentPlayer = currentState.currentPlayer;

			if(playerTypes[currentPlayer] == CNPC_Constants.PLAYER_TYPE_HUMAN)
			{
				if(currentInputMessage != null)
					uiController.renderGameState(currentState, false);

				currentInputMessage = uiController.getInput();

				if(currentInputMessage != null)
				{
					try
					{
						currentState = engine.processInput(currentInputMessage);
					}
					catch(CN_InputMessageException e)
					{
						handleInputMessageException(e);
						currentState = null;
					}
				}
			}
			else if(playerTypes[currentPlayer] == CNPC_Constants.PLAYER_TYPE_COMPUTER)
			{
				uiController.renderGameState(currentState, true);
				tryingAgain = false;
				while(true)
				{
					if(!tryingAgain)
						currentInputMessage = aiController.processGameState(currentState);
					else
						currentInputMessage = aiController.tryAgain(currentState);

					try
					{
						currentState = engine.processInput(currentInputMessage);
					}
					catch(CN_InputMessageException e)
					{
						tryingAgain = true;
						continue;
					}
					break;
				}

				try
				{
					Thread.sleep(1000);
				}
				catch(Exception e){}
			}
			else
			{
				System.out.println("Invalid player type!  Type: " + String.valueOf(playerTypes[currentState.currentPlayer]));
				System.exit(0);
			}

			if(checkForEndGame(currentState))
				return false;
		}
	}

	private CN_InputMessage getInput()
	{
		return uiController.getInput();
	}
	private void handleInputMessageException(CN_InputMessageException e)
	{
		uiController.handleInputMessageException(e);
	}

	private void handleFatalException(CN_FatalException e)
	{
		uiController.kill();
		CNPC_ExceptionFrame.showFrame(e);
	}
	private void handleUiKillingException(CNPC_UiKillingException e)
	{
		uiController.kill();
	}


	private boolean checkForEndGame(CN_GameState gameState)
	{
		if(gameState == null)
			return false;

		if(gameState.victoriousPlayer != CN_Constants.NO_VICTORIOUS_PLAYER)
		{
			if(gameState.victoriousPlayer == CN_Constants.END_OF_UNIVERSE)
			{
				uiController.endOfUniverse(gameState, gameState.previousPlayer);
				return true;
			}
			else
			{
				uiController.victory(gameState, gameState.victoriousPlayer);
				return true;
			}
		}
		return false;
	}





	private class OpeningDialog extends JDialog
	{
		private final SpringLayout layout = new SpringLayout();
		private final Container content = getContentPane();

		private final JPanel playerNumberPanel = new JPanel();
		private final SpringLayout playerNumberPanelLayout = new SpringLayout();
		private final JLabel playersLabel = new JLabel("Number of Players");
		private final JComboBox playersComboBox = new JComboBox();

		private final JPanel playersPanel = new JPanel();
		private final SpringLayout playersPanelLayout = new SpringLayout();
		private final ArrayList<JLabel> nameLabelList = new ArrayList<JLabel>();
		private final ArrayList<JTextField> playerNameList = new ArrayList<JTextField>();
		private final ArrayList<JComboBox> playerTypeList = new ArrayList<JComboBox>();
		private final ArrayList<JComboBox> playerPersonalityList = new ArrayList<JComboBox>();

		private final JPanel variantPanel = new JPanel();
		private final SpringLayout variantPanelLayout = new SpringLayout();
		private final JLabel variantLabel = new JLabel("Select Variant:");
		private final DefaultListModel variantSelectorModel = new DefaultListModel();
		private final JList variantSelector = new JList(variantSelectorModel);
		private final JTextArea variantTextArea = new JTextArea();
		private final JScrollPane variantPane = new JScrollPane(variantTextArea, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

		private final JPanel buttonPanel = new JPanel();
		private final SpringLayout buttonPanelLayout = new SpringLayout();
		private final JButton acceptButton = new JButton("Accept");
		private final JButton cancelButton = new JButton("Exit");

		private final int titleBarHeight = uiController.getTitleBarHeight();

		private boolean allowAi = false;

		private final ArrayList<VariantData> variants = new ArrayList<VariantData>();

		private final ArrayList<CachedPlayerPrefs> cachedPlayerPrefs = new ArrayList<CachedPlayerPrefs>();

		public OpeningDialog(Frame owner)
		{
			super(owner, true);

			content.setLayout(layout);
			playersPanel.setLayout(playersPanelLayout);
			variantPanel.setLayout(variantPanelLayout);
			buttonPanel.setLayout(buttonPanelLayout);

			setTitle("Chrononauts PC Setup");
			setResizable(false);
			setLocation(100, 50);
			setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);

			layoutDialog();

			setVisible(true);
		}

		private void layoutDialog()
		{
			layoutPlayerNumberPanel();
			layoutVariantPanel();
			updatePlayersPanel();
			layoutButtonPanel();

			playersComboBox.addItemListener
			(
				new ItemListener()
				{
					public void itemStateChanged(ItemEvent e)
					{
						updatePlayersPanel();
						resetSize();
					}
				}
			);


			layout.putConstraint(SpringLayout.NORTH, playerNumberPanel, CNPC_Constants.DIALOG_NORTH_BUFFER, SpringLayout.NORTH, content);
			layout.putConstraint(SpringLayout.WEST, playerNumberPanel, CNPC_Constants.DIALOG_WEST_BUFFER, SpringLayout.WEST, content);
			layout.putConstraint(SpringLayout.NORTH, playersPanel, CNPC_Constants.DIALOG_PANEL_SEPARATION, SpringLayout.SOUTH, playerNumberPanel);
			layout.putConstraint(SpringLayout.WEST, playersPanel, CNPC_Constants.DIALOG_WEST_BUFFER, SpringLayout.WEST, content);
			layout.putConstraint(SpringLayout.NORTH, variantPanel, CNPC_Constants.DIALOG_PANEL_SEPARATION, SpringLayout.SOUTH, playersPanel);
			layout.putConstraint(SpringLayout.WEST, variantPanel, CNPC_Constants.DIALOG_WEST_BUFFER, SpringLayout.WEST, content);
			layout.putConstraint(SpringLayout.NORTH, buttonPanel, CNPC_Constants.DIALOG_PANEL_SEPARATION, SpringLayout.SOUTH, variantPanel);
			layout.putConstraint(SpringLayout.WEST, buttonPanel, CNPC_Constants.DIALOG_WEST_BUFFER, SpringLayout.WEST, content);
			content.add(playerNumberPanel);
			content.add(playersPanel);
			content.add(variantPanel);
			content.add(buttonPanel);

			getRootPane().setDefaultButton(acceptButton);

			resetSize();
		}

		private void resetSize()
		{
			int widestPanelWidth = 0;
			if(playerNumberPanel.getPreferredSize().width > widestPanelWidth)
				widestPanelWidth = playerNumberPanel.getPreferredSize().width;
			if(playersPanel.getPreferredSize().width > widestPanelWidth)
				widestPanelWidth = playersPanel.getPreferredSize().width;
			if(variantPanel.getPreferredSize().width > widestPanelWidth)
				widestPanelWidth = variantPanel.getPreferredSize().width;
			if(buttonPanel.getPreferredSize().width > widestPanelWidth)
				widestPanelWidth = buttonPanel.getPreferredSize().width;

			int width = CNPC_Constants.DIALOG_WEST_BUFFER + widestPanelWidth + CNPC_Constants.DIALOG_EAST_BUFFER;
			int height = titleBarHeight + CNPC_Constants.DIALOG_NORTH_BUFFER + (3 * CNPC_Constants.DIALOG_PANEL_SEPARATION) + playerNumberPanel.getPreferredSize().height + playersPanel.getPreferredSize().height + variantPanel.getPreferredSize().height + buttonPanel.getPreferredSize().height + CNPC_Constants.DIALOG_SOUTH_BUFFER;
			setSize(width, height);
		}

		private void updatePlayersPanel()
		{
			int numberOfPlayers = getNumberOfPlayers();

			savePlayerPrefs();
			playersPanel.removeAll();
			resetLists();

			for(int i = 0; i < numberOfPlayers; i++)
			{
				if(i == 0)
				{
					playersPanelLayout.putConstraint(SpringLayout.WEST, nameLabelList.get(i), CNPC_Constants.PLAYERS_PANEL_INTERNAL_WEST_BUFFER, SpringLayout.WEST, playersPanel);
					playersPanelLayout.putConstraint(SpringLayout.NORTH, nameLabelList.get(i), CNPC_Constants.PLAYERS_PANEL_INTERNAL_NORTH_BUFFER, SpringLayout.NORTH, playersPanel);
				}
				else
				{
					playersPanelLayout.putConstraint(SpringLayout.WEST, nameLabelList.get(i), CNPC_Constants.PLAYERS_PANEL_INTERNAL_WEST_BUFFER, SpringLayout.WEST, playersPanel);
					playersPanelLayout.putConstraint(SpringLayout.NORTH, nameLabelList.get(i), CNPC_Constants.PLAYERS_PANEL_NS_SEPARATION, SpringLayout.SOUTH, nameLabelList.get(i - 1));
				}
				playersPanelLayout.putConstraint(SpringLayout.WEST, playerNameList.get(i), CNPC_Constants.PLAYERS_PANEL_EW_SEPARATION, SpringLayout.EAST, nameLabelList.get(i));
				playersPanelLayout.putConstraint(SpringLayout.NORTH, playerNameList.get(i), -1, SpringLayout.NORTH, nameLabelList.get(i));
				playersPanelLayout.putConstraint(SpringLayout.WEST, playerTypeList.get(i), CNPC_Constants.PLAYERS_PANEL_EW_SEPARATION, SpringLayout.EAST, playerNameList.get(i));
				playersPanelLayout.putConstraint(SpringLayout.NORTH, playerTypeList.get(i), -2, SpringLayout.NORTH, nameLabelList.get(i));
				playersPanelLayout.putConstraint(SpringLayout.WEST, playerPersonalityList.get(i), CNPC_Constants.PLAYERS_PANEL_EW_SEPARATION, SpringLayout.EAST, playerTypeList.get(i));
				playersPanelLayout.putConstraint(SpringLayout.NORTH, playerPersonalityList.get(i), -2, SpringLayout.NORTH, nameLabelList.get(i));

				playersPanel.add(nameLabelList.get(i));
				playersPanel.add(playerNameList.get(i));
				playersPanel.add(playerTypeList.get(i));
				playersPanel.add(playerPersonalityList.get(i));
			}

			int preferredWidth = CNPC_Constants.PLAYERS_PANEL_INTERNAL_WEST_BUFFER + nameLabelList.get(0).getPreferredSize().width + CNPC_Constants.PLAYERS_PANEL_EW_SEPARATION + playerNameList.get(0).getPreferredSize().width + CNPC_Constants.PLAYERS_PANEL_EW_SEPARATION + playerTypeList.get(0).getPreferredSize().width + CNPC_Constants.PLAYERS_PANEL_EW_SEPARATION + playerPersonalityList.get(0).getPreferredSize().width + CNPC_Constants.PLAYERS_PANEL_INTERNAL_EAST_BUFFER;
			int preferredHeight = CNPC_Constants.PLAYERS_PANEL_INTERNAL_NORTH_BUFFER + ((numberOfPlayers - 1) * CNPC_Constants.PLAYERS_PANEL_NS_SEPARATION) + (numberOfPlayers * playerNameList.get(0).getPreferredSize().height) + CNPC_Constants.PLAYERS_PANEL_INTERNAL_SOUTH_BUFFER;
			playersPanel.setPreferredSize(new Dimension(preferredWidth, preferredHeight));

			initializeComboBoxValues();
			updatePersonalityComboBoxesEnabled();
			updatePlayerTypeComboBoxesEnabled();
			loadPlayerPrefs();

			validate();
		}

		private void resetLists()
		{
			if(aiController != null)
			{
				try
				{
					aiController.setVariant(variants.get(variantSelector.getSelectedIndex()).path);
				}
				catch(CNPC_UiKillingException e)
				{
					handleUiKillingException(e);
					return;
				}
				setAllowAi();
			}

			int numberOfPlayers = getNumberOfPlayers();

			nameLabelList.clear();
			playerNameList.clear();
			playerTypeList.clear();
			playerPersonalityList.clear();

			for(int i = 0; i < numberOfPlayers; i++)
			{
				nameLabelList.add(new JLabel("Player " + String.valueOf(i + 1) + " name:"));
				playerNameList.add(new CNPC_RestrictedTextField(7, 15));
				playerTypeList.add(new JComboBox());
				playerPersonalityList.add(new JComboBox());
			}

			String[] randomNames = getRandomNames(playerNameList.size());
			for(int i = 0; i < playerNameList.size(); i++)
			{
				playerNameList.get(i).setText(randomNames[i]);
			}

			for(int i = 0; i < playerTypeList.size(); i++)
			{
				JComboBox comboBox = playerTypeList.get(i);
				comboBox.setEditable(false);
				comboBox.addItem("Human");
				if(allowAi)
					comboBox.addItem("Computer");
				if(i == 0) comboBox.setEnabled(false);

				comboBox.addItemListener
				(
					new ItemListener()
					{
						public void itemStateChanged(ItemEvent e)
						{
							updatePersonalityComboBoxesEnabled();
						}
					}
				);
			}


			String[] personalities;
			if(allowAi)
			{
				personalities = aiController.getPersonalitiesForCurrentVariant();
				for(int i = 0; i < playerPersonalityList.size(); i++)
				{
					JComboBox comboBox = playerPersonalityList.get(i);
					comboBox.setEditable(false);
					for(int j = 0; j < personalities.length; j++)
					{
						comboBox.addItem(personalities[j]);
					}
					comboBox.addItem(CNPC_Constants.RANDOM_STRING);
					comboBox.addItem(CNPC_Constants.WEIGHTED_RANDOM_STRING);
					if(playerTypeList.get(i).getSelectedIndex() != 1)
						comboBox.setEnabled(false);
				}
			}
			else
			{
				for(int i = 0; i < playerPersonalityList.size(); i++)
				{
					JComboBox comboBox = playerPersonalityList.get(i);
					comboBox.setEditable(false);
					comboBox.addItem(CNPC_Constants.RANDOM_STRING);
					comboBox.addItem(CNPC_Constants.WEIGHTED_RANDOM_STRING);
					comboBox.setEnabled(false);
				}
			}
		}

		private void updatePersonalityComboBoxesEnabled()
		{
			if(!allowAi)
				return;

			for(int i = 0; i < playerTypeList.size(); i++)
			{
				if(playerTypeList.get(i).getSelectedIndex() == 1)
					playerPersonalityList.get(i).setEnabled(true);
				else
					playerPersonalityList.get(i).setEnabled(false);
			}
		}

		private void updatePlayerTypeComboBoxesEnabled()
		{
			for(int i = 0; i < playerTypeList.size(); i++)
			{
				if(playerTypeList.get(i).getItemCount() <= 1)
					playerTypeList.get(i).setEnabled(false);
				else
					playerTypeList.get(i).setEnabled(true);
			}
		}



		private void initializeComboBoxValues()
		{
			int personalityIndex = 0;

			String currentString;
			for(int i = 0; i < playerPersonalityList.get(0).getItemCount(); i++)
			{
				currentString = (String)(playerPersonalityList.get(0).getItemAt(i));
				if(currentString.equals(CNPC_Constants.WEIGHTED_RANDOM_STRING))
				{
					personalityIndex = i;
					break;
				}
			}

			for(int i = 0; i < playerTypeList.size(); i++)
			{
				if((i > 0)&&(allowAi))
					playerTypeList.get(i).setSelectedIndex(1);
				playerPersonalityList.get(i).setSelectedIndex(personalityIndex);
			}
		}



		private void layoutPlayerNumberPanel()
		{
			int maxSupportedPlayers = uiController.getMaxSupportedPlayers();
			playersComboBox.setEditable(false);
			for(int i = 1; i < maxSupportedPlayers; i++)
			{
				playersComboBox.addItem(" " + String.valueOf(i + 1) + " ");
			}
			playersComboBox.setSelectedIndex(maxSupportedPlayers - 2);

			playerNumberPanelLayout.putConstraint(SpringLayout.WEST, playersLabel, CNPC_Constants.PLAYER_NUMBER_PANEL_INTERNAL_WEST_BUFFER, SpringLayout.WEST, playerNumberPanel);
			playerNumberPanelLayout.putConstraint(SpringLayout.NORTH, playersLabel, CNPC_Constants.PLAYER_NUMBER_PANEL_INTERNAL_NORTH_BUFFER, SpringLayout.NORTH, playerNumberPanel);
			playerNumberPanelLayout.putConstraint(SpringLayout.WEST, playersComboBox, CNPC_Constants.PLAYER_NUMBER_PANEL_EW_SEPARATION, SpringLayout.EAST, playersLabel);
			playerNumberPanelLayout.putConstraint(SpringLayout.NORTH, playersComboBox, CNPC_Constants.PLAYER_NUMBER_PANEL_INTERNAL_NORTH_BUFFER, SpringLayout.NORTH, playerNumberPanel);
			playerNumberPanel.add(playersLabel);
			playerNumberPanel.add(playersComboBox);

			int preferredWidth = CNPC_Constants.PLAYER_NUMBER_PANEL_INTERNAL_WEST_BUFFER + playersLabel.getPreferredSize().width + CNPC_Constants.PLAYER_NUMBER_PANEL_EW_SEPARATION + playersComboBox.getPreferredSize().width + CNPC_Constants.PLAYER_NUMBER_PANEL_INTERNAL_EAST_BUFFER;
			int preferredHeight = CNPC_Constants.PLAYER_NUMBER_PANEL_INTERNAL_NORTH_BUFFER + playersComboBox.getPreferredSize().height + CNPC_Constants.PLAYER_NUMBER_PANEL_INTERNAL_SOUTH_BUFFER;
			playerNumberPanel.setPreferredSize(new Dimension(preferredWidth, preferredHeight));
		}

		private int getNumberOfPlayers()
		{
			return playersComboBox.getSelectedIndex() + 2;
		}


		private void layoutVariantPanel()
		{
			resetVariants();

			variantSelector.setPreferredSize(new Dimension(CNPC_Constants.VARIANT_SELECTOR_WIDTH, CNPC_Constants.VARIANT_SELECTOR_HEIGHT));
			variantSelector.setBorder(BorderFactory.createLoweredBevelBorder());

			variantTextArea.setLineWrap(true);
			variantTextArea.setWrapStyleWord(true);
			variantTextArea.setBackground(null);
			variantTextArea.setEditable(false);
			variantTextArea.setFont(CNPC_Constants.VARIANT_TEXT_AREA_FONT);

			variantPane.setPreferredSize(new Dimension(variantSelector.getPreferredSize().width, CNPC_Constants.VARIANT_TEXT_AREA_HEIGHT));

			VariantData variant;
			for(int i = 0; i < variants.size(); i++)
			{
				variant = variants.get(i);
				variantSelectorModel.addElement(variant.name);
			}
			variantSelector.setSelectedIndex(0);

			variantSelector.addListSelectionListener
			(
				new ListSelectionListener()
				{
					public void valueChanged(ListSelectionEvent e)
					{
						updateVariantTextArea();
						updatePlayersPanel();
					}
				}
			);
			updateVariantTextArea();

			variantPanelLayout.putConstraint(SpringLayout.WEST, variantLabel, CNPC_Constants.VARIANT_PANEL_INTERNAL_WEST_BUFFER, SpringLayout.WEST, variantPanel);
			variantPanelLayout.putConstraint(SpringLayout.NORTH, variantLabel, CNPC_Constants.VARIANT_PANEL_INTERNAL_NORTH_BUFFER, SpringLayout.NORTH, variantPanel);
			variantPanelLayout.putConstraint(SpringLayout.WEST, variantSelector, CNPC_Constants.VARIANT_PANEL_INTERNAL_WEST_BUFFER, SpringLayout.WEST, variantPanel);
			variantPanelLayout.putConstraint(SpringLayout.NORTH, variantSelector, CNPC_Constants.VARIANT_PANEL_LABEL_LIST_SEPARATION, SpringLayout.SOUTH, variantLabel);
			variantPanelLayout.putConstraint(SpringLayout.WEST, variantPane, CNPC_Constants.VARIANT_PANEL_INTERNAL_WEST_BUFFER, SpringLayout.WEST, variantPanel);
			variantPanelLayout.putConstraint(SpringLayout.NORTH, variantPane, CNPC_Constants.VARIANT_PANEL_LIST_TEXT_AREA_SEPARATION, SpringLayout.SOUTH, variantSelector);
			variantPanel.add(variantLabel);
			variantPanel.add(variantSelector);
			variantPanel.add(variantPane);

			int preferredWidth = CNPC_Constants.VARIANT_PANEL_INTERNAL_WEST_BUFFER + variantSelector.getPreferredSize().width + CNPC_Constants.VARIANT_PANEL_INTERNAL_EAST_BUFFER;
			int preferredHeight = CNPC_Constants.VARIANT_PANEL_INTERNAL_NORTH_BUFFER + variantLabel.getPreferredSize().height + CNPC_Constants.VARIANT_PANEL_LABEL_LIST_SEPARATION + variantSelector.getPreferredSize().height + CNPC_Constants.VARIANT_PANEL_LIST_TEXT_AREA_SEPARATION + variantPane.getPreferredSize().height + CNPC_Constants.VARIANT_PANEL_INTERNAL_SOUTH_BUFFER;
			variantPanel.setPreferredSize(new Dimension(preferredWidth, preferredHeight));
		}


		private void layoutButtonPanel()
		{
			Dimension buttonDimension;
			if(acceptButton.getPreferredSize().width > cancelButton.getPreferredSize().width)
				buttonDimension = acceptButton.getPreferredSize();
			else
				buttonDimension = cancelButton.getPreferredSize();

			acceptButton.setPreferredSize(buttonDimension);
			cancelButton.setPreferredSize(buttonDimension);

			buttonPanelLayout.putConstraint(SpringLayout.WEST, acceptButton, CNPC_Constants.BUTTON_PANEL_INTERNAL_WEST_BUFFER, SpringLayout.WEST, buttonPanel);
			buttonPanelLayout.putConstraint(SpringLayout.NORTH, acceptButton, CNPC_Constants.BUTTON_PANEL_INTERNAL_WEST_BUFFER, SpringLayout.NORTH, buttonPanel);
			buttonPanelLayout.putConstraint(SpringLayout.WEST, cancelButton, CNPC_Constants.BUTTON_PANEL_EW_SEPARATION, SpringLayout.EAST, acceptButton);
			buttonPanelLayout.putConstraint(SpringLayout.NORTH, cancelButton, CNPC_Constants.BUTTON_PANEL_INTERNAL_WEST_BUFFER, SpringLayout.NORTH, buttonPanel);
			buttonPanel.add(acceptButton);
			buttonPanel.add(cancelButton);

			int preferredWidth = CNPC_Constants.BUTTON_PANEL_INTERNAL_WEST_BUFFER + acceptButton.getPreferredSize().width + CNPC_Constants.BUTTON_PANEL_EW_SEPARATION + cancelButton.getPreferredSize().width + CNPC_Constants.BUTTON_PANEL_INTERNAL_EAST_BUFFER;
			int preferredHeight = CNPC_Constants.BUTTON_PANEL_INTERNAL_NORTH_BUFFER + acceptButton.getPreferredSize().height + CNPC_Constants.BUTTON_PANEL_INTERNAL_SOUTH_BUFFER;
			buttonPanel.setPreferredSize(new Dimension(preferredWidth, preferredHeight));

			cancelButton.addActionListener
			(
				new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						System.exit(0);
					}
				}
			);

			acceptButton.addActionListener
			(
				new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						variantPath = variants.get(variantSelector.getSelectedIndex()).path;
						if(allowAi)
						{
							try
							{
								aiController.setVariant(variantPath);
							}
							catch(CNPC_UiKillingException f)
							{
								handleUiKillingException(f);
								return;
							}
						}

						int numberOfPlayers = getNumberOfPlayers();
						playerNames = new String[numberOfPlayers];
						playerTypes = new int[numberOfPlayers];
						playerPersonalities = new String[numberOfPlayers];

						for(int i = 0; i < numberOfPlayers; i++)
						{
							playerNames[i] = playerNameList.get(i).getText();
							playerTypes[i] = playerTypeList.get(i).getSelectedIndex();
							if(playerTypes[i] == 1)
							{
								playerPersonalities[i] = (String)(playerPersonalityList.get(i).getSelectedItem());
								if(playerPersonalities[i].equals(CNPC_Constants.RANDOM_STRING))
									playerPersonalities[i] = aiController.getRandomPersonality();
								else if(playerPersonalities[i].equals(CNPC_Constants.WEIGHTED_RANDOM_STRING))
									playerPersonalities[i] = aiController.getWeightedRandomPersonality();
							}
							else
							{
								playerPersonalities[i] = null;
							}
						}

						dispose();
					}
				}
			);
		}

		private void resetVariants()
		{
			variants.clear();

			File directory = new File(CNPC_Constants.VARIANT_SEARCH_DIRECTORY);
			File[] children = directory.listFiles();

			File[] lowerChildren;

			for(File file : children)
			{
				if(file.isDirectory())
				{
					lowerChildren = file.listFiles();
					for(File lowerFile : lowerChildren)
					{
						if(lowerFile.getName().toLowerCase().equals(CNPC_Constants.VARIANT_FILE_NAME))
						{
							variants.add(getVariantFromFile(lowerFile));
						}
					}
				}
			}
		}

		private VariantData getVariantFromFile(File variantFile)
		{
			VariantData returnValue;
			String path = variantFile.getParentFile().getName();
			CNPC_VariantProcessor processor = new CNPC_VariantProcessor();
			processor.loadVariant(variantFile);
			returnValue = new VariantData(processor.getName(), path, processor.getDescription());
			return returnValue;
		}



		private String[] getRandomNames(int numNames)
		{
			String[] returnValue = new String[numNames];
			ArrayList<String> names = loadNames();

			Random random = new Random();

			for(int i = 0; i < numNames; i++)
			{
				if((names == null)||(names.size() <= 0))
					returnValue[i] = "";
				else
					returnValue[i] = names.get(random.nextInt(names.size()));
			}

			return returnValue;
		}

		private ArrayList<String> loadNames()
		{
			ArrayList<String> returnValue = new ArrayList<String>();

			BufferedReader reader;
			try
			{
				reader = new BufferedReader(new FileReader(CNPC_Constants.NAMES_FILE));
			}
			catch(FileNotFoundException e)
			{
				return null;
			}

			String currentString;
			while(true)
			{
				try
				{
					currentString = reader.readLine();
				}
				catch(IOException e)
				{
					return null;
				}

				if(currentString == null) break;

				returnValue.add(currentString);
			}

			return returnValue;
		}


		private void updateVariantTextArea()
		{
			int variantIndex = variantSelector.getSelectedIndex();
			VariantData data = variants.get(variantIndex);
			String newText = getDescriptionText(data.name, data.path, data.description);
			variantTextArea.setText(newText);
			variantTextArea.getCaret().setDot(0);
		}

		private String getDescriptionText(String name, String path, String rawDescription)
		{
			String returnValue = "";

			returnValue += "Variant Name: " + name + "\n";
			returnValue += "Path: " + path + "\n";
			returnValue += "\n";
			returnValue += rawDescription;

			return returnValue;
		}

		private void setAllowAi()
		{
			if(aiController == null)
				allowAi = false;
			else if(aiController.getPersonalitiesForCurrentVariant().length == 0)
				allowAi = false;
			else
				allowAi = true;
		}



		private void loadPlayerPrefs()
		{
			int numberOfPlayers = getNumberOfPlayers();

			String name;
			int playerType;
			String personality;

			CachedPlayerPrefs playerPrefs;
			for(int i = 0; i < numberOfPlayers; i++)
			{
				if(i >= cachedPlayerPrefs.size())
					break;
				playerPrefs = cachedPlayerPrefs.get(i);

				name = playerPrefs.name;
				playerType = playerPrefs.playerType;
				personality = playerPrefs.personality;

				if(!name.trim().equals(""))
					playerNameList.get(i).setText(name);
				if(playerType < playerTypeList.get(i).getItemCount())
					playerTypeList.get(i).setSelectedIndex(playerType);
				playerPersonalityList.get(i).setSelectedItem(personality);
			}

			savePlayerPrefs();
		}

		private void savePlayerPrefs()
		{
			int numberOfPlayers = getNumberOfPlayers();

			String name;
			int playerType;
			String personality;

			for(int i = 0; i < playerNameList.size(); i++)
			{
				name = playerNameList.get(i).getText();
				playerType = playerTypeList.get(i).getSelectedIndex();
				personality = (String)(playerPersonalityList.get(i).getSelectedItem());

				if(cachedPlayerPrefs.size() > i)
					cachedPlayerPrefs.set(i, new CachedPlayerPrefs(name, playerType, personality));
				else
					cachedPlayerPrefs.add(new CachedPlayerPrefs(name, playerType, personality));
			}
		}

	}


	private class CachedPlayerPrefs
	{
		public final String name;
		public final int playerType;
		public final String personality;

		public CachedPlayerPrefs(String name_in, int playerType_in, String personality_in)
		{
			name = name_in;
			playerType = playerType_in;
			personality = personality_in;
		}
	}


	private class VariantData
	{
		public final String name;
		public final String path;
		public final String description;

		public VariantData(String name_in, String path_in, String description_in)
		{
			name = name_in;
			path = path_in;
			description = description_in;
		}
	}

}


class CNPC_VariantProcessor
{
	private static final String NONE_LOADED_STRING = "(none loaded)";

	private String variantName = NONE_LOADED_STRING;
	private String variantDescription = NONE_LOADED_STRING;

	private void reset()
	{
		variantName = NONE_LOADED_STRING;
		variantDescription = NONE_LOADED_STRING;
	}

	public String getName()
	{
		return variantName;
	}
	public String getDescription()
	{
		return variantDescription;
	}

	public boolean loadVariant(String path)
	{
		return loadVariant(new File(path));
	}

	public boolean loadVariant(File xmlFile)
	{
		reset();

		DocumentBuilderFactory factory;
		DocumentBuilder builder;
		org.w3c.dom.Document document;
		NodeList list;
		NodeList childList;
		Node currentNode;

		try
		{
			factory = DocumentBuilderFactory.newInstance();
			builder = factory.newDocumentBuilder();
			document = builder.parse(xmlFile);
		}
		catch(Exception e)
		{
			return false;
		}

		list = document.getElementsByTagName("Variant");
		if(list.getLength() != 1) return false;

		childList = list.item(0).getChildNodes();

		for(int i = 0; childList.item(i) != null; i++)
		{
			currentNode = childList.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("name"))
			{
				variantName = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("description"))
			{
				variantDescription = currentNode.getTextContent();
			}
		}
		return true;
	}
}



class CNPC_RestrictedTextField extends JTextField
{
	Cursor oldCursor = null;

	public CNPC_RestrictedTextField(int size, int limit)
	{
		super(size);
		setDocument(new RestrictedDocument(limit));
		addMouseListener
		(
			new MouseAdapter()
			{
				public void mouseEntered(MouseEvent e)
				{
					oldCursor = getTopLevelAncestor().getCursor();
					getTopLevelAncestor().setCursor(java.awt.Cursor.getPredefinedCursor(java.awt.Cursor.TEXT_CURSOR));
				}

				public void mouseExited(MouseEvent e)
				{
					getTopLevelAncestor().setCursor(oldCursor);
				}
			}
		);
	}





	private class RestrictedDocument extends PlainDocument
	{
		int limit;

		public RestrictedDocument(int limit_in)
		{
			super();
			limit = limit_in;
		}

		public void insertString(int offset, String str, AttributeSet attr) throws BadLocationException
		{
			if(str == null) return;
			if(getLength() + str.length() <= limit)
			{
				super.insertString(offset, str, attr);
			}
		}
	}
}


class CNPC_Constants
{
	public static final int PLAYER_TYPE_HUMAN = 0;
	public static final int PLAYER_TYPE_COMPUTER = 1;

	public static final String RANDOM_STRING = "(Random Personality)";
	public static final String WEIGHTED_RANDOM_STRING = "(Weighted Random Personality)";

	public static final int DIALOG_WEST_BUFFER = 10;
	public static final int DIALOG_NORTH_BUFFER = 10;
	public static final int DIALOG_EAST_BUFFER = 10;
	public static final int DIALOG_SOUTH_BUFFER = 10;
	public static final int DIALOG_PANEL_SEPARATION = 5;

	public static final int PLAYER_NUMBER_PANEL_INTERNAL_WEST_BUFFER = 10;
	public static final int PLAYER_NUMBER_PANEL_INTERNAL_NORTH_BUFFER = 10;
	public static final int PLAYER_NUMBER_PANEL_INTERNAL_EAST_BUFFER = 10;
	public static final int PLAYER_NUMBER_PANEL_INTERNAL_SOUTH_BUFFER = 10;
	public static final int PLAYER_NUMBER_PANEL_EW_SEPARATION = 2;

	public static final int PLAYERS_PANEL_INTERNAL_WEST_BUFFER = 10;
	public static final int PLAYERS_PANEL_INTERNAL_NORTH_BUFFER = 10;
	public static final int PLAYERS_PANEL_INTERNAL_EAST_BUFFER = 10;
	public static final int PLAYERS_PANEL_INTERNAL_SOUTH_BUFFER = 10;
	public static final int PLAYERS_PANEL_NS_SEPARATION = 10;
	public static final int PLAYERS_PANEL_EW_SEPARATION = 5;

	public static final int VARIANT_SELECTOR_WIDTH = 250;
	public static final int VARIANT_SELECTOR_HEIGHT = 100;
	public static final int VARIANT_TEXT_AREA_HEIGHT = 100;
	public static final int VARIANT_PANEL_INTERNAL_WEST_BUFFER = 10;
	public static final int VARIANT_PANEL_INTERNAL_NORTH_BUFFER = 10;
	public static final int VARIANT_PANEL_INTERNAL_EAST_BUFFER = 10;
	public static final int VARIANT_PANEL_INTERNAL_SOUTH_BUFFER = 10;
	public static final int VARIANT_PANEL_LABEL_LIST_SEPARATION = 2;
	public static final int VARIANT_PANEL_LIST_TEXT_AREA_SEPARATION = 10;
	public static final Font VARIANT_TEXT_AREA_FONT = new Font("Tahoma", Font.PLAIN, 12);

	public static final int BUTTON_PANEL_INTERNAL_WEST_BUFFER = 10;
	public static final int BUTTON_PANEL_INTERNAL_NORTH_BUFFER = 10;
	public static final int BUTTON_PANEL_INTERNAL_EAST_BUFFER = 10;
	public static final int BUTTON_PANEL_INTERNAL_SOUTH_BUFFER = 10;
	public static final int BUTTON_PANEL_EW_SEPARATION = 5;

	public static final String VARIANT_SEARCH_DIRECTORY = "../data/";
	public static final String NAMES_FILE = "../data/names.txt";

	public static final String VARIANT_FILE_NAME = "variant.xml";

	public static final String MY_EMAIL = "barbieri.michael@gmail.com";
	public static final String MY_EMAIL_URL = "mailto:" + MY_EMAIL;
	public static final String MY_EMAIL_HTML = "<a href=\"" + MY_EMAIL_URL + "\">" + MY_EMAIL + "</a>";
}