package cnpc;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class CNPC_ExceptionFrame extends JFrame
{
	private static final int NORTH_BUFFER = 10;
	private static final int SOUTH_BUFFER = 10;
	private static final int EAST_BUFFER = 10;
	private static final int WEST_BUFFER = 10;
	private static final int LABEL_TEXT_AREA_SEPARATION = 10;
	private static final int TEXT_AREA_BUTTON_SEPARATION = 10;
	private static final Font TEXT_AREA_FONT = new Font("Tahoma", Font.PLAIN, 12);
	private static final Dimension TEXT_AREA_DIMENSION = new Dimension(400, 400);


	private final SpringLayout layout = new SpringLayout();
	private final Container content = getContentPane();

	private final JPanel panel = new JPanel();
	private final SpringLayout panelLayout = new SpringLayout();

	private final JPanel labelPanel = new JPanel();
	private final SpringLayout labelPanelLayout = new SpringLayout();
	private final String openingText =
	"<html>" +
	"Oops!  An error has occurred from which Chrononauts PC must now, sadly, die.  If you<br>" +
	"would like to help me find and fix this bug, please send an email to " + CNPC_Constants.MY_EMAIL_HTML + ",<br>" +
	"copying and pasting the mumbo-jumbo in the box below.  If you want to be extra<br>" +
	"helpful, also tell me what you were doing leading up to the error.  Thanks, and sorry<br>" +
	"about the problem!" +
	"</html>";

	private final JLabel openingLabel = new JLabel(openingText);

	private final JTextArea exceptionTextArea = new JTextArea();
	private final JScrollPane exceptionPane = new JScrollPane(exceptionTextArea, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

	private final JButton exitButton = new JButton("Exit");



	public static void showFrame(Exception exception)
	{
		new CNPC_ExceptionFrame(exception);
	}

	private CNPC_ExceptionFrame(Exception exception)
	{
		super();
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		exitButton.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					System.exit(0);
				}
			}
		);

		initTextArea(exception);
		layOutFrame();
		setVisible(true);
	}



	private void initTextArea(Exception exception)
	{
		exceptionTextArea.setBackground(null);
		exceptionTextArea.setEditable(false);
		exceptionTextArea.setFont(TEXT_AREA_FONT);
		exceptionPane.setPreferredSize(TEXT_AREA_DIMENSION);

		exceptionTextArea.setText(getStackTraceString(exception));
	}


	private void layOutFrame()
	{
		setLocation(200, 200);
		setUndecorated(true);

		layOutLabelPanel();

		layOutPanel();
		setSize(panel.getPreferredSize());

		setLayout(layout);
		layout.putConstraint(SpringLayout.WEST, panel, 0, SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, panel, 0, SpringLayout.NORTH, content);
		content.add(panel);

	}

	private void layOutLabelPanel()
	{
		labelPanel.setLayout(labelPanelLayout);
		labelPanelLayout.putConstraint(SpringLayout.WEST, openingLabel, 0, SpringLayout.WEST, labelPanel);
		labelPanelLayout.putConstraint(SpringLayout.NORTH, openingLabel, 0, SpringLayout.NORTH, labelPanel);
		labelPanel.add(openingLabel);
		labelPanel.setPreferredSize(openingLabel.getPreferredSize());

		labelPanel.addMouseListener
		(
			new MouseAdapter()
			{
				public void mouseClicked(MouseEvent e)
				{
					CNPC_BrowserLauncher.openURL(CNPC_Constants.MY_EMAIL_URL);
				}
				public void mouseEntered(MouseEvent e)
				{
					setCursor(new Cursor(Cursor.HAND_CURSOR));
				}
				public void mouseExited(MouseEvent e)
				{
					setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				}
			}
		);
	}


	private void layOutPanel()
	{
		panel.setBorder(BorderFactory.createEtchedBorder());

		panel.setLayout(panelLayout);

		panelLayout.putConstraint(SpringLayout.WEST, labelPanel, WEST_BUFFER, SpringLayout.WEST, panel);
		panelLayout.putConstraint(SpringLayout.NORTH, labelPanel, NORTH_BUFFER, SpringLayout.NORTH, panel);
		panelLayout.putConstraint(SpringLayout.WEST, exceptionPane, WEST_BUFFER, SpringLayout.WEST, panel);
		panelLayout.putConstraint(SpringLayout.NORTH, exceptionPane, LABEL_TEXT_AREA_SEPARATION, SpringLayout.SOUTH, labelPanel);
		panelLayout.putConstraint(SpringLayout.WEST, exitButton, WEST_BUFFER, SpringLayout.WEST, panel);
		panelLayout.putConstraint(SpringLayout.NORTH, exitButton, TEXT_AREA_BUTTON_SEPARATION, SpringLayout.SOUTH, exceptionPane);
		panel.add(labelPanel);
		panel.add(exceptionPane);
		panel.add(exitButton);

		int widestComponentWidth = getWidestComponentWidth(new JComponent[] {openingLabel, exceptionPane});

		int width = WEST_BUFFER + widestComponentWidth + EAST_BUFFER;
		int height = NORTH_BUFFER + labelPanel.getPreferredSize().height + LABEL_TEXT_AREA_SEPARATION + exceptionPane.getPreferredSize().height + TEXT_AREA_BUTTON_SEPARATION + exitButton.getPreferredSize().height + SOUTH_BUFFER;
		panel.setPreferredSize(new Dimension(width, height));
	}

	private int getWidestComponentWidth(JComponent[] components)
	{
		if(components.length <= 0) return 0;

		int widestComponentWidth = components[0].getPreferredSize().width;
		int currentWidth;
		for(JComponent component : components)
		{
			currentWidth = component.getPreferredSize().width;
			if(currentWidth > widestComponentWidth)
				widestComponentWidth = currentWidth;
		}

		return widestComponentWidth;
	}




	private String getStackTraceString(Exception exception)
	{
		String returnValue = "";
		ArrayList<String> stackTrace = getStackTrace(exception);
		for(int i = 0; i < stackTrace.size(); i++)
		{
			returnValue = returnValue + stackTrace.get(i) + "\n";
		}
		return returnValue;
	}


	private ArrayList<String> getStackTrace(Throwable exception)
	{
		return getStackTrace(exception, false);
	}

	private ArrayList<String> getStackTrace(Throwable exception, boolean isACause)
	{
		if(exception.getCause() == null)
		{
			return createStackTrace(exception, isACause);
		}
		else
		{
			ArrayList<String> theRest = getStackTrace(exception.getCause(), true);
			ArrayList<String> currentTrace = createStackTrace(exception, isACause);
			ArrayList<String> returnValue = new ArrayList<String>();
			for(int i = 0; i < currentTrace.size(); i++)
			{
				returnValue.add(currentTrace.get(i));
			}
			for(int i = 0; i < theRest.size(); i++)
			{
				returnValue.add(theRest.get(i));
			}
			return returnValue;
		}
	}

	private ArrayList<String> createStackTrace(Throwable exception, boolean isACause)
	{
		ArrayList<String> returnValue = new ArrayList<String>();
		String initialString = "";
		if(isACause)
			initialString = initialString + "Caused by: ";
		initialString = initialString + exception.getClass().getName() + ": " + exception.getMessage();
		returnValue.add(initialString);

		StackTraceElement[] currentTrace = exception.getStackTrace();
		for(int i = 0; i < currentTrace.length; i++)
		{
			returnValue.add("     " + currentTrace[i].toString());
		}
		return returnValue;
	}



}
