package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_CardScenarioGroup
{
	private enum ScenarioListId
	{
		MAIN,
		GET_CARD_PLAY,
		GET_DECK_SELECTION,
		GET_DISCARDS_SELECTION,
		GET_ARTIFACT_SELECTION,
		GET_HAND_SELECTION,
		GET_LINCHPIN_SELECTION,
		GET_RIPPLEPOINT_SELECTION,
		GET_ID_SELECTION,
		GET_MISSION_SELECTION,
		GET_DIRECTION_SELECTION,
		END_TURN,
		BETWEEN_TURNS;

		private static final ScenarioListId[] array = {MAIN, GET_CARD_PLAY, GET_DECK_SELECTION, GET_DISCARDS_SELECTION, GET_ARTIFACT_SELECTION, GET_HAND_SELECTION, GET_LINCHPIN_SELECTION, GET_RIPPLEPOINT_SELECTION, GET_ID_SELECTION, GET_MISSION_SELECTION, GET_DIRECTION_SELECTION, END_TURN, BETWEEN_TURNS};
		public static ScenarioListId[] array()
		{
			ScenarioListId[] returnValue = new ScenarioListId[array.length];
			for(int i = 0; i < array.length; i++)
			{
				returnValue[i] = array[i];
			}
			return returnValue;
		}
		public static ScenarioListId array(int index)
		{
			return array[index];
		}
	}

	private final AI_PlayCard playCard;
	private final Hashtable<ScenarioListId, ArrayList<AI_Scenario>> scenarios = new Hashtable<ScenarioListId, ArrayList<AI_Scenario>>();

	private int tryAgainCount;
	private int skips;

	public AI_CardScenarioGroup(Node cardScenarioGroupNode, String playCard_in) throws AI_FatalException
	{
		playCard = AI_Globals.playCardTable.get(playCard_in);

		NodeList list = cardScenarioGroupNode.getChildNodes();
		Node inputStateScenariosNode;

		ScenarioListId currentListId;
		ArrayList<AI_Scenario> currentScenarioList;
		for(int i = 0; list.item(i) != null; i++)
		{
			inputStateScenariosNode = list.item(i);
			if(inputStateScenariosNode.getNodeName().toLowerCase().equals("inputstatescenarios"))
			{
				currentListId = getState(inputStateScenariosNode);
				currentScenarioList = getScenarios(inputStateScenariosNode);
				currentScenarioList.add(getDefaultScenario(currentListId));
				scenarios.put(currentListId, currentScenarioList);
			}
			else if(inputStateScenariosNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_CardScenarioGroup, constructor: Invalid node name!  Name:" + inputStateScenariosNode.getNodeName());
			}
		}
		addDefaultScenarios();
	}




	private AI_Scenario getDefaultScenario(ScenarioListId listId) throws AI_FatalException
	{
		switch(listId)
		{
			case MAIN:
				return new AI_Scenario(new AI_MainSpecifier());
			case GET_CARD_PLAY:
				return new AI_Scenario(new AI_CardPlaySpecifier());
			case GET_DECK_SELECTION:
				return new AI_Scenario(new AI_DeckSelectionSpecifier());
			case GET_DISCARDS_SELECTION:
				return new AI_Scenario(new AI_DiscardsSelectionSpecifier());
			case GET_ARTIFACT_SELECTION:
				return new AI_Scenario(new AI_ArtifactSelectionSpecifier());
			case GET_HAND_SELECTION:
				return new AI_Scenario(new AI_HandSelectionSpecifier());
			case GET_LINCHPIN_SELECTION:
				return new AI_Scenario(new AI_LinchpinSelectionSpecifier());
			case GET_RIPPLEPOINT_SELECTION:
				return new AI_Scenario(new AI_RipplepointSelectionSpecifier());
			case GET_ID_SELECTION:
				return new AI_Scenario(new AI_IdSelectionSpecifier());
			case GET_MISSION_SELECTION:
				return new AI_Scenario(new AI_MissionSelectionSpecifier());
			case GET_DIRECTION_SELECTION:
				return new AI_Scenario(new AI_DirectionSelectionSpecifier());
			case END_TURN:
				return new AI_Scenario(new AI_EndTurnSpecifier());
			case BETWEEN_TURNS:
				return new AI_Scenario(new AI_BetweenTurnsSpecifier());
			default:
				throw new AI_FatalException("AI_CardScenarioGroup.getDefaultScenario(): Invalid scenario list id!");
		}
	}

	public CN_InputMessage checkAndExecute(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission, boolean tryAgain) throws AI_FatalException
	{
		if(!tryAgain)
		{
			tryAgainCount = 0;
			skips = 0;
		}
		else
		{
			tryAgainCount++;
			if(tryAgainCount >= AI_Constants.MAX_TRIES)
			{
				tryAgainCount = 0;
				skips++;
			}
		}

		ScenarioListId listId = getStateFromInputState(gameState.currentInputState);

		if(listId == null)
		{
			throw new AI_FatalException("AI_CardScenarioGroup.checkAndExecute(): Invalid Input State!");
		}

		ArrayList<AI_Scenario> scenarioList = scenarios.get(listId);

		int skipsUsed = 0;

		CN_InputMessage currentMessage;
		for(int i = 0; i < scenarioList.size(); i++)
		{
			currentMessage = scenarioList.get(i).checkAndExecute(gameState, timeline, id, mission);
			if(currentMessage != null)
			{
				if(skipsUsed < skips)
				{
					skipsUsed++;
					if(i == scenarioList.size() - 1)
						i--;
					continue;
				}
				return currentMessage;
			}
			if(i == scenarioList.size() - 1)
				i--;
		}


		throw new AI_FatalException("AI_CardScenarioGroup.checkAndExecute(): No valid input message was returned!  State: " + listId);
	}


	private ScenarioListId getStateFromInputState(CN_Constants.InputState state)
	{
		switch(state)
		{
			case MAIN:
				return ScenarioListId.MAIN;
			case GET_CARD_PLAY:
				return ScenarioListId.GET_CARD_PLAY;
			case GET_DECK_SELECTION:
				return ScenarioListId.GET_DECK_SELECTION;
			case GET_DISCARDS_SELECTION:
				return ScenarioListId.GET_DISCARDS_SELECTION;
			case GET_ARTIFACT_SELECTION:
				return ScenarioListId.GET_ARTIFACT_SELECTION;
			case GET_HAND_SELECTION:
				return ScenarioListId.GET_HAND_SELECTION;
			case GET_LINCHPIN_SELECTION:
				return ScenarioListId.GET_LINCHPIN_SELECTION;
			case GET_RIPPLEPOINT_SELECTION:
				return ScenarioListId.GET_RIPPLEPOINT_SELECTION;
			case GET_ID_SELECTION:
				return ScenarioListId.GET_ID_SELECTION;
			case GET_MISSION_SELECTION:
				return ScenarioListId.GET_MISSION_SELECTION;
			case GET_DIRECTION_SELECTION:
				return ScenarioListId.GET_DIRECTION_SELECTION;
			case END_TURN:
				return ScenarioListId.END_TURN;
			case BETWEEN_TURNS:
				return ScenarioListId.BETWEEN_TURNS;
			default:
				return null;
		}
	}



	private ScenarioListId getState(Node inputStateScenariosNode) throws AI_FatalException
	{
		NamedNodeMap attributes = inputStateScenariosNode.getAttributes();
		Node attributeNode = attributes.getNamedItem(AI_Constants.STATE_ATTRIBUTE_STRING);

		ScenarioListId returnValue;
		if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.INPUT_STATE_SCENARIOS_STATE_MAIN.toLowerCase()))
			returnValue = ScenarioListId.MAIN;
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.INPUT_STATE_SCENARIOS_STATE_GET_CARD_PLAY.toLowerCase()))
			returnValue = ScenarioListId.GET_CARD_PLAY;
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.INPUT_STATE_SCENARIOS_STATE_GET_DECK_SELECTION.toLowerCase()))
			returnValue = ScenarioListId.GET_DECK_SELECTION;
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.INPUT_STATE_SCENARIOS_STATE_GET_DISCARDS_SELECTION.toLowerCase()))
			returnValue = ScenarioListId.GET_DISCARDS_SELECTION;
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.INPUT_STATE_SCENARIOS_STATE_GET_ARTIFACT_SELECTION.toLowerCase()))
			returnValue = ScenarioListId.GET_ARTIFACT_SELECTION;
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.INPUT_STATE_SCENARIOS_STATE_GET_HAND_SELECTION.toLowerCase()))
			returnValue = ScenarioListId.GET_HAND_SELECTION;
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.INPUT_STATE_SCENARIOS_STATE_GET_LINCHPIN_SELECTION.toLowerCase()))
			returnValue = ScenarioListId.GET_LINCHPIN_SELECTION;
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.INPUT_STATE_SCENARIOS_STATE_GET_RIPPLEPOINT_SELECTION.toLowerCase()))
			returnValue = ScenarioListId.GET_RIPPLEPOINT_SELECTION;
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.INPUT_STATE_SCENARIOS_STATE_GET_ID_SELECTION.toLowerCase()))
			returnValue = ScenarioListId.GET_ID_SELECTION;
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.INPUT_STATE_SCENARIOS_STATE_GET_MISSION_SELECTION.toLowerCase()))
			returnValue = ScenarioListId.GET_MISSION_SELECTION;
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.INPUT_STATE_SCENARIOS_STATE_GET_DIRECTION_SELECTION.toLowerCase()))
			returnValue = ScenarioListId.GET_DIRECTION_SELECTION;
		else
			throw new AI_FatalException("AI_CardScenarioGroup.getState(): Invalid action type!  Type: " + attributeNode.getTextContent());

		return returnValue;
	}


	private ArrayList<AI_Scenario> getScenarios(Node inputStateScenariosNode) throws AI_FatalException
	{
		NodeList list = inputStateScenariosNode.getChildNodes();
		Node scenarioNode;

		ArrayList<AI_Scenario> returnValue = new ArrayList<AI_Scenario>();
		for(int i = 0; list.item(i) != null; i++)
		{
			scenarioNode = list.item(i);
			if(scenarioNode.getNodeName().toLowerCase().equals("scenario"))
			{
				returnValue.add(new AI_Scenario(scenarioNode));
			}
			else if(scenarioNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_CardScenarioGroup.getScenarios: Invalid node name!  Name:" + scenarioNode.getNodeName());
			}
		}

		return returnValue;
	}


	private void addDefaultScenarios() throws AI_FatalException
	{
		ArrayList<AI_Scenario> newList;
		for(ScenarioListId listId : ScenarioListId.array())
		{
			if(scenarios.get(listId) == null)
			{
//if(listId == ScenarioListId.GET_LINCHPIN_SELECTION)
//System.out.println("yo");
				newList = new ArrayList<AI_Scenario>();
				newList.add(getDefaultScenario(listId));
				scenarios.put(listId, newList);
			}
		}
	}




	public void debugPrint()
	{
		if(playCard != null)
			System.out.println("PlayCard: " + playCard.identity);
		else
			System.out.println("PlayCard: null");

		Enumeration<ScenarioListId> keys = scenarios.keys();
		ScenarioListId key;
		while(keys.hasMoreElements())
		{
			key = keys.nextElement();
			System.out.println(key + ":");
			for(AI_Scenario scenario : scenarios.get(key))
			{
				scenario.debugPrint();
			}
		}
	}

}
