package cnpc.ai;

import cnpc.engine.CN_Constants;
import cnpc.engine.CN_GameState;

import java.util.*;

class AI_GameStateTimeline
{
	private final Hashtable<Integer, TimelineCardData> byCardNumber = new Hashtable<Integer, TimelineCardData>();
	private final Hashtable<String, TimelineCardData> byIdentity = new Hashtable<String, TimelineCardData>();

	public AI_GameStateTimeline(CN_GameState gameState)
	{
		CN_GameState.TimelineCardData[] gameStateTimeline = gameState.timeline;
		TimelineCardData[] timeline = new TimelineCardData[gameStateTimeline.length];
		int patchedBy;
		for(int i = 0; i < timeline.length; i++)
		{
			if(gameStateTimeline[i].patchedBy == CN_Constants.NULL_PATCH)
				patchedBy = AI_Constants.NULL_PATCH;
			else
				patchedBy = gameStateTimeline[i].patchedBy;
			timeline[i] = new TimelineCardData(gameStateTimeline[i].cardNumber, gameStateTimeline[i].isFlipped, patchedBy, gameStateTimeline[i].isTerminal, gameStateTimeline[i].isLegal, gameStateTimeline[i].year);
		}

		for(TimelineCardData card : timeline)
		{
			byCardNumber.put(card.cardNumber, card);
			byIdentity.put(AI_Constants.timelineCardDirectory.get(card.cardNumber).identity, card);
		}
	}


	public TimelineCardData getByNumber(int cardNumber)
	{
		return byCardNumber.get(cardNumber);
	}

	public TimelineCardData getByIdentity(String identity)
	{
		return byIdentity.get(identity);
	}

	public TimelineCardData getByCard(AI_TimelineCard card)
	{
		return getByIdentity(card.identity);
	}




	public static class TimelineCardData
	{
		public final int cardNumber;
		public final boolean isFlipped;
		public final int patchedBy;
		public final boolean isTerminal;
		public final boolean isLegal;
		public final int year;

		public TimelineCardData(int cardNumber_in, boolean isFlipped_in, int patchedBy_in, boolean isTerminal_in, boolean isLegal_in, int year_in)
		{
			cardNumber = cardNumber_in;
			isFlipped = isFlipped_in;
			patchedBy = patchedBy_in;
			isTerminal = isTerminal_in;
			isLegal = isLegal_in;
			year = year_in;
		}
	}


}
