package cnpc.ai;

import cnpc.engine.*;

import java.util.*;

class AI_IdCard extends AI_GameCard
{
	private final ArrayList<VictoryOption> victoryOptions;

	public AI_IdCard(String identity_in, String cardName_in, ArrayList<ArrayList<String>> victoryOptions_in) throws AI_FatalException
	{
		super(identity_in, cardName_in);

		victoryOptions = new ArrayList<VictoryOption>();

		for(int i = 0; i < victoryOptions_in.size(); i++)
		{
			victoryOptions.add(new VictoryOption(victoryOptions_in.get(i)));
		}
	}



	public boolean isLinchpinForId(AI_Linchpin linchpin) throws AI_FatalException
	{
		AI_Ripplepoint[] ripplepoints = linchpin.getRipples();

		for(AI_Ripplepoint ripplepoint : ripplepoints)
		{
			for(VictoryOption option : victoryOptions)
			{
				if(option.containsTarget(ripplepoint.identity))
					return true;
			}
		}
		return false;
	}

	public boolean isRipplepointForId(AI_Ripplepoint ripplepoint)
	{
		for(VictoryOption option : victoryOptions)
		{
			if(option.containsTarget(ripplepoint.identity))
				return true;
		}
		return false;
	}

	public boolean isPatchForId(AI_Patch patch)
	{
		for(VictoryOption option : victoryOptions)
		{
			if(option.containsPatchedBy(patch.identity))
				return true;
		}
		return false;
	}


	public boolean isLinchpinStateForId(AI_Linchpin linchpin, boolean isFlipped) throws AI_FatalException
	{
		for(VictoryOption option : victoryOptions)
		{
			if(option.linchpinSatisfies(linchpin, isFlipped))
				return true;
		}
		return false;
	}

	public boolean isRipplepointStateForId(AI_Ripplepoint ripplepoint, AI_Patch patch, boolean isFlipped)
	{
		for(VictoryOption option : victoryOptions)
		{
			if(option.patchSatisfies(ripplepoint, patch, isFlipped))
				return true;
		}
		return false;
	}


	private class VictoryOption
	{
		public final ArrayList<PrereqPair> prereqs;

		public VictoryOption(ArrayList<String> prereqs_in) throws AI_FatalException
		{
			if(prereqs_in.size() % 2 != 0)
			{
				throw new AI_FatalException("AI_IdCard.VictoryOption, constructor: input ArrayList must have an even number of items!");
			}

			prereqs = new ArrayList<PrereqPair>();

			String currentPrereqString;
			String currentTarget = null;
			String currentPatchedBy = null;

			for(int i = 0; i < prereqs_in.size(); i++)
			{
				currentPrereqString = prereqs_in.get(i);

				if(i % 2 == 0)
				{
					currentTarget = currentPrereqString;
					if(currentTarget == null)
					{
						throw new AI_FatalException("AI_IdCard.VictoryOption, constructor: Targets are ripplepoints and cannot be null!");
					}

				}
				else
				{
					currentPatchedBy = currentPrereqString;
					prereqs.add(new PrereqPair(currentTarget, currentPatchedBy));
				}
			}
		}

		public boolean containsTarget(String target)
		{
			for(PrereqPair prereq : prereqs)
			{
				if(prereq.target.equals(target))
					return true;
			}
			return false;
		}

		public boolean containsPatchedBy(String target)
		{
			for(PrereqPair prereq : prereqs)
			{
				if(prereq.patchedBy == null)
					return false;
				else if(prereq.patchedBy.equals(target))
					return true;
			}
			return false;
		}


		public boolean linchpinSatisfies(AI_Linchpin linchpin, boolean isFlipped) throws AI_FatalException
		{
			for(PrereqPair prereq : prereqs)
			{
				if(prereq.linchpinSatisfies(linchpin, isFlipped))
					return true;
			}
			return false;
		}

		public boolean patchSatisfies(AI_Ripplepoint ripplepoint, AI_Patch patch, boolean isFlipped)
		{
			for(PrereqPair prereq : prereqs)
			{
				if(prereq.patchSatisfies(ripplepoint, patch, isFlipped))
					return true;
			}
			return false;
		}
	}


	private class PrereqPair
	{
		public final String target;
		public final String patchedBy;

		public PrereqPair(String target_in, String patchedBy_in)
		{
			patchedBy = patchedBy_in;
			target = target_in;
		}

		public PrereqPair(String target_in)
		{
			this(target_in, null);
		}


		public boolean linchpinSatisfies(AI_Linchpin linchpin, boolean isFlipped) throws AI_FatalException
		{
			AI_Ripplepoint[] ripplepoints = linchpin.getRipples();
			for(AI_Ripplepoint ripplepoint : ripplepoints)
			{
				if((ripplepoint.identity.equals(target))&&((patchedBy == null) != isFlipped))
					return true;
			}
			return false;
		}

		public boolean patchSatisfies(AI_Ripplepoint ripplepoint, AI_Patch patch, boolean isFlipped)
		{
			if(ripplepoint.identity.equals(target))
			{
				if((patch == null)&&(patchedBy == null))
				{
					if(!isFlipped)
						return true;
				}
				else if((patch == null)||(patchedBy == null))
				{
					return false;
				}
				else if(patchedBy.equals(patch.identity))
				{
					return true;
				}
			}
			return false;
		}

	}


}
