package cnpc.ai;

import cnpc.engine.*;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;

class AI_Initializer
{
	public class PersonalityInfo
	{
		public final String path;
		public final String name;
		public final int weight;

		public PersonalityInfo(String path_in, String name_in, int weight_in)
		{
			path = path_in;
			name = name_in;
			weight = weight_in;
		}
	}





	private String pathToEngineData = null;
	private String pathToAiData = null;

	public void setVariant(String variant)
	{
		pathToEngineData = AI_Constants.PATH_TO_DATA + variant + AI_Constants.PATH_TO_ENGINE_DATA;
		pathToAiData = AI_Constants.PATH_TO_DATA + variant + AI_Constants.PATH_TO_AI_DATA;
	}




	public ArrayList<PersonalityInfo> getPersonalityInfo() throws AI_FatalException
	{
		ArrayList<PersonalityInfo> returnValue = new ArrayList<PersonalityInfo>();

		String path;
		String name;
		int weight;

		File[] xmlFiles = getPersonalityFiles();

		Document document;
		Node rootNode;
		String personalityName;
		for(File xmlFile : xmlFiles)
		{
			document = AI_XmlUtils.loadDocument(xmlFile);
			rootNode = AI_XmlUtils.getRoot(document, "Personality");
			path = xmlFile.getName();
			name = getPersonalityName(rootNode);
			weight = getPersonalityWeight(rootNode);
			returnValue.add(new PersonalityInfo(path, name, weight));
		}

		return returnValue;
	}




	public void initializeGame(CN_GameState initialState) throws AI_FatalException
	{
		clearGlobals();
		initializeGlobals();
		initializeDirectories(initialState);
	}


	public AI_Personality getPersonality(String path) throws AI_FatalException
	{
		File xmlFile = null;
		File[] xmlFiles = getPersonalityFiles();
		for(File file : xmlFiles)
		{
			if(file.getName().equals(path))
			{
				xmlFile = file;
				break;
			}
		}
		if(xmlFile == null)
			throw new AI_FatalException("AI_Initializer.getPersonality(): File \"" + path + "\" was not found!");

		Document document = AI_XmlUtils.loadDocument(xmlFile);
		Node rootNode = AI_XmlUtils.getRoot(document, "Personality");

		return new AI_Personality(rootNode);
	}

	private File[] getPersonalityFiles() throws AI_FatalException
	{
		File directory = new File(pathToAiData);
		if(!directory.isDirectory())
			throw new AI_FatalException("AI_Initializer.getPersonalityFiles(): File \"" + directory.getAbsolutePath() + "\" is not a directory!");

		File[] allFiles = directory.listFiles();
		ArrayList<File> xmlFiles = new ArrayList<File>();

		for(File file : allFiles)
		{
			if(file.getName().toLowerCase().endsWith(AI_Constants.XML_FILE_EXTENSION))
				xmlFiles.add(file);
		}

		File[] returnValue = new File[xmlFiles.size()];
		for(int i = 0; i < xmlFiles.size(); i++)
		{
			returnValue[i] = xmlFiles.get(i);
		}

		return returnValue;
	}

	private String getPersonalityName(Node personalityRoot)
	{
		return AI_XmlUtils.getAttributeValue(personalityRoot, AI_Constants.NAME_ATTRIBUTE_STRING);
	}

	private int getPersonalityWeight(Node personalityRoot)
	{
		String weightString = AI_XmlUtils.getAttributeValue(personalityRoot, AI_Constants.WEIGHT_ATTRIBUTE_STRING);
		if(weightString == null)
			return 0;
		else
			return Integer.valueOf(weightString);
	}






	private void clearGlobals()
	{
		AI_Globals.playCardTable.clear();
		AI_Globals.playCardList.clear();

		AI_Globals.linchpinTable.clear();
		AI_Globals.linchpinList.clear();

		AI_Globals.ripplepointTable.clear();
		AI_Globals.ripplepointList.clear();

		AI_Globals.patchTable.clear();
		AI_Globals.patchList.clear();

		AI_Globals.artifactTable.clear();
		AI_Globals.artifactList.clear();

		AI_Globals.idTable.clear();

		AI_Globals.missionTable.clear();
	}

	//Order:  Play cards first, then timeline cards, then ids and missions
	private void initializeGlobals() throws AI_FatalException
	{
		Node playCardRoot = getPlayCardRoot();
		ArrayList<AI_PlayCard> playCards = createPlayCards(playCardRoot);
		loadPlayCardGlobals(playCards);

		Node timelineCardRoot = getTimelineCardRoot();
		createAndLoadTimelineGlobals(timelineCardRoot);

		ArrayList<AI_IdCard> idCards = loadIds();
		loadIdCardGlobals(idCards);

		ArrayList<AI_MissionCard> missionCards = loadMissions();
		loadMissionCardGlobals(missionCards);
	}

	private void initializeDirectories(CN_GameState initialState) throws AI_FatalException
	{
		initializePlayCardDirectory(initialState.playCardDirectory);
		initializeTimelineCardDirectory(initialState.timelineCardDirectory);
		initializeIdCardDirectory(initialState.idCardDirectory);
		initializeMissionCardDirectory(initialState.missionCardDirectory);
	}

	private void initializePlayCardDirectory(Hashtable<Integer, String> gameStateTable)
	{
		AI_Constants.playCardDirectory.clear();
		Enumeration<Integer> keys = gameStateTable.keys();
		Integer key;
		AI_PlayCard playCard;
		while(keys.hasMoreElements())
		{
			key = keys.nextElement();
			playCard = AI_Globals.playCardTable.get(gameStateTable.get(key));
			AI_Constants.playCardDirectory.put(key, playCard);
		}
	}
	private void initializeTimelineCardDirectory(Hashtable<Integer, String> gameStateTable) throws AI_FatalException
	{
		AI_Constants.timelineCardDirectory.clear();
		Enumeration<Integer> keys = gameStateTable.keys();
		Integer key;
		AI_TimelineCard timelineCard;
		while(keys.hasMoreElements())
		{
			key = keys.nextElement();
			timelineCard = AI_Globals.linchpinTable.get(gameStateTable.get(key));
			if(timelineCard == null)
				timelineCard = AI_Globals.ripplepointTable.get(gameStateTable.get(key));

			if(timelineCard == null)
				throw new AI_FatalException("AI_Initializer.initializeTimelineCardDirectory(): A timeline card was not found!  Card: " + gameStateTable.get(key));

			AI_Constants.timelineCardDirectory.put(key, timelineCard);
		}
	}
	private void initializeIdCardDirectory(Hashtable<Integer, String> gameStateTable)
	{
		AI_Constants.idCardDirectory.clear();
		Enumeration<Integer> keys = gameStateTable.keys();
		Integer key;
		AI_IdCard idCard;
		while(keys.hasMoreElements())
		{
			key = keys.nextElement();
			idCard = AI_Globals.idTable.get(gameStateTable.get(key));
			AI_Constants.idCardDirectory.put(key, idCard);
		}
	}
	private void initializeMissionCardDirectory(Hashtable<Integer, String> gameStateTable)
	{
		AI_Constants.missionCardDirectory.clear();
		Enumeration<Integer> keys = gameStateTable.keys();
		Integer key;
		AI_MissionCard missionCard;
		while(keys.hasMoreElements())
		{
			key = keys.nextElement();
			missionCard = AI_Globals.missionTable.get(gameStateTable.get(key));
			AI_Constants.missionCardDirectory.put(key, missionCard);
		}
	}


	private Node getPlayCardRoot() throws AI_FatalException
	{
		Document document = AI_XmlUtils.loadDocument(pathToEngineData + AI_Constants.ENGINE_PLAY_CARDS_PATH);
		return AI_XmlUtils.getRoot(document, "PlayCards");
	}

	private Node getTimelineCardRoot() throws AI_FatalException
	{
		Document document = AI_XmlUtils.loadDocument(pathToEngineData + AI_Constants.ENGINE_TIMELINE_CARDS_PATH);
		return AI_XmlUtils.getRoot(document, "Timeline");
	}






	//NOTE: Initializations in this method must be done in precise order
	private void createAndLoadTimelineGlobals(Node rootNode) throws AI_FatalException
	{
		ArrayList<AI_Linchpin> linchpins = new ArrayList<AI_Linchpin>();
		ArrayList<AI_Ripplepoint> ripplepoints = new ArrayList<AI_Ripplepoint>();

		NodeList list = rootNode.getChildNodes();
		Node childNode;

		for(int i = 0; list.item(i) != null; i++)
		{
			childNode = list.item(i);
			if(childNode.getNodeName().toLowerCase().equals("timelinecard"))
			{
				if(trueIfLinchpin(childNode))
				{
					linchpins.add(new AI_Linchpin(getTimelineCardIdentity(childNode), getTimelineCardName(childNode)));
				}
			}
		}

		for(AI_Linchpin linchpin : linchpins)
		{
			AI_Globals.linchpinList.add(linchpin);
			AI_Globals.linchpinTable.put(linchpin.identity, linchpin);
		}

		for(int i = 0; list.item(i) != null; i++)
		{
			childNode = list.item(i);
			if(childNode.getNodeName().toLowerCase().equals("timelinecard"))
			{
				if(!trueIfLinchpin(childNode))
				{
					ripplepoints.add(new AI_Ripplepoint(getTimelineCardIdentity(childNode), getTimelineCardName(childNode), childNode));
				}
			}
		}

		for(AI_Ripplepoint ripplepoint : ripplepoints)
		{
			AI_Globals.ripplepointList.add(ripplepoint);
			AI_Globals.ripplepointTable.put(ripplepoint.identity, ripplepoint);
		}

		AI_Ripplepoint[] allRipplepoints = new AI_Ripplepoint[ripplepoints.size()];
		for(int i = 0; i < ripplepoints.size(); i++)
		{
			allRipplepoints[i] = ripplepoints.get(i);
		}

		for(AI_Linchpin linchpin : linchpins)
		{
			linchpin.setRipples(allRipplepoints);
		}
	}




	private boolean trueIfLinchpin(Node timelineCardNode) throws AI_FatalException
	{
		NodeList list = timelineCardNode.getChildNodes();
		Node childNode;

		for(int i = 0; list.item(i) != null; i++)
		{
			childNode = list.item(i);
			if(childNode.getNodeName().toLowerCase().equals("cardtype"))
			{
				if(childNode.getTextContent().toLowerCase().equals(AI_Constants.TIMELINE_CARD_TYPE_LINCHPIN.toLowerCase()))
					return true;
				else if(childNode.getTextContent().toLowerCase().equals(AI_Constants.TIMELINE_CARD_TYPE_RIPPLEPOINT.toLowerCase()))
					return false;
				else
					throw new AI_FatalException("AI_Initializer.trueIfLinchpin(): Invalid card type!  Type: " + childNode.getTextContent());
			}
		}

		throw new AI_FatalException("AI_Initializer.trueIfLinchpin(): No card type!");
	}

	private String getTimelineCardIdentity(Node timelineCardNode) throws AI_FatalException
	{
		String returnValue = AI_XmlUtils.getElementValue(timelineCardNode, "Identity", false);

		if(returnValue == null)
			throw new AI_FatalException("AI_Initializer.getTimelineCardIdentity(): No identity present!");

		return returnValue;
	}

	private String getTimelineCardName(Node timelineCardNode) throws AI_FatalException
	{
		String returnValue = AI_XmlUtils.getElementValue(timelineCardNode, "CardName", false);

		if(returnValue == null)
			throw new AI_FatalException("AI_Initializer.getTimelineCardName(): No card name present!");

		return returnValue;
	}








	private void loadPlayCardGlobals(ArrayList<AI_PlayCard> cardList)
	{
		for(AI_PlayCard playCard : cardList)
		{
			AI_Globals.playCardList.add(playCard);
			AI_Globals.playCardTable.put(playCard.identity, playCard);

			if(playCard instanceof AI_Patch)
			{
				AI_Globals.patchList.add((AI_Patch)playCard);
				AI_Globals.patchTable.put(playCard.identity, (AI_Patch)playCard);
			}
			else if(playCard instanceof AI_Artifact)
			{
				AI_Globals.artifactList.add((AI_Artifact)playCard);
				AI_Globals.artifactTable.put(playCard.identity, (AI_Artifact)playCard);
			}
		}
	}









	private ArrayList<AI_PlayCard> createPlayCards(Node rootNode) throws AI_FatalException
	{
		ArrayList<AI_PlayCard> returnValue = new ArrayList<AI_PlayCard>();

		NodeList list = rootNode.getChildNodes();
		Node childNode;

		for(int i = 0; list.item(i) != null; i++)
		{
			childNode = list.item(i);
			if(childNode.getNodeName().toLowerCase().equals("playcard"))
			{
				returnValue.add(createPlayCardFromNode(childNode));
			}
		}

		return returnValue;
	}
	
		


	private AI_PlayCard createPlayCardFromNode(Node playCardNode) throws AI_FatalException
	{
		AI_PlayCard returnValue;

		String identity = null;
		String cardName = null;
		String playCardClassString = null;

		String authClass = null;
		String authenticityString = null;
		int authenticity = AI_Constants.NONEXISTENT_AUTHENTICITY;

		NodeList list = playCardNode.getChildNodes();
		Node childNode;

		for(int i = 0; list.item(i) != null; i++)
		{
			childNode = list.item(i);
			if(childNode.getNodeName().toLowerCase().equals("identity"))
			{
				identity = childNode.getTextContent();
			}
			if(childNode.getNodeName().toLowerCase().equals("cardname"))
			{
				cardName = childNode.getTextContent();
			}
			else if(childNode.getNodeName().toLowerCase().equals("cardtype"))
			{
				playCardClassString = childNode.getTextContent();
			}
			else if(childNode.getNodeName().toLowerCase().equals("authclass"))
			{
				authClass = childNode.getTextContent();
			}
			else if(childNode.getNodeName().toLowerCase().equals("authenticity"))
			{
				authenticityString = childNode.getTextContent();
				authenticity = Integer.valueOf(authenticityString);
			}
		}

		if(identity == null)
			throw new AI_FatalException("AI_Initializer.createPlayCardFromNode(): Identity not found!");
		if(cardName == null)
			throw new AI_FatalException("AI_Initializer.createPlayCardFromNode(): Card Name not found!");
		if(playCardClassString == null)
			throw new AI_FatalException("AI_Initializer.createPlayCardFromNode(): Card Type not found!");

		if(playCardClassString.toLowerCase().equals("artifact"))
		{
			if(authClass == null)
				returnValue = new AI_Artifact(identity, cardName);
			else
				returnValue = new AI_Artifact(identity, cardName, authClass, authenticity);
		}
		else if(playCardClassString.toLowerCase().equals("patch"))
			returnValue = new AI_Patch(identity, cardName);
		else if(playCardClassString.toLowerCase().equals("inverter"))
			returnValue = new AI_Inverter(identity, cardName);
		else if(playCardClassString.toLowerCase().equals("action"))
			returnValue = new AI_Action(identity, cardName);
		else if(playCardClassString.toLowerCase().equals("timewarp"))
			returnValue = new AI_Timewarp(identity, cardName);
		else
			throw new AI_FatalException("AI_Initializer.createPlayCardFromNode(): Invalid Card Type!  Type: " + playCardClassString);

		return returnValue;
	}






















	private void loadIdCardGlobals(ArrayList<AI_IdCard> cardList)
	{
		for(AI_IdCard idCard : cardList)
		{
			AI_Globals.idTable.put(idCard.identity, idCard);
		}
	}


	//Borrowed from Engine's initializer
	//NOTE: This should be done AFTER timeline cards are created and added to AI_Globals.ripplepointTable, etc.

	public ArrayList<AI_IdCard> loadIds() throws AI_FatalException
	{
		ArrayList<AI_IdCard> loadedIds = new ArrayList<AI_IdCard>();

		Document document = AI_XmlUtils.loadDocument(pathToEngineData + AI_Constants.ENGINE_ID_CARDS_PATH);

		NodeList list;
		Node currentNode;

		list = document.getElementsByTagName("IdCard");
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			loadedIds.add(createIdCard(loadIdCard(currentNode)));
		}

		return loadedIds;
	}



	private AI_IdCard createIdCard(LoadedIdCard card) throws AI_FatalException
	{
		AI_IdCard returnValue = new AI_IdCard(card.identity, card.cardName, card.victoryOptions);
		return returnValue;
	}


	private LoadedIdCard loadIdCard(Node idCardNode) throws AI_FatalException
	{
		NodeList list = idCardNode.getChildNodes();
		Node currentNode;

		String newIdentity = null;
		String newCardName = null;
		ArrayList<ArrayList<String>> newVictoryOptions = new ArrayList<ArrayList<String>>();
		int newAmountInDeck = 0;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("identity"))
			{
				newIdentity = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("cardname"))
			{
				newCardName = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("victoryoption"))
			{
				newVictoryOptions.add(idCard_loadVictoryOption(currentNode));
			}
			else if(currentNode.getNodeName().toLowerCase().equals("amountindeck"))
			{
				newAmountInDeck = Integer.parseInt(currentNode.getTextContent());
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_Initializer.loadIdCard(): Invalid node name!  Name: " + currentNode.getNodeName());
			}
		}
		
		if(newIdentity == null)
			throw new AI_FatalException("AI_Initializer.loadIdCard(): Identity not found!");
		if(newCardName == null)
			throw new AI_FatalException("AI_Initializer.loadIdCard(): Card name not found!");

		return new LoadedIdCard(newIdentity, newCardName, newVictoryOptions, newAmountInDeck);
	}



	private ArrayList<String> idCard_loadVictoryOption(Node victoryOptionNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = victoryOptionNode.getChildNodes();
		Node currentNode;
		IdCard_LoadedPrereqPair currentPrereq;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("prereq"))
			{
				currentPrereq = idCard_loadPrereq(currentNode);
				returnValue.add(currentPrereq.target);
				returnValue.add(currentPrereq.patchedBy);
			}
		}


		return returnValue;
	}



	private IdCard_LoadedPrereqPair idCard_loadPrereq(Node prereqNode) throws AI_FatalException
	{
		String newTarget = null;
		String newPatchedBy = null;

		NodeList list = prereqNode.getChildNodes();
		Node currentNode;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("target"))
			{
				newTarget = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("patchedby"))
			{
				newPatchedBy = currentNode.getTextContent();
				if(newPatchedBy.equals(""))
				{
					newPatchedBy = null;
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_Initializer.idCard_loadPrereq(): Invalid node name!");
			}
		}

		if(AI_Globals.ripplepointTable.get(newTarget) == null)
		{
			throw new AI_FatalException("AI_Initializer.idCard_loadPrereq(): Target ripplepoint " + newTarget + " does not exist!");
		}

		if(newPatchedBy != null)
		{
			AI_PlayCard validatingPlayCard = AI_Globals.playCardTable.get(newPatchedBy);
			if(validatingPlayCard == null)
			{
				throw new AI_FatalException("AI_Initializer.idCard_loadPrereq(): newPatchedBy patch " + newPatchedBy + " does not exist!");
			}
			else if(!(validatingPlayCard instanceof AI_Patch))
			{
				throw new AI_FatalException("AI_Initializer.idCard_loadPrereq(): PatchedBy " + newPatchedBy + " is not a patch!");
			}
		}

		return new IdCard_LoadedPrereqPair(newTarget, newPatchedBy);

	}



	private class LoadedIdCard
	{
		public final String identity;
		public final String cardName;
		public final ArrayList<ArrayList<String>> victoryOptions;
		public int amountInDeck;

		public LoadedIdCard(String identity_in, String cardName_in, ArrayList<ArrayList<String>> victoryOptions_in, int amountInDeck_in)
		{
			identity = identity_in;
			cardName = cardName_in;
			victoryOptions = victoryOptions_in;
			amountInDeck = amountInDeck_in;
		}
	}



	private class IdCard_LoadedPrereqPair
	{
		public final String target;
		public final String patchedBy;

		public IdCard_LoadedPrereqPair(String target_in, String patchedBy_in)
		{
			target = target_in;
			patchedBy = patchedBy_in;
		}
	}




























	private void loadMissionCardGlobals(ArrayList<AI_MissionCard> cardList)
	{
		for(AI_MissionCard missionCard : cardList)
		{
			AI_Globals.missionTable.put(missionCard.identity, missionCard);
		}
	}

	//Borrowed from Engine's initializer
	//NOTE: This should be done AFTER play cards are created and added to AI_Globals.playCardTable, etc.

	public ArrayList<AI_MissionCard> loadMissions() throws AI_FatalException
	{
		ArrayList<AI_MissionCard> loadedMissions = new ArrayList<AI_MissionCard>();

		Document document = AI_XmlUtils.loadDocument(pathToEngineData + AI_Constants.ENGINE_MISSION_CARDS_PATH);

		NodeList list;
		Node currentNode;

		list = document.getElementsByTagName("MissionCard");
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			loadedMissions.add(createMissionCard(loadMissionCard(currentNode)));
		}

		return loadedMissions;
	}



	private AI_MissionCard createMissionCard(LoadedMissionCard card) throws AI_FatalException
	{
		AI_MissionCard returnValue = new AI_MissionCard(card.identity, card.cardName, card.victoryOptionTargets, card.victoryOptionTargetTypes, card.victoryOptionRequiredAmounts);
		return returnValue;
	}


	private LoadedMissionCard loadMissionCard(Node missionCardNode) throws AI_FatalException
	{
		NodeList list = missionCardNode.getChildNodes();
		Node currentNode;

		String newIdentity = null;
		String newCardName = null;
		ArrayList<ArrayList<String>> newVictoryOptionTargets = new ArrayList<ArrayList<String>>();
		ArrayList<ArrayList<AI_MissionCard.TargetType>> newVictoryOptionTargetTypes = new ArrayList<ArrayList<AI_MissionCard.TargetType>>();
		ArrayList<ArrayList<Integer>> newVictoryOptionRequiredAmounts = new ArrayList<ArrayList<Integer>>();

		int newAmountInDeck = 0;

		MissionCard_LoadedVictoryOption currentVictoryOption;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("identity"))
			{
				newIdentity = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("cardname"))
			{
				newCardName = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("victoryoption"))
			{
				currentVictoryOption = missionCard_loadVictoryOption(currentNode);
				newVictoryOptionTargets.add(currentVictoryOption.targets);
				newVictoryOptionTargetTypes.add(currentVictoryOption.targetTypes);
				newVictoryOptionRequiredAmounts.add(currentVictoryOption.requiredAmounts);
			}
			else if(currentNode.getNodeName().toLowerCase().equals("amountindeck"))
			{
				newAmountInDeck = Integer.parseInt(currentNode.getTextContent());
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_Initializer, loadMissionCard(): Invalid node name!  Name: " + currentNode.getNodeName());
			}
		}
		
		if(newIdentity == null)
			throw new AI_FatalException("AI_Initializer.loadMissionCard(): Identity not found!");
		if(newCardName == null)
			throw new AI_FatalException("AI_Initializer.loadMissionCard(): Card name not found!");

		return new LoadedMissionCard(newIdentity, newCardName, newVictoryOptionTargets, newVictoryOptionTargetTypes, newVictoryOptionRequiredAmounts, newAmountInDeck);
	}




	private MissionCard_LoadedVictoryOption missionCard_loadVictoryOption(Node victoryOptionNode) throws AI_FatalException
	{
		ArrayList<String> targets = new ArrayList<String>();
		ArrayList<AI_MissionCard.TargetType> targetTypes = new ArrayList<AI_MissionCard.TargetType>();
		ArrayList<Integer> requiredAmounts = new ArrayList<Integer>();

		NodeList list = victoryOptionNode.getChildNodes();
		Node currentNode;
		MissionCard_LoadedPrereqTriple currentPrereq;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("prereq"))
			{
				currentPrereq = missionCard_loadPrereq(currentNode);
				targets.add(currentPrereq.target);
				targetTypes.add(currentPrereq.targetType);
				requiredAmounts.add(currentPrereq.requiredAmount);
			}
		}

		return new MissionCard_LoadedVictoryOption(targets, targetTypes, requiredAmounts);
	}




	private MissionCard_LoadedPrereqTriple missionCard_loadPrereq(Node prereqNode) throws AI_FatalException
	{
		String newTarget = null;
		String newTargetType = null;
		String newRequiredAmount = null;

		NodeList list = prereqNode.getChildNodes();
		Node currentNode;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("target"))
			{
				newTarget = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("targettype"))
			{
				newTargetType = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("requiredamount"))
			{
				newRequiredAmount = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_Initializer.missionCard_loadPrereq(): Invalid node name!");
			}
		}

		if(!newTargetType.toLowerCase().equals(AI_Constants.EXACT_TARGET_TYPE.toLowerCase()))
		{
			Enumeration<AI_PlayCard> allCards = AI_Globals.playCardTable.elements();

			boolean validated = false;

			AI_PlayCard currentCard;
			AI_Artifact currentArtifact;
			while(allCards.hasMoreElements())
			{
				currentCard = allCards.nextElement();
				if(currentCard instanceof AI_Artifact)
				{
					currentArtifact = (AI_Artifact)currentCard;
					if(currentArtifact.authClass.equals(newTarget))
					{
						validated = true;
					}
				}
			}
			if(!validated)
			{
				throw new AI_FatalException("AI_Initializer.missionCard_loadPrereq(): artifact with authClass " + newTarget + " does not exist!");
			}
		}
		else
		{
			AI_PlayCard validatingPlayCard = AI_Globals.playCardTable.get(newTarget);
			if(validatingPlayCard == null)
			{
				throw new AI_FatalException("AI_Initializer.missionCard_loadPrereq(): newTarget artifact " + newTarget + " does not exist!");
			}
			else if(!(validatingPlayCard instanceof AI_Artifact))
			{
				throw new AI_FatalException("AI_Initializer.missionCard_loadPrereq(): Mission target " + newTarget + " is not an artifact!");
			}
		}

		return new MissionCard_LoadedPrereqTriple(newTarget, newTargetType, newRequiredAmount);
	}








	private class LoadedMissionCard
	{
		public final String identity;
		public final String cardName;
		public final ArrayList<ArrayList<String>> victoryOptionTargets;
		public final ArrayList<ArrayList<AI_MissionCard.TargetType>> victoryOptionTargetTypes;
		public final ArrayList<ArrayList<Integer>> victoryOptionRequiredAmounts;

		public int amountInDeck;

		public LoadedMissionCard(String identity_in, String cardName_in, ArrayList<ArrayList<String>> victoryOptionTargets_in, ArrayList<ArrayList<AI_MissionCard.TargetType>> victoryOptionTargetTypes_in, ArrayList<ArrayList<Integer>> victoryOptionRequiredAmounts_in, int amountInDeck_in)
		{
			identity = identity_in;
			cardName = cardName_in;
			victoryOptionTargets = victoryOptionTargets_in;
			victoryOptionTargetTypes = victoryOptionTargetTypes_in;
			victoryOptionRequiredAmounts = victoryOptionRequiredAmounts_in;
			amountInDeck = amountInDeck_in;
		}
	}


	private class MissionCard_LoadedVictoryOption
	{
		public final ArrayList<String> targets;
		public final ArrayList<AI_MissionCard.TargetType> targetTypes;
		public final ArrayList<Integer> requiredAmounts;

		public MissionCard_LoadedVictoryOption(ArrayList<String> targets_in, ArrayList<AI_MissionCard.TargetType> targetTypes_in ,ArrayList<Integer> requiredAmounts_in)
		{
			targets = targets_in;
			targetTypes = targetTypes_in;
			requiredAmounts = requiredAmounts_in;
		}
	}


	private class MissionCard_LoadedPrereqTriple
	{
		public final String target;
		public final AI_MissionCard.TargetType targetType;
		public final int requiredAmount;

		public MissionCard_LoadedPrereqTriple(String target_in, String targetTypeString_in, String requiredAmountString_in) throws AI_FatalException, NumberFormatException
		{
			target = target_in;
			targetType = getTargetTypeFromString(targetTypeString_in);
			requiredAmount = Integer.parseInt(requiredAmountString_in);
		}

		private AI_MissionCard.TargetType getTargetTypeFromString(String targetTypeString) throws AI_FatalException
		{
			if(targetTypeString.toLowerCase().equals("exact"))
			{
				return AI_MissionCard.TargetType.EXACT;
			}
			else if(targetTypeString.toLowerCase().equals("mostauthentic"))
			{
				return AI_MissionCard.TargetType.MOST_AUTHENTIC;
			}
			else if(targetTypeString.toLowerCase().equals("notmostauthentic"))
			{
				return AI_MissionCard.TargetType.NOT_MOST_AUTHENTIC;
			}
			else
			{
				throw new AI_FatalException("AI_Initializer.getTargetTypeFromString(): invalid target type string!");
			}
		}

	}





}
