package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_Personality
{
	private final Hashtable<String, AI_CardScenarioGroup> scenarioGroupTable = new Hashtable<String, AI_CardScenarioGroup>();

	public AI_Personality(Node personalityNode) throws AI_FatalException
	{
		NodeList list = personalityNode.getChildNodes();
		Node cardScenarioGroupNode;

		String playCard;
		for(int i = 0; list.item(i) != null; i++)
		{
			cardScenarioGroupNode = list.item(i);
			if(cardScenarioGroupNode.getNodeName().toLowerCase().equals("cardscenariogroup"))
			{
				playCard = getPlayCard(cardScenarioGroupNode);
				scenarioGroupTable.put(playCard, new AI_CardScenarioGroup(cardScenarioGroupNode, playCard));
			}
			else if(cardScenarioGroupNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_Personality, constructor: Invalid node name!  Name: " + cardScenarioGroupNode.getNodeName());
			}
		}
	}


	public CN_InputMessage getInputMessage(CN_GameState gameState, boolean tryAgain) throws AI_FatalException
	{
		int currentPlayer = gameState.currentPlayer;

		AI_GameStateTimeline timeline = new AI_GameStateTimeline(gameState);
		AI_IdCard id = AI_Constants.idCardDirectory.get(gameState.players[currentPlayer].id);
		AI_MissionCard mission = AI_Constants.missionCardDirectory.get(gameState.players[currentPlayer].mission);

		AI_PlayCard sourceOfMostRecentBascom = AI_Constants.playCardDirectory.get(gameState.sourceOfMostRecentBascom);
		AI_CardScenarioGroup currentGroup;
		if(sourceOfMostRecentBascom == null)
		{
			currentGroup = scenarioGroupTable.get(AI_Constants.NULL_STRING);
		}
		else
		{
			currentGroup = scenarioGroupTable.get(sourceOfMostRecentBascom.identity);
			if(currentGroup == null)
				currentGroup = scenarioGroupTable.get(AI_Constants.NULL_STRING);
		}
		return currentGroup.checkAndExecute(gameState, timeline, id, mission, tryAgain);
	}




	private String getPlayCard(Node cardScenarioGroupNode) throws AI_FatalException
	{
		NamedNodeMap attributes = cardScenarioGroupNode.getAttributes();
		Node attributeNode = attributes.getNamedItem(AI_Constants.CARD_ATTRIBUTE_STRING);

		String returnValue;
		if(attributeNode.getTextContent().equals(AI_Constants.DEFAULT_STRING))
		{
			returnValue = AI_Constants.NULL_STRING;
		}
		else
		{
			returnValue = attributeNode.getTextContent();
			if(AI_Globals.playCardTable.get(returnValue) == null)
				throw new AI_FatalException("AI_Personality.getPlayCard: Play card \"" + returnValue + "\" does not exist!");
		}

		return returnValue;
	}



	public void debugPrint()
	{
		Enumeration<String> keys = scenarioGroupTable.keys();
		String key;
		while(keys.hasMoreElements())
		{
			key = keys.nextElement();
			System.out.println(key);
			scenarioGroupTable.get(key).debugPrint();
		}
	}


}
