package cnpc.ai;

import java.util.*;
import org.w3c.dom.*;

class AI_Ripplepoint extends AI_TimelineCard
{
	private final AI_Linchpin[] rippledBy;


	//NOTE: Create linchpins and fill AI_Globals.linchpinTable BEFORE constructing any ripplepoints!
	public AI_Ripplepoint(String identity_in, String cardName_in, Node timelineCardNode) throws AI_FatalException
	{
		super(identity_in, cardName_in);

		ArrayList<AI_Linchpin> rippledByList = getRippledByFromNode(timelineCardNode);
		rippledBy = new AI_Linchpin[rippledByList.size()];
		for(int i = 0; i < rippledByList.size(); i++)
		{
			rippledBy[i] = rippledByList.get(i);
		}
	}


	private ArrayList<AI_Linchpin> getRippledByFromNode(Node timelineCardNode) throws AI_FatalException
	{
		ArrayList<AI_Linchpin> currentList;
		ArrayList<AI_Linchpin> returnValue = new ArrayList<AI_Linchpin>();

		NodeList list = timelineCardNode.getChildNodes();
		Node childNode;

		for(int i = 0; list.item(i) != null; i++)
		{
			childNode = list.item(i);
			if(childNode.getNodeName().toLowerCase().equals("patchingoption"))
			{
				currentList = getRippledByFromPatchingOptionNode(childNode);
				for(AI_Linchpin linchpin : currentList)
				{
					returnValue.add(linchpin);
				}
			}
		}
		return returnValue;
	}

	private ArrayList<AI_Linchpin> getRippledByFromPatchingOptionNode(Node patchingOptionNode) throws AI_FatalException
	{
		ArrayList<AI_Linchpin> returnValue = new ArrayList<AI_Linchpin>();

		NodeList list = patchingOptionNode.getChildNodes();
		Node childNode;

		for(int i = 0; list.item(i) != null; i++)
		{
			childNode = list.item(i);
			if(childNode.getNodeName().toLowerCase().equals("prereq"))
			{
				returnValue.add(getRippledByFromPrereqNode(childNode));
			}
		}
		return returnValue;
	}

	private AI_Linchpin getRippledByFromPrereqNode(Node prereqNode) throws AI_FatalException
	{
		String linchpinString;
		AI_Linchpin returnValue = null;

		NodeList list = prereqNode.getChildNodes();
		Node childNode;

		for(int i = 0; list.item(i) != null; i++)
		{
			childNode = list.item(i);
			if(childNode.getNodeName().toLowerCase().equals("linchpinidentity"))
			{
				linchpinString = childNode.getTextContent();
				returnValue = AI_Globals.linchpinTable.get(linchpinString);
			}
		}

		if(returnValue == null)
		{
			throw new AI_FatalException("AI_Ripplepoint.getRippledByFromPrereqNode(): No linchpin identity, or bad linchpin identity!");
		}

		return returnValue;
	}



	public AI_Linchpin[] getRippledBy()
	{
		AI_Linchpin[] returnValue = new AI_Linchpin[rippledBy.length];
		for(int i = 0; i < rippledBy.length; i++)
		{
			returnValue[i] = rippledBy[i];
		}
		return returnValue;
	}


}
