package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_Scenario
{
	private final ComparisonList comparisonList;
	private final AI_ActionSpecifier action;



	public AI_Scenario(AI_ActionSpecifier specifier)
	{
		comparisonList = new ComparisonList();
		action = specifier;
	}

	public AI_Scenario(Node scenarioNode) throws AI_FatalException
	{
		comparisonList = new ComparisonList(scenarioNode);

		NodeList list = scenarioNode.getChildNodes();
		Node actionNode;

		AI_ActionSpecifier currentAction = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			actionNode = list.item(i);
			if(actionNode.getNodeName().toLowerCase().equals("action"))
			{
				currentAction = getAction(actionNode);
				break;
			}
			else if(actionNode.getNodeName().toLowerCase().equals("comparison"))
			{
			}
			else if(actionNode.getNodeName().toLowerCase().equals("operator"))
			{
			}
			else if(actionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_Scenario, constructor: Invalid node name!  Name:" + actionNode.getNodeName());
			}
		}

		if(currentAction == null)
			throw new AI_FatalException("AI_Scenario, constructor: No action was specified!");
		else
			action = currentAction;
	}

	private AI_ActionSpecifier getAction(Node actionNode) throws AI_FatalException
	{
		NamedNodeMap attributes = actionNode.getAttributes();
		Node attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

		AI_ActionSpecifier returnValue;
		if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.ARTIFACT_SELECTION_SPECIFIER_STRING.toLowerCase()))
			returnValue = new AI_ArtifactSelectionSpecifier(actionNode);
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.CARD_PLAY_SPECIFIER_STRING.toLowerCase()))
			returnValue = new AI_CardPlaySpecifier(actionNode);
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.DECK_SELECTION_SPECIFIER_STRING.toLowerCase()))
			returnValue = new AI_DeckSelectionSpecifier(actionNode);
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.DIRECTION_SELECTION_SPECIFIER_STRING.toLowerCase()))
			returnValue = new AI_DirectionSelectionSpecifier(actionNode);
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.DISCARDS_SELECTION_SPECIFIER_STRING.toLowerCase()))
			returnValue = new AI_DiscardsSelectionSpecifier(actionNode);
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.HAND_SELECTION_SPECIFIER_STRING.toLowerCase()))
			returnValue = new AI_HandSelectionSpecifier(actionNode);
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.ID_SELECTION_SPECIFIER_STRING.toLowerCase()))
			returnValue = new AI_IdSelectionSpecifier(actionNode);
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.LINCHPIN_SELECTION_SPECIFIER_STRING.toLowerCase()))
			returnValue = new AI_LinchpinSelectionSpecifier(actionNode);
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.MAIN_SPECIFIER_STRING.toLowerCase()))
			returnValue = new AI_MainSpecifier(actionNode);
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.MISSION_SELECTION_SPECIFIER_STRING.toLowerCase()))
			returnValue = new AI_MissionSelectionSpecifier(actionNode);
		else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.RIPPLEPOINT_SELECTION_SPECIFIER_STRING.toLowerCase()))
			returnValue = new AI_RipplepointSelectionSpecifier(actionNode);
		else
			throw new AI_FatalException("AI_Scenario.getAction(): Invalid action type!  Type: " + attributeNode.getTextContent());

		return returnValue;
	}


	public CN_InputMessage checkAndExecute(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		if(comparisonList.checkForSuccess(gameState, timeline, id, mission))
			return executeAction(gameState, timeline, id, mission);
		else
			return null;
	}

	private CN_InputMessage executeAction(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		return action.getAction(gameState, timeline, id, mission);
	}




	private enum Operator
	{
		AND,
		OR,
		NOT,
		COMPARISON,
		TRUE,
		FALSE;

		public static Operator getFromString(String string)
		{
			if(string.toLowerCase().equals("and"))
				return AND;
			else if(string.toLowerCase().equals("or"))
				return OR;
			else if(string.toLowerCase().equals("not"))
				return NOT;
			else
				return null;
		}
	}

	private class ComparisonList
	{
		private final ArrayList<Operator> operators = new ArrayList<Operator>();
		private final ArrayList<AI_Comparison> comparisons = new ArrayList<AI_Comparison>();

		private int comparisonNumber;



		public ComparisonList()
		{
		}

		public ComparisonList(Node scenarioNode) throws AI_FatalException
		{
			NodeList list = scenarioNode.getChildNodes();
			Node operatorNode;

			for(int i = 0; list.item(i) != null; i++)
			{
				operatorNode = list.item(i);
				if(operatorNode.getNodeName().toLowerCase().equals("comparison"))
				{
					operators.add(Operator.COMPARISON);
					comparisons.add(getComparison(operatorNode));
				}
				else if(operatorNode.getNodeName().toLowerCase().equals("operator"))
				{
					operators.add(Operator.getFromString(operatorNode.getTextContent()));
				}
				else if(operatorNode.getNodeName().toLowerCase().equals("action"))
				{
				}
				else if(operatorNode.getNodeName().toLowerCase().equals("#text"))
				{
				}
				else
				{
					throw new AI_FatalException("AI_Scenario.ComparisonList, constructor: Invalid node name!  Name:" + operatorNode.getNodeName());
				}
			}
		}

		private AI_Comparison getComparison(Node comparisonNode) throws AI_FatalException
		{
			NamedNodeMap attributes = comparisonNode.getAttributes();
			Node attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

			AI_Comparison returnValue;
			if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.ARTIFACT_COMPARISON_STRING.toLowerCase()))
				returnValue = new AI_ArtifactComparison(comparisonNode);
			else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.DECK_COMPARISON_STRING.toLowerCase()))
				returnValue = new AI_DeckComparison(comparisonNode);
			else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.DISCARDS_COMPARISON_STRING.toLowerCase()))
				returnValue = new AI_DiscardsComparison(comparisonNode);
			else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.HAND_CARD_COMPARISON_STRING.toLowerCase()))
				returnValue = new AI_HandCardComparison(comparisonNode);
			else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.HAND_SIZE_COMPARISON_STRING.toLowerCase()))
				returnValue = new AI_HandSizeComparison(comparisonNode);
			else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.LINCHPIN_COMPARISON_STRING.toLowerCase()))
				returnValue = new AI_LinchpinComparison(comparisonNode);
			else if(attributeNode.getTextContent().toLowerCase().equals(AI_Constants.RIPPLEPOINT_COMPARISON_STRING.toLowerCase()))
				returnValue = new AI_RipplepointComparison(comparisonNode);
			else
				throw new AI_FatalException("AI_Scenario.ComparisonList.getComparison(): Invalid comparison type!  Type: " + attributeNode.getTextContent());

			return returnValue;
		}



		public boolean checkForSuccess(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
		{
			if(operators.size() <= 0)
				return true;

			boolean result;

			ArrayList<Operator> currentOperators = new ArrayList<Operator>();
			for(int i = 0; i < operators.size(); i++)
			{
				currentOperators.add(operators.get(i));
			}

			currentOperators = evaluateComparisons(currentOperators, gameState, timeline, id, mission);
//System.out.println();
//for(int i = 0; i < currentOperators.size(); i++)
//{
//System.out.println(currentOperators.get(i));
//}
			currentOperators = evaluateNots(currentOperators);
			result = evaluateAndsAndOrs(currentOperators);

//System.out.println("Result: " + result);
			return result;
		}

		private ArrayList<Operator> evaluateComparisons(ArrayList<Operator> currentOperators, CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
		{
			resetComparison();

			ArrayList<Operator> newOperators = new ArrayList<Operator>();
			AI_Comparison comparison;

			Operator operator;
			for(int i = 0; i < currentOperators.size(); i++)
			{
				operator = currentOperators.get(i);
				if(operator == Operator.COMPARISON)
				{
					comparison = getNextComparison();
					if(comparison.compare(gameState, timeline, id, mission))
						newOperators.add(Operator.TRUE);
					else
						newOperators.add(Operator.FALSE);
				}
				else
				{
					newOperators.add(operator);
				}
			}

			return newOperators;
		}

		private ArrayList<Operator> evaluateNots(ArrayList<Operator> currentOperators) throws AI_FatalException
		{
			ArrayList<Operator> newOperators = new ArrayList<Operator>();

			Operator operator;
			Operator prevOperator;
			for(int i = 1; i < currentOperators.size(); i++)
			{
				operator = currentOperators.get(i);
				prevOperator = currentOperators.get(i - 1);

				if(prevOperator == Operator.NOT)
				{
					if(operator == Operator.TRUE)
					{
						newOperators.add(Operator.FALSE);
						i++;
					}
					else if(operator == Operator.FALSE)
					{
						newOperators.add(Operator.TRUE);
						i++;
					}
					else
					{
						throw new AI_FatalException("AI_Scenario.ComparisonList.evaluateNots(): Only a TRUE or FALSE operator may follow a NOT operator!");
					}
				}
				else
				{
					newOperators.add(prevOperator);
				}
			}
			newOperators.add(currentOperators.get(currentOperators.size() - 1));

			return newOperators;
		}

		private boolean evaluateAndsAndOrs(ArrayList<Operator> currentOperators) throws AI_FatalException
		{
			Operator first, second, third;
			if(currentOperators.size() >= 3)
			{
				first = currentOperators.get(0);
				second = currentOperators.get(1);
				third = currentOperators.get(2);
				currentOperators.remove(0);
				currentOperators.remove(0);
				currentOperators.remove(0);

				currentOperators.add(0, evaluateTriple(first, second, third));

				return evaluateAndsAndOrs(currentOperators);
			}
			else if(currentOperators.size() == 1)
			{
				switch(currentOperators.get(0))
				{
					case TRUE:
						return true;
					case FALSE:
						return false;
					default:
						throw new AI_FatalException("AI_Scenario.ComparisonList.evaluateAndsAndOrs(): Only a TRUE or FALSE operator may be used here!");
				}
			}
			else
			{
				throw new AI_FatalException("AI_Scenario.ComparisonList.evaluateAndsAndOrs(): Size of currentOperators must be equal to 1 or greater than 2!");
			}
		}

		private Operator evaluateTriple(Operator first, Operator second, Operator third) throws AI_FatalException
		{
			boolean firstBool, thirdBool;
			boolean resultBool;

			switch(first)
			{
				case TRUE:
					firstBool = true;
					break;
				case FALSE:
					firstBool = false;
					break;
				default:
					throw new AI_FatalException("AI_Scenario.ComparisonList.evaluateTriple(): Only a TRUE or FALSE operator may be used for first!");
			}

			switch(third)
			{
				case TRUE:
					thirdBool = true;
					break;
				case FALSE:
					thirdBool = false;
					break;
				default:
					throw new AI_FatalException("AI_Scenario.ComparisonList.evaluateTriple(): Only a TRUE or FALSE operator may be used for third!");
			}

			switch(second)
			{
				case AND:
					resultBool = (firstBool && thirdBool);
					break;
				case OR:
					resultBool = (firstBool || thirdBool);
					break;
				default:
					throw new AI_FatalException("AI_Scenario.ComparisonList.evaluateTriple(): Only a AND or OR operator may be used for second!");
			}

			if(resultBool)
				return Operator.TRUE;
			else
				return Operator.FALSE;

		}

		private void resetComparison()
		{
			comparisonNumber = 0;
		}

		private AI_Comparison getNextComparison()
		{
			AI_Comparison returnValue = comparisons.get(comparisonNumber);
			comparisonNumber++;
			return returnValue;
		}

		private void checkComparisonCount() throws AI_FatalException
		{
			int comparisonCount = 0;
			for(Operator operator : operators)
			{
				if(operator == Operator.COMPARISON)
					comparisonCount++;
			}
			if(comparisonCount != comparisons.size())
				throw new AI_FatalException("AI_Scenario.ComparisonList.checkComparisonCount(): Comparison counts in operators and comparisons do not match!");
		}



		public void debugPrint()
		{
			System.out.println("Operators:");
			for(Operator operator : operators)
			{
				System.out.println(operator);
			}
			System.out.println("Comparisons:");
			for(AI_Comparison comparison : comparisons)
			{
				comparison.debugPrint();
			}
		}



	}




	public void debugPrint()
	{
		comparisonList.debugPrint();
		action.debugPrint();
	}

	public void printActionClass()
	{
		System.out.println(action.getClass());
	}


}
