package cnpc.ai;

import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;

class AI_XmlUtils
{

	public static Document loadDocument(String xmlPath) throws AI_FatalException
	{
		return loadDocument(new File(xmlPath));
	}

	public static Document loadDocument(File xmlFile) throws AI_FatalException
	{
		DocumentBuilderFactory factory;
		DocumentBuilder builder;
		Document document;

		try
		{
			factory = DocumentBuilderFactory.newInstance();
			builder = factory.newDocumentBuilder();
			document = builder.parse(xmlFile);
		}
		catch(Exception e)
		{
			throw new AI_FatalException("AI_XmlUtils.loadDocument(): Exception in XML file loading and parsing; see cause for details!", e);
		}

		return document;
	}


	public static Node getRoot(Document document, String rootNodeName) throws AI_FatalException
	{
		NodeList rootList = document.getElementsByTagName(rootNodeName);

		if(rootList.getLength() <= 0)
			throw new AI_FatalException("AI_XmlUtils.getRoot(): Root node \"" + rootNodeName + "\" was not found!");

		return rootList.item(0);
	}



	public static String getElementValue(Node node, String elementName, boolean caseSensitive)
	{
		NodeList list = node.getChildNodes();
		Node childNode;

		if(caseSensitive)
		{
			for(int i = 0; list.item(i) != null; i++)
			{
				childNode = list.item(i);
				if(childNode.getNodeName().equals(elementName))
				{
					return childNode.getTextContent();
				}
			}
		}
		else
		{
			for(int i = 0; list.item(i) != null; i++)
			{
				childNode = list.item(i);
				if(childNode.getNodeName().toLowerCase().equals(elementName.toLowerCase()))
				{
					return childNode.getTextContent();
				}
			}
		}

		return null;
	}


	public static String getAttributeValue(Node node, String attributeName)
	{
		NamedNodeMap attributes = node.getAttributes();
		Node attributeNode = attributes.getNamedItem(attributeName);
		if(attributeNode == null)
			return null;
		else
			return attributeNode.getTextContent();
	}


}
