package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_ArtifactSelectionSpecifier extends AI_ActionSpecifier
{
	private enum CardOption {IDENTITY, ANY}
	private enum CardRelevanceOption {FOR_MISSION, NOT_FOR_MISSION, DONT_CARE}
	private enum CardOwnerOption {CURRENT_PLAYER, NOT_CURRENT_PLAYER, MOST_ARTIFACTS, LEAST_ARTIFACTS, MOST_ARTIFACTS_NOT_CURRENT, LEAST_ARTIFACTS_NOT_CURRENT, DONT_CARE}

	private static final String CARD_OPTION_IDENTITY_STRING = "Identity";
	private static final String CARD_OPTION_ANY_STRING = "Any";
	private static final String CARD_RELEVANCE_OPTION_FOR_MISSION_STRING = "ForMission";
	private static final String CARD_RELEVANCE_OPTION_NOT_FOR_MISSION_STRING = "NotForMission";
	private static final String CARD_RELEVANCE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String CARD_OWNER_OPTION_CURRENT_PLAYER_STRING = "CurrentPlayer";
	private static final String CARD_OWNER_OPTION_NOT_CURRENT_PLAYER_STRING = "NotCurrentPlayer";
	private static final String CARD_OWNER_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String CARD_OWNER_OPTION_MOST_ARTIFACTS_STRING = "MostArtifacts";
	private static final String CARD_OWNER_OPTION_LEAST_ARTIFACTS_STRING = "LeastArtifacts";
	private static final String CARD_OWNER_OPTION_MOST_ARTIFACTS_NOT_CURRENT_STRING = "MostArtifactsNotCurrent";
	private static final String CARD_OWNER_OPTION_LEAST_ARTIFACTS_NOT_CURRENT_STRING = "LeastArtifactsNotCurrent";

	private final CardOption cardOption;
	private final CardRelevanceOption cardRelevanceOption;
	private final CardOwnerOption cardOwnerOption;

	private final AI_Artifact card;



	public AI_ArtifactSelectionSpecifier()
	{
		cardOption = CardOption.ANY;
		cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		cardOwnerOption = CardOwnerOption.DONT_CARE;
		card = null;
	}

	public AI_ArtifactSelectionSpecifier(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String cardOption_in = inputValues.get(0);
		String cardRelevanceOption_in = inputValues.get(1);
		String cardOwnerOption_in = inputValues.get(2);
		String card_in = inputValues.get(3);


		if(card_in != null)
		{
			card = AI_Globals.artifactTable.get(card_in);
			if(card == null)
				throw new AI_FatalException("AI_GetArtifactSelectionSpecifier, constructor: Specified linchpin does not exist!");
		}
		else
		{
			card = null;
		}

		if(cardOption_in.toLowerCase().equals(CARD_OPTION_IDENTITY_STRING.toLowerCase()))
			cardOption = CardOption.IDENTITY;
		else if(cardOption_in.toLowerCase().equals(CARD_OPTION_ANY_STRING.toLowerCase()))
			cardOption = CardOption.ANY;
		else
			throw new AI_FatalException("AI_GetArtifactSelectionSpecifier, constructor: Invalid card option string!");

		if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_FOR_MISSION_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.FOR_MISSION;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_NOT_FOR_MISSION_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.NOT_FOR_MISSION;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_GetArtifactSelectionSpecifier, constructor: Invalid card relevance option string!");

		if(cardOwnerOption_in.toLowerCase().equals(CARD_OWNER_OPTION_CURRENT_PLAYER_STRING.toLowerCase()))
			cardOwnerOption = CardOwnerOption.CURRENT_PLAYER;
		else if(cardOwnerOption_in.toLowerCase().equals(CARD_OWNER_OPTION_NOT_CURRENT_PLAYER_STRING.toLowerCase()))
			cardOwnerOption = CardOwnerOption.NOT_CURRENT_PLAYER;
		else if(cardOwnerOption_in.toLowerCase().equals(CARD_OWNER_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardOwnerOption = CardOwnerOption.DONT_CARE;
		else if(cardOwnerOption_in.toLowerCase().equals(CARD_OWNER_OPTION_MOST_ARTIFACTS_STRING.toLowerCase()))
			cardOwnerOption = CardOwnerOption.MOST_ARTIFACTS;
		else if(cardOwnerOption_in.toLowerCase().equals(CARD_OWNER_OPTION_LEAST_ARTIFACTS_STRING.toLowerCase()))
			cardOwnerOption = CardOwnerOption.LEAST_ARTIFACTS;
		else if(cardOwnerOption_in.toLowerCase().equals(CARD_OWNER_OPTION_MOST_ARTIFACTS_NOT_CURRENT_STRING.toLowerCase()))
			cardOwnerOption = CardOwnerOption.MOST_ARTIFACTS_NOT_CURRENT;
		else if(cardOwnerOption_in.toLowerCase().equals(CARD_OWNER_OPTION_LEAST_ARTIFACTS_NOT_CURRENT_STRING.toLowerCase()))
			cardOwnerOption = CardOwnerOption.LEAST_ARTIFACTS_NOT_CURRENT;
		else
			throw new AI_FatalException("AI_GetArtifactSelectionSpecifier, constructor: Invalid card owner option string!");
	}


	private ArrayList<String> initializeFromXmlNode(Node actionNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = actionNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newCardOption = null;
		String newCardRelevanceOption = null;
		String newCardOwnerOption = null;
		String newCard = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("cardoption"))
				{
					newCardOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardrelevanceoption"))
				{
					newCardRelevanceOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardowneroption"))
				{
					newCardOwnerOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("card"))
				{
					newCard = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_ArtifactSelectionSpecifier.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_ArtifactSelectionSpecifier.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newCardOption);
		returnValue.add(newCardRelevanceOption);
		returnValue.add(newCardOwnerOption);
		returnValue.add(newCard);

		return returnValue;
	}






	public CN_InputMessage getAction(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<ArrayList<Integer>> artifacts = createEmptyList(gameState);
		for(int i = 0; i < gameState.players.length; i++)
		{
			for(int j = 0; j < gameState.players[i].artifacts.size(); j++)
			{
				artifacts.get(i).add(gameState.players[i].artifacts.get(j));
			}
		}

		ArrayList<Integer> currentArtifacts = pruneForCardOwner(artifacts, gameState);
		currentArtifacts = pruneForCard(currentArtifacts);
		currentArtifacts = pruneForCardRelevance(currentArtifacts, gameState, mission);

		if(currentArtifacts.size() <= 0)
			return null;
			//throw new AI_FatalException("AI_ArtifactSelectionSpecifier.getAction(): Hits were zero, so couldn't create an input message!");

		Integer selectedArtifact = currentArtifacts.get(new Random().nextInt(currentArtifacts.size()));
		AI_PlayCard selectedPlayCard = AI_Constants.playCardDirectory.get(selectedArtifact);

		CN_InputMessage returnValue = new CN_InputMessage(CN_InputMessage.MessageType.PLAY_CARD_CLICK, selectedArtifact, null, selectedPlayCard.cardName + " was selected.");

 		return returnValue;
	}


	private ArrayList<ArrayList<Integer>> createEmptyList(CN_GameState gameState)
	{
		ArrayList<ArrayList<Integer>> returnValue = new ArrayList<ArrayList<Integer>>();
		for(int i = 0; i < gameState.players.length; i++)
		{
			returnValue.add(new ArrayList<Integer>());
		}
		return returnValue;
	}



	private ArrayList<Integer> pruneForCardOwner(ArrayList<ArrayList<Integer>> currentArtifacts, CN_GameState gameState) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();
		ArrayList<ArrayList<Integer>> intermediate = createEmptyList(gameState);

		int mostArtifactsPlayer = 0;
		int leastArtifactsPlayer = 0;
		for(CN_GameState.PlayerData player : gameState.players)
		{
			if(player.artifacts.size() > gameState.players[mostArtifactsPlayer].artifacts.size())
				mostArtifactsPlayer = player.playerIndex;
			if(player.artifacts.size() < gameState.players[leastArtifactsPlayer].artifacts.size())
				leastArtifactsPlayer = player.playerIndex;
		}

		switch(cardOwnerOption)
		{
			case CURRENT_PLAYER:
				for(int i = 0; i < currentArtifacts.size(); i++)
				{
					if(gameState.currentPlayer == i)
					{
						for(int j = 0; j < currentArtifacts.get(i).size(); j++)
						{
							intermediate.get(i).add(currentArtifacts.get(i).get(j));
						}
					}
				}
				break;
			case NOT_CURRENT_PLAYER:
				for(int i = 0; i < currentArtifacts.size(); i++)
				{
					if(gameState.currentPlayer != i)
					{
						for(int j = 0; j < currentArtifacts.get(i).size(); j++)
						{
							intermediate.get(i).add(currentArtifacts.get(i).get(j));
						}
					}
				}
				break;
			case MOST_ARTIFACTS:
				for(int i = 0; i < currentArtifacts.size(); i++)
				{
					if(mostArtifactsPlayer == i)
					{
						for(int j = 0; j < currentArtifacts.get(i).size(); j++)
						{
							intermediate.get(i).add(currentArtifacts.get(i).get(j));
						}
					}
				}
				break;
			case LEAST_ARTIFACTS:
				for(int i = 0; i < currentArtifacts.size(); i++)
				{
					if(leastArtifactsPlayer == i)
					{
						for(int j = 0; j < currentArtifacts.get(i).size(); j++)
						{
							intermediate.get(i).add(currentArtifacts.get(i).get(j));
						}
					}
				}
				break;
			case MOST_ARTIFACTS_NOT_CURRENT:
				if(mostArtifactsPlayer == gameState.currentPlayer)
				{
					for(int i = 0; i < currentArtifacts.size(); i++)
					{
						if(gameState.currentPlayer != i)
						{
							for(int j = 0; j < currentArtifacts.get(i).size(); j++)
							{
								intermediate.get(i).add(currentArtifacts.get(i).get(j));
							}
						}
					}
				}
				else
				{
					for(int i = 0; i < currentArtifacts.size(); i++)
					{
						if(mostArtifactsPlayer == i)
						{
							for(int j = 0; j < currentArtifacts.get(i).size(); j++)
							{
								intermediate.get(i).add(currentArtifacts.get(i).get(j));
							}
						}
					}
				}
				break;
			case LEAST_ARTIFACTS_NOT_CURRENT:
				if(leastArtifactsPlayer == gameState.currentPlayer)
				{
					for(int i = 0; i < currentArtifacts.size(); i++)
					{
						if(gameState.currentPlayer != i)
						{
							for(int j = 0; j < currentArtifacts.get(i).size(); j++)
							{
								intermediate.get(i).add(currentArtifacts.get(i).get(j));
							}
						}
					}
				}
				else
				{
					for(int i = 0; i < currentArtifacts.size(); i++)
					{
						if(leastArtifactsPlayer == i)
						{
							for(int j = 0; j < currentArtifacts.get(i).size(); j++)
							{
								intermediate.get(i).add(currentArtifacts.get(i).get(j));
							}
						}
					}
				}
				break;
			case DONT_CARE:
				for(int i = 0; i < currentArtifacts.size(); i++)
				{
					for(int j = 0; j < currentArtifacts.get(i).size(); j++)
					{
						intermediate.get(i).add(currentArtifacts.get(i).get(j));
					}
				}
				break;
			default:
				throw new AI_FatalException("AI_ArtifactSelectionSpecifier.pruneForCard(): Invalid card owner option!");
		}

		for(int i = 0; i < intermediate.size(); i++)
		{
			for(int j = 0; j < intermediate.get(i).size(); j++)
			{
				returnValue.add(intermediate.get(i).get(j));
			}
		}

		return returnValue;
	}



	private ArrayList<Integer> pruneForCard(ArrayList<Integer> currentArtifacts) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		switch(cardOption)
		{
			case IDENTITY:
				for(Integer artifact : currentArtifacts)
				{
					if(AI_Constants.playCardDirectory.get(artifact).identity.equals(card.identity))
						returnValue.add(artifact);
				}
				break;
			case ANY:
				for(Integer artifact : currentArtifacts)
				{
					returnValue.add(artifact);
				}
				break;
			default:
				throw new AI_FatalException("AI_ArtifactSelectionSpecifier.pruneForCard(): Invalid card option!");
		}

		return returnValue;
	}


	private ArrayList<Integer> pruneForCardRelevance(ArrayList<Integer> currentArtifacts, CN_GameState gameState, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();
		ArrayList<AI_Artifact> artifactContext = createContext(gameState);

		switch(cardRelevanceOption)
		{
			case FOR_MISSION:
				for(Integer artifact : currentArtifacts)
				{
					if(mission.isArtifactForMission((AI_Artifact)AI_Constants.playCardDirectory.get(artifact), gameState, artifactContext))
						returnValue.add(artifact);
				}
				break;
			case NOT_FOR_MISSION:
				for(Integer artifact : currentArtifacts)
				{
					if(!mission.isArtifactForMission((AI_Artifact)AI_Constants.playCardDirectory.get(artifact), gameState, artifactContext))
						returnValue.add(artifact);
				}
				break;
			case DONT_CARE:
				for(Integer artifact : currentArtifacts)
				{
					returnValue.add(artifact);
				}
				break;
			default:
				throw new AI_FatalException("AI_ArtifactSelectionSpecifier.pruneForCard(): Invalid card relevance option!");
		}

		return returnValue;
	}

	private ArrayList<AI_Artifact> createContext(CN_GameState gameState)
	{
		return null;
	}

}
