package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_DeckSelectionSpecifier extends AI_ActionSpecifier
{
	private enum CardOption {IDENTITY, ANY}
	private enum CardTypeOption {ARTIFACT, PATCH, INVERTER, ACTION, TIMEWARP, DONT_CARE}
	private enum CardRelevanceOption {PATCH_FOR_ID, PATCH_NOT_FOR_ID, ARTIFACT_FOR_MISSION, ARTIFACT_NOT_FOR_MISSION, FOR_EITHER, NOT_FOR_EITHER, DONT_CARE}

	private static final String CARD_OPTION_IDENTITY_STRING = "Identity";
	private static final String CARD_OPTION_ANY_STRING = "Any";
	private static final String CARD_TYPE_OPTION_ARTIFACT_STRING = "Artifact";
	private static final String CARD_TYPE_OPTION_PATCH_STRING = "Patch";
	private static final String CARD_TYPE_OPTION_INVERTER_STRING = "Inverter";
	private static final String CARD_TYPE_OPTION_ACTION_STRING = "Action";
	private static final String CARD_TYPE_OPTION_TIMEWARP_STRING = "Timewarp";
	private static final String CARD_TYPE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String CARD_RELEVANCE_OPTION_PATCH_FOR_ID_STRING = "PatchForId";
	private static final String CARD_RELEVANCE_OPTION_PATCH_NOT_FOR_ID_STRING = "PatchNotForId";
	private static final String CARD_RELEVANCE_OPTION_ARTIFACT_FOR_MISSION_STRING = "ArtifactForMission";
	private static final String CARD_RELEVANCE_OPTION_ARTIFACT_NOT_FOR_MISSION_STRING = "ArtifactNotForMission";
	private static final String CARD_RELEVANCE_OPTION_FOR_EITHER_STRING = "ForEither";
	private static final String CARD_RELEVANCE_OPTION_NOT_FOR_EITHER_STRING = "NotForEither";
	private static final String CARD_RELEVANCE_OPTION_DONT_CARE_STRING = "DontCare";

	private final CardOption cardOption;
	private final CardTypeOption cardTypeOption;
	private final CardRelevanceOption cardRelevanceOption;

	private final AI_PlayCard card;


	public AI_DeckSelectionSpecifier()
	{
		cardOption = CardOption.ANY;
		cardTypeOption = CardTypeOption.DONT_CARE;
		cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		card = null;
	}

	public AI_DeckSelectionSpecifier(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String cardOption_in = inputValues.get(0);
		String cardTypeOption_in = inputValues.get(1);
		String cardRelevanceOption_in = inputValues.get(2);
		String card_in = inputValues.get(3);


		if(card_in != null)
		{
			card = AI_Globals.playCardTable.get(card_in);
			if(card == null)
				throw new AI_FatalException("AI_DeckSelectionSpecifier, constructor: Specified card does not exist!");
		}
		else
		{
			card = null;
		}

		if(cardOption_in.toLowerCase().equals(CARD_OPTION_IDENTITY_STRING.toLowerCase()))
			cardOption = CardOption.IDENTITY;
		else if(cardOption_in.toLowerCase().equals(CARD_OPTION_ANY_STRING.toLowerCase()))
			cardOption = CardOption.ANY;
		else
			throw new AI_FatalException("AI_DeckSelectionSpecifier, constructor: Invalid card option string!");

		if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_ARTIFACT_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.ARTIFACT;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_PATCH_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.PATCH;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_INVERTER_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.INVERTER;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_ACTION_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.ACTION;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_TIMEWARP_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.TIMEWARP;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_DeckSelectionSpecifier, constructor: Invalid card type option string!");

		if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_PATCH_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.PATCH_FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_PATCH_NOT_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.PATCH_NOT_FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_ARTIFACT_FOR_MISSION_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.ARTIFACT_FOR_MISSION;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_ARTIFACT_NOT_FOR_MISSION_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.ARTIFACT_NOT_FOR_MISSION;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_FOR_EITHER_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.FOR_EITHER;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_NOT_FOR_EITHER_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.NOT_FOR_EITHER;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_DeckSelectionSpecifier, constructor: Invalid card relevance option string!");
	}



	private ArrayList<String> initializeFromXmlNode(Node actionNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = actionNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newCardOption = null;
		String newCardTypeOption = null;
		String newCardRelevanceOption = null;
		String newCard = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("cardoption"))
				{
					newCardOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardtypeoption"))
				{
					newCardTypeOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardrelevanceoption"))
				{
					newCardRelevanceOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("card"))
				{
					newCard = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_DeckSelectionSpecifier.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_DeckSelectionSpecifier.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newCardOption);
		returnValue.add(newCardTypeOption);
		returnValue.add(newCardRelevanceOption);
		returnValue.add(newCard);

		return returnValue;
	}



	public CN_InputMessage getAction(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<Integer> currentDeckCards = new ArrayList<Integer>();

		for(Integer deckCard : gameState.deck)
		{
			currentDeckCards.add(deckCard);
		}

		currentDeckCards = pruneForCard(currentDeckCards);
		currentDeckCards = pruneForCardType(currentDeckCards);
		currentDeckCards = pruneForCardRelevance(currentDeckCards, gameState, id, mission);

		if(currentDeckCards.size() <= 0)
			return null;
			//throw new AI_FatalException("AI_DeckSelectionSpecifier.getAction(): Hits were zero, so couldn't create an input message!");

		Integer selectedDeckCard = currentDeckCards.get(new Random().nextInt(currentDeckCards.size()));
		AI_PlayCard selectedPlayCard = AI_Constants.playCardDirectory.get(selectedDeckCard);

		CN_InputMessage returnValue = new CN_InputMessage(CN_InputMessage.MessageType.PLAY_CARD_CLICK, selectedDeckCard, null, selectedPlayCard.cardName + " was selected.");

 		return returnValue;
	}


	private ArrayList<Integer> pruneForCard(ArrayList<Integer> currentDeckCards) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		switch(cardOption)
		{
			case IDENTITY:
				for(Integer deckCard : currentDeckCards)
				{
					if(AI_Constants.playCardDirectory.get(deckCard).identity.equals(card.identity))
						returnValue.add(deckCard);
				}
				break;
			case ANY:
				for(Integer deckCard : currentDeckCards)
				{
					returnValue.add(deckCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_DeckSelectionSpecifier.pruneForCard(): Invalid card option!");
		}

		return returnValue;
	}


	private ArrayList<Integer> pruneForCardType(ArrayList<Integer> currentDeckCards) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		AI_PlayCard playCard;
		switch(cardTypeOption)
		{
			case ARTIFACT:
				for(Integer deckCard : currentDeckCards)
				{
					playCard = AI_Constants.playCardDirectory.get(deckCard);
					if(playCard instanceof AI_Artifact)
						returnValue.add(deckCard);
				}
				break;
			case PATCH:
				for(Integer deckCard : currentDeckCards)
				{
					playCard = AI_Constants.playCardDirectory.get(deckCard);
					if(playCard instanceof AI_Patch)
						returnValue.add(deckCard);
				}
				break;
			case INVERTER:
				for(Integer deckCard : currentDeckCards)
				{
					playCard = AI_Constants.playCardDirectory.get(deckCard);
					if(playCard instanceof AI_Inverter)
						returnValue.add(deckCard);
				}
				break;
			case ACTION:
				for(Integer deckCard : currentDeckCards)
				{
					playCard = AI_Constants.playCardDirectory.get(deckCard);
					if(playCard instanceof AI_Action)
						returnValue.add(deckCard);
				}
				break;
			case TIMEWARP:
				for(Integer deckCard : currentDeckCards)
				{
					playCard = AI_Constants.playCardDirectory.get(deckCard);
					if(playCard instanceof AI_Timewarp)
						returnValue.add(deckCard);
				}
				break;
			case DONT_CARE:
				for(Integer deckCard : currentDeckCards)
				{
					returnValue.add(deckCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_DeckSelectionSpecifier.pruneForCard(): Invalid card type option!");
		}

		return returnValue;
	}



	private ArrayList<Integer> pruneForCardRelevance(ArrayList<Integer> currentDeckCards, CN_GameState gameState, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();
		ArrayList<AI_Artifact> artifactContext = createContext(gameState);

		AI_PlayCard playCard;
		switch(cardRelevanceOption)
		{
			case PATCH_FOR_ID:
				for(Integer deckCard : currentDeckCards)
				{
					playCard = AI_Constants.playCardDirectory.get(deckCard);
					if(playCard instanceof AI_Patch)
					{
						if(id.isPatchForId((AI_Patch)playCard))
							returnValue.add(deckCard);
					}
				}
				break;
			case PATCH_NOT_FOR_ID:
				for(Integer deckCard : currentDeckCards)
				{
					playCard = AI_Constants.playCardDirectory.get(deckCard);
					if(playCard instanceof AI_Patch)
					{
						if(!id.isPatchForId((AI_Patch)playCard))
							returnValue.add(deckCard);
					}
				}
				break;
			case ARTIFACT_FOR_MISSION:
				for(Integer deckCard : currentDeckCards)
				{
					playCard = AI_Constants.playCardDirectory.get(deckCard);
					if(playCard instanceof AI_Artifact)
					{
						if(mission.isArtifactForMission((AI_Artifact)playCard, gameState, artifactContext))
							returnValue.add(deckCard);
					}
				}
				break;
			case ARTIFACT_NOT_FOR_MISSION:
				for(Integer deckCard : currentDeckCards)
				{
					playCard = AI_Constants.playCardDirectory.get(deckCard);
					if(playCard instanceof AI_Artifact)
					{
						if(!mission.isArtifactForMission((AI_Artifact)playCard, gameState, artifactContext))
							returnValue.add(deckCard);
					}
				}
				break;
			case FOR_EITHER:
				for(Integer deckCard : currentDeckCards)
				{
					playCard = AI_Constants.playCardDirectory.get(deckCard);
					if(playCard instanceof AI_Patch)
					{
						if(id.isPatchForId((AI_Patch)playCard))
							returnValue.add(deckCard);
					}
					else if(playCard instanceof AI_Artifact)
					{
						if(mission.isArtifactForMission((AI_Artifact)playCard, gameState, artifactContext))
							returnValue.add(deckCard);
					}
				}
				break;
			case NOT_FOR_EITHER:
				for(Integer deckCard : currentDeckCards)
				{
					playCard = AI_Constants.playCardDirectory.get(deckCard);
					if(playCard instanceof AI_Patch)
					{
						if(id.isPatchForId((AI_Patch)playCard))
							break;
					}
					else if(playCard instanceof AI_Artifact)
					{
						if(mission.isArtifactForMission((AI_Artifact)playCard, gameState, artifactContext))
							break;
					}
					returnValue.add(deckCard);
				}
				break;
			case DONT_CARE:
				for(Integer deckCard : currentDeckCards)
				{
					returnValue.add(deckCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_DeckSelectionSpecifier.pruneForCard(): Invalid card relevance option!");
		}

		return returnValue;
	}

	private ArrayList<AI_Artifact> createContext(CN_GameState gameState)
	{
		ArrayList<AI_Artifact> returnValue = new ArrayList<AI_Artifact>();
		AI_PlayCard currentPlayCard;
		for(Integer integer : gameState.deck)
		{
			currentPlayCard = AI_Constants.playCardDirectory.get(integer);
			if(currentPlayCard instanceof AI_Artifact)
				returnValue.add((AI_Artifact)currentPlayCard);
		}
		return returnValue;
	}

}
