package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_DirectionSelectionSpecifier extends AI_ActionSpecifier
{
	private enum DirectionOption {LEFT, RIGHT, FROM_MOST_CARDS, FROM_LEAST_CARDS, DONT_CARE}

	private static final String DIRECTION_OPTION_LEFT_STRING = "Left";
	private static final String DIRECTION_OPTION_RIGHT_STRING = "Right";
	private static final String DIRECTION_OPTION_FROM_MOST_CARDS_STRING = "FromMostCards";
	private static final String DIRECTION_OPTION_FROM_LEAST_CARDS_STRING = "FromLeastCards";
	private static final String DIRECTION_OPTION_DONT_CARE_STRING = "DontCare";

	private final DirectionOption directionOption;


	public AI_DirectionSelectionSpecifier()
	{
		directionOption = DirectionOption.DONT_CARE;
	}

	public AI_DirectionSelectionSpecifier(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String directionOption_in = inputValues.get(0);


		if(directionOption_in.toLowerCase().equals(DIRECTION_OPTION_LEFT_STRING.toLowerCase()))
			directionOption = DirectionOption.LEFT;
		else if(directionOption_in.toLowerCase().equals(DIRECTION_OPTION_RIGHT_STRING.toLowerCase()))
			directionOption = DirectionOption.RIGHT;
		else if(directionOption_in.toLowerCase().equals(DIRECTION_OPTION_FROM_MOST_CARDS_STRING.toLowerCase()))
			directionOption = DirectionOption.FROM_MOST_CARDS;
		else if(directionOption_in.toLowerCase().equals(DIRECTION_OPTION_FROM_LEAST_CARDS_STRING.toLowerCase()))
			directionOption = DirectionOption.FROM_LEAST_CARDS;
		else if(directionOption_in.toLowerCase().equals(DIRECTION_OPTION_DONT_CARE_STRING.toLowerCase()))
			directionOption = DirectionOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_DirectionSelectionSpecifier, constructor: Invalid direction option string!");
	}


	private ArrayList<String> initializeFromXmlNode(Node actionNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = actionNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newDirectionOption = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("directionoption"))
				{
					newDirectionOption = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_DirectionSelectionSpecifier.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_DirectionSelectionSpecifier.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newDirectionOption);

		return returnValue;
	}


	public CN_InputMessage getAction(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		CN_InputMessage returnValue;

		if(determineIfLeft(gameState))
			returnValue = new CN_InputMessage(CN_InputMessage.MessageType.DIRECTION_SELECTION, null, CN_InputMessage.Direction.LEFT, "Left was selected");
		else
			returnValue = new CN_InputMessage(CN_InputMessage.MessageType.DIRECTION_SELECTION, null, CN_InputMessage.Direction.RIGHT, "Right was selected");

 		return returnValue;
	}


	private boolean determineIfLeft(CN_GameState gameState) throws AI_FatalException
	{
		switch(directionOption)
		{
			case LEFT:
				return true;
			case RIGHT:
				return false;
			case FROM_MOST_CARDS:
				return (!leftPlayerHasMoreCards(gameState));
			case FROM_LEAST_CARDS:
				return leftPlayerHasMoreCards(gameState);
			case DONT_CARE:
				if(new Random().nextInt(2) == 0)
					return true;
				else
					return false;
			default:
				throw new AI_FatalException("AI_DirectionSelectionSpecifier.determineIfLeft(): Invalid direction option!");
		}
	}

	private boolean leftPlayerHasMoreCards(CN_GameState gameState)
	{
		int numberOfPlayers = gameState.players.length;
		int currentPlayer = gameState.currentPlayer;

		int leftPlayer = currentPlayer - 1;
		if(leftPlayer < 0)
			leftPlayer = numberOfPlayers - 1;

		int rightPlayer = currentPlayer + 1;
		if(rightPlayer >= numberOfPlayers)
			rightPlayer = 0;

		int leftPlayerHandSize = gameState.players[leftPlayer].hand.size();
		int rightPlayerHandSize = gameState.players[rightPlayer].hand.size();

		if(leftPlayerHandSize > rightPlayerHandSize)
			return true;
		else
			return false;
	}
}
