package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_IdSelectionSpecifier extends AI_ActionSpecifier
{
	private enum CardOption {IDENTITY, ANY}
	private enum PlayerOption {CURRENT_PLAYER, NOT_CURRENT_PLAYER, DONT_CARE}

	private static final String CARD_OPTION_IDENTITY_STRING = "Identity";
	private static final String CARD_OPTION_ANY_STRING = "Any";
	private static final String PLAYER_OPTION_CURRENT_PLAYER_STRING = "CurrentPlayer";
	private static final String PLAYER_OPTION_NOT_CURRENT_PLAYER_STRING = "NotCurrentPlayer";
	private static final String PLAYER_OPTION_DONT_CARE_STRING = "DontCare";

	private final CardOption cardOption;
	private final PlayerOption playerOption;

	private final AI_IdCard card;


	public AI_IdSelectionSpecifier()
	{
		cardOption = CardOption.ANY;
		playerOption = PlayerOption.DONT_CARE;
		card = null;
	}

	public AI_IdSelectionSpecifier(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String cardOption_in = inputValues.get(0);
		String playerOption_in = inputValues.get(1);
		String card_in = inputValues.get(2);


		if(card_in != null)
		{
			card = AI_Globals.idTable.get(card_in);
			if(card == null)
				throw new AI_FatalException("AI_IdSelectionSpecifier, constructor: Specified id does not exist!");
		}
		else
		{
			card = null;
		}

		if(cardOption_in.toLowerCase().equals(CARD_OPTION_IDENTITY_STRING.toLowerCase()))
			cardOption = CardOption.IDENTITY;
		else if(cardOption_in.toLowerCase().equals(CARD_OPTION_ANY_STRING.toLowerCase()))
			cardOption = CardOption.ANY;
		else
			throw new AI_FatalException("AI_IdSelectionSpecifier, constructor: Invalid card option string!");

		if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_CURRENT_PLAYER_STRING.toLowerCase()))
			playerOption = PlayerOption.CURRENT_PLAYER;
		else if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_NOT_CURRENT_PLAYER_STRING.toLowerCase()))
			playerOption = PlayerOption.NOT_CURRENT_PLAYER;
		else if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_DONT_CARE_STRING.toLowerCase()))
			playerOption = PlayerOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_IdSelectionSpecifier, constructor: Invalid player option string!");
	}


	private ArrayList<String> initializeFromXmlNode(Node actionNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = actionNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newCardOption = null;
		String newPlayerOption = null;
		String newCard = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("cardoption"))
				{
					newCardOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("playeroption"))
				{
					newPlayerOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("card"))
				{
					newCard = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_IdSelectionSpecifier.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_IdSelectionSpecifier.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newCardOption);
		returnValue.add(newPlayerOption);
		returnValue.add(newCard);

		return returnValue;
	}


	public CN_InputMessage getAction(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<Integer> currentIdCards = new ArrayList<Integer>();

		for(CN_GameState.PlayerData player : gameState.players)
		{
			currentIdCards.add(player.id);
		}

		currentIdCards = pruneForCard(currentIdCards);
		currentIdCards = pruneForPlayer(currentIdCards, gameState);

		if(currentIdCards.size() <= 0)
			return null;
			//throw new AI_FatalException("AI_IdSelectionSpecifier.getAction(): Hits were zero, so couldn't create an input message!");

		Integer selectedIdCard = currentIdCards.get(new Random().nextInt(currentIdCards.size()));
		AI_IdCard selectedPlayCard = AI_Constants.idCardDirectory.get(selectedIdCard);

		CN_InputMessage returnValue = new CN_InputMessage(CN_InputMessage.MessageType.ID_CARD_SELECTION, selectedIdCard, null, selectedPlayCard.cardName + " was selected.");

 		return returnValue;
	}


	private ArrayList<Integer> pruneForCard(ArrayList<Integer> currentIdCards) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		switch(cardOption)
		{
			case IDENTITY:
				for(Integer idCard : currentIdCards)
				{
					if(AI_Constants.timelineCardDirectory.get(idCard).identity.equals(card.identity))
						returnValue.add(idCard);
				}
				break;
			case ANY:
				for(Integer idCard : currentIdCards)
				{
					returnValue.add(idCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_IdSelectionSpecifier.pruneForCard(): Invalid card option!");
		}

		return returnValue;
	}


	private ArrayList<Integer> pruneForPlayer(ArrayList<Integer> currentIdCards, CN_GameState gameState) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		switch(playerOption)
		{
			case CURRENT_PLAYER:
				for(Integer idCard : currentIdCards)
				{
					for(CN_GameState.PlayerData playerData : gameState.players)
					{
						if((playerData.id == idCard)&&(playerData.playerIndex == gameState.currentPlayer))
							returnValue.add(idCard);
					}
				}
				break;
			case NOT_CURRENT_PLAYER:
				for(Integer idCard : currentIdCards)
				{
					for(CN_GameState.PlayerData playerData : gameState.players)
					{
						if((playerData.id == idCard)&&(playerData.playerIndex != gameState.currentPlayer))
							returnValue.add(idCard);
					}
				}
				break;
			case DONT_CARE:
				for(Integer idCard : currentIdCards)
				{
					returnValue.add(idCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_IdSelectionSpecifier.pruneForPlayer(): Invalid player option!");
		}

		return returnValue;
	}


}
