package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_LinchpinSelectionSpecifier extends AI_ActionSpecifier
{
	private enum CardOption {IDENTITY, ANY}
	private enum CardRelevanceOption {FOR_ID, NOT_FOR_ID, DONT_CARE}
	private enum CardStateOption {FLIPPED, UNFLIPPED, FOR_ID, NOT_FOR_ID, DONT_CARE}
	private enum LegalityOption {LEGAL, ILLEGAL, DONT_CARE}

	private static final String CARD_OPTION_IDENTITY_STRING = "Identity";
	private static final String CARD_OPTION_ANY_STRING = "Any";
	private static final String CARD_RELEVANCE_OPTION_FOR_ID_STRING = "ForId";
	private static final String CARD_RELEVANCE_OPTION_NOT_FOR_ID_STRING = "NotForId";
	private static final String CARD_RELEVANCE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String CARD_STATE_OPTION_FLIPPED_STRING = "Flipped";
	private static final String CARD_STATE_OPTION_UNFLIPPED_STRING = "Unflipped";
	private static final String CARD_STATE_OPTION_FOR_ID_STRING = "ForId";
	private static final String CARD_STATE_OPTION_NOT_FOR_ID_STRING = "NotForId";
	private static final String CARD_STATE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String LEGALITY_OPTION_LEGAL_STRING = "Legal";
	private static final String LEGALITY_OPTION_ILLEGAL_STRING = "Illegal";
	private static final String LEGALITY_OPTION_DONT_CARE_STRING = "DontCare";

	private final CardOption cardOption;
	private final CardRelevanceOption cardRelevanceOption;
	private final CardStateOption cardStateOption;
	private final LegalityOption legalityOption;

	private final AI_Linchpin card;


	public AI_LinchpinSelectionSpecifier()
	{
		cardOption = CardOption.ANY;
		cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		cardStateOption = CardStateOption.DONT_CARE;
		legalityOption = LegalityOption.DONT_CARE;
		card = null;
	}

	public AI_LinchpinSelectionSpecifier(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String cardOption_in = inputValues.get(0);
		String cardRelevanceOption_in = inputValues.get(1);
		String cardStateOption_in = inputValues.get(2);
		String legalityOption_in = inputValues.get(3);
		String card_in = inputValues.get(4);


		if(card_in != null)
		{
			card = AI_Globals.linchpinTable.get(card_in);
			if(card == null)
				throw new AI_FatalException("AI_LinchpinSelectionSpecifier, constructor: Specified linchpin does not exist!");
		}
		else
		{
			card = null;
		}

		if(cardOption_in.toLowerCase().equals(CARD_OPTION_IDENTITY_STRING.toLowerCase()))
			cardOption = CardOption.IDENTITY;
		else if(cardOption_in.toLowerCase().equals(CARD_OPTION_ANY_STRING.toLowerCase()))
			cardOption = CardOption.ANY;
		else
			throw new AI_FatalException("AI_LinchpinSelectionSpecifier, constructor: Invalid card option string!");

		if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_NOT_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.NOT_FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_LinchpinSelectionSpecifier, constructor: Invalid card relevance option string!");

		if(cardStateOption_in.toLowerCase().equals(CARD_STATE_OPTION_FLIPPED_STRING.toLowerCase()))
			cardStateOption = CardStateOption.FLIPPED;
		else if(cardStateOption_in.toLowerCase().equals(CARD_STATE_OPTION_UNFLIPPED_STRING.toLowerCase()))
			cardStateOption = CardStateOption.UNFLIPPED;
		else if(cardStateOption_in.toLowerCase().equals(CARD_STATE_OPTION_FOR_ID_STRING.toLowerCase()))
			cardStateOption = CardStateOption.FOR_ID;
		else if(cardStateOption_in.toLowerCase().equals(CARD_STATE_OPTION_NOT_FOR_ID_STRING.toLowerCase()))
			cardStateOption = CardStateOption.NOT_FOR_ID;
		else if(cardStateOption_in.toLowerCase().equals(CARD_STATE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardStateOption = CardStateOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_LinchpinSelectionSpecifier, constructor: Invalid card state option string!");

		if(legalityOption_in.toLowerCase().equals(LEGALITY_OPTION_LEGAL_STRING.toLowerCase()))
			legalityOption = LegalityOption.LEGAL;
		else if(legalityOption_in.toLowerCase().equals(LEGALITY_OPTION_ILLEGAL_STRING.toLowerCase()))
			legalityOption = LegalityOption.ILLEGAL;
		else if(legalityOption_in.toLowerCase().equals(LEGALITY_OPTION_DONT_CARE_STRING.toLowerCase()))
			legalityOption = LegalityOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_LinchpinSelectionSpecifier, constructor: Invalid legality option string!");
	}



	private ArrayList<String> initializeFromXmlNode(Node actionNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = actionNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newCardOption = null;
		String newCardRelevanceOption = null;
		String newCardStateOption = null;
		String newLegalityOption = null;
		String newCard = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("cardoption"))
				{
					newCardOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardrelevanceoption"))
				{
					newCardRelevanceOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardstateoption"))
				{
					newCardStateOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("legalityoption"))
				{
					newLegalityOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("card"))
				{
					newCard = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_LinchpinSelectionSpecifier.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_LinchpinSelectionSpecifier.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newCardOption);
		returnValue.add(newCardRelevanceOption);
		returnValue.add(newCardStateOption);
		returnValue.add(newLegalityOption);
		returnValue.add(newCard);

		return returnValue;
	}



	public CN_InputMessage getAction(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
//System.out.println("ACTION LINCHPIN");

		ArrayList<Integer> currentLinchpins = new ArrayList<Integer>();

		for(CN_GameState.TimelineCardData timelineCard : gameState.timeline)
		{
			if(AI_Constants.timelineCardDirectory.get(timelineCard.cardNumber) instanceof AI_Linchpin)
			{
				currentLinchpins.add(timelineCard.cardNumber);
			}
		}

		currentLinchpins = pruneForCard(currentLinchpins);
		currentLinchpins = pruneForCardRelevance(currentLinchpins, id);
		currentLinchpins = pruneForCardState(currentLinchpins, timeline, id);
		currentLinchpins = pruneForLegality(currentLinchpins, timeline);

		if(currentLinchpins.size() <= 0)
			return null;
			//throw new AI_FatalException("AI_LinchpinSelectionSpecifier.getAction(): Hits were zero, so couldn't create an input message!");

		Integer selectedLinchpin = currentLinchpins.get(new Random().nextInt(currentLinchpins.size()));
		AI_TimelineCard selectedPlayCard = AI_Constants.timelineCardDirectory.get(selectedLinchpin);

		CN_InputMessage returnValue = new CN_InputMessage(CN_InputMessage.MessageType.TIMELINE_CARD_CLICK, selectedLinchpin, null, selectedPlayCard.cardName + " was selected.");

 		return returnValue;
	}


	private ArrayList<Integer> pruneForCard(ArrayList<Integer> currentLinchpins) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		switch(cardOption)
		{
			case IDENTITY:
				for(Integer linchpin : currentLinchpins)
				{
					if(AI_Constants.timelineCardDirectory.get(linchpin).identity.equals(card.identity))
						returnValue.add(linchpin);
				}
				break;
			case ANY:
				for(Integer linchpin : currentLinchpins)
				{
					returnValue.add(linchpin);
				}
				break;
			default:
				throw new AI_FatalException("AI_LinchpinSelectionSpecifier.pruneForCard(): Invalid card option!");
		}

		return returnValue;
	}


	private ArrayList<Integer> pruneForCardRelevance(ArrayList<Integer> currentLinchpins, AI_IdCard id) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		AI_Linchpin oLinchpin;
		switch(cardRelevanceOption)
		{
			case FOR_ID:
				for(Integer linchpin : currentLinchpins)
				{
					oLinchpin = (AI_Linchpin)AI_Constants.timelineCardDirectory.get(linchpin);
					if(id.isLinchpinForId(oLinchpin))
						returnValue.add(linchpin);
				}
				break;
			case NOT_FOR_ID:
				for(Integer linchpin : currentLinchpins)
				{
					oLinchpin = (AI_Linchpin)AI_Constants.timelineCardDirectory.get(linchpin);
					if(!id.isLinchpinForId(oLinchpin))
						returnValue.add(linchpin);
				}
				break;
			case DONT_CARE:
				for(Integer linchpin : currentLinchpins)
				{
					returnValue.add(linchpin);
				}
				break;
			default:
				throw new AI_FatalException("AI_LinchpinSelectionSpecifier.pruneForCardRelevance(): Invalid card relevance option!");
		}

		return returnValue;
	}


	private ArrayList<Integer> pruneForCardState(ArrayList<Integer> currentLinchpins, AI_GameStateTimeline timeline, AI_IdCard id) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		AI_Linchpin oLinchpin;
		switch(cardStateOption)
		{
			case FLIPPED:
				for(Integer linchpin : currentLinchpins)
				{
					oLinchpin = (AI_Linchpin)AI_Constants.timelineCardDirectory.get(linchpin);
					if(timeline.getByIdentity(oLinchpin.identity).isFlipped)
						returnValue.add(linchpin);
				}
				break;
			case UNFLIPPED:
				for(Integer linchpin : currentLinchpins)
				{
					oLinchpin = (AI_Linchpin)AI_Constants.timelineCardDirectory.get(linchpin);
					if(!timeline.getByIdentity(oLinchpin.identity).isFlipped)
						returnValue.add(linchpin);
				}
				break;
			case FOR_ID:
				for(Integer linchpin : currentLinchpins)
				{
					oLinchpin = (AI_Linchpin)AI_Constants.timelineCardDirectory.get(linchpin);
					if(id.isLinchpinStateForId(oLinchpin, timeline.getByIdentity(oLinchpin.identity).isFlipped))
						returnValue.add(linchpin);
				}
				break;
			case NOT_FOR_ID:
				for(Integer linchpin : currentLinchpins)
				{
					oLinchpin = (AI_Linchpin)AI_Constants.timelineCardDirectory.get(linchpin);
					if(!id.isLinchpinStateForId(oLinchpin, timeline.getByIdentity(oLinchpin.identity).isFlipped))
						returnValue.add(linchpin);
				}
				break;
			case DONT_CARE:
				for(Integer linchpin : currentLinchpins)
				{
					returnValue.add(linchpin);
				}
				break;
			default:
				throw new AI_FatalException("AI_LinchpinSelectionSpecifier.pruneForCardState(): Invalid card state option!");
		}

		return returnValue;
	}


	private ArrayList<Integer> pruneForLegality(ArrayList<Integer> currentLinchpins, AI_GameStateTimeline timeline) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		AI_Linchpin oLinchpin;
		switch(legalityOption)
		{
			case LEGAL:
				for(Integer linchpin : currentLinchpins)
				{
					oLinchpin = (AI_Linchpin)AI_Constants.timelineCardDirectory.get(linchpin);
					if(timeline.getByIdentity(oLinchpin.identity).isLegal)
						returnValue.add(linchpin);
				}
				break;
			case ILLEGAL:
				for(Integer linchpin : currentLinchpins)
				{
					oLinchpin = (AI_Linchpin)AI_Constants.timelineCardDirectory.get(linchpin);
					if(!timeline.getByIdentity(oLinchpin.identity).isLegal)
						returnValue.add(linchpin);
				}
				break;
			case DONT_CARE:
				for(Integer linchpin : currentLinchpins)
				{
					returnValue.add(linchpin);
				}
				break;
			default:
				throw new AI_FatalException("AI_LinchpinSelectionSpecifier.pruneForLegality(): Invalid legality option!");
		}

		return returnValue;
	}

}
