package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_MainSpecifier extends AI_ActionSpecifier
{
	private enum CardOption {IDENTITY, ANY}
	private enum CardTypeOption {ARTIFACT, PATCH, INVERTER, ACTION, TIMEWARP, DONT_CARE}
	private enum CardRelevanceOption {PATCH_FOR_ID, PATCH_NOT_FOR_ID, ARTIFACT_FOR_MISSION, ARTIFACT_NOT_FOR_MISSION, FOR_EITHER, NOT_FOR_EITHER, DONT_CARE}
	private enum DiscardOption {DISCARD_ONE, DISCARD_TWO, NO_DISCARD, DONE_RANDOMLY}

	private static final String CARD_OPTION_IDENTITY_STRING = "Identity";
	private static final String CARD_OPTION_ANY_STRING = "Any";
	private static final String CARD_TYPE_OPTION_ARTIFACT_STRING = "Artifact";
	private static final String CARD_TYPE_OPTION_PATCH_STRING = "Patch";
	private static final String CARD_TYPE_OPTION_INVERTER_STRING = "Inverter";
	private static final String CARD_TYPE_OPTION_ACTION_STRING = "Action";
	private static final String CARD_TYPE_OPTION_TIMEWARP_STRING = "Timewarp";
	private static final String CARD_TYPE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String CARD_RELEVANCE_OPTION_PATCH_FOR_ID_STRING = "PatchForId";
	private static final String CARD_RELEVANCE_OPTION_PATCH_NOT_FOR_ID_STRING = "PatchNotForId";
	private static final String CARD_RELEVANCE_OPTION_ARTIFACT_FOR_MISSION_STRING = "ArtifactForMission";
	private static final String CARD_RELEVANCE_OPTION_ARTIFACT_NOT_FOR_MISSION_STRING = "ArtifactNotForMission";
	private static final String CARD_RELEVANCE_OPTION_FOR_EITHER_STRING = "ForEither";
	private static final String CARD_RELEVANCE_OPTION_NOT_FOR_EITHER_STRING = "NotForEither";
	private static final String CARD_RELEVANCE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String DISCARD_OPTION_DISCARD_ONE_STRING = "DiscardOne";
	private static final String DISCARD_OPTION_DISCARD_TWO_STRING = "DiscardTwo";
	private static final String DISCARD_OPTION_NO_DISCARD_STRING = "NoDiscard";
	private static final String DISCARD_OPTION_DONE_RANDOMLY_STRING = "DoneRandomly";

	private final CardOption cardOption;
	private final CardTypeOption cardTypeOption;
	private final CardRelevanceOption cardRelevanceOption;
	private final DiscardOption discardOption;

	private final AI_PlayCard card;


	public AI_MainSpecifier()
	{
		cardOption = CardOption.ANY;
		cardTypeOption = CardTypeOption.DONT_CARE;
		cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		discardOption = DiscardOption.DONE_RANDOMLY;
		card = null;
	}

	public AI_MainSpecifier(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String cardOption_in = inputValues.get(0);
		String cardTypeOption_in = inputValues.get(1);
		String cardRelevanceOption_in = inputValues.get(2);
		String discardOption_in = inputValues.get(3);
		String card_in = inputValues.get(4);


		if(card_in != null)
		{
			card = AI_Globals.playCardTable.get(card_in);
			if(card == null)
				throw new AI_FatalException("AI_MainSpecifier, constructor: Specified card does not exist!  Card: " + card_in);
		}
		else
		{
			card = null;
		}

		if(cardOption_in.toLowerCase().equals(CARD_OPTION_IDENTITY_STRING.toLowerCase()))
			cardOption = CardOption.IDENTITY;
		else if(cardOption_in.toLowerCase().equals(CARD_OPTION_ANY_STRING.toLowerCase()))
			cardOption = CardOption.ANY;
		else
			throw new AI_FatalException("AI_MainSpecifier, constructor: Invalid card option string!");

		if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_ARTIFACT_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.ARTIFACT;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_PATCH_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.PATCH;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_INVERTER_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.INVERTER;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_ACTION_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.ACTION;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_TIMEWARP_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.TIMEWARP;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_MainSpecifier, constructor: Invalid card type option string!");

		if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_PATCH_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.PATCH_FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_PATCH_NOT_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.PATCH_NOT_FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_ARTIFACT_FOR_MISSION_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.ARTIFACT_FOR_MISSION;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_ARTIFACT_NOT_FOR_MISSION_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.ARTIFACT_NOT_FOR_MISSION;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_FOR_EITHER_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.FOR_EITHER;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_NOT_FOR_EITHER_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.NOT_FOR_EITHER;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_MainSpecifier, constructor: Invalid card relevance option string!");


		if(discardOption_in.toLowerCase().equals(DISCARD_OPTION_DISCARD_ONE_STRING.toLowerCase()))
			discardOption = DiscardOption.DISCARD_ONE;
		else if(discardOption_in.toLowerCase().equals(DISCARD_OPTION_DISCARD_TWO_STRING.toLowerCase()))
			discardOption = DiscardOption.DISCARD_TWO;
		else if(discardOption_in.toLowerCase().equals(DISCARD_OPTION_NO_DISCARD_STRING.toLowerCase()))
			discardOption = DiscardOption.NO_DISCARD;
		else if(discardOption_in.toLowerCase().equals(DISCARD_OPTION_DONE_RANDOMLY_STRING.toLowerCase()))
			discardOption = DiscardOption.DONE_RANDOMLY;
		else
			throw new AI_FatalException("AI_MainSpecifier, constructor: Invalid card relevance option string!  String: " + discardOption_in);

	}



	private ArrayList<String> initializeFromXmlNode(Node actionNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = actionNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newCardOption = null;
		String newCardTypeOption = null;
		String newCardRelevanceOption = null;
		String newDiscardOption = null;
		String newCard = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("cardoption"))
				{
					newCardOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardtypeoption"))
				{
					newCardTypeOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardrelevanceoption"))
				{
					newCardRelevanceOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("discardoption"))
				{
					newDiscardOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("card"))
				{
					newCard = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_MainSpecifier.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_MainSpecifier.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newCardOption);
		returnValue.add(newCardTypeOption);
		returnValue.add(newCardRelevanceOption);
		returnValue.add(newDiscardOption);
		returnValue.add(newCard);

		return returnValue;
	}



	public CN_InputMessage getAction(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
//System.out.println("ACTION MAIN");

		CN_InputMessage possibleDiscardOverride = getPossibleDiscardOverride();
		if(possibleDiscardOverride != null)
			return possibleDiscardOverride;

		ArrayList<Integer> currentHandCards = new ArrayList<Integer>();

		for(Integer handCard : gameState.players[gameState.currentPlayer].hand)
		{
			currentHandCards.add(handCard);
		}

		currentHandCards = pruneForCard(currentHandCards);
		currentHandCards = pruneForCardType(currentHandCards);
		currentHandCards = pruneForCardRelevance(currentHandCards, gameState, id, mission);

		if(currentHandCards.size() <= 0)
			return null;
			//throw new AI_FatalException("AI_MainSpecifier.getAction(): Hits were zero, so couldn't create an input message!");

		Integer selectedHandCard = currentHandCards.get(new Random().nextInt(currentHandCards.size()));
		AI_PlayCard selectedPlayCard = AI_Constants.playCardDirectory.get(selectedHandCard);

		CN_InputMessage returnValue = new CN_InputMessage(CN_InputMessage.MessageType.PLAY_CARD_CLICK, selectedHandCard, null, selectedPlayCard.cardName + " was played.");

 		return returnValue;
	}


	private CN_InputMessage getPossibleDiscardOverride()
	{
		if(discardOption == DiscardOption.DISCARD_ONE)
			return new CN_InputMessage(CN_InputMessage.MessageType.DISCARD_CLICK, null, null);
		if(discardOption == DiscardOption.DISCARD_TWO)
			return new CN_InputMessage(CN_InputMessage.MessageType.DISCARD_TWO_CLICK, null, null);

		Random random;
		int randomNumber;
		if(discardOption == DiscardOption.DONE_RANDOMLY)
		{
			random = new Random();
			randomNumber = random.nextInt(8);
			if(randomNumber == 0)
				return new CN_InputMessage(CN_InputMessage.MessageType.DISCARD_CLICK, null, null);
			else if(randomNumber == 1)
				return new CN_InputMessage(CN_InputMessage.MessageType.DISCARD_TWO_CLICK, null, null);
			else if(randomNumber == 2)
				return new CN_InputMessage(CN_InputMessage.MessageType.DISCARD_TWO_CLICK, null, null);
		}

		return null;
	}


	private ArrayList<Integer> pruneForCard(ArrayList<Integer> currentHandCards) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		switch(cardOption)
		{
			case IDENTITY:
				for(Integer handCard : currentHandCards)
				{
					if(AI_Constants.playCardDirectory.get(handCard).identity.equals(card.identity))
						returnValue.add(handCard);
				}
				break;
			case ANY:
				for(Integer handCard : currentHandCards)
				{
					returnValue.add(handCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_MainSpecifier.pruneForCard(): Invalid card option!");
		}

		return returnValue;
	}


	private ArrayList<Integer> pruneForCardType(ArrayList<Integer> currentHandCards) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		AI_PlayCard playCard;
		switch(cardTypeOption)
		{
			case ARTIFACT:
				for(Integer handCard : currentHandCards)
				{
					playCard = AI_Constants.playCardDirectory.get(handCard);
					if(playCard instanceof AI_Artifact)
						returnValue.add(handCard);
				}
				break;
			case PATCH:
				for(Integer handCard : currentHandCards)
				{
					playCard = AI_Constants.playCardDirectory.get(handCard);
					if(playCard instanceof AI_Patch)
						returnValue.add(handCard);
				}
				break;
			case INVERTER:
				for(Integer handCard : currentHandCards)
				{
					playCard = AI_Constants.playCardDirectory.get(handCard);
					if(playCard instanceof AI_Inverter)
						returnValue.add(handCard);
				}
				break;
			case ACTION:
				for(Integer handCard : currentHandCards)
				{
					playCard = AI_Constants.playCardDirectory.get(handCard);
					if(playCard instanceof AI_Action)
						returnValue.add(handCard);
				}
				break;
			case TIMEWARP:
				for(Integer handCard : currentHandCards)
				{
					playCard = AI_Constants.playCardDirectory.get(handCard);
					if(playCard instanceof AI_Timewarp)
						returnValue.add(handCard);
				}
				break;
			case DONT_CARE:
				for(Integer handCard : currentHandCards)
				{
					returnValue.add(handCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_MainSpecifier.pruneForCard(): Invalid card type option!");
		}

		return returnValue;
	}



	private ArrayList<Integer> pruneForCardRelevance(ArrayList<Integer> currentHandCards, CN_GameState gameState, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();
		ArrayList<AI_Artifact> artifactContext = createContext(gameState);

		AI_PlayCard playCard;
		switch(cardRelevanceOption)
		{
			case PATCH_FOR_ID:
				for(Integer handCard : currentHandCards)
				{
					playCard = AI_Constants.playCardDirectory.get(handCard);
					if(playCard instanceof AI_Patch)
					{
						if(id.isPatchForId((AI_Patch)playCard))
							returnValue.add(handCard);
					}
				}
				break;
			case PATCH_NOT_FOR_ID:
				for(Integer handCard : currentHandCards)
				{
					playCard = AI_Constants.playCardDirectory.get(handCard);
					if(playCard instanceof AI_Patch)
					{
						if(!id.isPatchForId((AI_Patch)playCard))
							returnValue.add(handCard);
					}
				}
				break;
			case ARTIFACT_FOR_MISSION:
				for(Integer handCard : currentHandCards)
				{
					playCard = AI_Constants.playCardDirectory.get(handCard);
					if(playCard instanceof AI_Artifact)
					{
						if(mission.isArtifactForMission((AI_Artifact)playCard, gameState, artifactContext))
							returnValue.add(handCard);
					}
				}
				break;
			case ARTIFACT_NOT_FOR_MISSION:
				for(Integer handCard : currentHandCards)
				{
					playCard = AI_Constants.playCardDirectory.get(handCard);
					if(playCard instanceof AI_Artifact)
					{
						if(!mission.isArtifactForMission((AI_Artifact)playCard, gameState, artifactContext))
							returnValue.add(handCard);
					}
				}
				break;
			case FOR_EITHER:
				for(Integer handCard : currentHandCards)
				{
					playCard = AI_Constants.playCardDirectory.get(handCard);
					if(playCard instanceof AI_Patch)
					{
						if(id.isPatchForId((AI_Patch)playCard))
							returnValue.add(handCard);
					}
					else if(playCard instanceof AI_Artifact)
					{
						if(mission.isArtifactForMission((AI_Artifact)playCard, gameState, artifactContext))
							returnValue.add(handCard);
					}
				}
				break;
			case NOT_FOR_EITHER:
				for(Integer handCard : currentHandCards)
				{
					playCard = AI_Constants.playCardDirectory.get(handCard);
					if(playCard instanceof AI_Patch)
					{
						if(id.isPatchForId((AI_Patch)playCard))
							break;
					}
					else if(playCard instanceof AI_Artifact)
					{
						if(mission.isArtifactForMission((AI_Artifact)playCard, gameState, artifactContext))
							break;
					}
					returnValue.add(handCard);
				}
				break;
			case DONT_CARE:
				for(Integer handCard : currentHandCards)
				{
					returnValue.add(handCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_MainSpecifier.pruneForCard(): Invalid card relevance option!");
		}

		return returnValue;
	}
	
	private ArrayList<AI_Artifact> createContext(CN_GameState gameState)
	{
		ArrayList<AI_Artifact> returnValue = new ArrayList<AI_Artifact>();
		AI_PlayCard currentPlayCard;
		for(Integer integer : gameState.players[gameState.currentPlayer].hand)
		{
			currentPlayCard = AI_Constants.playCardDirectory.get(integer);
			if(currentPlayCard instanceof AI_Artifact)
				returnValue.add((AI_Artifact)currentPlayCard);
		}
		return returnValue;
	}

}
