package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_MissionSelectionSpecifier extends AI_ActionSpecifier
{
	private enum CardOption {IDENTITY, ANY}
	private enum PlayerOption {CURRENT_PLAYER, NOT_CURRENT_PLAYER, MOST_ARTIFACTS, LEAST_ARTIFACTS, DONT_CARE}

	private static final String CARD_OPTION_IDENTITY_STRING = "Identity";
	private static final String CARD_OPTION_ANY_STRING = "Any";
	private static final String PLAYER_OPTION_CURRENT_PLAYER_STRING = "CurrentPlayer";
	private static final String PLAYER_OPTION_NOT_CURRENT_PLAYER_STRING = "NotCurrentPlayer";
	private static final String PLAYER_OPTION_MOST_ARTIFACTS_STRING = "MostArtifacts";
	private static final String PLAYER_OPTION_LEAST_ARTIFACTS_STRING = "LeastArtifacts";
	private static final String PLAYER_OPTION_DONT_CARE_STRING = "DontCare";

	private final CardOption cardOption;
	private final PlayerOption playerOption;

	private final AI_MissionCard card;


	public AI_MissionSelectionSpecifier()
	{
		cardOption = CardOption.ANY;
		playerOption = PlayerOption.DONT_CARE;
		card = null;
	}

	public AI_MissionSelectionSpecifier(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String cardOption_in = inputValues.get(0);
		String playerOption_in = inputValues.get(1);
		String card_in = inputValues.get(2);


		if(card_in != null)
		{
			card = AI_Globals.missionTable.get(card_in);
			if(card == null)
				throw new AI_FatalException("AI_MissionSelectionSpecifier, constructor: Specified mission does not exist!");
		}
		else
		{
			card = null;
		}

		if(cardOption_in.toLowerCase().equals(CARD_OPTION_IDENTITY_STRING.toLowerCase()))
			cardOption = CardOption.IDENTITY;
		else if(cardOption_in.toLowerCase().equals(CARD_OPTION_ANY_STRING.toLowerCase()))
			cardOption = CardOption.ANY;
		else
			throw new AI_FatalException("AI_MissionSelectionSpecifier, constructor: Invalid card option string!");

		if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_CURRENT_PLAYER_STRING.toLowerCase()))
			playerOption = PlayerOption.CURRENT_PLAYER;
		else if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_NOT_CURRENT_PLAYER_STRING.toLowerCase()))
			playerOption = PlayerOption.NOT_CURRENT_PLAYER;
		else if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_MOST_ARTIFACTS_STRING.toLowerCase()))
			playerOption = PlayerOption.MOST_ARTIFACTS;
		else if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_LEAST_ARTIFACTS_STRING.toLowerCase()))
			playerOption = PlayerOption.LEAST_ARTIFACTS;
		else if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_DONT_CARE_STRING.toLowerCase()))
			playerOption = PlayerOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_MissionSelectionSpecifier, constructor: Invalid player option string!");
	}

	private ArrayList<String> initializeFromXmlNode(Node actionNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = actionNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newCardOption = null;
		String newPlayerOption = null;
		String newCard = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("cardoption"))
				{
					newCardOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("playeroption"))
				{
					newPlayerOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("card"))
				{
					newCard = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_MissionSelectionSpecifier.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_MissionSelectionSpecifier.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newCardOption);
		returnValue.add(newPlayerOption);
		returnValue.add(newCard);

		return returnValue;
	}


	public CN_InputMessage getAction(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<Integer> currentMissionCards = new ArrayList<Integer>();

		for(CN_GameState.PlayerData player : gameState.players)
		{
			currentMissionCards.add(player.mission);
		}

		currentMissionCards = pruneForCard(currentMissionCards);
		currentMissionCards = pruneForPlayer(currentMissionCards, gameState);

		if(currentMissionCards.size() <= 0)
			return null;
			//throw new AI_FatalException("AI_MissionSelectionSpecifier.getAction(): Hits were zero, so couldn't create an input message!");

		Integer selectedMissionCard = currentMissionCards.get(new Random().nextInt(currentMissionCards.size()));
		AI_MissionCard selectedPlayCard = AI_Constants.missionCardDirectory.get(selectedMissionCard);

		CN_InputMessage returnValue = new CN_InputMessage(CN_InputMessage.MessageType.MISSION_CARD_SELECTION, selectedMissionCard, null, selectedPlayCard.cardName + " was selected.");

 		return returnValue;
	}

	private ArrayList<Integer> pruneForCard(ArrayList<Integer> currentMissionCards) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		switch(cardOption)
		{
			case IDENTITY:
				for(Integer missionCard : currentMissionCards)
				{
					if(AI_Constants.timelineCardDirectory.get(missionCard).identity.equals(card.identity))
						returnValue.add(missionCard);
				}
				break;
			case ANY:
				for(Integer missionCard : currentMissionCards)
				{
					returnValue.add(missionCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_MissionSelectionSpecifier.pruneForCard(): Invalid card option!");
		}

		return returnValue;
	}


	private ArrayList<Integer> pruneForPlayer(ArrayList<Integer> currentMissionCards, CN_GameState gameState) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		switch(playerOption)
		{
			case CURRENT_PLAYER:
				for(Integer missionCard : currentMissionCards)
				{
					for(CN_GameState.PlayerData playerData : gameState.players)
					{
						if((playerData.mission == missionCard)&&(playerData.playerIndex == gameState.currentPlayer))
							returnValue.add(missionCard);
					}
				}
				break;
			case NOT_CURRENT_PLAYER:
				for(Integer missionCard : currentMissionCards)
				{
					for(CN_GameState.PlayerData playerData : gameState.players)
					{
						if((playerData.mission == missionCard)&&(playerData.playerIndex != gameState.currentPlayer))
							returnValue.add(missionCard);
					}
				}
				break;
			case MOST_ARTIFACTS:
				int mostArtifactsPlayer = playerWithMostArtifacts(gameState);
				for(Integer missionCard : currentMissionCards)
				{
					for(CN_GameState.PlayerData playerData : gameState.players)
					{
						if((playerData.mission == missionCard)&&(playerData.playerIndex == mostArtifactsPlayer))
							returnValue.add(missionCard);
					}
				}
				break;
			case LEAST_ARTIFACTS:
				int leastArtifactsPlayer = playerWithLeastArtifacts(gameState);
				for(Integer missionCard : currentMissionCards)
				{
					for(CN_GameState.PlayerData playerData : gameState.players)
					{
						if((playerData.mission == missionCard)&&(playerData.playerIndex == leastArtifactsPlayer))
							returnValue.add(missionCard);
					}
				}
				break;
			case DONT_CARE:
				for(Integer missionCard : currentMissionCards)
				{
					returnValue.add(missionCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_MissionSelectionSpecifier.pruneForPlayer(): Invalid player option!");
		}

		return returnValue;
	}

	private int playerWithMostArtifacts(CN_GameState gameState)
	{
		int currentHighestPlayer = 0;
		for(CN_GameState.PlayerData player : gameState.players)
		{
			if(player.artifacts.size() > gameState.players[currentHighestPlayer].artifacts.size())
			{
				currentHighestPlayer = player.playerIndex;
			}
		}
		return currentHighestPlayer;
	}
	private int playerWithLeastArtifacts(CN_GameState gameState)
	{
		int currentLowestPlayer = 0;
		for(CN_GameState.PlayerData player : gameState.players)
		{
			if(player.artifacts.size() < gameState.players[currentLowestPlayer].artifacts.size())
			{
				currentLowestPlayer = player.playerIndex;
			}
		}
		return currentLowestPlayer;
	}
}
