package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_ArtifactComparison extends AI_Comparison
{
	private enum CardOption {IDENTITY, ANY}
	private enum CardRelevanceOption {FOR_MISSION, NOT_FOR_MISSION, DONT_CARE}
	private enum CardOwnerOption {CURRENT_PLAYER, NOT_CURRENT_PLAYER, NOT_OWNED, DONT_CARE}

	private static final String CARD_OPTION_IDENTITY_STRING = "Identity";
	private static final String CARD_OPTION_ANY_STRING = "Any";
	private static final String CARD_RELEVANCE_OPTION_FOR_MISSION_STRING = "ForMission";
	private static final String CARD_RELEVANCE_OPTION_NOT_FOR_MISSION_STRING = "NotForMission";
	private static final String CARD_RELEVANCE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String CARD_OWNER_OPTION_CURRENT_PLAYER_STRING = "CurrentPlayer";
	private static final String CARD_OWNER_OPTION_NOT_CURRENT_PLAYER_STRING = "NotCurrentPlayer";
	private static final String CARD_OWNER_OPTION_NOT_OWNED_STRING = "NotOwned";
	private static final String CARD_OWNER_OPTION_DONT_CARE_STRING = "DontCare";

	private final CardOption cardOption;
	private final CardRelevanceOption cardRelevanceOption;
	private final CardOwnerOption cardOwnerOption;

	private final AI_Artifact card;



	public AI_ArtifactComparison(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String cardOption_in = inputValues.get(0);
		String cardRelevanceOption_in = inputValues.get(1);
		String cardOwnerOption_in = inputValues.get(2);
		String card_in = inputValues.get(3);

		if(card_in != null)
		{
			card = AI_Globals.artifactTable.get(card_in);
			if(card == null)
				throw new AI_FatalException("AI_ArtifactComparison, constructor: Specified linchpin does not exist!");
		}
		else
		{
			card = null;
		}

		if(cardOption_in.toLowerCase().equals(CARD_OPTION_IDENTITY_STRING.toLowerCase()))
			cardOption = CardOption.IDENTITY;
		else if(cardOption_in.toLowerCase().equals(CARD_OPTION_ANY_STRING.toLowerCase()))
			cardOption = CardOption.ANY;
		else
			throw new AI_FatalException("AI_ArtifactComparison, constructor: Invalid card option string!  String: " + cardOption_in);

		if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_FOR_MISSION_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.FOR_MISSION;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_NOT_FOR_MISSION_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.NOT_FOR_MISSION;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_ArtifactComparison, constructor: Invalid card relevance option string!  String: " + cardRelevanceOption_in);

		if(cardOwnerOption_in.toLowerCase().equals(CARD_OWNER_OPTION_CURRENT_PLAYER_STRING.toLowerCase()))
			cardOwnerOption = CardOwnerOption.CURRENT_PLAYER;
		else if(cardOwnerOption_in.toLowerCase().equals(CARD_OWNER_OPTION_NOT_CURRENT_PLAYER_STRING.toLowerCase()))
			cardOwnerOption = CardOwnerOption.NOT_CURRENT_PLAYER;
		else if(cardOwnerOption_in.toLowerCase().equals(CARD_OWNER_OPTION_NOT_OWNED_STRING.toLowerCase()))
			cardOwnerOption = CardOwnerOption.NOT_OWNED;
		else if(cardOwnerOption_in.toLowerCase().equals(CARD_OWNER_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardOwnerOption = CardOwnerOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_ArtifactComparison, constructor: Invalid card owner option string!  String: " + cardOwnerOption_in);
	}



	private ArrayList<String> initializeFromXmlNode(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = comparisonNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newCardOption = null;
		String newCardRelevanceOption = null;
		String newCardOwnerOption = null;
		String newCard = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("cardoption"))
				{
					newCardOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardrelevanceoption"))
				{
					newCardRelevanceOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardowneroption"))
				{
					newCardOwnerOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("card"))
				{
					newCard = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_ArtifactComparison.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_ArtifactComparison.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newCardOption);
		returnValue.add(newCardRelevanceOption);
		returnValue.add(newCardOwnerOption);
		returnValue.add(newCard);

		return returnValue;
	}


	public boolean compare(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<AI_Artifact> satisfyingArtifacts = getCardOptionList();

		satisfyingArtifacts = pruneForCardRelevance(satisfyingArtifacts, gameState, mission);
		satisfyingArtifacts = pruneForCardOwner(satisfyingArtifacts, gameState);

		if(satisfyingArtifacts.size() > 0)
			return true;
		else
			return false;
	}

	private ArrayList<AI_Artifact> getCardOptionList() throws AI_FatalException
	{
		ArrayList<AI_Artifact> returnValue = new ArrayList<AI_Artifact>();

		switch(cardOption)
		{
			case IDENTITY:
				returnValue.add(card);
				break;
			case ANY:
				for(AI_Artifact artifact : AI_Globals.artifactList)
				{
					returnValue.add(artifact);
				}
				break;
			default:
				throw new AI_FatalException("AI_ArtifactComparison.getCardOptionList(): Invalid card option!");
		}

		return returnValue;
	}


	private ArrayList<AI_Artifact> pruneForCardRelevance(ArrayList<AI_Artifact> currentArtifacts, CN_GameState gameState, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<AI_Artifact> returnValue = new ArrayList<AI_Artifact>();
		ArrayList<AI_Artifact> artifactContext = createContext(gameState);

		switch(cardRelevanceOption)
		{
			case FOR_MISSION:
				for(AI_Artifact artifact : currentArtifacts)
				{
					if(mission.isArtifactForMission(artifact, gameState, artifactContext))
						returnValue.add(artifact);
				}
				break;
			case NOT_FOR_MISSION:
				for(AI_Artifact artifact : currentArtifacts)
				{
					if(!mission.isArtifactForMission(artifact, gameState, artifactContext))
						returnValue.add(artifact);
				}
				break;
			case DONT_CARE:
				for(AI_Artifact artifact : currentArtifacts)
				{
					returnValue.add(artifact);
				}
				break;
			default:
				throw new AI_FatalException("AI_ArtifactComparison.pruneForCardRelevance(): Invalid card relevance option!");
		}

		return returnValue;
	}


	private ArrayList<AI_Artifact> pruneForCardOwner(ArrayList<AI_Artifact> currentArtifacts, CN_GameState gameState) throws AI_FatalException
	{
		ArrayList<AI_Artifact> returnValue = new ArrayList<AI_Artifact>();

		ArrayList<AI_Artifact> currentPlayers = new ArrayList<AI_Artifact>();
		ArrayList<AI_Artifact> notCurrentPlayers = new ArrayList<AI_Artifact>();

		for(CN_GameState.PlayerData player : gameState.players)
		{
			if(player.playerIndex == gameState.currentPlayer)
			{
				for(Integer artIndex : player.artifacts)
				{
					currentPlayers.add((AI_Artifact)AI_Constants.playCardDirectory.get(artIndex));
				}
			}
			else
			{
				for(Integer artIndex : player.artifacts)
				{
					notCurrentPlayers.add((AI_Artifact)AI_Constants.playCardDirectory.get(artIndex));
				}
			}
		}

		switch(cardOwnerOption)
		{
			case CURRENT_PLAYER:
				for(AI_Artifact artifact : currentArtifacts)
				{
					for(AI_Artifact artifact1 : currentPlayers)
					{
						if(artifact.identity.equals(artifact1.identity))
							returnValue.add(artifact);
					}
				}
				break;
			case NOT_CURRENT_PLAYER:
				for(AI_Artifact artifact : currentArtifacts)
				{
					for(AI_Artifact artifact1 : notCurrentPlayers)
					{
						if(artifact.identity.equals(artifact1.identity))
							returnValue.add(artifact);
					}
				}
				break;
			case NOT_OWNED:
				for(AI_Artifact artifact : currentArtifacts)
				{
					for(AI_Artifact artifact1 : currentPlayers)
					{
						if(artifact.identity.equals(artifact1.identity))
							break;
					}
					for(AI_Artifact artifact1 : notCurrentPlayers)
					{
						if(artifact.identity.equals(artifact1.identity))
							break;
					}
					returnValue.add(artifact);
				}
				break;
			case DONT_CARE:
				for(AI_Artifact artifact : currentArtifacts)
				{
					returnValue.add(artifact);
				}
				break;
			default:
				throw new AI_FatalException("AI_ArtifactComparison.pruneForCardOwner(): Invalid card owner option!");
		}

		return returnValue;
	}

	private ArrayList<AI_Artifact> createContext(CN_GameState gameState)
	{
		return null;
	}

}
