package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_HandCardComparison extends AI_Comparison
{
	private enum CardOption {IDENTITY, ANY}
	private enum CardTypeOption {ARTIFACT, PATCH, INVERTER, ACTION, TIMEWARP, DONT_CARE}
	private enum CardRelevanceOption {PATCH_FOR_ID, PATCH_NOT_FOR_ID, ARTIFACT_FOR_MISSION, ARTIFACT_NOT_FOR_MISSION, FOR_EITHER, NOT_FOR_EITHER, DONT_CARE}

	private static final String CARD_OPTION_IDENTITY_STRING = "Identity";
	private static final String CARD_OPTION_ANY_STRING = "Any";
	private static final String CARD_TYPE_OPTION_ARTIFACT_STRING = "Artifact";
	private static final String CARD_TYPE_OPTION_PATCH_STRING = "Patch";
	private static final String CARD_TYPE_OPTION_INVERTER_STRING = "Inverter";
	private static final String CARD_TYPE_OPTION_ACTION_STRING = "Action";
	private static final String CARD_TYPE_OPTION_TIMEWARP_STRING = "Timewarp";
	private static final String CARD_TYPE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String CARD_RELEVANCE_OPTION_PATCH_FOR_ID_STRING = "PatchForId";
	private static final String CARD_RELEVANCE_OPTION_PATCH_NOT_FOR_ID_STRING = "PatchNotForId";
	private static final String CARD_RELEVANCE_OPTION_ARTIFACT_FOR_MISSION_STRING = "ArtifactForMission";
	private static final String CARD_RELEVANCE_OPTION_ARTIFACT_NOT_FOR_MISSION_STRING = "ArtifactNotForMission";
	private static final String CARD_RELEVANCE_OPTION_FOR_EITHER_STRING = "ForEither";
	private static final String CARD_RELEVANCE_OPTION_NOT_FOR_EITHER_STRING = "NotForEither";
	private static final String CARD_RELEVANCE_OPTION_DONT_CARE_STRING = "DontCare";

	private final CardOption cardOption;
	private final CardTypeOption cardTypeOption;
	private final CardRelevanceOption cardRelevanceOption;

	private final AI_PlayCard card;


	public AI_HandCardComparison(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String cardOption_in = inputValues.get(0);
		String cardTypeOption_in = inputValues.get(1);
		String cardRelevanceOption_in = inputValues.get(2);
		String card_in = inputValues.get(3);

		if(card_in != null)
		{
			card = AI_Globals.playCardTable.get(card_in);
			if(card == null)
				throw new AI_FatalException("AI_HandCardComparison, constructor: Specified card does not exist!");
		}
		else
		{
			card = null;
		}

		if(cardOption_in.toLowerCase().equals(CARD_OPTION_IDENTITY_STRING.toLowerCase()))
			cardOption = CardOption.IDENTITY;
		else if(cardOption_in.toLowerCase().equals(CARD_OPTION_ANY_STRING.toLowerCase()))
			cardOption = CardOption.ANY;
		else
			throw new AI_FatalException("AI_HandCardComparison, constructor: Invalid card option string!  String: " + cardOption_in);

		if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_ARTIFACT_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.ARTIFACT;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_PATCH_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.PATCH;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_INVERTER_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.INVERTER;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_ACTION_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.ACTION;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_TIMEWARP_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.TIMEWARP;
		else if(cardTypeOption_in.toLowerCase().equals(CARD_TYPE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardTypeOption = CardTypeOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_HandCardComparison, constructor: Invalid card type option string!  String: " + cardTypeOption_in);

		if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_PATCH_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.PATCH_FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_PATCH_NOT_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.PATCH_NOT_FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_ARTIFACT_FOR_MISSION_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.ARTIFACT_FOR_MISSION;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_ARTIFACT_NOT_FOR_MISSION_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.ARTIFACT_NOT_FOR_MISSION;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_FOR_EITHER_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.FOR_EITHER;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_NOT_FOR_EITHER_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.NOT_FOR_EITHER;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_HandCardComparison, constructor: Invalid card relevance option string!  String: " + cardRelevanceOption_in);
	}

	private ArrayList<String> initializeFromXmlNode(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = comparisonNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newCardOption = null;
		String newCardTypeOption = null;
		String newCardRelevanceOption = null;
		String newCard = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("cardoption"))
				{
					newCardOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardtypeoption"))
				{
					newCardTypeOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardrelevanceoption"))
				{
					newCardRelevanceOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("card"))
				{
					newCard = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_HandCardComparison.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_HandCardComparison.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newCardOption);
		returnValue.add(newCardTypeOption);
		returnValue.add(newCardRelevanceOption);
		returnValue.add(newCard);

		return returnValue;
	}





	public boolean compare(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<AI_PlayCard> currentHandCards = getCardOptionList(gameState);

		currentHandCards = pruneForCardType(currentHandCards);
		currentHandCards = pruneForCardRelevance(currentHandCards, gameState, id, mission);

		if(currentHandCards.size() > 0)
			return true;
		else
			return false;
	}



	private ArrayList<AI_PlayCard> getCardOptionList(CN_GameState gameState) throws AI_FatalException
	{
		ArrayList<AI_PlayCard> returnValue = new ArrayList<AI_PlayCard>();

		switch(cardOption)
		{
			case IDENTITY:
				if(cardInCurrentPlayersHand(card, gameState))
					returnValue.add(card);
				break;
			case ANY:
				for(AI_PlayCard playCard : AI_Globals.playCardList)
				{
					if(cardInCurrentPlayersHand(playCard, gameState))
						returnValue.add(playCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_HandCardComparison.getCardOptionList(): Invalid card option!");
		}

		return returnValue;
	}

	private boolean cardInCurrentPlayersHand(AI_PlayCard card, CN_GameState gameState)
	{
		CN_GameState.PlayerData player = gameState.players[gameState.currentPlayer];

		for(Integer cardIndex : player.hand)
		{
			if(AI_Constants.playCardDirectory.get(cardIndex).identity.equals(card.identity))
				return true;
		}

		return false;
	}


	private ArrayList<AI_PlayCard> pruneForCardType(ArrayList<AI_PlayCard> currentHandCards) throws AI_FatalException
	{
		ArrayList<AI_PlayCard> returnValue = new ArrayList<AI_PlayCard>();

		switch(cardTypeOption)
		{
			case ARTIFACT:
				for(AI_PlayCard handCard : currentHandCards)
				{
					if(handCard instanceof AI_Artifact)
						returnValue.add(handCard);
				}
				break;
			case PATCH:
				for(AI_PlayCard handCard : currentHandCards)
				{
					if(handCard instanceof AI_Patch)
						returnValue.add(handCard);
				}
				break;
			case INVERTER:
				for(AI_PlayCard handCard : currentHandCards)
				{
					if(handCard instanceof AI_Inverter)
						returnValue.add(handCard);
				}
				break;
			case ACTION:
				for(AI_PlayCard handCard : currentHandCards)
				{
					if(handCard instanceof AI_Action)
						returnValue.add(handCard);
				}
				break;
			case TIMEWARP:
				for(AI_PlayCard handCard : currentHandCards)
				{
					if(handCard instanceof AI_Timewarp)
						returnValue.add(handCard);
				}
				break;
			case DONT_CARE:
				for(AI_PlayCard handCard : currentHandCards)
				{
					returnValue.add(handCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_HandCardComparison.pruneForCard(): Invalid card type option!");
		}

		return returnValue;
	}



	private ArrayList<AI_PlayCard> pruneForCardRelevance(ArrayList<AI_PlayCard> currentHandCards, CN_GameState gameState, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<AI_PlayCard> returnValue = new ArrayList<AI_PlayCard>();
		ArrayList<AI_Artifact> artifactContext = createContext(gameState);

		switch(cardRelevanceOption)
		{
			case PATCH_FOR_ID:
				for(AI_PlayCard handCard : currentHandCards)
				{
					if(handCard instanceof AI_Patch)
					{
						if(id.isPatchForId((AI_Patch)handCard))
							returnValue.add(handCard);
					}
				}
				break;
			case PATCH_NOT_FOR_ID:
				for(AI_PlayCard handCard : currentHandCards)
				{
					if(handCard instanceof AI_Patch)
					{
						if(!id.isPatchForId((AI_Patch)handCard))
							returnValue.add(handCard);
					}
				}
				break;
			case ARTIFACT_FOR_MISSION:
				for(AI_PlayCard handCard : currentHandCards)
				{
					if(handCard instanceof AI_Artifact)
					{
						if(mission.isArtifactForMission((AI_Artifact)handCard, gameState, artifactContext))
							returnValue.add(handCard);
					}
				}
				break;
			case ARTIFACT_NOT_FOR_MISSION:
				for(AI_PlayCard handCard : currentHandCards)
				{
					if(handCard instanceof AI_Artifact)
					{
						if(!mission.isArtifactForMission((AI_Artifact)handCard, gameState, artifactContext))
							returnValue.add(handCard);
					}
				}
				break;
			case FOR_EITHER:
				for(AI_PlayCard handCard : currentHandCards)
				{
					if(handCard instanceof AI_Patch)
					{
						if(id.isPatchForId((AI_Patch)handCard))
							returnValue.add(handCard);
					}
					else if(handCard instanceof AI_Artifact)
					{
						if(mission.isArtifactForMission((AI_Artifact)handCard, gameState, artifactContext))
							returnValue.add(handCard);
					}
				}
				break;
			case NOT_FOR_EITHER:
				for(AI_PlayCard handCard : currentHandCards)
				{
					if(handCard instanceof AI_Patch)
					{
						if(id.isPatchForId((AI_Patch)handCard))
							break;
					}
					else if(handCard instanceof AI_Artifact)
					{
						if(mission.isArtifactForMission((AI_Artifact)handCard, gameState, artifactContext))
							break;
					}
					returnValue.add(handCard);
				}
				break;
			case DONT_CARE:
				for(AI_PlayCard handCard : currentHandCards)
				{
					returnValue.add(handCard);
				}
				break;
			default:
				throw new AI_FatalException("AI_HandCardComparison.pruneForCard(): Invalid card relevance option!");
		}

		return returnValue;
	}

	private ArrayList<AI_Artifact> createContext(CN_GameState gameState)
	{
		ArrayList<AI_Artifact> returnValue = new ArrayList<AI_Artifact>();
		AI_PlayCard currentPlayCard;
		for(Integer integer : gameState.players[gameState.currentPlayer].hand)
		{
			currentPlayCard = AI_Constants.playCardDirectory.get(integer);
			if(currentPlayCard instanceof AI_Artifact)
				returnValue.add((AI_Artifact)currentPlayCard);
		}
		return returnValue;
	}

}
