package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_HandSizeComparison extends AI_Comparison
{
	private enum PlayerOption {CURRENT_PLAYER, NOT_CURRENT_PLAYER, NOBODY, DONT_CARE}
	private enum OperatorOption {EQUAL, NOT_EQUAL, LESS, GREATER, LESS_OR_EQUAL, GREATER_OR_EQUAL}

	private static final String PLAYER_OPTION_CURRENT_PLAYER_STRING = "CurrentPlayer";
	private static final String PLAYER_OPTION_NOT_CURRENT_PLAYER_STRING = "NotCurrentPlayer";
	private static final String PLAYER_OPTION_NOBODY_STRING = "Nobody";
	private static final String PLAYER_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String OPERATOR_OPTION_EQUAL_STRING = "Equal";
	private static final String OPERATOR_OPTION_NOT_EQUAL_STRING = "NotEqual";
	private static final String OPERATOR_OPTION_LESS_STRING = "Less";
	private static final String OPERATOR_OPTION_GREATER_STRING = "Greater";
	private static final String OPERATOR_OPTION_LESS_OR_EQUAL_STRING = "LessOrEqual";
	private static final String OPERATOR_OPTION_GREATER_OR_EQUAL_STRING = "GreaterOrEqual";

	private final PlayerOption playerOption;
	private final OperatorOption operatorOption;

	private final int amount;


	private boolean invertOperators = false;


	public AI_HandSizeComparison(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String playerOption_in = inputValues.get(0);
		String operatorOption_in = inputValues.get(1);
		String amount_in = inputValues.get(2);

		try
		{
			amount = Integer.valueOf(amount_in);
		}
		catch(NumberFormatException e)
		{
			throw new AI_FatalException("AI_HandSizeComparison, constructor: Amount could not be parsed!", e);
		}


		if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_CURRENT_PLAYER_STRING.toLowerCase()))
			playerOption = PlayerOption.CURRENT_PLAYER;
		else if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_NOT_CURRENT_PLAYER_STRING.toLowerCase()))
			playerOption = PlayerOption.NOT_CURRENT_PLAYER;
		else if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_NOBODY_STRING.toLowerCase()))
			playerOption = PlayerOption.NOBODY;
		else if(playerOption_in.toLowerCase().equals(PLAYER_OPTION_DONT_CARE_STRING.toLowerCase()))
			playerOption = PlayerOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_HandSizeComparison, constructor: Invalid player option string!");

		if(operatorOption_in.toLowerCase().equals(OPERATOR_OPTION_EQUAL_STRING.toLowerCase()))
			operatorOption = OperatorOption.EQUAL;
		else if(operatorOption_in.toLowerCase().equals(OPERATOR_OPTION_NOT_EQUAL_STRING.toLowerCase()))
			operatorOption = OperatorOption.NOT_EQUAL;
		else if(operatorOption_in.toLowerCase().equals(OPERATOR_OPTION_LESS_STRING.toLowerCase()))
			operatorOption = OperatorOption.LESS;
		else if(operatorOption_in.toLowerCase().equals(OPERATOR_OPTION_GREATER_STRING.toLowerCase()))
			operatorOption = OperatorOption.GREATER;
		else if(operatorOption_in.toLowerCase().equals(OPERATOR_OPTION_LESS_OR_EQUAL_STRING.toLowerCase()))
			operatorOption = OperatorOption.LESS_OR_EQUAL;
		else if(operatorOption_in.toLowerCase().equals(OPERATOR_OPTION_GREATER_OR_EQUAL_STRING.toLowerCase()))
			operatorOption = OperatorOption.GREATER_OR_EQUAL;
		else
			throw new AI_FatalException("AI_HandSizeComparison, constructor: Invalid operator option string!");

	}

	private ArrayList<String> initializeFromXmlNode(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = comparisonNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newPlayerOption = null;
		String newOperatorOption = null;
		String newAmount = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("playeroption"))
				{
					newPlayerOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("operatoroption"))
				{
					newOperatorOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("amount"))
				{
					newAmount = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_HandSizeComparison.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_HandSizeComparison.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newPlayerOption);
		returnValue.add(newOperatorOption);
		returnValue.add(newAmount);

		return returnValue;
	}





	public boolean compare(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		invertOperators = false;
		ArrayList<Integer> satisfyingPlayers = getPlayerOptionList(gameState);

		OperatorOption effectiveOperator = operatorOption;
		if(invertOperators)
			effectiveOperator = getInvertedOperator();

		satisfyingPlayers = pruneForOperator(satisfyingPlayers, gameState, effectiveOperator);

		if(satisfyingPlayers.size() > 0)
			return true;
		else
			return false;
	}


	private ArrayList<Integer> getPlayerOptionList(CN_GameState gameState) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		switch(playerOption)
		{
			case CURRENT_PLAYER:
				for(CN_GameState.PlayerData player : gameState.players)
				{
					if(player.playerIndex == gameState.currentPlayer)
						returnValue.add(player.playerIndex);
				}
				break;
			case NOT_CURRENT_PLAYER:
				for(CN_GameState.PlayerData player : gameState.players)
				{
					if(player.playerIndex != gameState.currentPlayer)
						returnValue.add(player.playerIndex);
				}
				break;
			case NOBODY:
				invertOperators = true;
				for(CN_GameState.PlayerData player : gameState.players)
				{
					returnValue.add(player.playerIndex);
				}
				break;
			case DONT_CARE:
				for(CN_GameState.PlayerData player : gameState.players)
				{
					returnValue.add(player.playerIndex);
				}
				break;
			default:
				throw new AI_FatalException("AI_HandSizeComparison.getPlayerOptionList(): Invalid player option!");
		}

		return returnValue;
	}


	private ArrayList<Integer> pruneForOperator(ArrayList<Integer> satisfyingPlayers, CN_GameState gameState, OperatorOption effectiveOperator) throws AI_FatalException
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		int[] handSizes = new int[gameState.players.length];
		for(CN_GameState.PlayerData player : gameState.players)
		{
			handSizes[player.playerIndex] = player.hand.size();
		}

		switch(effectiveOperator)
		{
			case EQUAL:
				for(Integer player : satisfyingPlayers)
				{
					if(handSizes[player] == amount)
						returnValue.add(player);
				}
				break;
			case NOT_EQUAL:
				for(Integer player : satisfyingPlayers)
				{
					if(handSizes[player] != amount)
						returnValue.add(player);
				}
				break;
			case LESS:
				for(Integer player : satisfyingPlayers)
				{
					if(handSizes[player] < amount)
						returnValue.add(player);
				}
				break;
			case GREATER:
				for(Integer player : satisfyingPlayers)
				{
					if(handSizes[player] > amount)
						returnValue.add(player);
				}
				break;
			case LESS_OR_EQUAL:
				for(Integer player : satisfyingPlayers)
				{
					if(handSizes[player] <= amount)
						returnValue.add(player);
				}
				break;
			case GREATER_OR_EQUAL:
				for(Integer player : satisfyingPlayers)
				{
					if(handSizes[player] >= amount)
						returnValue.add(player);
				}
				break;
			default:
				throw new AI_FatalException("AI_HandSizeComparison.pruneForOperator(): Invalid operator option!");
		}

		return returnValue;
	}



	private OperatorOption getInvertedOperator() throws AI_FatalException
	{
		OperatorOption returnValue;
		switch(operatorOption)
		{
			case EQUAL:
				returnValue = OperatorOption.NOT_EQUAL;
				break;
			case NOT_EQUAL:
				returnValue = OperatorOption.EQUAL;
				break;
			case LESS:
				returnValue = OperatorOption.GREATER_OR_EQUAL;
				break;
			case GREATER:
				returnValue = OperatorOption.LESS_OR_EQUAL;
				break;
			case LESS_OR_EQUAL:
				returnValue = OperatorOption.GREATER;
				break;
			case GREATER_OR_EQUAL:
				returnValue = OperatorOption.LESS;
				break;
			default:
				throw new AI_FatalException("AI_HandSizeComparison.getInvertedOperator(): Invalid operator option!");
		}
		return returnValue;
	}

}
