package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_LinchpinComparison extends AI_Comparison
{
	private enum CardOption {IDENTITY, ANY}
	private enum CardRelevanceOption {FOR_ID, NOT_FOR_ID, DONT_CARE}
	private enum CardStateOption {FLIPPED, UNFLIPPED, FOR_ID, NOT_FOR_ID, DONT_CARE}
	private enum LegalityOption {LEGAL, ILLEGAL, DONT_CARE}

	private static final String CARD_OPTION_IDENTITY_STRING = "Identity";
	private static final String CARD_OPTION_ANY_STRING = "Any";
	private static final String CARD_RELEVANCE_OPTION_FOR_ID_STRING = "ForId";
	private static final String CARD_RELEVANCE_OPTION_NOT_FOR_ID_STRING = "NotForId";
	private static final String CARD_RELEVANCE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String CARD_STATE_OPTION_FLIPPED_STRING = "Flipped";
	private static final String CARD_STATE_OPTION_UNFLIPPED_STRING = "Unflipped";
	private static final String CARD_STATE_OPTION_FOR_ID_STRING = "ForId";
	private static final String CARD_STATE_OPTION_NOT_FOR_ID_STRING = "NotForId";
	private static final String CARD_STATE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String LEGALITY_OPTION_LEGAL_STRING = "Legal";
	private static final String LEGALITY_OPTION_ILLEGAL_STRING = "Illegal";
	private static final String LEGALITY_OPTION_DONT_CARE_STRING = "DontCare";

	private final CardOption cardOption;
	private final CardRelevanceOption cardRelevanceOption;
	private final CardStateOption cardStateOption;
	private final LegalityOption legalityOption;

	private final AI_Linchpin card;


	public AI_LinchpinComparison(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String cardOption_in = inputValues.get(0);
		String cardRelevanceOption_in = inputValues.get(1);
		String cardStateOption_in = inputValues.get(2);
		String legalityOption_in = inputValues.get(3);
		String card_in = inputValues.get(4);

		if(card_in != null)
		{
			card = AI_Globals.linchpinTable.get(card_in);
			if(card == null)
				throw new AI_FatalException("AI_LinchpinComparison, constructor: Specified linchpin does not exist!  Linchpin: " + card_in);
		}
		else
		{
			card = null;
		}

		if(cardOption_in.toLowerCase().equals(CARD_OPTION_IDENTITY_STRING.toLowerCase()))
			cardOption = CardOption.IDENTITY;
		else if(cardOption_in.toLowerCase().equals(CARD_OPTION_ANY_STRING.toLowerCase()))
			cardOption = CardOption.ANY;
		else
			throw new AI_FatalException("AI_LinchpinComparison, constructor: Invalid card option string!");

		if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_NOT_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.NOT_FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_LinchpinComparison, constructor: Invalid card relevance option string!");

		if(cardStateOption_in.toLowerCase().equals(CARD_STATE_OPTION_FLIPPED_STRING.toLowerCase()))
			cardStateOption = CardStateOption.FLIPPED;
		else if(cardStateOption_in.toLowerCase().equals(CARD_STATE_OPTION_UNFLIPPED_STRING.toLowerCase()))
			cardStateOption = CardStateOption.UNFLIPPED;
		else if(cardStateOption_in.toLowerCase().equals(CARD_STATE_OPTION_FOR_ID_STRING.toLowerCase()))
			cardStateOption = CardStateOption.FOR_ID;
		else if(cardStateOption_in.toLowerCase().equals(CARD_STATE_OPTION_NOT_FOR_ID_STRING.toLowerCase()))
			cardStateOption = CardStateOption.NOT_FOR_ID;
		else if(cardStateOption_in.toLowerCase().equals(CARD_STATE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardStateOption = CardStateOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_LinchpinComparison, constructor: Invalid card state option string!");

		if(legalityOption_in.toLowerCase().equals(LEGALITY_OPTION_LEGAL_STRING.toLowerCase()))
			legalityOption = LegalityOption.LEGAL;
		else if(legalityOption_in.toLowerCase().equals(LEGALITY_OPTION_ILLEGAL_STRING.toLowerCase()))
			legalityOption = LegalityOption.ILLEGAL;
		else if(legalityOption_in.toLowerCase().equals(LEGALITY_OPTION_DONT_CARE_STRING.toLowerCase()))
			legalityOption = LegalityOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_LinchpinComparison, constructor: Invalid legality option string!");
	}


	private ArrayList<String> initializeFromXmlNode(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = comparisonNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newCardOption = null;
		String newCardRelevanceOption = null;
		String newCardStateOption = null;
		String newLegalityOption = null;
		String newCard = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("cardoption"))
				{
					newCardOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardrelevanceoption"))
				{
					newCardRelevanceOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardstateoption"))
				{
					newCardStateOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("legalityoption"))
				{
					newLegalityOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("card"))
				{
					newCard = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_LinchpinComparison.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_LinchpinComparison.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newCardOption);
		returnValue.add(newCardRelevanceOption);
		returnValue.add(newCardStateOption);
		returnValue.add(newLegalityOption);
		returnValue.add(newCard);

		return returnValue;
	}




	public boolean compare(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<AI_Linchpin> satisfyingLinchpins = getCardOptionList();

		satisfyingLinchpins = pruneForCardRelevance(satisfyingLinchpins, id);
		satisfyingLinchpins = pruneForCardState(satisfyingLinchpins, timeline, id);
		satisfyingLinchpins = pruneForLegality(satisfyingLinchpins, timeline);

//System.out.println("linchpins: " + satisfyingLinchpins.size());
//for(AI_Linchpin linchpin : satisfyingLinchpins)
//{
//	System.out.println(linchpin.identity);
//}

		if(satisfyingLinchpins.size() > 0)
			return true;
		else
			return false;
	}

	private ArrayList<AI_Linchpin> getCardOptionList() throws AI_FatalException
	{
		ArrayList<AI_Linchpin> returnValue = new ArrayList<AI_Linchpin>();

		switch(cardOption)
		{
			case IDENTITY:
				returnValue.add(card);
				break;
			case ANY:
				for(AI_Linchpin linchpin : AI_Globals.linchpinList)
				{
					returnValue.add(linchpin);
				}
				break;
			default:
				throw new AI_FatalException("AI_LinchpinComparison.getCardOptionList(): Invalid card option!");
		}

		return returnValue;
	}


	private ArrayList<AI_Linchpin> pruneForCardRelevance(ArrayList<AI_Linchpin> currentLinchpins, AI_IdCard id) throws AI_FatalException
	{
		ArrayList<AI_Linchpin> returnValue = new ArrayList<AI_Linchpin>();

		switch(cardRelevanceOption)
		{
			case FOR_ID:
				for(AI_Linchpin linchpin : currentLinchpins)
				{
					if(id.isLinchpinForId(linchpin))
						returnValue.add(linchpin);
				}
				break;
			case NOT_FOR_ID:
				for(AI_Linchpin linchpin : currentLinchpins)
				{
					if(!id.isLinchpinForId(linchpin))
						returnValue.add(linchpin);
				}
				break;
			case DONT_CARE:
				for(AI_Linchpin linchpin : currentLinchpins)
				{
					returnValue.add(linchpin);
				}
				break;
			default:
				throw new AI_FatalException("AI_LinchpinComparison.pruneForCardRelevance(): Invalid card relevance option!");
		}

		return returnValue;
	}


	private ArrayList<AI_Linchpin> pruneForCardState(ArrayList<AI_Linchpin> currentLinchpins, AI_GameStateTimeline timeline, AI_IdCard id) throws AI_FatalException
	{
		ArrayList<AI_Linchpin> returnValue = new ArrayList<AI_Linchpin>();

		switch(cardStateOption)
		{
			case FLIPPED:
				for(AI_Linchpin linchpin : currentLinchpins)
				{
					if(timeline.getByIdentity(linchpin.identity).isFlipped)
						returnValue.add(linchpin);
				}
				break;
			case UNFLIPPED:
				for(AI_Linchpin linchpin : currentLinchpins)
				{
					if(!timeline.getByIdentity(linchpin.identity).isFlipped)
						returnValue.add(linchpin);
				}
				break;
			case FOR_ID:
				for(AI_Linchpin linchpin : currentLinchpins)
				{
					if(id.isLinchpinStateForId(linchpin, timeline.getByIdentity(linchpin.identity).isFlipped))
						returnValue.add(linchpin);
				}
				break;
			case NOT_FOR_ID:
				for(AI_Linchpin linchpin : currentLinchpins)
				{
					if(!id.isLinchpinStateForId(linchpin, timeline.getByIdentity(linchpin.identity).isFlipped))
						returnValue.add(linchpin);
				}
				break;
			case DONT_CARE:
				for(AI_Linchpin linchpin : currentLinchpins)
				{
					returnValue.add(linchpin);
				}
				break;
			default:
				throw new AI_FatalException("AI_LinchpinComparison.pruneForCardState(): Invalid card state option!");
		}

		return returnValue;
	}


	private ArrayList<AI_Linchpin> pruneForLegality(ArrayList<AI_Linchpin> currentLinchpins, AI_GameStateTimeline timeline) throws AI_FatalException
	{
		ArrayList<AI_Linchpin> returnValue = new ArrayList<AI_Linchpin>();

		switch(legalityOption)
		{
			case LEGAL:
				for(AI_Linchpin linchpin : currentLinchpins)
				{
					if(timeline.getByIdentity(linchpin.identity).isLegal)
						returnValue.add(linchpin);
				}
				break;
			case ILLEGAL:
				for(AI_Linchpin linchpin : currentLinchpins)
				{
					if(!timeline.getByIdentity(linchpin.identity).isLegal)
						returnValue.add(linchpin);
				}
				break;
			case DONT_CARE:
				for(AI_Linchpin linchpin : currentLinchpins)
				{
					returnValue.add(linchpin);
				}
				break;
			default:
				throw new AI_FatalException("AI_LinchpinComparison.pruneForLegality(): Invalid legality option!");
		}

		return returnValue;
	}

}