package cnpc.ai;

import cnpc.engine.*;
import java.util.*;

import org.w3c.dom.*;

class AI_RipplepointComparison extends AI_Comparison
{
	private enum CardOption {IDENTITY, ANY}
	private enum CardRelevanceOption {FOR_ID, NOT_FOR_ID, DONT_CARE}
	private enum PatchStateOption {IDENTITY, UNFLIPPED, FLIPPED_NONE, PATCHED_BUT_ANY, FLIPPED_DONT_CARE, FOR_ID, NOT_FOR_ID, DONT_CARE}
	private enum LegalityOption {LEGAL, ILLEGAL, DONT_CARE}

	private static final String CARD_OPTION_IDENTITY_STRING = "Identity";
	private static final String CARD_OPTION_ANY_STRING = "Any";
	private static final String CARD_RELEVANCE_OPTION_FOR_ID_STRING = "ForId";
	private static final String CARD_RELEVANCE_OPTION_NOT_FOR_ID_STRING = "NotForId";
	private static final String CARD_RELEVANCE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String PATCH_STATE_OPTION_IDENTITY_STRING = "Identity";
	private static final String PATCH_STATE_OPTION_UNFLIPPED_STRING = "Unflipped";
	private static final String PATCH_STATE_OPTION_FLIPPED_NONE_STRING = "FlippedNone";
	private static final String PATCH_STATE_OPTION_PATCHED_BUT_ANY_STRING = "PatchedButAny";
	private static final String PATCH_STATE_OPTION_FLIPPED_DONT_CARE_STRING = "FlippedDontCare";
	private static final String PATCH_STATE_OPTION_FOR_ID_STRING = "ForId";
	private static final String PATCH_STATE_OPTION_NOT_FOR_ID_STRING = "NotForId";
	private static final String PATCH_STATE_OPTION_DONT_CARE_STRING = "DontCare";
	private static final String LEGALITY_OPTION_LEGAL_STRING = "Legal";
	private static final String LEGALITY_OPTION_ILLEGAL_STRING = "Illegal";
	private static final String LEGALITY_OPTION_DONT_CARE_STRING = "DontCare";

	private final CardOption cardOption;
	private final CardRelevanceOption cardRelevanceOption;
	private final PatchStateOption patchStateOption;
	private final LegalityOption legalityOption;

	private final AI_Ripplepoint card;
	private final AI_Patch patch;


	public AI_RipplepointComparison(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> inputValues = initializeFromXmlNode(comparisonNode);

		String cardOption_in = inputValues.get(0);
		String cardRelevanceOption_in = inputValues.get(1);
		String patchStateOption_in = inputValues.get(2);
		String legalityOption_in = inputValues.get(3);
		String card_in = inputValues.get(4);
		String patch_in = inputValues.get(5);

		if(card_in != null)
		{
			card = AI_Globals.ripplepointTable.get(card_in);
			if(card == null)
				throw new AI_FatalException("AI_RipplepointComparison, constructor: Specified ripplepoint does not exist!");
		}
		else
		{
			card = null;
		}

		if(patch_in != null)
		{
			patch = AI_Globals.patchTable.get(patch_in);
			if(patch == null)
				throw new AI_FatalException("AI_RipplepointComparison, constructor: Specified patch does not exist!");
		}
		else
		{
			patch = null;
		}

		if(cardOption_in.toLowerCase().equals(CARD_OPTION_IDENTITY_STRING.toLowerCase()))
			cardOption = CardOption.IDENTITY;
		else if(cardOption_in.toLowerCase().equals(CARD_OPTION_ANY_STRING.toLowerCase()))
			cardOption = CardOption.ANY;
		else
			throw new AI_FatalException("AI_RipplepointComparison: Invalid card option string!");

		if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_NOT_FOR_ID_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.NOT_FOR_ID;
		else if(cardRelevanceOption_in.toLowerCase().equals(CARD_RELEVANCE_OPTION_DONT_CARE_STRING.toLowerCase()))
			cardRelevanceOption = CardRelevanceOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_RipplepointComparison, constructor: Invalid card relevance option string!");

		if(patchStateOption_in.toLowerCase().equals(PATCH_STATE_OPTION_IDENTITY_STRING.toLowerCase()))
			patchStateOption = PatchStateOption.IDENTITY;
		else if(patchStateOption_in.toLowerCase().equals(PATCH_STATE_OPTION_UNFLIPPED_STRING.toLowerCase()))
			patchStateOption = PatchStateOption.UNFLIPPED;
		else if(patchStateOption_in.toLowerCase().equals(PATCH_STATE_OPTION_FLIPPED_NONE_STRING.toLowerCase()))
			patchStateOption = PatchStateOption.FLIPPED_NONE;
		else if(patchStateOption_in.toLowerCase().equals(PATCH_STATE_OPTION_PATCHED_BUT_ANY_STRING.toLowerCase()))
			patchStateOption = PatchStateOption.PATCHED_BUT_ANY;
		else if(patchStateOption_in.toLowerCase().equals(PATCH_STATE_OPTION_FLIPPED_DONT_CARE_STRING.toLowerCase()))
			patchStateOption = PatchStateOption.FLIPPED_DONT_CARE;
		else if(patchStateOption_in.toLowerCase().equals(PATCH_STATE_OPTION_FOR_ID_STRING.toLowerCase()))
			patchStateOption = PatchStateOption.FOR_ID;
		else if(patchStateOption_in.toLowerCase().equals(PATCH_STATE_OPTION_NOT_FOR_ID_STRING.toLowerCase()))
			patchStateOption = PatchStateOption.NOT_FOR_ID;
		else if(patchStateOption_in.toLowerCase().equals(PATCH_STATE_OPTION_DONT_CARE_STRING.toLowerCase()))
			patchStateOption = PatchStateOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_RipplepointComparison: Invalid patch state option string!");

		if(legalityOption_in.toLowerCase().equals(LEGALITY_OPTION_LEGAL_STRING.toLowerCase()))
			legalityOption = LegalityOption.LEGAL;
		else if(legalityOption_in.toLowerCase().equals(LEGALITY_OPTION_ILLEGAL_STRING.toLowerCase()))
			legalityOption = LegalityOption.ILLEGAL;
		else if(legalityOption_in.toLowerCase().equals(LEGALITY_OPTION_DONT_CARE_STRING.toLowerCase()))
			legalityOption = LegalityOption.DONT_CARE;
		else
			throw new AI_FatalException("AI_RipplepointComparison, constructor: Invalid legality option string!");
	}

	private ArrayList<String> initializeFromXmlNode(Node comparisonNode) throws AI_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = comparisonNode.getChildNodes();
		Node optionNode;
		NamedNodeMap attributes;
		Node attributeNode;

		String newCardOption = null;
		String newCardRelevanceOption = null;
		String newPatchStateOption = null;
		String newLegalityOption = null;
		String newCard = null;
		String newPatch = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			optionNode = list.item(i);
			if(optionNode.getNodeName().toLowerCase().equals("option"))
			{
				attributes = optionNode.getAttributes();
				attributeNode = attributes.getNamedItem(AI_Constants.TYPE_ATTRIBUTE_STRING);

				if(attributeNode.getTextContent().toLowerCase().equals("cardoption"))
				{
					newCardOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("cardrelevanceoption"))
				{
					newCardRelevanceOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("patchstateoption"))
				{
					newPatchStateOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("legalityoption"))
				{
					newLegalityOption = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("card"))
				{
					newCard = optionNode.getTextContent();
				}
				else if(attributeNode.getTextContent().toLowerCase().equals("patch"))
				{
					newPatch = optionNode.getTextContent();
				}
				else
				{
					throw new AI_FatalException("AI_RipplepointComparison.initializeFromXmlNode(): Invalid type attribute!  Value:" + attributeNode.getTextContent());
				}
			}
			else if(optionNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new AI_FatalException("AI_RipplepointComparison.initializeFromXmlNode(): Invalid node name!  Name:" + optionNode.getNodeName());
			}
		}

		returnValue.add(newCardOption);
		returnValue.add(newCardRelevanceOption);
		returnValue.add(newPatchStateOption);
		returnValue.add(newLegalityOption);
		returnValue.add(newCard);
		returnValue.add(newPatch);

		return returnValue;
	}





	public boolean compare(CN_GameState gameState, AI_GameStateTimeline timeline, AI_IdCard id, AI_MissionCard mission) throws AI_FatalException
	{
		ArrayList<AI_Ripplepoint> satisfyingRipplepoints = getCardOptionList();

		satisfyingRipplepoints = pruneForCardRelevance(satisfyingRipplepoints, id);
		satisfyingRipplepoints = pruneForPatchState(satisfyingRipplepoints, timeline, id);
		satisfyingRipplepoints = pruneForLegality(satisfyingRipplepoints, timeline);

		if(satisfyingRipplepoints.size() > 0)
			return true;
		else
			return false;
	}


	private ArrayList<AI_Ripplepoint> getCardOptionList() throws AI_FatalException
	{
		ArrayList<AI_Ripplepoint> returnValue = new ArrayList<AI_Ripplepoint>();

		switch(cardOption)
		{
			case IDENTITY:
				returnValue.add(card);
				break;
			case ANY:
				for(AI_Ripplepoint ripplepoint : AI_Globals.ripplepointList)
				{
					returnValue.add(ripplepoint);
				}
				break;
			default:
				throw new AI_FatalException("AI_RipplepointComparison.getCardOptionList(): Invalid card option!");
		}

		return returnValue;
	}


	private ArrayList<AI_Ripplepoint> pruneForCardRelevance(ArrayList<AI_Ripplepoint> currentRipplepoints, AI_IdCard id) throws AI_FatalException
	{
		ArrayList<AI_Ripplepoint> returnValue = new ArrayList<AI_Ripplepoint>();

		switch(cardRelevanceOption)
		{
			case FOR_ID:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					if(id.isRipplepointForId(ripplepoint))
						returnValue.add(ripplepoint);
				}
				break;
			case NOT_FOR_ID:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					if(!id.isRipplepointForId(ripplepoint))
						returnValue.add(ripplepoint);
				}
				break;
			case DONT_CARE:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					returnValue.add(ripplepoint);
				}
				break;
			default:
				throw new AI_FatalException("AI_RipplepointComparison.pruneForCardRelevance(): Invalid card relevance option!");
		}

		return returnValue;
	}

	private ArrayList<AI_Ripplepoint> pruneForPatchState(ArrayList<AI_Ripplepoint> currentRipplepoints, AI_GameStateTimeline timeline, AI_IdCard id) throws AI_FatalException
	{
		ArrayList<AI_Ripplepoint> returnValue = new ArrayList<AI_Ripplepoint>();

		AI_GameStateTimeline.TimelineCardData cardData;
		AI_Patch currentPatch;
		switch(patchStateOption)
		{
			case IDENTITY:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					if(AI_Constants.playCardDirectory.get(timeline.getByCard(ripplepoint).patchedBy).identity.equals(patch.identity))
					{
						returnValue.add(ripplepoint);
					}
				}
				break;
			case UNFLIPPED:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					cardData = timeline.getByCard(ripplepoint);
					if((!cardData.isFlipped)&&(cardData.patchedBy == AI_Constants.NULL_PATCH))
					{
						returnValue.add(ripplepoint);
					}
				}
				break;
			case FLIPPED_NONE:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					cardData = timeline.getByCard(ripplepoint);
					if((cardData.isFlipped)&&(cardData.patchedBy == AI_Constants.NULL_PATCH))
					{
						returnValue.add(ripplepoint);
					}
				}
				break;
			case PATCHED_BUT_ANY:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					cardData = timeline.getByCard(ripplepoint);
					if((cardData.isFlipped)&&(cardData.patchedBy != AI_Constants.NULL_PATCH))
					{
						returnValue.add(ripplepoint);
					}
				}
				break;
			case FOR_ID:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					cardData = timeline.getByCard(ripplepoint);
					if(cardData.patchedBy == AI_Constants.NULL_PATCH)
						currentPatch = null;
					else
						currentPatch = (AI_Patch)AI_Constants.playCardDirectory.get(cardData.patchedBy);

					if(id.isRipplepointStateForId(ripplepoint, currentPatch, cardData.isFlipped))
					{
						returnValue.add(ripplepoint);
					}
				}
				break;
			case NOT_FOR_ID:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					cardData = timeline.getByCard(ripplepoint);
					if(cardData.patchedBy == AI_Constants.NULL_PATCH)
						currentPatch = null;
					else
						currentPatch = (AI_Patch)AI_Constants.playCardDirectory.get(cardData.patchedBy);

					if(!id.isRipplepointStateForId(ripplepoint, currentPatch, cardData.isFlipped))
					{
						returnValue.add(ripplepoint);
					}
				}
				break;
			case FLIPPED_DONT_CARE:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					cardData = timeline.getByCard(ripplepoint);
					if(cardData.isFlipped)
					{
						returnValue.add(ripplepoint);
					}
				}
				break;
			case DONT_CARE:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					returnValue.add(ripplepoint);
				}
				break;
			default:
				throw new AI_FatalException("AI_RipplepointComparison.pruneForPatchState(): Invalid card relevance option!");
		}

		return returnValue;
	}



	private ArrayList<AI_Ripplepoint> pruneForLegality(ArrayList<AI_Ripplepoint> currentRipplepoints, AI_GameStateTimeline timeline) throws AI_FatalException
	{
		ArrayList<AI_Ripplepoint> returnValue = new ArrayList<AI_Ripplepoint>();

		switch(legalityOption)
		{
			case LEGAL:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					if(timeline.getByIdentity(ripplepoint.identity).isLegal)
						returnValue.add(ripplepoint);
				}
				break;
			case ILLEGAL:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					if(!timeline.getByIdentity(ripplepoint.identity).isLegal)
						returnValue.add(ripplepoint);
				}
				break;
			case DONT_CARE:
				for(AI_Ripplepoint ripplepoint : currentRipplepoints)
				{
					returnValue.add(ripplepoint);
				}
				break;
			default:
				throw new AI_FatalException("AI_RipplepointComparison.pruneForLegality(): Invalid legality option!");
		}

		return returnValue;
	}


}