package cnpc.engine;

import java.util.*;

class CN_CardDirectory<V extends CN_GameCard>
{
	private final Hashtable<Integer, V> directory;
	private boolean isLocked;

	public CN_CardDirectory()
	{
		directory = new Hashtable<Integer, V>();
		isLocked = false;
	}

	public void lock() throws CN_FatalException
	{
		if(!isLocked)
			isLocked = true;
		else
			throw new CN_FatalException("CN_CardDirectory.lock(): CardDirectory is already locked!");
	}

	public V getById(Integer cardId) throws CN_FatalException
	{
		if(isLocked)
			return directory.get(cardId);
		else
			throw new CN_FatalException("CN_CardDirectory.getById(): CardDirectory must be locked before it can be read!");
	}


	public V getAnInstance(String identity_in) throws CN_FatalException
	{
		if(isLocked)
			return getAnInstanceByString(identity_in);
		else
			throw new CN_FatalException("CN_CardDirectory.getAnInstance(): CardDirectory must be locked before it can be read!");

	}


	public Enumeration<V> getEnumeration() throws CN_FatalException
	{
		if(isLocked)
			return directory.elements();
		else
			throw new CN_FatalException("CN_CardDirectory.getEnumeration(): CardDirectory must be locked before it can be read!");
	}

	public void reset()
	{
		directory.clear();
		isLocked = false;
	}



	private V getAnInstanceByString(String identity_in)
	{
		Enumeration<V> enumeration = directory.elements();

		V currentCard;
		while(enumeration.hasMoreElements())
		{
			currentCard = enumeration.nextElement();
			if(currentCard.identity.equals(identity_in))
			{
				return currentCard;
			}
		}
		return null;
	}


	public void put(Integer index, V card) throws CN_FatalException
	{
		if(!isLocked)
			directory.put(index, card);
		else
			throw new CN_FatalException("CN_CardDirectory.put(): attempt to modify a locked CardDirectory!");
	}


	public Hashtable<Integer, String> toGameStateHashtable() throws CN_FatalException
	{
		if(isLocked)
		{
			Hashtable<Integer, String> returnValue = new Hashtable<Integer, String>();
			Set<Integer> keys;
			keys = directory.keySet();
			Iterator<Integer> iter = keys.iterator();

			Integer currentKey;
			while(iter.hasNext())
			{
				currentKey = iter.next();
				returnValue.put(currentKey, directory.get(currentKey).identity);
			}

			return returnValue;
		}
		else
		{
			throw new CN_FatalException("CN_CardDirectory.toGameStateHashtable(): function called before directory was locked!");
		}
	}

}
