package cnpc.engine;

//TODO: UI Messages to allow for button names in GET_ID_SELECTION and GET_MISSION_SELECTION

//NOTE: if reverse player order functionality is ever added, bascom_passHands will break

//TODO: make more object-oriented by eliminating static enums and classes, putting initialization stuff within class itself

import cnpc.*;

import java.util.*;

public class CN_Engine
{
	enum Location{HAND, ARTIFACTS, DECK, DISCARDS, TIMELINE, BACKED_UP_HAND};
	private class CardLocation
	{
		public CardLocation()
		{
			location = null;
			player = null;
		}

		public Location location;
		public CN_Player player;
	}



	CN_Player currentPlayer;
	CN_Player victoriousPlayer;
	boolean endOfUniverse;

	CN_Player[] players;
	CN_CardStack<CN_PlayCard> deck;
	CN_CardStack<CN_PlayCard> discards;
	CN_CardStack<CN_IdCard> idCards;
	CN_CardStack<CN_IdCard> idDiscards;
	CN_CardStack<CN_MissionCard> missionCards;
	CN_CardStack<CN_MissionCard> missionDiscards;
	CN_Timeline timeline;

	CN_Constants.InputState gameInputState;
	String gameInputHash;
	CN_PlayCard patchInQuestion;
	CN_PlayCard sourceOfMostRecentBascom;

	String inputStateMessage;
	String aiMessage;

	LinkedList<CN_BascomSpecifier> bascomQueue;

	final ArrayList<CN_Constants.InputConstraint> inputConstraints = new ArrayList<CN_Constants.InputConstraint>();



	Hashtable<String, CN_PlayCard> bascomPlayCardParams;
	Hashtable<String, CN_Player> bascomPlayerParams;
	Hashtable<String, String> bascomStringParams;

	int ifCounter;

	int victoryType;

	CN_GameState savedGameState;
	CN_GameState pendingGameState;

	boolean cancelable;



	public CN_GameState newGame(int numberOfPlayers, String variant) throws CN_FatalException
	{
		if(numberOfPlayers < 2)
		{
			throw new CN_FatalException("CN_Engine, newGame(): too few players!");
		}

		CN_Initializer initializer = new CN_Initializer();
		initializer.setVariant(variant);

		CNPC_ProgressFrame.instance().setProcessLabel("Loading Engine...");

		CN_Constants.timelineCardDirectory.reset();
		CNPC_ProgressFrame.instance().setProgress(0, "Loading Timeline...");
		timeline = initializer.loadTimeline();
		CN_Constants.timelineCardDirectory.lock();

		CN_Constants.playCardDirectory.reset();
		CNPC_ProgressFrame.instance().setProgress(25, "Loading Deck...");
		deck = initializer.loadDeck();
		CN_Constants.playCardDirectory.lock();
		deck.shuffle();
		discards = new CN_CardStack<CN_PlayCard>();

		CN_Constants.idCardDirectory.reset();
		CNPC_ProgressFrame.instance().setProgress(50, "Loading ID's...");
		idCards = initializer.loadIds();
		CN_Constants.idCardDirectory.lock();
		idCards.shuffle();
		idDiscards = new CN_CardStack<CN_IdCard>();

		CN_Constants.missionCardDirectory.reset();
		CNPC_ProgressFrame.instance().setProgress(75, "Loading Missions...");
		missionCards = initializer.loadMissions();
		CN_Constants.missionCardDirectory.lock();
		missionCards.shuffle();
		missionDiscards = new CN_CardStack<CN_MissionCard>();

		CNPC_ProgressFrame.instance().setProgress(100, "");

		players = new CN_Player[numberOfPlayers];
		for(int i = 0; i < players.length; i++)
		{
			players[i] = new CN_Player(i);
			players[i].setId(idCards.pop());
			players[i].setMission(missionCards.pop());
		}

		for(int i = 0; i < players.length; i++)
		{
			for(int j = 0; j < CN_Constants.STARTING_NUMBER_OF_CARDS; j++)
			{
				players[i].addHandCard(deck.pop());
			}

			if(i == 0)
			{
				players[i].nextPlayer = players[i + 1];
				players[i].prevPlayer = players[players.length - 1];
			}
			else if(i == players.length - 1)
			{
				players[i].nextPlayer = players[0];
				players[i].prevPlayer = players[i - 1];
			}
			else
			{
				players[i].nextPlayer = players[i + 1];
				players[i].prevPlayer = players[i - 1];
			}
		}

		currentPlayer = players[0];
		victoriousPlayer = null;
		endOfUniverse = false;

		gameInputState = CN_Constants.InputState.BETWEEN_TURNS;
		gameInputHash = "";
		patchInQuestion = null;
		sourceOfMostRecentBascom = null;

		inputStateMessage = "";
		aiMessage = null;

		bascomQueue = new LinkedList<CN_BascomSpecifier>();

		bascomPlayCardParams = new Hashtable<String, CN_PlayCard>();
		bascomPlayerParams = new Hashtable<String, CN_Player>();
		bascomStringParams = new Hashtable<String, String>();

		ifCounter = 0;

		victoryType = CN_Constants.VICTORY_TYPE_NULL;

		savedGameState = null;
		pendingGameState = null;

		cancelable = false;

//fixPlayer2Hand();
//fixPlayer2Hand2();
//fixPlayer2Hand3();

		return generateFirstGameState();
	}








void fixPlayer2Hand3() throws CN_FatalException
{
	int playerNumber = 1;

	String desiredMission = "cretaceouspark";
	String handCard1 = "livestegosaurus";
	String handCard2 = "livetyrannosaurus";
	String handCard3 = "livebrontosaurus";
	String handCard4 = "tomorrowsnewstoday";
	String handCard5 = "rongorongotablets";

	CN_Player player = players[playerNumber];

	ArrayList<CN_PlayCard> hand = player.clearHand();
	for(CN_PlayCard card : hand)
	{
		deck.push(card);
	}

	player.setMission(getDesiredMission(desiredMission));
	player.addHandCard(findHandCard(handCard1, playerNumber));
	player.addHandCard(findHandCard(handCard2, playerNumber));
	player.addHandCard(findHandCard(handCard3, playerNumber));
	player.addHandCard(findHandCard(handCard4, playerNumber));
	player.addHandCard(findHandCard(handCard5, playerNumber));
}


void fixPlayer2Hand2() throws CN_FatalException
{
	int playerNumber = 0;

//	String desiredId = "gunther";
	String handCard1 = "performamiraclebibl";
	String handCard2 = "crownofthorns";
	String handCard3 = "fastforward";
//	String handCard4 = "rewind";
//	String handCard5 = "fastforward";
//	String handCard6 = "fastforward";

	CN_Player player = players[playerNumber];

	ArrayList<CN_PlayCard> hand = player.clearHand();
	for(CN_PlayCard card : hand)
	{
		deck.push(card);
	}

//	player.setId(getDesiredId(desiredId));
	player.addHandCard(findHandCard(handCard1, playerNumber));
	player.addHandCard(findHandCard(handCard2, playerNumber));
	player.addHandCard(findHandCard(handCard3, playerNumber));
//	player.addHandCard(findHandCard(handCard4, playerNumber));
//	player.addHandCard(findHandCard(handCard5, playerNumber));
//	player.addHandCard(findHandCard(handCard6, playerNumber));
}

void fixPlayer2Hand() throws CN_FatalException
{
	int playerNumber = 1;

	String desiredId = "gunther";
	String handCard1 = "reversefate";
	String handCard2 = "reversefate";
	String handCard3 = "patchA6";
	String handCard4 = "patchA7";
	String handCard5 = "quicktrip";
	String handCard6 = "rewind";

	CN_Player player = players[playerNumber];

	ArrayList<CN_PlayCard> hand = player.clearHand();
	for(CN_PlayCard card : hand)
	{
		deck.push(card);
	}

	player.setId(getDesiredId(desiredId));
	player.addHandCard(findHandCard(handCard1, playerNumber));
	player.addHandCard(findHandCard(handCard2, playerNumber));
	player.addHandCard(findHandCard(handCard3, playerNumber));
	player.addHandCard(findHandCard(handCard4, playerNumber));
	player.addHandCard(findHandCard(handCard5, playerNumber));
	player.addHandCard(findHandCard(handCard6, playerNumber));
}

CN_IdCard getDesiredId(String identity) throws CN_FatalException
{
	CN_IdCard returnValue;
	for(CN_Player player : players)
	{
		if(player.getId().identity.equals(identity))
		{
			returnValue = player.setId(idCards.pop());
			return returnValue;
		}
	}

	CN_IdCard card;
	for(int i = 0; i < idCards.size(); i++)
	{
		card = idCards.get(i);
		if(card.identity.equals(identity))
		{
			idCards.remove(i);
			return card;
		}
	}

	throw new CN_FatalException("TEST EXCEPTION: getDesiredId()");
}

CN_MissionCard getDesiredMission(String identity) throws CN_FatalException
{
	CN_MissionCard returnValue;
	for(CN_Player player : players)
	{
		if(player.getMission().identity.equals(identity))
		{
			returnValue = player.setMission(missionCards.pop());
			return returnValue;
		}
	}

	CN_MissionCard card;
	for(int i = 0; i < missionCards.size(); i++)
	{
		card = missionCards.get(i);
		if(card.identity.equals(identity))
		{
			missionCards.remove(i);
			return card;
		}
	}

	throw new CN_FatalException("TEST EXCEPTION: getDesiredId()");
}


CN_PlayCard findHandCard(String identity, int playerException) throws CN_FatalException
{
	ArrayList<CN_PlayCard> handCards;

	CN_Player player;
	for(int i = 0; i < players.length; i++)
	{
		if(i == playerException) continue;
		player = players[i];
		handCards = player.getListOfHandCards();
		for(CN_PlayCard card : handCards)
		{
			if(card.identity.equals(identity))
			{
				player.removeHandCard(card);
				return card;
			}
		}
	}

	CN_PlayCard card;
	for(int i = 0; i < deck.size(); i++)
	{
		card = deck.get(i);
		if(card.identity.equals(identity))
		{
			deck.remove(i);
			return card;
		}
	}
	throw new CN_FatalException("TEST EXCEPTION: findInDeck()");
}




	private void processPlayCardClick(CN_PlayCard referencedCard) throws CN_FatalException
	{
		bascomPlayCardParams.remove(CN_Constants.SPECIALPARAM_PLAYEDCARDHASH);
		bascomPlayCardParams.put(CN_Constants.SPECIALPARAM_PLAYEDCARDHASH, referencedCard);

		CN_BascomSpecifier currentSpecifier;
		CN_PlayCard.StoredBascom currentBascom;
		for(int i = referencedCard.bascomSize() - 1; i >= 0; i--)
		{
			currentBascom = referencedCard.getBascom(i);
			currentSpecifier = new CN_BascomSpecifier(currentBascom.bascomId, referencedCard);
			if(currentBascom.bascomId == CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL)
			{
				for(int j = 0; j < currentBascom.specialStringsSize(); j++)
				{
					currentSpecifier.stringParams.add(currentBascom.getSpecialString(j));
				}
			}
			else if(currentBascom.bascomId == CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_HASH)
			{
				for(int j = 0; j < currentBascom.specialStringsSize(); j++)
				{
					currentSpecifier.stringParams.add(currentBascom.getSpecialString(j));
				}
			}
			else if(currentBascom.bascomId == CN_Constants.BASCOM_SPECIAL_REMOVE_PARAMETER)
			{
				for(int j = 0; j < currentBascom.specialStringsSize(); j++)
				{
					currentSpecifier.stringParams.add(currentBascom.getSpecialString(j));
				}
			}
			bascomQueue.addFirst(currentSpecifier);
		}

		commenceBascomExecution();
	}


	private void commenceBascomExecution() throws CN_FatalException
	{
		if(pendingGameState != null)
		{
			savedGameState = pendingGameState;
			pendingGameState = null;
		}

		gameInputState = CN_Constants.InputState.PROCESSING;
		while(!bascomQueue.isEmpty())
		{
			if(gameInputState == CN_Constants.InputState.PROCESSING)
			{
				executeBasicCommand(bascomQueue.removeFirst());
			}
			else
			{
				break;
			}
		}
	}



	private void executeBasicCommand(CN_BascomSpecifier command) throws CN_FatalException
	{
		sourceOfMostRecentBascom = command.source;
//debugPrintBasicCommandString(command);
		switch(command.commandIndex)
		{
			case CN_Constants.BASCOM_IDENT_CONDITIONAL:
				bascom_identConditional(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_ELSE:
				bascom_else(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_END_IF:
				bascom_endIf(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_TRANSFER_ARTIFACT:
				bascom_transferArtifact(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_PLAY_ARTIFACT:
				bascom_playArtifact(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_DISCARD_ARTIFACT:
				bascom_discardArtifact(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_DISCARD_CARD:
				bascom_discardCard(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_DRAW:
				bascom_draw(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_TRANSFER_DECK_TO_DISCARDS:
				bascom_transferDeckToDiscards(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_TRANSFER_DECK_TO_ARTIFACTS:
				bascom_transferDeckToArtifacts(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_TRANSFER_DECK_TO_HAND:
				bascom_transferDeckToHand(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_MOVE_DISCARD_TO_TOP:
				bascom_moveDiscardToTop(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_TRANSFER_DISCARDS_TO_ARTIFACTS:
				bascom_transferDiscardsToArtifacts(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_TRANSFER_DISCARDS_TO_HAND:
				bascom_transferDiscardsToHand(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_EXECUTE_CARD:
				bascom_executeCard(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_BACK_UP_HAND:
				bascom_backUpHand(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_RESTORE_BACKED_UP_HAND:
				bascom_restoreBackedUpHand(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_FLIP_LINCHPIN:
				bascom_flipLinchpin(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_TRANSFER_HAND_TO_RIPPLEPOINT:
				bascom_transferHandToRipplepoint(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_DO_VORTEX:
				bascom_doVortex(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_PASS_HANDS:
				bascom_passHands(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_NEW_ID:
				bascom_newId(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_SWITCH_IDS:
				bascom_switchIds(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_NEW_MISSION:
				bascom_newMission(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_SWITCH_MISSIONS:
				bascom_switchMissions(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_SAVE_GAME_STATE:
				bascom_saveGameState(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_LOAD_GAME_STATE:
				bascom_loadGameState(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_CLEAR_GAME_STATE:
				bascom_clearGameState(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_RESHUFFLE_DECK:
				bascom_reshuffleDeck(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_RESHUFFLE_IDS:
				bascom_reshuffleIds(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_RESHUFFLE_MISSIONS:
				bascom_reshuffleMissions(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_GET_ARTIFACT_SELECTION:
				bascom_getArtifactSelection(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_GET_HAND_SELECTION:
				bascom_getHandSelection(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_GET_LINCHPIN_SELECTION:
				bascom_getLinchpinSelection(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_GET_RIPPLEPOINT_SELECTION:
				bascom_getRipplepointSelection(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_GET_ID_SELECTION:
				bascom_getIdSelection(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_GET_MISSION_SELECTION:
				bascom_getMissionSelection(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_GET_CARD_PLAY:
				bascom_getCardPlay(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_GET_DIRECTION_SELECTION:
				bascom_getDirectionSelection(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_GET_DECK_SELECTION:
				bascom_getDeckSelection(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_GET_DISCARDS_SELECTION:
				bascom_getDiscardsSelection(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_END_TURN:
				bascom_endTurn(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_SKIP_QUEUE_ITEMS:
				bascom_skipQueueItems(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_DRAW_REAL:
				bascom_drawReal(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL:
				bascomSpecial_loadParameterLiteral(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_HASH:
				bascomSpecial_loadParameterHash(command.playerParams, command.playCardParams, command.stringParams);
				break;
			case CN_Constants.BASCOM_SPECIAL_REMOVE_PARAMETER:
				bascomSpecial_removeParameter(command.playerParams, command.playCardParams, command.stringParams);
				break;
			default:
				throw new CN_FatalException("executeBasicCommand(): commandIndex not recognized!");
		}

	}




	public CN_GameState processInput(CN_InputMessage message) throws CN_FatalException, CN_InputMessageException
	{
		boolean isPlayCardClick = false;
		boolean isTimelineCardClick = false;
		boolean isIdSelection = false;
		boolean isMissionSelection = false;
		boolean isDiscardClick = false;
		boolean isDiscardTwoClick = false;
		boolean isDirectionSelection = false;
		boolean isCancel = false;
		boolean isUndoTurn = false;
		boolean isFinishTurn = false;
		boolean isBeginTurn = false;

		CN_PlayCard referencedPlayCard = null;
		CardLocation referencedCardLocation = null;

		CN_TimelineCard referencedTimelineCard = null;
		CN_IdCard referencedIdCard = null;
		CN_MissionCard referencedMissionCard = null;
		boolean referencedDirectionIsLeft = false;

		if(message.messageType == CN_InputMessage.MessageType.PLAY_CARD_CLICK)
		{
			isPlayCardClick = true;
			referencedPlayCard = CN_Constants.playCardDirectory.getById(message.cardNumber);
			referencedCardLocation = locatePlayCard(referencedPlayCard);
		}
		else if(message.messageType == CN_InputMessage.MessageType.TIMELINE_CARD_CLICK)
		{
			isTimelineCardClick = true;
			referencedTimelineCard = CN_Constants.timelineCardDirectory.getById(message.cardNumber);
		}
		else if(message.messageType == CN_InputMessage.MessageType.ID_CARD_SELECTION)
		{
			isIdSelection = true;
			referencedIdCard = CN_Constants.idCardDirectory.getById(message.cardNumber);
		}
		else if(message.messageType == CN_InputMessage.MessageType.MISSION_CARD_SELECTION)
		{
			isMissionSelection = true;
			referencedMissionCard = CN_Constants.missionCardDirectory.getById(message.cardNumber);
		}
		else if(message.messageType == CN_InputMessage.MessageType.DISCARD_CLICK) isDiscardClick = true;
		else if(message.messageType == CN_InputMessage.MessageType.DISCARD_TWO_CLICK) isDiscardTwoClick = true;
		else if(message.messageType == CN_InputMessage.MessageType.DIRECTION_SELECTION)
		{
			isDirectionSelection = true;
			switch(message.direction)
			{
				case LEFT:
					referencedDirectionIsLeft = true;
					break;
				case RIGHT:
					referencedDirectionIsLeft = false;
					break;
				default:
					throw new CN_FatalException("CN_Engine, processInput(): invalid direction!");
			}

		}
		else if(message.messageType == CN_InputMessage.MessageType.CANCEL) isCancel = true;
		else if(message.messageType == CN_InputMessage.MessageType.UNDO_TURN) isUndoTurn = true;
		else if(message.messageType == CN_InputMessage.MessageType.FINISH_TURN) isFinishTurn = true;
		else if(message.messageType == CN_InputMessage.MessageType.BEGIN_TURN) isBeginTurn = true;
		else
			throw new CN_InputMessageException("processInput: MessageType not set!");



		int numberOfLocations = -1;
		if(referencedPlayCard != null)
		{
			numberOfLocations = findNumberOfLocations(referencedPlayCard);
			if(numberOfLocations < 1)
			{
				throw new CN_FatalException("CN_Engine, processInput(): referenced play card \"" + referencedPlayCard.identity + "\" was not found anywhere!");
			}
			if(numberOfLocations > 1)
			{
				throw new CN_FatalException("CN_Engine, processInput(): referenced play card \"" + referencedPlayCard.identity + "\" was found more than once!  Number of locations: " + String.valueOf(numberOfLocations));
			}
		}




		testAgainstInputConstraints(referencedPlayCard, referencedTimelineCard, referencedIdCard, referencedMissionCard, referencedDirectionIsLeft, referencedCardLocation);

		aiMessage = message.aiMessage;


		if(gameInputState == CN_Constants.InputState.MAIN)
		{

			if(isPlayCardClick)
			{
				if(referencedCardLocation.location != Location.HAND)
				{
					throw new CN_InputMessageException("Natural constraint of CARD_CLICK violated in MAIN - Card not in hand.");
				}
				else if(referencedCardLocation.player != currentPlayer)
				{
					throw new CN_InputMessageException("Natural constraint of CARD_CLICK violated in MAIN - Card not owned by current player.");
				}
				else if(testUnplayable(referencedPlayCard))
				{
					throw new CN_InputMessageException("Automatic artificial constraint of CARD_CLICK violated in MAIN - Card is unplayable.");
				}
				processPlayCardClick(referencedPlayCard);
			}
			else if(isDiscardClick)
			{
				setUpDiscardClick();
				commenceBascomExecution();
			}
			else if(isDiscardTwoClick)
			{
				setUpDiscardTwoClick();
				commenceBascomExecution();
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in MAIN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by MAIN.");
			}
		}


		else if(gameInputState == CN_Constants.InputState.GET_ARTIFACT_SELECTION)
		{
			if(isPlayCardClick)
			{
				if(referencedCardLocation.location != Location.ARTIFACTS)
				{
					throw new CN_InputMessageException("Natural constraint of CARD_CLICK violated in GET_ARTIFACT_SELECTION - selected card is not an artifact.");
				}

				bascomPlayCardParams.put(gameInputHash, referencedPlayCard);
				commenceBascomExecution();
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in MAIN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by GET_ARTIFACT_SELECTION.");
			}
		}


		else if(gameInputState == CN_Constants.InputState.GET_HAND_SELECTION)
		{
			if(isPlayCardClick)
			{
				if(referencedCardLocation.location != Location.HAND)
				{
					throw new CN_InputMessageException("Natural constraint of CARD_CLICK violated in GET_HAND_SELECTION - Card not in hand.");
				}
				else if(referencedCardLocation.player != currentPlayer)
				{
					throw new CN_InputMessageException("Natural constraint of CARD_CLICK violated in GET_HAND_SELECTION - Card not owned by current player.");
				}
				bascomPlayCardParams.put(gameInputHash, referencedPlayCard);
				commenceBascomExecution();
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in MAIN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by GET_HAND_SELECTION.");
			}
		}


		else if(gameInputState == CN_Constants.InputState.GET_CARD_PLAY)
		{
			if(isPlayCardClick)
			{
				if(referencedCardLocation.location != Location.HAND)
				{
					throw new CN_InputMessageException("Natural constraint of CARD_CLICK violated in GET_CARD_PLAY - Card not in hand.");
				}
				else if(referencedCardLocation.player != currentPlayer)
				{
					throw new CN_InputMessageException("Natural constraint of CARD_CLICK violated in GET_CARD_PLAY - Card not owned by current player.");
				}
				else if(testUnplayable(referencedPlayCard))
				{
					throw new CN_InputMessageException("Automatic artificial constraint of CARD_CLICK violated in GET_CARD_PLAY - Card is unplayable.");
				}
				processPlayCardClick(referencedPlayCard);
			}
			else if(isDiscardClick)
			{
				setUpDiscardClick();
				commenceBascomExecution();
			}
			else if(isDiscardTwoClick)
			{
				setUpDiscardTwoClick();
				commenceBascomExecution();
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in MAIN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by GET_CARD_PLAY.");
			}
		}


		else if(gameInputState == CN_Constants.InputState.GET_LINCHPIN_SELECTION)
		{
			if(isTimelineCardClick)
			{
				if(!(referencedTimelineCard instanceof CN_Linchpin))
				{
					throw new CN_InputMessageException("Natural constraint of CARD_CLICK violated in GET_LINCHPIN_SELECTION - Card is not a linchpin.");
				}

				if(timeline.isLinchpinLegal(referencedTimelineCard.identity))
				{
					timeline.banAllLinchpins();
					bascomStringParams.put(gameInputHash, referencedTimelineCard.identity);
					commenceBascomExecution();
				}
				else
				{
					throw new CN_InputMessageException("Linchpin " + referencedTimelineCard.identity + " cannot legally be flipped in GET_LINCHPIN_SELECTION.");
				}
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in MAIN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by GET_LINCHPIN_SELECTION.");
			}
		}


		else if(gameInputState == CN_Constants.InputState.GET_RIPPLEPOINT_SELECTION)
		{
			if(isTimelineCardClick)
			{
				if(!(referencedTimelineCard instanceof CN_Ripplepoint))
				{
					throw new CN_InputMessageException("Natural constraint of CARD_CLICK violated in GET_RIPPLEPOINT_SELECTION - Card is not a ripplepoint.");
				}

				if(timeline.isPatchLegal(patchInQuestion, referencedTimelineCard.identity))
				{
					patchInQuestion = null;
					bascomStringParams.put(gameInputHash, referencedTimelineCard.identity);
					commenceBascomExecution();
				}
				else
				{
					throw new CN_InputMessageException("Patch " + patchInQuestion.identity + " cannot legally be applied to " + referencedTimelineCard.identity + " in GET_RIPPLEPOINT_SELECTION.");
				}
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in MAIN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by GET_RIPPLEPOINT_SELECTION.");
			}
		}


		else if(gameInputState == CN_Constants.InputState.GET_ID_SELECTION)
		{
			if(isIdSelection)
			{
				bascomPlayerParams.put(gameInputHash, locateIdCard(referencedIdCard));
				commenceBascomExecution();
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in MAIN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by GET_ID_SELECTION.");
			}
		}


		else if(gameInputState == CN_Constants.InputState.GET_MISSION_SELECTION)
		{
			if(isMissionSelection)
			{
				bascomPlayerParams.put(gameInputHash, locateMissionCard(referencedMissionCard));
				commenceBascomExecution();
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in MAIN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by GET_MISSION_SELECTION.");
			}
		}


		else if(gameInputState == CN_Constants.InputState.GET_DIRECTION_SELECTION)
		{
			if(isDirectionSelection)
			{
				if(referencedDirectionIsLeft)
				{
					bascomStringParams.put(gameInputHash, CN_Constants.LEFT_STRING);
				}
				else
				{
					bascomStringParams.put(gameInputHash, CN_Constants.RIGHT_STRING);
				}
				commenceBascomExecution();
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in MAIN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by GET_DIRECTION_SELECTION.");
			}
		}


		else if(gameInputState == CN_Constants.InputState.GET_DECK_SELECTION)
		{
			if(isPlayCardClick)
			{
				if(referencedCardLocation.location != Location.DECK)
				{
					throw new CN_InputMessageException("Natural constraint of CARD_CLICK violated in GET_DECK_SELECTION - Card not in deck.");
				}
				bascomPlayCardParams.put(gameInputHash, referencedPlayCard);
				commenceBascomExecution();
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in MAIN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by GET_DECK_SELECTION.");
			}
		}



		else if(gameInputState == CN_Constants.InputState.GET_DISCARDS_SELECTION)
		{
			if(isPlayCardClick)
			{
				if(referencedCardLocation.location != Location.DISCARDS)
				{
					throw new CN_InputMessageException("Natural constraint of CARD_CLICK violated in GET_DISCARDS_SELECTION - Card not in discards.");
				}
				bascomPlayCardParams.put(gameInputHash, referencedPlayCard);
				commenceBascomExecution();
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in MAIN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by GET_DISCARDS_SELECTION.");
			}
		}





		else if(gameInputState == CN_Constants.InputState.END_TURN)
		{
			if(isUndoTurn)
			{
				//ON UNDO TURN GOES HERE
			}
			else if(isFinishTurn)
			{
				savedGameState = null;
				cancelable = false;
				victoriousPlayer = getVictoriousPlayer();
				endOfUniverse = getEndOfUniverse();

				bascomQueue.clear();
				bascomPlayCardParams.clear();
				bascomPlayerParams.clear();
				bascomStringParams.clear();
				inputConstraints.clear();

				currentPlayer = currentPlayer.nextPlayer;

				gameInputState = CN_Constants.InputState.BETWEEN_TURNS;
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in END_TURN - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by END_TURN.");
			}
		}


		else if(gameInputState == CN_Constants.InputState.BETWEEN_TURNS)
		{
			if(isBeginTurn)
			{
				CN_BascomSpecifier endTurnSpecifier = new CN_BascomSpecifier(CN_Constants.BASCOM_END_TURN, null);
				endTurnSpecifier.stringParams.add("Your turn is over.  Press \"End Turn\" to continue.");
				bascomQueue.addFirst(endTurnSpecifier);
				bascomPlayerParams.put(CN_Constants.SPECIALPARAM_CURRENTPLAYERHASH, currentPlayer);
				gameInputState = CN_Constants.InputState.MAIN;
				if(deck.empty())
				{
					if(discards.empty())
					{
						throw new CN_FatalException("CN_Engine, processInput(): both deck and discards are empty!");
					}
					mergeDiscardsIntoDeck();
				}
				currentPlayer.addHandCard(deck.pop());
				saveGameState(false);
			}
			else if(isCancel)
			{
				if(savedGameState == null)
				{
					throw new CN_InputMessageException("Natural constraint of CANCEL violated in BETWEEN_TURNS - There is no currently saved game state.");
				}
				loadGameState(savedGameState);
				if(gameInputState == CN_Constants.InputState.PROCESSING)
				{
					commenceBascomExecution();
				}
			}
			else
			{
				throw new CN_InputMessageException("Unrecognized by BETWEEN_TURNS.");
			}
		}

		else
		{
			throw new CN_FatalException("CN_Engine, processInput(): Game is in an unrecgonized Input State!");
		}


		if(gameInputState == CN_Constants.InputState.PROCESSING)
		{
			throw new CN_FatalException("CN_Engine, processInput(): Illegal attempt to return Processing state to the UI!");
		}

		return generateGameState();
	}





	private void mergeDiscardsIntoDeck()
	{
		while(!discards.empty())
		{
			deck.push(discards.pop());
		}
		deck.shuffle();
	}

	private void mergeIdDiscardsIntoIdCards()
	{
		while(!idDiscards.empty())
		{
			idCards.push(idDiscards.pop());
		}
		idCards.shuffle();
	}

	private void mergeMissionDiscardsIntoMissionCards()
	{
		while(!missionDiscards.empty())
		{
			missionCards.push(missionDiscards.pop());
		}
		missionCards.shuffle();
	}














	/**
	 * Transfers an artifact
	 *
	 * @param playerParams			Index 1 is the destination player
	 * @param playCardParams		Index 0 is the target card, which must be an artifact in a player's artifact list
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_transferArtifact(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_Player destination;
		CN_PlayCard artifact;

		CardLocation location;

		try
		{
			destination = playerParams.get(0);
			artifact = playCardParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_transferArtifact: parameter arrays not properly filled!");
		}

		if(artifact.cardType != CN_PlayCard.PlayCardType.ARTIFACT)
		{
			throw new CN_FatalException("bascom_transferArtifact: card is not an artifact!");
		}

		location = locatePlayCard(artifact);

		if(location.location != Location.ARTIFACTS)
		{
			throw new CN_FatalException("bascom_transferArtifact: card is not a member of a player's artifacts!");
		}

		if(location.player.removeArtifact(artifact))
		{
			destination.addArtifact(artifact);
		}
		else
		{
			throw new CN_FatalException("bascom_transferArtifact: artifact not present in source's artifacts!");
		}
	}


	/**
	 * Plays an artifact from the specified player's hand
	 *
	 * @param playerParams			Index 0 is the owner of the artifact
	 * @param playCardParams		Index 0 is the target card, which must be an artifact and in the owner's hand
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_playArtifact(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_PlayCard artifact;

		CardLocation location;

		try
		{
			artifact = playCardParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_playArtifact: parameter arrays not properly filled!");
		}

		if(artifact.cardType != CN_PlayCard.PlayCardType.ARTIFACT)
		{
			throw new CN_FatalException("bascom_playArtifact: card is not an artifact!");
		}

		location = locatePlayCard(artifact);

		if(location.location != Location.HAND)
		{
			throw new CN_FatalException("bascom_playArtifact: card is not a member of a player's hand!");
		}


		if(location.player.removeHandCard(artifact))
		{
			location.player.addArtifact(artifact);
		}
		else
		{
			throw new CN_FatalException("bascom_playArtifact: artifact not present in hand!");
		}
	}

	/**
	 * Discards an artifact from the specified player's artifacts
	 *
	 * @param playerParams			Index 0 is the owner of the artifact
	 * @param playCardParams		Index 0 is the target card, which must be in the owner's artifacts
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_discardArtifact(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_PlayCard artifact;

		CardLocation location;

		try
		{
			artifact = playCardParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_discardArtifact: parameter arrays not properly filled!");
		}

		if(artifact.cardType != CN_PlayCard.PlayCardType.ARTIFACT)
		{
			throw new CN_FatalException("bascom_discardArtifact: card is not an artifact!");
		}

		location = locatePlayCard(artifact);

		if(location.location != Location.ARTIFACTS)
		{
			throw new CN_FatalException("bascom_discardArtifact: card is not a member of a player's artifacts!");
		}

		if(location.player.removeArtifact(artifact))
		{
			discards.push(artifact);
		}
		else
		{
			throw new CN_FatalException("bascom_discardArtifact: artifact not present in source's artifacts!");
		}
	}

	/**
	 * Discards the specified card from the specified player's hand
	 *
	 * @param playerParams			Index 0 is the owner of the card
	 * @param playCardParams		Index 0 is the target card, which must be in the owner's hand
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_discardCard(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_PlayCard card;

		CardLocation location;

		try
		{
			card = playCardParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_discardCard: playcard parameter array not properly filled!");
		}

		location = locatePlayCard(card);

		if(location.location != Location.HAND)
		{
			throw new CN_FatalException("bascom_discardCard: card is not a member of a player's hand!");
		}

		if(location.player.removeHandCard(card))
		{
			discards.push(card);
		}
		else
		{
			throw new CN_FatalException("bascom_discardCard: card not present in source's hand!");
		}
	}

	/**
	 * Draws a card for the specified player
	 *
	 * @param playerParams			Index 0 is the player who will receive the card
	 * @param playCardParams		unused
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_draw(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_Player player;

		try
		{
			player = playerParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_draw: parameter arrays not properly filled!");
		}

		if(deck.empty())
		{
			if(discards.empty())
			{
				throw new CN_FatalException("bascom_draw: both deck and discards are empty!");
			}
			mergeDiscardsIntoDeck();
		}

		player.addHandCard(deck.pop());
	}


	/**
	 * Draws a card for the specified player and ensures that it is in the player's real hand
	 *
	 * @param playerParams			Index 0 is the player who will receive the card
	 * @param playCardParams		unused
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_drawReal(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_Player player;

		try
		{
			player = playerParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_draw: parameter arrays not properly filled!");
		}

		if(deck.empty())
		{
			if(discards.empty())
			{
				throw new CN_FatalException("bascom_draw: both deck and discards are empty!");
			}
			mergeDiscardsIntoDeck();
		}

		player.addRealHandCard(deck.pop());
	}




	/**
	 * Moves a card from the deck to the discard pile
	 *
	 * @param playerParams			unused
	 * @param playCardParams		Index 0 is the target card, which must be in the deck
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_transferDeckToDiscards(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_PlayCard card;

		try
		{
			card = playCardParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_transferDeckToDiscards: parameter arrays not properly filled!");
		}


		if(deck.remove(card))
		{
			discards.push(card);
		}
		else
		{
			throw new CN_FatalException("bascom_transferDeckToDiscards: card not present in deck!");
		}
	}

	/**
	 * Plays an artifact for the specified player from the deck
	 *
	 * @param playerParams			Index 0 is the player who will receive the artifact
	 * @param playCardParams		Index 0 is the target card, which must be an artifact and in the deck
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_transferDeckToArtifacts(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_PlayCard artifact;
		CN_Player destination;

		try
		{
			artifact = playCardParams.get(0);
			destination = playerParams.get(0);

		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_transferDeckToArtifacts: parameter arrays not properly filled!");
		}


		if(artifact.cardType != CN_PlayCard.PlayCardType.ARTIFACT)
		{
			throw new CN_FatalException("bascom_transferDeckToArtifacts: card is not an artifact!");
		}

		if(deck.remove(artifact))
		{
			destination.addArtifact(artifact);
		}
		else
		{
			throw new CN_FatalException("bascom_transferDeckToArtifacts: artifact not present in deck!");
		}
	}

	/**
	 * Transfers the specified card from the deck to the specified player's hand
	 *
	 * @param playerParams			Index 0 is the player who will receive the card
	 * @param playCardParams		Index 0 is the target card, which must be in the deck
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_transferDeckToHand(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_PlayCard card;
		CN_Player destination;

		try
		{
			card = playCardParams.get(0);
			destination = playerParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_transferDeckToHand: parameter arrays not properly filled!");
		}


		if(deck.remove(card))
		{
			destination.addHandCard(card);
		}
		else
		{
			throw new CN_FatalException("bascom_transferDeckToHand: card not present in deck!");
		}
	}

	/**
	 * Moves a card from within the discard pile to the top of the pile
	 *
	 * @param playerParams			unused
	 * @param playCardParams		Index 0 is the target card, which must be in the discard pile
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_moveDiscardToTop(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_PlayCard card;

		try
		{
			card = playCardParams.get(0);

		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_moveDiscardToTop: parameter arrays not properly filled!");
		}


		if(discards.remove(card))
		{
			discards.push(card);
		}
		else
		{
			throw new CN_FatalException("bascom_moveDiscardToTop: card not present in discards!");
		}
	}

	/**
	 * Plays an artifact for the specified player from the discard pile
	 *
	 * @param playerParams			Index 0 is the player who will receive the artifact
	 * @param playCardParams		Index 0 is the target card, which must be an artifact and in the discard pile
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_transferDiscardsToArtifacts(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_PlayCard artifact;
		CN_Player destination;

		try
		{
			artifact = playCardParams.get(0);
			destination = playerParams.get(0);

		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_transferDiscardsToArtifacts: parameter arrays not properly filled!");
		}

		if(artifact.cardType != CN_PlayCard.PlayCardType.ARTIFACT)
		{
			throw new CN_FatalException("bascom_transferDiscardsToArtifacts: card is not an artifact!");
		}

		if(discards.remove(artifact))
		{
			destination.addArtifact(artifact);
		}
		else
		{
			throw new CN_FatalException("bascom_transferDiscardsToArtifacts: artifact not present in deck!");
		}
	}

	/**
	 * Transfers the specified card from the discard pile to the specified player's hand
	 *
	 * @param playerParams			Index 0 is the player who will receive the card
	 * @param playCardParams		Index 0 is the target card, which must be in the discard pile
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_transferDiscardsToHand(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_PlayCard card;
		CN_Player destination;

		try
		{
			card = playCardParams.get(0);
			destination = playerParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_transferDiscardsToHand: parameter arrays not properly filled!");
		}


		if(discards.remove(card))
		{
			destination.addHandCard(card);
		}
		else
		{
			throw new CN_FatalException("bascom_transferDiscardsToHand: card not present in discard pile!");
		}
	}



	/**
	 * Executes the commands of the specified card as though it had been played normally.
	 *
	 * @param playerParams			unused
	 * @param playCardParams		Index 0 is the target card
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_executeCard(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_PlayCard card;

		try
		{
			card = playCardParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_executeCard: parameter arrays not properly filled!");
		}

		processPlayCardClick(card);
	}




	/**
	 * Backs up a player's hand, allowing that player to temporarily use a different
	 * hand (see the Chrononauts card "Perform a Miracle")
	 *
	 * @param playerParams			Index 0 is the specified player
	 * @param playCardParams		unused
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_backUpHand(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_Player owner;

		try
		{
			owner = playerParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_backUpHand: parameter arrays not properly filled!");
		}

		owner.backUpHand();
	}

	/**
	 * Restores a backed-up hand to a player
	 *
	 * @param playerParams			Index 0 is the specified player
	 * @param playCardParams		unused
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_restoreBackedUpHand(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_Player owner;

		try
		{
			owner = playerParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_restoreBackedUpHand: parameter arrays not properly filled!");
		}

		ArrayList<CN_PlayCard> newDiscards;
		try
		{
			newDiscards = owner.restoreBackedUpHand();
		}
		catch(CN_FatalException e)
		{
			throw new CN_FatalException("bascom_restoreBackedUpHand: error! --- details: " + e.getMessage());
		}

		for(CN_PlayCard card : newDiscards)
		{
			discards.push(card);
		}
	}



	/**
	 * Flips a linchpin
	 *
	 * @param playerParams			unused
	 * @param playCardParams		unused
	 * @param stringParams			Index 0 is the identity of the linchpin
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_flipLinchpin(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		String linchpin;
		ArrayList<Integer> discardedPatches;

		try
		{
			linchpin = stringParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_flipLinchpin: parameter arrays not properly filled!");
		}

		discardedPatches = timeline.flipLinchpin(linchpin);
		if(discardedPatches == null)
			throw new CN_FatalException("bascom_flipLinchpin: specified linchpin does not exist!");

		for(Integer patch : discardedPatches)
		{
			if(patch != null)
				discards.push(CN_Constants.playCardDirectory.getById(patch));
		}
	}


	/**
	 * Plays a patch from the specified player's hand
	 *
	 * @param playerParams			Index 0 is the owner of the patch
	 * @param playCardParams		Index 0 is the target card, which must be a patch
	 * @param stringParams			Index 0 is the identity of the target ripplepoint, Index 1 is an optional "true" or "false", for whether this card ends the world (default is false)
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_transferHandToRipplepoint(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_PlayCard patch;
		String ripplepoint;
		boolean endsTheWorld = false;

		CardLocation location;

		try
		{
			patch = playCardParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_transferHandToRipplepoint: playcard parameter array not properly filled!");
		}
		try
		{
			ripplepoint = stringParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_transferHandToRipplepoint: string parameter array not properly filled!");
		}

		if(stringParams.size() > 1)
		{
			if(stringParams.get(1).toLowerCase().equals("true"))
			{
				endsTheWorld = true;
			}
			else if(stringParams.get(1).toLowerCase().equals("false"))
			{
				endsTheWorld = false;
			}
			else
			{
				throw new CN_FatalException("bascom_transferHandToRipplepoint: string parameter array index 1 must be either \"true\" or \"false\"!");
			}
		}


		if(patch.cardType != CN_PlayCard.PlayCardType.PATCH)
		{
			throw new CN_FatalException("bascom_transferHandToRipplepoint: card is not a patch!");
		}


		location = locatePlayCard(patch);

		if(location.location != Location.HAND)
		{
			throw new CN_FatalException("bascom_transferHandToRipplepoint: card is not a member of a player's hand!");
		}


		if(location.player.removeHandCard(patch))
		{
			timeline.applyPatch(patch, ripplepoint, endsTheWorld);
		}
		else
		{
			throw new CN_FatalException("bascom_transferHandToRipplepoint: patch not present in source's hand!");
		}
	}




	/**
	 * Performs a time vortex.  All cards in all players hands are shuffled together and dealt out.
	 *
	 * @param playerParams			Index 0 is the player with whom the dealing starts.
	 * @param playCardParams		unused
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_doVortex(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_Player startingPlayer;

		try
		{
			startingPlayer = playerParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_doVortex: parameter arrays not properly filled!");
		}

		ArrayList<CN_PlayCard> playersHandCards = new ArrayList<CN_PlayCard>();
		ArrayList<CN_PlayCard> currentPlayerHandCards;

		for(CN_Player player : players)
		{
			currentPlayerHandCards = player.getListOfHandCards();
			for(CN_PlayCard card : currentPlayerHandCards)
			{
				player.removeHandCard(card);
				playersHandCards.add(card);
			}
		}

		Collections.shuffle(playersHandCards);

		CN_Player currentPlayerVortex;
		currentPlayerVortex = startingPlayer;

		while(!playersHandCards.isEmpty())
		{
			currentPlayerVortex.addHandCard(playersHandCards.remove(0));
			currentPlayerVortex = currentPlayerVortex.nextPlayer;
		}

	}




	/**
	 * Passes hands either to the left or to the right.
	 *
	 * @param playerParams			unused
	 * @param playCardParams		unused
	 * @param stringParams			Index 0 is either "left" or "right", and specifies the direction of card passing
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_passHands(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		String directionString;

		try
		{
			directionString = stringParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_passHands: parameter arrays not properly filled!");
		}

		boolean isLeft;

		if(directionString.toLowerCase().equals("left"))
		{
			isLeft = true;
		}
		else if(directionString.toLowerCase().equals("right"))
		{
			isLeft = false;
		}
		else
		{
			throw new CN_FatalException("bascom_passHands: Direction string \"" + directionString + "\" is invalid!");
		}

		ArrayList<ArrayList<CN_PlayCard>> newPlayerHands = new ArrayList<ArrayList<CN_PlayCard>>();
		for(int i = 0; i < players.length; i++)
		{
			newPlayerHands.add(null);
		}

		CN_Player myCurrentPlayer;
		ArrayList<CN_PlayCard> currentHand;
		for(int i = 0; i < players.length; i++)
		{
			myCurrentPlayer = players[i];
			currentHand = myCurrentPlayer.getListOfRealHandCards();

			for(CN_PlayCard card : currentHand)
			{
				myCurrentPlayer.removeRealHandCard(card);
			}

			if(isLeft)
			{
				newPlayerHands.set(myCurrentPlayer.prevPlayer.playerIndex, currentHand);
			}
			else
			{
				newPlayerHands.set(myCurrentPlayer.nextPlayer.playerIndex, currentHand);
			}
		}

		for(int i = 0; i < newPlayerHands.size(); i++)
		{
			currentHand = newPlayerHands.get(i);
			players[i].addListOfRealHandCards(currentHand);
		}

	}





	/**
	 * Gives the specified player a new ID from the deck
	 *
	 * @param playerParams			Index 0 is the specified player
	 * @param playCardParams		unused
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_newId(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_Player source;

		try
		{
			source = playerParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_newId: parameter arrays not properly filled!");
		}

		idDiscards.push(source.getId());
		if(idCards.empty())
		{
			if(idDiscards.empty())
			{
				throw new CN_FatalException("bascom_newId: both idCards and idDiscards are empty!");
			}
			mergeIdDiscardsIntoIdCards();
		}
		source.setId(idCards.pop());
	}

	/**
	 * Switches the missions of the two specified players
	 *
	 * @param playerParams			Index 0 and 1 are the two players
	 * @param playCardParams		unused
	 * @param stringParams			Index 0 is a boolean representing whether the option to get a new ID exists.  If it is "true", then if player1 and player2 are the same, that player gets a new ID from the stack.
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_switchIds(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_Player player1;
		CN_Player player2;
		String newIdIfSameString;

		try
		{
			player1 = playerParams.get(0);
			player2 = playerParams.get(1);
			newIdIfSameString = stringParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_switchIds: parameter arrays not properly filled!");
		}


		boolean newIdIfSame;
		if(newIdIfSameString.toLowerCase().equals("true"))
		{
			newIdIfSame = true;
		}
		else if(newIdIfSameString.toLowerCase().equals("false"))
		{
			newIdIfSame = false;
		}
		else
		{
			throw new CN_FatalException("bascom_switchIds: string parameter must be \"true\" or \"false\"!");
		}



		if((player1.playerIndex == player2.playerIndex)&&(newIdIfSame == true))
		{
			idDiscards.push(player1.getId());
			if(idCards.empty())
			{
				if(idDiscards.empty())
				{
					throw new CN_FatalException("bascom_switchIds: both idCards and idDiscards are empty!");
				}
				mergeIdDiscardsIntoIdCards();
			}
			player1.setId(idCards.pop());
		}
		else
		{
			CN_IdCard tempId;
			tempId = player1.getId();
			player1.setId(player2.getId());
			player2.setId(tempId);
		}
	}

	/**
	 * Gives the specified player a new Mission from the deck
	 *
	 * @param playerParams			Index 0 is the specified player
	 * @param playCardParams		unused
	 * @param stringParams			unused
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_newMission(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_Player source;

		try
		{
			source = playerParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_newMission: parameter arrays not properly filled!");
		}

		missionDiscards.push(source.getMission());
		if(missionCards.empty())
		{
			if(missionDiscards.empty())
			{
				throw new CN_FatalException("bascom_newMission: both missionCards and missionDiscards are empty!");
			}
			mergeMissionDiscardsIntoMissionCards();
		}
		source.setMission(missionCards.pop());
	}

	/**
	 * Switches the missions of the two specified players
	 *
	 * @param playerParams			Index 0 and 1 are the two players
	 * @param playCardParams		unused
	 * @param stringParams			Index 0 is a boolean representing whether the option to get a new Mission exists.  If it is "true", then if player1 and player2 are the same, that player gets a new mission from the stack.
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_switchMissions(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		CN_Player player1;
		CN_Player player2;
		String newMissionIfSameString;

		try
		{
			player1 = playerParams.get(0);
			player2 = playerParams.get(1);
			newMissionIfSameString = stringParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_switchMissions: parameter arrays not properly filled!");
		}

		boolean newMissionIfSame;
		if(newMissionIfSameString.toLowerCase().equals("true"))
		{
			newMissionIfSame = true;
		}
		else if(newMissionIfSameString.toLowerCase().equals("false"))
		{
			newMissionIfSame = false;
		}
		else
		{
			throw new CN_FatalException("bascom_switchMissions: string parameter must be \"true\" or \"false\"!");
		}

		if((player1.playerIndex == player2.playerIndex)&&(newMissionIfSame == true))
		{
			missionDiscards.push(player1.getMission());
			if(missionCards.empty())
			{
				if(missionDiscards.empty())
				{
					throw new CN_FatalException("bascom_switchMissions: both missionCards and missionDiscards are empty!");
				}
				mergeMissionDiscardsIntoMissionCards();
			}
			player1.setMission(missionCards.pop());
		}
		else
		{
			CN_MissionCard tempMission;
			tempMission = player1.getMission();
			player1.setMission(player2.getMission());
			player2.setMission(tempMission);
		}

	}


	private void bascom_saveGameState(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		saveGameState(true);
	}

	private void bascom_loadGameState(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		loadGameState(savedGameState);
	}

	private void bascom_clearGameState(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		savedGameState = null;
	}



	private void bascom_reshuffleDeck(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		deck.shuffle();
	}

	private void bascom_reshuffleIds(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		idCards.shuffle();
	}

	private void bascom_reshuffleMissions(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		missionCards.shuffle();
	}

	private void bascom_getArtifactSelection(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		inputConstraints.clear();

		try
		{
			gameInputHash = stringParams.get(0);
			inputStateMessage = stringParams.get(1);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_getArtifactSelection: parameter arrays not properly filled!");
		}

		cancelable = false;
		if(stringParams.size() > 2)
		{
			Boolean cancelableFromString = getCancelableFromString(stringParams, 2);
			int startingIndex = 2;
			if(cancelableFromString != null)
			{
				cancelable = cancelableFromString;
				startingIndex = 3;
			}
			try
			{
				for(int i = startingIndex; i < stringParams.size(); i++)
				{
					inputConstraints.add(parseInputConstraintString(stringParams.get(i)));
				}
			}
			catch(CN_FatalException e)
			{
				throw new CN_FatalException("bascom_getArtifactSelection: fatal exception; details -- " + e.getMessage());
			}
		}
		else
		{
		}
		gameInputState = CN_Constants.InputState.GET_ARTIFACT_SELECTION;
	}

	private void bascom_getHandSelection(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		inputConstraints.clear();

		try
		{
			gameInputHash = stringParams.get(0);
			inputStateMessage = stringParams.get(1);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_getHandSelection: parameter arrays not properly filled!");
		}

		cancelable = false;
		if(stringParams.size() > 2)
		{
			Boolean cancelableFromString = getCancelableFromString(stringParams, 2);
			int startingIndex = 2;
			if(cancelableFromString != null)
			{
				cancelable = cancelableFromString;
				startingIndex = 3;
			}
			try
			{
				for(int i = startingIndex; i < stringParams.size(); i++)
				{
					inputConstraints.add(parseInputConstraintString(stringParams.get(i)));
				}
			}
			catch(CN_FatalException e)
			{
				throw new CN_FatalException("bascom_getHandSelection: fatal exception; details -- " + e.getMessage());
			}
		}
		else
		{
		}
		gameInputState = CN_Constants.InputState.GET_HAND_SELECTION;
	}

	/**
	 * Changes the Input State to Get Linchpin Selection, and sets linchpin legalities
	 *
	 * @param playerParams			unused
	 * @param playCardParams		unused
	 * @param stringParams			The first parameter here is the input hash for the selection state.
	 *								The rest of the parameters must be in an even number, and alternating between a linchpin
	 *								identity and a "legality" string.  The legality string is either "flip", "unflip", or "both",
	 *								depending on what kind of linchpin flip is legal.
	 *								The other option is, instead of having an even number of remaining parameters in the list,
	 *								there is exactly one.  This can be "__flip", "__unflip", and "__both".  They affect all linchpins.
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_getLinchpinSelection(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		inputConstraints.clear();

		Boolean cancelableFromString = getCancelableFromString(stringParams, 2);
		try
		{
			gameInputHash = stringParams.get(0);
			stringParams.remove(0);
			inputStateMessage = stringParams.get(0);
			stringParams.remove(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_getLinchpinSelection: parameter arrays not properly filled!");
		}

		cancelable = false;
		if(cancelableFromString != null)
		{
			stringParams.remove(0);
			cancelable = cancelableFromString;
		}

		try
		{
			timeline.parseBascomLegalityArray(stringParams);
		}
		catch(CN_FatalException e)
		{
			throw new CN_FatalException("bascom_getLinchpinSelection: timeline error!  Message: " + e.getMessage());
		}
		gameInputState = CN_Constants.InputState.GET_LINCHPIN_SELECTION;
	}


	/**
	 * Changes the Input State to Get Ripplepoint Selection, and sets the current patch in question
	 *
	 * @param playerParams			unused
	 * @param playCardParams		Parameter 0 is the patch
	 * @param stringParams			Parameter 0 is the input hash for the selection state
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_getRipplepointSelection(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		inputConstraints.clear();

		try
		{
			patchInQuestion = playCardParams.get(0);
			gameInputHash = stringParams.get(0);
			inputStateMessage = stringParams.get(1);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_getRipplepointSelection: parameter arrays not properly filled!");
		}

		Boolean cancelableFromString = getCancelableFromString(stringParams, 2);
		cancelable = false;
		if(cancelableFromString != null)
		{
			cancelable = cancelableFromString;
		}

		if(patchInQuestion.cardType != CN_PlayCard.PlayCardType.PATCH)
		{
			throw new CN_FatalException("bascom_getRipplepointSelection: Input \"" + patchInQuestion.identity + "\" is not a patch!");
		}
		gameInputState = CN_Constants.InputState.GET_RIPPLEPOINT_SELECTION;
	}


	private void bascom_getIdSelection(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		inputConstraints.clear();

		try
		{
			gameInputHash = stringParams.get(0);
			inputStateMessage = stringParams.get(1);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_getIdSelection: parameter arrays not properly filled!");
		}

		cancelable = false;
		if(stringParams.size() > 2)
		{
			Boolean cancelableFromString = getCancelableFromString(stringParams, 2);
			int startingIndex = 2;
			if(cancelableFromString != null)
			{
				cancelable = cancelableFromString;
				startingIndex = 3;
			}
			try
			{
				for(int i = startingIndex; i < stringParams.size(); i++)
				{
					inputConstraints.add(parseInputConstraintString(stringParams.get(i)));
				}
			}
			catch(CN_FatalException e)
			{
				throw new CN_FatalException("bascom_getIdSelection: fatal exception; details -- " + e.getMessage());
			}
		}
		else
		{
		}
		gameInputState = CN_Constants.InputState.GET_ID_SELECTION;
	}

	private void bascom_getMissionSelection(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		inputConstraints.clear();

		try
		{
			gameInputHash = stringParams.get(0);
			inputStateMessage = stringParams.get(1);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_getMissionSelection: parameter arrays not properly filled!");
		}

		cancelable = false;
		if(stringParams.size() > 2)
		{
			Boolean cancelableFromString = getCancelableFromString(stringParams, 2);
			int startingIndex = 2;
			if(cancelableFromString != null)
			{
				cancelable = cancelableFromString;
				startingIndex = 3;
			}
			try
			{
				for(int i = startingIndex; i < stringParams.size(); i++)
				{
					inputConstraints.add(parseInputConstraintString(stringParams.get(i)));
				}
			}
			catch(CN_FatalException e)
			{
				throw new CN_FatalException("bascom_getMissionSelection: fatal exception; details -- " + e.getMessage());
			}
		}
		else
		{
		}
		gameInputState = CN_Constants.InputState.GET_MISSION_SELECTION;
	}



	private void bascom_getCardPlay(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		inputConstraints.clear();

		try
		{
			inputStateMessage = stringParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_getCardPlay: parameter arrays not properly filled!");
		}

		Boolean cancelableFromString = getCancelableFromString(stringParams, 1);
		cancelable = false;
		if(cancelableFromString != null)
		{
			cancelable = cancelableFromString;
		}

		gameInputState = CN_Constants.InputState.GET_CARD_PLAY;
	}




	private void bascom_getDirectionSelection(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		inputConstraints.clear();

		try
		{
			gameInputHash = stringParams.get(0);
			inputStateMessage = stringParams.get(1);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_getDirectionSelection: parameter arrays not properly filled!");
		}

		Boolean cancelableFromString = getCancelableFromString(stringParams, 2);
		cancelable = false;
		if(cancelableFromString != null)
		{
			cancelable = cancelableFromString;
		}

		gameInputState = CN_Constants.InputState.GET_DIRECTION_SELECTION;
	}



	private void bascom_getDeckSelection(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		inputConstraints.clear();

		try
		{
			gameInputHash = stringParams.get(0);
			inputStateMessage = stringParams.get(1);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_getDeckSelection: parameter arrays not properly filled!");
		}

		cancelable = false;
		if(stringParams.size() > 2)
		{
			Boolean cancelableFromString = getCancelableFromString(stringParams, 2);
			int startingIndex = 2;
			if(cancelableFromString != null)
			{
				cancelable = cancelableFromString;
				startingIndex = 3;
			}
			try
			{
				for(int i = startingIndex; i < stringParams.size(); i++)
				{
					inputConstraints.add(parseInputConstraintString(stringParams.get(i)));
				}
			}
			catch(CN_FatalException e)
			{
				throw new CN_FatalException("bascom_getDeckSelection: fatal exception; details -- " + e.getMessage());
			}
		}
		else
		{
		}

		if(deck.empty())
		{
			if(discards.empty())
			{
				throw new CN_FatalException("bascom_getDeckSelection: both deck and discards are empty!");
			}
			mergeDiscardsIntoDeck();
		}

		gameInputState = CN_Constants.InputState.GET_DECK_SELECTION;
	}



	private void bascom_getDiscardsSelection(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		inputConstraints.clear();

		try
		{
			gameInputHash = stringParams.get(0);
			inputStateMessage = stringParams.get(1);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_getDiscardsSelection: parameter arrays not properly filled!");
		}

		cancelable = false;
		if(stringParams.size() > 2)
		{
			Boolean cancelableFromString = getCancelableFromString(stringParams, 2);
			int startingIndex = 2;
			if(cancelableFromString != null)
			{
				cancelable = cancelableFromString;
				startingIndex = 3;
			}
			try
			{
				for(int i = startingIndex; i < stringParams.size(); i++)
				{
					inputConstraints.add(parseInputConstraintString(stringParams.get(i)));
				}
			}
			catch(CN_FatalException e)
			{
				throw new CN_FatalException("bascom_getDiscardsSelection: fatal exception; details -- " + e.getMessage());
			}
		}
		else
		{
		}
		gameInputState = CN_Constants.InputState.GET_DISCARDS_SELECTION;
	}




	private void bascom_endTurn(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		inputConstraints.clear();

		try
		{
			inputStateMessage = stringParams.get(0);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_endTurn: parameter arrays not properly filled!");
		}

		gameInputState = CN_Constants.InputState.END_TURN;
	}

	/**
	 * Skips the specified number of items in the bascomQueue
	 *
	 * @param playerParams			unused
	 * @param playCardParams		unused
	 * @param stringParams			Index 0 is the number of items; this parameter must be parsable into an integer
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascom_skipQueueItems(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		int numberSkipped;

		try
		{
			numberSkipped = Integer.parseInt(stringParams.get(0));
		}
		catch(NumberFormatException e)
		{
			throw new CN_FatalException("bascom_skipQueueItems: Input string does not contain a parsable integer!");
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascom_skipQueueItems: parameter arrays not properly filled!");
		}

		for(int i = 0; i < numberSkipped; i++)
		{
			bascomQueue.removeFirst();
		}
	}

	/*
	 * This function requires a list of cards, a list of string specifying
	 * the type of comparison, and a list of names.  If any of the
	 * comparisons made are true, the IF statement succeeds.  Thus, this
	 * function uses "OR" logic.
	 */
	private void bascom_identConditional(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		if(stringParams.size() != playCardParams.size() * 2)
		{
			throw new CN_FatalException("bascom_identConditional: size of stringParams (" + String.valueOf(stringParams.size()) + ") must be twice the size of playCardParams (" + String.valueOf(playCardParams.size()) + ")!");
		}

		ArrayList<String> typeStrings = new ArrayList<String>();
		ArrayList<String> valueStrings = new ArrayList<String>();

		for(int i = 0; i < stringParams.size(); i++)
		{
			if(i % 2 == 0)
			{
				typeStrings.add(stringParams.get(i));
			}
			else
			{
				valueStrings.add(stringParams.get(i));
			}
		}

		boolean conditionState = false;
		for(int i = 0; i < playCardParams.size(); i++)
		{
			if(typeStrings.get(i).toLowerCase().equals("identity"))
			{
				if(playCardParams.get(i).identity.equals(valueStrings.get(i)))
				{
					conditionState = true;
				}
			}
			else if(typeStrings.get(i).toLowerCase().equals("mostauthentic"))
			{
				if((playCardParams.get(i).authClass.equals(valueStrings.get(i)))&&(isMostAuthentic(playCardParams.get(i))))
				{
					conditionState = true;
				}
			}
			else
			{
				throw new CN_FatalException("bascom_identConditional: string \"" + typeStrings.get(i) + "\" is not a valid comparison type!");
			}
		}

		ifCounter++;
		if(conditionState)
		{
			//Success is empty.
		}
		else
		{
			int tempIfCounter = 0;
			while(!bascomQueue.isEmpty())
			{
				CN_BascomSpecifier currentBascom;
				currentBascom = bascomQueue.getFirst();
				if(currentBascom.commandIndex == CN_Constants.BASCOM_IDENT_CONDITIONAL)
				{
					tempIfCounter++;
					bascomQueue.removeFirst();
				}
				else if((currentBascom.commandIndex == CN_Constants.BASCOM_ELSE)&&(tempIfCounter > 0))
				{
					bascomQueue.removeFirst();
				}
				else if((currentBascom.commandIndex == CN_Constants.BASCOM_END_IF)&&(tempIfCounter > 0))
				{
					tempIfCounter--;
					bascomQueue.removeFirst();
				}
				else if((currentBascom.commandIndex == CN_Constants.BASCOM_ELSE)&&(tempIfCounter <= 0))
				{
					bascomQueue.removeFirst();
					return;
				}
				else if((currentBascom.commandIndex == CN_Constants.BASCOM_END_IF)&&(tempIfCounter <= 0))
				{
					return;
				}
				else
				{
					bascomQueue.removeFirst();
				}
			}
			throw new CN_FatalException("bascom_identConditional: Condition failed, but no ELSE or END IF was encountered!");
		}
	}


	/*
	 * This function skips everything between it and its respective END IF.  Thus,
	 * those things are only executed if an IDENTCONDITIONAL fails, which takes you
	 * straight to the first BASCOM in the ELSE block.
	 */
	private void bascom_else(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		int tempIfCounter = 0;
		while(!bascomQueue.isEmpty())
		{
			CN_BascomSpecifier currentBascom;
			currentBascom = bascomQueue.getFirst();
			if(currentBascom.commandIndex == CN_Constants.BASCOM_IDENT_CONDITIONAL)
			{
				tempIfCounter++;
				bascomQueue.removeFirst();
			}
			else if((currentBascom.commandIndex == CN_Constants.BASCOM_END_IF)&&(tempIfCounter > 0))
			{
				tempIfCounter--;
				bascomQueue.removeFirst();
			}
			else if((currentBascom.commandIndex == CN_Constants.BASCOM_END_IF)&&(tempIfCounter <= 0))
			{
				return;
			}
			else
			{
				bascomQueue.removeFirst();
			}
		}
		throw new CN_FatalException("bascom_Else: no END IF was encountered!");
	}


	private void bascom_endIf(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		ifCounter--;
		if(ifCounter < 0)
		{
			throw new CN_FatalException("bascom_EndIf: more END IF's were encountered than conditionals!");
		}
	}


	/**
	 * Loads a literal parameter into a Basic Command
	 *
	 * @param playerParams			unused
	 * @param playCardParams		unused
	 * @param stringParams			Index 0 is an integer representing the parameter type; index 1 is the literal itself, in string form; index 2 is an integer representing the queue index of the destination command; 0 and 2 must be parsable into integers
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascomSpecial_loadParameterLiteral(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		int paramType;
		String param;
		int destinationQueueIndex;

		try
		{
			paramType = getParamTypeInt(stringParams.get(0));
			param = stringParams.get(1);
			destinationQueueIndex = Integer.parseInt(stringParams.get(2));
		}
		catch(NumberFormatException e)
		{
			throw new CN_FatalException("bascomSpecial_loadParameterLiteral: Input string for paramType does not contain a parsable integer!");
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascomSpecial_loadParameterLiteral: parameter arrays not properly filled!");
		}


		CN_BascomSpecifier command;

		try
		{
			command = bascomQueue.get(destinationQueueIndex);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascomSpecial_loadParameterLiteral: Index specified was out of bounds of the bascomQueue!");
		}

		switch(paramType)
		{
			case CN_BascomSpecifier.BascomParamType.PLAYCARD:
				throw new CN_FatalException("bascomSpecial_loadParameterLiteral: Function does not support PlayCards!");
			case CN_BascomSpecifier.BascomParamType.PLAYER:
				try
				{
					command.playerParams.add(players[Integer.parseInt(param)]);
				}
				catch(NumberFormatException e)
				{
					throw new CN_FatalException("bascomSpecial_loadParameterLiteral: Player index '" + param + "' does not contain a parsable integer!");
				}
				break;
			case CN_BascomSpecifier.BascomParamType.STRING:
				command.stringParams.add(param);
				break;
			default:
				throw new CN_FatalException("bascomSpecial_loadParameterLiteral: paramType was not recognized!");
		}

	}


	/**
	 * Loads a hashed parameter into a Basic Command
	 *
	 * @param playerParams			unused
	 * @param playCardParams		unused
	 * @param stringParams			Index 0 is an integer representing the parameter type; index 1 is the hash code, in string form; index 2 is an integer representing the queue index of the destination command; 0 and 2 must be parsable into integers
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascomSpecial_loadParameterHash(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		int paramType;
		String paramHash;
		int destinationQueueIndex;


		try
		{
			paramType = getParamTypeInt(stringParams.get(0));
			paramHash = stringParams.get(1);
			destinationQueueIndex = Integer.parseInt(stringParams.get(2));
		}
		catch(NumberFormatException e)
		{
			throw new CN_FatalException("bascomSpecial_loadParameterHash: Input string does not contain a parsable integer!");
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascomSpecial_loadParameterHash: parameter arrays not properly filled!");
		}


		CN_BascomSpecifier command;

		try
		{
			command = bascomQueue.get(destinationQueueIndex);
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascomSpecial_loadParameterHash: Index specified was out of bounds of the bascomQueue!");
		}

		switch(paramType)
		{
			case CN_BascomSpecifier.BascomParamType.PLAYCARD:
				if(bascomPlayCardParams.get(paramHash) == null)
					throw new CN_FatalException("bascomSpecial_loadParameterHash: Key '" + paramHash + "' was not found in bascomPlayCardParams!");
				command.playCardParams.add(bascomPlayCardParams.get(paramHash));
				//if(!paramHash.equals(CN_Constants.SPECIALPARAM_PLAYEDCARDHASH))
					//bascomPlayerParams.remove(paramHash);
				break;
			case CN_BascomSpecifier.BascomParamType.PLAYER:
				if(bascomPlayerParams.get(paramHash) == null)
					throw new CN_FatalException("bascomSpecial_loadParameterHash: Key '" + paramHash + "' was not found in bascomPlayerParams!");
				command.playerParams.add(bascomPlayerParams.get(paramHash));
				//if(!paramHash.equals(CN_Constants.SPECIALPARAM_CURRENTPLAYERHASH))
					//bascomPlayerParams.remove(paramHash);
				break;
			case CN_BascomSpecifier.BascomParamType.STRING:
				if(bascomStringParams.get(paramHash) == null)
					throw new CN_FatalException("bascomSpecial_loadParameterHash: Key '" + paramHash + "' was not found in bascomStringParams!");
				command.stringParams.add(bascomStringParams.get(paramHash));
				//bascomStringParams.remove(paramHash);
				break;
			default:
				throw new CN_FatalException("bascomSpecial_loadParameterHash: paramType was not recognized!");
		}
	}


	/**
	 * Removes a hashed parameter
	 *
	 * @param playerParams			unused
	 * @param playCardParams		unused
	 * @param stringParams			Index 0 is an integer representing the parameter type; index 1 is the hash code, in string form; index 0 must be parsable into an integer
	 * @return						void
	 * @throws CN_FatalException	If an error has occurred from which the engine cannot recover
	 */
	private void bascomSpecial_removeParameter(ArrayList<CN_Player> playerParams, ArrayList<CN_PlayCard> playCardParams, ArrayList<String> stringParams) throws CN_FatalException
	{
		int paramType;
		String paramHash;

		try
		{
			paramType = Integer.parseInt(stringParams.get(0));
			paramHash = stringParams.get(1);
		}
		catch(NumberFormatException e)
		{
			throw new CN_FatalException("bascomSpecial_removeParameter: Input string does not contain a parsable integer!");
		}
		catch(IndexOutOfBoundsException e)
		{
			throw new CN_FatalException("bascomSpecial_removeParameter: parameter arrays not properly filled!");
		}


		switch(paramType)
		{
			case CN_BascomSpecifier.BascomParamType.PLAYCARD:
				if(bascomPlayCardParams.get(paramHash) == null)
					throw new CN_FatalException("bascomSpecial_removeParameter: Key was not found in bascomPlayCardParams!");
				bascomPlayCardParams.remove(paramHash);
				break;
			case CN_BascomSpecifier.BascomParamType.PLAYER:
				if(bascomPlayerParams.get(paramHash) == null)
					throw new CN_FatalException("bascomSpecial_removeParameter: Key was not found in bascomPlayerParams!");
				bascomPlayerParams.remove(paramHash);
				break;
			case CN_BascomSpecifier.BascomParamType.STRING:
				if(bascomStringParams.get(paramHash) == null)
					throw new CN_FatalException("bascomSpecial_removeParameter: Key was not found in bascomStringParams!");
				bascomStringParams.remove(paramHash);
				break;
			default:
				throw new CN_FatalException("bascomSpecial_removeParameter: paramType was not recognized!");
		}
	}






	private Boolean getCancelableFromString(ArrayList<String> stringParams, int index)
	{
		if(stringParams.size() <= index)
			return null;

		String possibleString = stringParams.get(index);
		if(possibleString.equals(CN_Constants.CANCELABLE_STRING))
			return true;
		else if(possibleString.equals(CN_Constants.NOT_CANCELABLE_STRING))
			return false;
		else
			return null;
	}

	private void setUpDiscardClick()
	{
		CN_BascomSpecifier hashParamLoader = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL, null);
		hashParamLoader.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.STRING));
		hashParamLoader.stringParams.add(CN_Constants.DISCARDONEHASH);
		hashParamLoader.stringParams.add("1");
		CN_BascomSpecifier hashParamMessageLoader = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL, null);
		hashParamMessageLoader.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.STRING));
		hashParamMessageLoader.stringParams.add("Select a card to discard.");
		hashParamMessageLoader.stringParams.add("0");
		CN_BascomSpecifier stateChanger = new CN_BascomSpecifier(CN_Constants.BASCOM_GET_HAND_SELECTION, null);
		CN_BascomSpecifier cardParamLoader = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_HASH, null);
		cardParamLoader.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.PLAYCARD));
		cardParamLoader.stringParams.add(CN_Constants.DISCARDONEHASH);
		cardParamLoader.stringParams.add("1");
		CN_BascomSpecifier cardParamMessageLoader = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL, null);
		cardParamMessageLoader.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.STRING));
		cardParamMessageLoader.stringParams.add("");
		cardParamMessageLoader.stringParams.add("0");
		CN_BascomSpecifier discarder = new CN_BascomSpecifier(CN_Constants.BASCOM_DISCARD_CARD, null);
		bascomQueue.addFirst(discarder);
		bascomQueue.addFirst(cardParamMessageLoader);
		bascomQueue.addFirst(cardParamLoader);
		bascomQueue.addFirst(stateChanger);
		bascomQueue.addFirst(hashParamMessageLoader);
		bascomQueue.addFirst(hashParamLoader);

		aiMessage = "The Discard One option was chosen.";
	}


	private void setUpDiscardTwoClick()
	{
		CN_BascomSpecifier hashParamLoader1 = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL, null);
		hashParamLoader1.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.STRING));
		hashParamLoader1.stringParams.add(CN_Constants.DISCARDTWOHASH);
		hashParamLoader1.stringParams.add("1");
		CN_BascomSpecifier hashParamMessageLoader1 = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL, null);
		hashParamMessageLoader1.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.STRING));
		hashParamMessageLoader1.stringParams.add("Select a card to discard.");
		hashParamMessageLoader1.stringParams.add("0");
		CN_BascomSpecifier stateChanger1 = new CN_BascomSpecifier(CN_Constants.BASCOM_GET_HAND_SELECTION, null);
		CN_BascomSpecifier cardParamLoader1 = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_HASH, null);
		cardParamLoader1.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.PLAYCARD));
		cardParamLoader1.stringParams.add(CN_Constants.DISCARDTWOHASH);
		cardParamLoader1.stringParams.add("1");
		CN_BascomSpecifier cardParamMessageLoader1 = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL, null);
		cardParamMessageLoader1.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.STRING));
		cardParamMessageLoader1.stringParams.add("");
		cardParamMessageLoader1.stringParams.add("0");
		CN_BascomSpecifier discarder1 = new CN_BascomSpecifier(CN_Constants.BASCOM_DISCARD_CARD, null);

		CN_BascomSpecifier hashParamLoader2 = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL, null);
		hashParamLoader2.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.STRING));
		hashParamLoader2.stringParams.add(CN_Constants.SECONDDISCARDHASH);
		hashParamLoader2.stringParams.add("2");
		CN_BascomSpecifier hashParamMessageLoader2 = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL, null);
		hashParamMessageLoader2.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.STRING));
		hashParamMessageLoader2.stringParams.add("Select another card to discard.");
		hashParamMessageLoader2.stringParams.add("1");
		CN_BascomSpecifier stateSaver = new CN_BascomSpecifier(CN_Constants.BASCOM_SAVE_GAME_STATE, null);
		CN_BascomSpecifier stateChanger2 = new CN_BascomSpecifier(CN_Constants.BASCOM_GET_HAND_SELECTION, null);
		CN_BascomSpecifier cardParamLoader2 = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_HASH, null);
		cardParamLoader2.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.PLAYCARD));
		cardParamLoader2.stringParams.add(CN_Constants.SECONDDISCARDHASH);
		cardParamLoader2.stringParams.add("1");
		CN_BascomSpecifier cardParamMessageLoader2 = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL, null);
		cardParamMessageLoader2.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.STRING));
		cardParamMessageLoader2.stringParams.add("");
		cardParamMessageLoader2.stringParams.add("0");
		CN_BascomSpecifier discarder2 = new CN_BascomSpecifier(CN_Constants.BASCOM_DISCARD_CARD, null);
		CN_BascomSpecifier drawPlayerParamLoader = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_HASH, null);
		drawPlayerParamLoader.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.PLAYER));
		drawPlayerParamLoader.stringParams.add(CN_Constants.SPECIALPARAM_CURRENTPLAYERHASH);
		drawPlayerParamLoader.stringParams.add("1");
		CN_BascomSpecifier drawPlayerParamMessageLoader = new CN_BascomSpecifier(CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL, null);
		drawPlayerParamMessageLoader.stringParams.add(Integer.toString(CN_BascomSpecifier.BascomParamType.STRING));
		drawPlayerParamMessageLoader.stringParams.add("");
		drawPlayerParamMessageLoader.stringParams.add("0");
		CN_BascomSpecifier cardDrawer = new CN_BascomSpecifier(CN_Constants.BASCOM_DRAW, null);
		CN_BascomSpecifier stateClearer = new CN_BascomSpecifier(CN_Constants.BASCOM_CLEAR_GAME_STATE, null);

		bascomQueue.addFirst(stateClearer);
		bascomQueue.addFirst(cardDrawer);
		bascomQueue.addFirst(drawPlayerParamMessageLoader);
		bascomQueue.addFirst(drawPlayerParamLoader);
		bascomQueue.addFirst(discarder2);
		bascomQueue.addFirst(cardParamMessageLoader2);
		bascomQueue.addFirst(cardParamLoader2);
		bascomQueue.addFirst(stateChanger2);
		bascomQueue.addFirst(stateSaver);
		bascomQueue.addFirst(hashParamMessageLoader2);
		bascomQueue.addFirst(hashParamLoader2);

		bascomQueue.addFirst(discarder1);
		bascomQueue.addFirst(cardParamMessageLoader1);
		bascomQueue.addFirst(cardParamLoader1);
		bascomQueue.addFirst(stateChanger1);
		bascomQueue.addFirst(hashParamMessageLoader1);
		bascomQueue.addFirst(hashParamLoader1);

		aiMessage = "The Discard Two option was chosen.";
	}




	private CN_Constants.InputConstraint parseInputConstraintString(String inputConstraintString) throws CN_FatalException
	{
		CN_Constants.InputConstraint returnValue;
		if(inputConstraintString.toLowerCase().equals(CN_Constants.CONSTRAINT_NONE.toLowerCase()))
		{
			returnValue = CN_Constants.InputConstraint.NONE;
		}
		else if(inputConstraintString.toLowerCase().equals(CN_Constants.CONSTRAINT_CURRENTPLAYER.toLowerCase()))
		{
			returnValue = CN_Constants.InputConstraint.CURRENT_PLAYER;
		}
		else if(inputConstraintString.toLowerCase().equals(CN_Constants.CONSTRAINT_NOTCURRENTPLAYER.toLowerCase()))
		{
			returnValue = CN_Constants.InputConstraint.NOT_CURRENT_PLAYER;
		}
		else if(inputConstraintString.toLowerCase().equals(CN_Constants.CONSTRAINT_PLAYABLE.toLowerCase()))
		{
			returnValue = CN_Constants.InputConstraint.PLAYABLE;
		}
		else if(inputConstraintString.toLowerCase().equals(CN_Constants.CONSTRAINT_NOT_A_TIMEWARP.toLowerCase()))
		{
			returnValue = CN_Constants.InputConstraint.NOT_A_TIMEWARP;
		}
		else
		{
			throw new CN_FatalException("parseInputConstraintString: input constraint string \"" + inputConstraintString + "\" not recognized!");
		}
		return returnValue;
	}




	private int getParamTypeInt(String paramTypeString) throws CN_FatalException
	{
		int returnValue;

		if(paramTypeString.toLowerCase().equals("player"))
		{
			returnValue = CN_BascomSpecifier.BascomParamType.PLAYER;
		}
		else if(paramTypeString.toLowerCase().equals("playcard"))
		{
			returnValue = CN_BascomSpecifier.BascomParamType.PLAYCARD;
		}
		else if(paramTypeString.toLowerCase().equals("string"))
		{
			returnValue = CN_BascomSpecifier.BascomParamType.STRING;
		}
		else
		{
			try
			{
				returnValue = Integer.parseInt(paramTypeString);
			}
			catch(NumberFormatException e)
			{
				throw new CN_FatalException("CN_Engine, getParamTypeInt(): Invalid paramTypeString - not a parsable integer!");
			}
			if((returnValue != CN_BascomSpecifier.BascomParamType.PLAYER)&&(returnValue != CN_BascomSpecifier.BascomParamType.PLAYCARD)&&(returnValue != CN_BascomSpecifier.BascomParamType.STRING))
			{
				throw new CN_FatalException("CN_Engine, getParamTypeInt(): Invalid paramTypeString - bad integer!");
			}
		}

		return returnValue;
	}



	//todo: get true number of locations (i.e. number of times appearing in deck, hand, etc.)
	private int findNumberOfLocations(CN_PlayCard card) throws CN_FatalException
	{
		int locationCount = 0;

		for(int i = 0; i < players.length; i++)
		{
			if(players[i].isInHand(card))
			{
				locationCount++;
			}

			if(players[i].isInArtifacts(card))
			{
				locationCount++;
			}

			if(players[i].isInBackedUpHands(card))
			{
				locationCount++;
			}

		}

		if(deck.contains(card))
		{
			locationCount++;
		}

		if(discards.contains(card))
		{
			locationCount++;
		}

		locationCount += timeline.timesPatchPresent(card);

		return locationCount;
	}




	private CardLocation locatePlayCard(CN_PlayCard card)
	{
		CardLocation returnValue = new CardLocation();

		for(int i = 0; i < players.length; i++)
		{
			if(players[i].isInHand(card))
			{
				returnValue.location = Location.HAND;
				returnValue.player = players[i];
				break;
			}
			else if(players[i].isInArtifacts(card))
			{
				returnValue.location = Location.ARTIFACTS;
				returnValue.player = players[i];
				break;
			}
			else if(players[i].isInBackedUpHands(card))
			{
				returnValue.location = Location.BACKED_UP_HAND;
				returnValue.player = players[i];
				break;
			}
		}

		if(returnValue.player == null)
		{
			if(deck.contains(card))
			{
				returnValue.location = Location.DECK;
			}
			else if(discards.contains(card))
			{
				returnValue.location = Location.DISCARDS;
			}
			else
			{
				returnValue.location = Location.TIMELINE;
			}
		}

		return returnValue;
	}


	private CN_Player locateIdCard(CN_IdCard card)
	{
		CN_Player returnValue = null;

		for(int i = 0; i < players.length; i++)
		{
			if(players[i].getId() == card)
			{
				returnValue = players[i];
				return returnValue;
			}
		}

		return returnValue;
	}

	private CN_Player locateMissionCard(CN_MissionCard card)
	{
		CN_Player returnValue = null;

		for(int i = 0; i < players.length; i++)
		{
			if(players[i].getMission() == card)
			{
				returnValue = players[i];
				return returnValue;
			}
		}

		return returnValue;
	}




	private boolean isMostAuthentic(CN_PlayCard artifact)
	{
		return (artifact.authenticity >= getHighestAuth(artifact.authClass));
	}


	private ArrayList<Integer> getHighestAuthsInPlay(ArrayList<CN_PlayCard> artifacts)
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();
		CN_PlayCard artifact;
		for(int i = 0; i < artifacts.size(); i++)
		{
			artifact = artifacts.get(i);
			returnValue.add(getHighestAuth(artifact.authClass));
		}
		return returnValue;
	}


	private int getHighestAuth(String testAuthClass)
	{
		int returnValue = -1;
		ArrayList<Integer> matchingAuths = new ArrayList<Integer>();

		for(CN_Player player : players)
		{
			for(CN_PlayCard artifact : player.getListOfArtifacts())
			{
				if(artifact.authClass.equals(testAuthClass))
				{
					matchingAuths.add(artifact.authenticity);
				}
			}
		}

		for(Integer currentInt : matchingAuths)
		{
			if(currentInt > returnValue)
			{
				returnValue = currentInt;
			}
		}

		return returnValue;
	}


	private void testAgainstInputConstraints(CN_PlayCard referencedPlayCard, CN_TimelineCard referencedTimelineCard, CN_IdCard referencedIdCard, CN_MissionCard referencedMissionCard, boolean referencedDirectionIsLeft, CardLocation referencedCardLocation) throws CN_InputMessageException, CN_FatalException
	{
		for(CN_Constants.InputConstraint constraint : inputConstraints)
		{
			switch(constraint)
			{
				case NONE:
					break;
				case CURRENT_PLAYER:
					if((referencedCardLocation == null)||(referencedPlayCard == null))
					{
						break;
					}
					if(referencedCardLocation.player != currentPlayer)
					{
						throw new CN_InputMessageException("Artificial constraint of CARD_CLICK violated - selected artifact is not owned by current player.");
					}
					break;
				case NOT_CURRENT_PLAYER:
					if((referencedCardLocation == null)||(referencedPlayCard == null))
					{
						break;
					}
					if(referencedCardLocation.player == currentPlayer)
					{
						throw new CN_InputMessageException("Artificial constraint of CARD_CLICK violated - selected artifact is owned by current player.");
					}
					break;
				case PLAYABLE:
					if(referencedPlayCard == null)
					{
						break;
					}
					if(testUnplayable(referencedPlayCard))
					{
						throw new CN_InputMessageException("Artificial constraint of CARD_CLICK violated - card is unplayable.");
					}
					break;
				case NOT_A_TIMEWARP:
					if(referencedPlayCard == null)
					{
						break;
					}
					if(referencedPlayCard.cardType == CN_PlayCard.PlayCardType.TIMEWARP)
					{
						throw new CN_InputMessageException("Artificial constraint of CARD_CLICK violated - card is a timewarp.");
					}
					break;
				default:
					throw new CN_FatalException("CN_Engine, testAgainstInputConstraints(): Invalid constraint!");
			}
		}
	}





	private boolean testUnplayable(CN_PlayCard card) throws CN_FatalException
	{
		if(!timeline.isBeforeTerminalYear(card))
		{
			return true;
		}

		ArrayList<CN_PlayCard.UnplayableCondition> unplayableConditions;
		unplayableConditions = card.getUnplayableConditions();
		for(int i = 0; i < unplayableConditions.size(); i++)
		{
			if(testUnplayableCondition(unplayableConditions.get(i), card))
			{
				return true;
			}
		}
		return false;
	}




	private boolean testUnplayableCondition(CN_PlayCard.UnplayableCondition unplayableCondition, CN_PlayCard card) throws CN_FatalException
	{
		switch(unplayableCondition)
		{
			case PATCH_STANDARD:
				return isPatchIllegalEverywhere(card);
			case INVERTER:
				return isInverterUnplayable(card.getInverterUnplayableCondition());
			case DISCARDS_EMPTY:
				return discards.empty();
			case NO_NON_TIMEWARP_DISCARDS_PLAYABLE:
				return noNonTimewarpDiscardsPlayable();
			case ARTIFACTS_NONE:
				return noArtifacts();
			case ARTIFACTS_NOT_CURRENT_PLAYER:
				return noCurrentPlayerArtifacts();
			case ARTIFACTS_NOT_OPPONENT:
				return noOpponentArtifacts();
			default:
				throw new CN_FatalException("CN_Engine: testUnplayableCondition(): Invalid condition!");
		}
	}






	private boolean isInverterUnplayable(ArrayList<CN_PlayCard.InverterUnplayableConditionPart> inverterCondition) throws CN_FatalException
	{
		for(CN_PlayCard.InverterUnplayableConditionPart conditionPart : inverterCondition)
		{
			if(conditionPart.linchpin.equals(CN_Constants.UNPLAYABLE_ALL_LINCHPINS))
			{
				if(!timeline.testAllLinchpins(conditionPart.unplayableIfFlipped))
				{
					return false;
				}
			}
			else
			{
				if(timeline.isLinchpinFlipped(conditionPart.linchpin) != conditionPart.unplayableIfFlipped)
				{
					return false;
				}
			}
		}
		return true;
	}


	private boolean isPatchIllegalEverywhere(CN_PlayCard patch) throws CN_FatalException
	{
		return !timeline.isPatchLegalAnywhere(patch);
	}



	private boolean noArtifacts()
	{
		for(CN_Player player : players)
		{
			if(player.hasArtifacts())
			{
				return false;
			}
		}
		return true;
	}

	private boolean noCurrentPlayerArtifacts()
	{
		return !currentPlayer.hasArtifacts();
	}

	private boolean noOpponentArtifacts()
	{
		for(CN_Player player : players)
		{
			if(player.playerIndex != currentPlayer.playerIndex)
			{
				if(player.hasArtifacts())
				{
					return false;
				}
			}
		}
		return true;
	}


	private boolean noNonTimewarpDiscardsPlayable() throws CN_FatalException
	{
		for(CN_PlayCard card : discards)
		{
			if(card.cardType == CN_PlayCard.PlayCardType.TIMEWARP)
			{
			}
			else if(card.containsUnplayableCondition(CN_PlayCard.UnplayableCondition.NO_NON_TIMEWARP_DISCARDS_PLAYABLE))
			{
			}
			else
			{
				if(!testUnplayable(card))
				{
					return false;
				}
			}
		}
		return true;
	}


	private CN_Player getVictoriousPlayer() throws CN_FatalException
	{
		CN_Player returnValue = null;

		ArrayList<CN_PlayCard> playerArtifacts;
		ArrayList<Integer> highestAuthsInPlay;

		int currentVictoryType;

		//This strange code structure is so that if multiple players have "won", the current player is favored for victory.
		//Also, it tests players whose turn it isn't, even though in CN_Player.testForVictory() they clearly cannot win if it's not their turn, b/c functionality may be added later in the form of a CN_Player property such as "canWinOnOtherPlayersTurns" or something
		for(CN_Player player : players)
		{
			if(player != currentPlayer)
			{
				playerArtifacts = player.getListOfArtifacts();
				highestAuthsInPlay = getHighestAuthsInPlay(playerArtifacts);

				currentVictoryType = player.testForVictory(timeline, playerArtifacts, highestAuthsInPlay, false);
				if(currentVictoryType != CN_Constants.VICTORY_TYPE_NULL)
				{
					returnValue = player;
				}
			}
		}

		playerArtifacts = currentPlayer.getListOfArtifacts();
		highestAuthsInPlay = getHighestAuthsInPlay(playerArtifacts);

		currentVictoryType = currentPlayer.testForVictory(timeline, playerArtifacts, highestAuthsInPlay, true);
		if(currentVictoryType != CN_Constants.VICTORY_TYPE_NULL)
		{
			returnValue = currentPlayer;
		}

		if(returnValue != null)
			victoryType = currentVictoryType;

		return returnValue;
	}

	private boolean getEndOfUniverse()
	{
		if(timeline.getUnpatchedParadoxes() >= CN_Constants.END_OF_UNIVERSE_PARADOX_COUNT)
			return true;
		else
			return false;
	}




	private ArrayList<CN_GameState.BascomSpecifier> getBascomQueueGs()
	{
		ArrayList<CN_GameState.BascomSpecifier> returnValue = new ArrayList<CN_GameState.BascomSpecifier>();

		CN_BascomSpecifier currentSpecifier;
		CN_GameState.BascomSpecifier currentGsSpecifier;
		int currentSpecifierSource;
		for(int i = 0; i < bascomQueue.size(); i++)
		{
			currentSpecifier = bascomQueue.get(i);
			if(currentSpecifier.source == null)
			{
				currentSpecifierSource = CN_Constants.NULL_BASCOM_SOURCE;
			}
			else
			{
				currentSpecifierSource = currentSpecifier.source.cardNumber;
			}
			currentGsSpecifier = new CN_GameState.BascomSpecifier(currentSpecifier.commandIndex, currentSpecifierSource);
			for(int j = 0; j < currentSpecifier.playerParams.size(); j++)
			{
				currentGsSpecifier.playerParams.add(currentSpecifier.playerParams.get(j).playerIndex);
			}
			for(int j = 0; j < currentSpecifier.playCardParams.size(); j++)
			{
				currentGsSpecifier.playCardParams.add(currentSpecifier.playCardParams.get(j).cardNumber);
			}
			for(int j = 0; j < currentSpecifier.stringParams.size(); j++)
			{
				currentGsSpecifier.stringParams.add(currentSpecifier.stringParams.get(j));
			}
			returnValue.add(currentGsSpecifier);
		}
		return returnValue;
	}

	private void loadBascomQueueFromGs(ArrayList<CN_GameState.BascomSpecifier> bascomQueueGs) throws CN_FatalException
	{
		bascomQueue.clear();

		CN_BascomSpecifier currentSpecifier;
		CN_GameState.BascomSpecifier currentGsSpecifier;
		CN_PlayCard currentSpecifierSource;
		for(int i = 0; i < bascomQueueGs.size(); i++)
		{
			currentGsSpecifier = bascomQueueGs.get(i);
			if(currentGsSpecifier.source == CN_Constants.NULL_BASCOM_SOURCE)
			{
				currentSpecifierSource = null;
			}
			else
			{
				currentSpecifierSource = CN_Constants.playCardDirectory.getById(currentGsSpecifier.source);
			}
			currentSpecifier = new CN_BascomSpecifier(currentGsSpecifier.commandIndex, currentSpecifierSource);
			for(int j = 0; j < currentGsSpecifier.playerParams.size(); j++)
			{
				currentSpecifier.playerParams.add(getPlayerByIndex(currentGsSpecifier.playerParams.get(j)));
			}
			for(int j = 0; j < currentGsSpecifier.playCardParams.size(); j++)
			{
				currentSpecifier.playCardParams.add(CN_Constants.playCardDirectory.getById(currentGsSpecifier.playCardParams.get(j)));
			}
			for(int j = 0; j < currentGsSpecifier.stringParams.size(); j++)
			{
				currentSpecifier.stringParams.add(currentGsSpecifier.stringParams.get(j));
			}
			bascomQueue.add(currentSpecifier);
		}
	}


	private Hashtable<String, Integer> getPlayCardParamsHashtableGs()
 	{
		Hashtable<String, Integer> returnValue = new Hashtable<String, Integer>();

		Set<String> keys;
		keys = bascomPlayCardParams.keySet();
		Iterator<String> iter = keys.iterator();

		String currentKey;
		while(iter.hasNext())
		{
			currentKey = iter.next();
			returnValue.put(currentKey, bascomPlayCardParams.get(currentKey).cardNumber);
		}

		return returnValue;
 	}

 	private void loadPlayCardParamsHashtableFromGs(Hashtable<String, Integer> bascomPlayCardParamsGs) throws CN_FatalException
 	{
 		bascomPlayCardParams.clear();

		Set<String> keys;
		keys = bascomPlayCardParamsGs.keySet();
		Iterator<String> iter = keys.iterator();

		String currentKey;
		while(iter.hasNext())
		{
			currentKey = iter.next();
			bascomPlayCardParams.put(currentKey, CN_Constants.playCardDirectory.getById(bascomPlayCardParamsGs.get(currentKey)));
		}
 	}


 	private Hashtable<String, Integer> getPlayerParamsHashtableGs()
 	{
		Hashtable<String, Integer> returnValue = new Hashtable<String, Integer>();

		Set<String> keys;
		keys = bascomPlayerParams.keySet();
		Iterator<String> iter = keys.iterator();

		String currentKey;
		while(iter.hasNext())
		{
			currentKey = iter.next();
			returnValue.put(currentKey, bascomPlayerParams.get(currentKey).playerIndex);
		}

		return returnValue;
 	}

 	private void loadPlayerParamsHashtableFromGs(Hashtable<String, Integer> bascomPlayerParamsGs) throws CN_FatalException
 	{
 		bascomPlayerParams.clear();

		Set<String> keys;
		keys = bascomPlayerParamsGs.keySet();
		Iterator<String> iter = keys.iterator();

		String currentKey;
		while(iter.hasNext())
		{
			currentKey = iter.next();
			bascomPlayerParams.put(currentKey, getPlayerByIndex(bascomPlayerParamsGs.get(currentKey)));
		}
 	}


 	private Hashtable<String, String> getStringParamsHashtableGs()
 	{
		Hashtable<String, String> returnValue = new Hashtable<String, String>();

		Set<String> keys;
		keys = bascomStringParams.keySet();
		Iterator<String> iter = keys.iterator();

		String currentKey;
		while(iter.hasNext())
		{
			currentKey = iter.next();
			returnValue.put(currentKey, bascomStringParams.get(currentKey));
		}

		return returnValue;
 	}

 	private void loadStringParamsHashtableFromGs(Hashtable<String, String> bascomStringParamsGs) throws CN_FatalException
 	{
 		bascomStringParams.clear();

		Set<String> keys;
		keys = bascomStringParamsGs.keySet();
		Iterator<String> iter = keys.iterator();

		String currentKey;
		while(iter.hasNext())
		{
			currentKey = iter.next();
			bascomStringParams.put(currentKey, bascomStringParamsGs.get(currentKey));
		}
 	}




	private CN_GameState generateGameState() throws CN_FatalException
	{
		return generateGameState(false);
	}

	private CN_GameState generateFirstGameState() throws CN_FatalException
	{
		return generateGameState(true);
	}


	//First one contains directories
	private CN_GameState generateGameState(boolean firstGameState) throws CN_FatalException
	{
		Hashtable<Integer, String> playCardDirectoryGs = null;
		Hashtable<Integer, String> timelineCardDirectoryGs = null;
		Hashtable<Integer, String> idCardDirectoryGs = null;
		Hashtable<Integer, String> missionCardDirectoryGs = null;

		if(firstGameState)
		{
			playCardDirectoryGs = CN_Constants.playCardDirectory.toGameStateHashtable();
			timelineCardDirectoryGs = CN_Constants.timelineCardDirectory.toGameStateHashtable();
			idCardDirectoryGs = CN_Constants.idCardDirectory.toGameStateHashtable();
			missionCardDirectoryGs = CN_Constants.missionCardDirectory.toGameStateHashtable();
		}

		Stack<Integer> deckGs = deck.toGameStateStack();
		Stack<Integer> discardsGs = discards.toGameStateStack();
		Stack<Integer> idCardsGs = idCards.toGameStateStack();
		Stack<Integer> idDiscardsGs = idDiscards.toGameStateStack();
		Stack<Integer> missionCardsGs = missionCards.toGameStateStack();
		Stack<Integer> missionDiscardsGs = missionDiscards.toGameStateStack();


		CN_GameState.PlayerData[] playersGs = new CN_GameState.PlayerData[players.length];
		for(int i = 0; i < playersGs.length; i++)
		{
			playersGs[i] = players[i].toPlayerGs();
		}

		CN_GameState.TimelineCardData[] timelineGs = timeline.toGameStateTimeline(patchInQuestion);

		int currentPlayerGs = currentPlayer.playerIndex;
		int previousPlayerGs = currentPlayer.prevPlayer.playerIndex;

		int victoriousPlayerGs;
		if(endOfUniverse)
		{
			victoriousPlayerGs = CN_Constants.END_OF_UNIVERSE;
		}
		else if(victoriousPlayer == null)
		{
			victoriousPlayerGs = CN_Constants.NO_VICTORIOUS_PLAYER;
		}
		else
		{
			victoriousPlayerGs = victoriousPlayer.playerIndex;
		}

		int victoryTypeGs = victoryType;

		int sourceOfMostRecentBascomGs;
		if(sourceOfMostRecentBascom != null)
		{
			sourceOfMostRecentBascomGs = sourceOfMostRecentBascom.cardNumber;
		}
		else
		{
			sourceOfMostRecentBascomGs = CN_Constants.NULL_BASCOM_SOURCE;
		}

		int unpatchedParadoxesGs = timeline.getUnpatchedParadoxes();
		int terminalYearGs = timeline.getTerminalYear();

		CN_Constants.InputState currentInputStateGs = gameInputState;

		String gameInputHashGs = gameInputHash;
		String aiMessageGs = aiMessage;

		int patchInQuestionGs;
		if(patchInQuestion == null)
		{
			patchInQuestionGs = CN_Constants.NULL_PATCH;
		}
		else
		{
			patchInQuestionGs = patchInQuestion.cardNumber;
		}

		int ifCounterGs = ifCounter;

		String inputStateMessageGs = inputStateMessage;

		ArrayList<CN_GameState.BascomSpecifier> bascomQueueGs = getBascomQueueGs();

		Hashtable<String, Integer> bascomPlayCardParamsGs = getPlayCardParamsHashtableGs();
		Hashtable<String, Integer> bascomPlayerParamsGs = getPlayerParamsHashtableGs();
		Hashtable<String, String> bascomStringParamsGs = getStringParamsHashtableGs();

		boolean savedGameStateExistsGs;
		if(savedGameState != null)
		{
			savedGameStateExistsGs = true;
		}
		else
		{
			savedGameStateExistsGs = false;
		}

		ArrayList<CN_Constants.InputConstraint> inputConstraintsGs = new ArrayList<CN_Constants.InputConstraint>();
		for(int i = 0; i < inputConstraints.size(); i++)
		{
			inputConstraintsGs.add(inputConstraints.get(i));
		}

		return new CN_GameState(deckGs, discardsGs, idCardsGs, idDiscardsGs, missionCardsGs, missionDiscardsGs, playCardDirectoryGs, timelineCardDirectoryGs, idCardDirectoryGs, missionCardDirectoryGs, playersGs, timelineGs, currentPlayerGs, previousPlayerGs, victoriousPlayerGs, victoryTypeGs, sourceOfMostRecentBascomGs, unpatchedParadoxesGs, terminalYearGs, currentInputStateGs, gameInputHashGs, patchInQuestionGs, ifCounterGs, inputStateMessageGs, aiMessageGs, bascomQueueGs, bascomPlayCardParamsGs, bascomPlayerParamsGs, bascomStringParamsGs, inputConstraintsGs, savedGameStateExistsGs);
	}


	private void saveGameState(boolean addBascom) throws CN_FatalException
	{
		if(addBascom)
		{
			CN_BascomSpecifier saveGameStateSpecifier = new CN_BascomSpecifier(CN_Constants.BASCOM_SAVE_GAME_STATE, sourceOfMostRecentBascom);
			bascomQueue.addFirst(saveGameStateSpecifier);
			pendingGameState = generateGameState();
			bascomQueue.removeFirst();
		}
		else
		{
			pendingGameState = generateGameState();
		}
	}


	private void loadGameState(CN_GameState gameState) throws CN_FatalException
	{
		if(gameState == null)
		{
			throw new CN_FatalException("CN_Engine, loadGameState(): gameState was null!");
		}

		clearGame();

		for(CN_GameState.PlayerData playerData : gameState.players)
		{
			players[playerData.playerIndex].addListOfHandCards(getActualCards(playerData.hand));
			players[playerData.playerIndex].addListOfArtifacts(getActualCards(playerData.artifacts));
			players[playerData.playerIndex].loadGsBackedUpHands(playerData.backedUpHands);
			players[playerData.playerIndex].setId(CN_Constants.idCardDirectory.getById(playerData.id));
			players[playerData.playerIndex].setMission(CN_Constants.missionCardDirectory.getById(playerData.mission));
		}

		deck.addFromStack(getActualPlayCardStack(gameState.deck));
		discards.addFromStack(getActualPlayCardStack(gameState.discards));
		idCards.addFromStack(getActualIdCardStack(gameState.idCards));
		idDiscards.addFromStack(getActualIdCardStack(gameState.idDiscards));
		missionCards.addFromStack(getActualMissionCardStack(gameState.missionCards));
		missionDiscards.addFromStack(getActualMissionCardStack(gameState.missionDiscards));

		timeline.fromGameStateTimeline(gameState.timeline);

		currentPlayer = getPlayerByIndex(gameState.currentPlayer);
		sourceOfMostRecentBascom = CN_Constants.playCardDirectory.getById(gameState.sourceOfMostRecentBascom);

		gameInputState = gameState.currentInputState;

		gameInputHash = gameState.gameInputHash;
		patchInQuestion = CN_Constants.playCardDirectory.getById(gameState.patchInQuestion);
		ifCounter = gameState.ifCounter;

		loadBascomQueueFromGs(gameState.bascomQueue);
		loadPlayCardParamsHashtableFromGs(gameState.bascomPlayCardParams);
		loadPlayerParamsHashtableFromGs(gameState.bascomPlayerParams);
		loadStringParamsHashtableFromGs(gameState.bascomStringParams);

		for(int i = 0; i < gameState.inputConstraints.size(); i++)
		{
			inputConstraints.add(gameState.inputConstraints.get(i));
		}

		pendingGameState = gameState;
	}



	private void clearGame() throws CN_FatalException
	{
		for(CN_Player player : players)
		{
			player.clearAllHands();
			player.clearArtifacts();
		}

		timeline.resetTimeline();

		deck.clear();
		discards.clear();
		idCards.clear();
		idDiscards.clear();
		missionCards.clear();
		missionDiscards.clear();

		bascomQueue.clear();
		bascomPlayerParams.clear();
		bascomPlayCardParams.clear();
		bascomStringParams.clear();

		inputConstraints.clear();

		savedGameState = null;
		pendingGameState = null;
	}




	private ArrayList<CN_PlayCard> getActualCards(ArrayList<Integer> cards) throws CN_FatalException
	{
		ArrayList<CN_PlayCard> returnValue = new ArrayList<CN_PlayCard>();
		for(int i = 0; i < cards.size(); i++)
		{
			returnValue.add(CN_Constants.playCardDirectory.getById(cards.get(i)));
		}
		return returnValue;
	}



	private Stack<CN_PlayCard> getActualPlayCardStack(Stack<Integer> stack) throws CN_FatalException
	{
		Stack<CN_PlayCard> returnValue = new Stack<CN_PlayCard>();
		Stack<Integer> invertedStack = new Stack<Integer>();

		while(!stack.empty())
		{
			invertedStack.push(stack.pop());
		}

		Integer currentInteger;
		while(!invertedStack.empty())
		{
			currentInteger = invertedStack.pop();
			stack.push(currentInteger);
			returnValue.push(CN_Constants.playCardDirectory.getById(currentInteger));
		}

		return returnValue;
	}


	private Stack<CN_IdCard> getActualIdCardStack(Stack<Integer> stack) throws CN_FatalException
	{
		Stack<CN_IdCard> returnValue = new Stack<CN_IdCard>();
		Stack<Integer> invertedStack = new Stack<Integer>();

		while(!stack.empty())
		{
			invertedStack.push(stack.pop());
		}

		Integer currentInteger;
		while(!invertedStack.empty())
		{
			currentInteger = invertedStack.pop();
			stack.push(currentInteger);
			returnValue.push(CN_Constants.idCardDirectory.getById(currentInteger));
		}

		return returnValue;
	}


	private Stack<CN_MissionCard> getActualMissionCardStack(Stack<Integer> stack) throws CN_FatalException
	{
		Stack<CN_MissionCard> returnValue = new Stack<CN_MissionCard>();
		Stack<Integer> invertedStack = new Stack<Integer>();

		while(!stack.empty())
		{
			invertedStack.push(stack.pop());
		}

		Integer currentInteger;
		while(!invertedStack.empty())
		{
			currentInteger = invertedStack.pop();
			stack.push(currentInteger);
			returnValue.push(CN_Constants.missionCardDirectory.getById(currentInteger));
		}

		return returnValue;
	}


	private CN_Player getPlayerByIndex(int playerIndex) throws CN_FatalException
	{
		for(CN_Player player : players)
		{
			if(player.playerIndex == playerIndex)
			{
				return player;
			}
		}
		throw new CN_FatalException("CN_Engine, getPlayerByIndex(): Index \"" + String.valueOf(playerIndex) + "\" is an invalid index!");
	}




	private void debugPrintBasicCommandString(CN_BascomSpecifier command)
	{
		System.out.println(getBasicCommandString(command) + "  ---  " + String.valueOf(command.commandIndex));
	}

	private String getBasicCommandString(CN_BascomSpecifier command)
	{
		String returnValue;

		switch(command.commandIndex)
		{
			case CN_Constants.BASCOM_IDENT_CONDITIONAL:
				returnValue = CN_Constants.INIT_IDENT_CONDITIONAL;
				break;
			case CN_Constants.BASCOM_ELSE:
				returnValue = CN_Constants.INIT_ELSE;
				break;
			case CN_Constants.BASCOM_END_IF:
				returnValue = CN_Constants.INIT_END_IF;
				break;
			case CN_Constants.BASCOM_TRANSFER_ARTIFACT:
				returnValue = CN_Constants.INIT_TRANSFER_ARTIFACT;
				break;
			case CN_Constants.BASCOM_PLAY_ARTIFACT:
				returnValue = CN_Constants.INIT_PLAY_ARTIFACT;
				break;
			case CN_Constants.BASCOM_DISCARD_ARTIFACT:
				returnValue = CN_Constants.INIT_DISCARD_ARTIFACT;
				break;
			case CN_Constants.BASCOM_DISCARD_CARD:
				returnValue = CN_Constants.INIT_DISCARD_CARD;
				break;
			case CN_Constants.BASCOM_DRAW:
				returnValue = CN_Constants.INIT_DRAW;
				break;
			case CN_Constants.BASCOM_TRANSFER_DECK_TO_DISCARDS:
				returnValue = CN_Constants.INIT_TRANSFER_DECK_TO_DISCARDS;
				break;
			case CN_Constants.BASCOM_TRANSFER_DECK_TO_ARTIFACTS:
				returnValue = CN_Constants.INIT_TRANSFER_DECK_TO_ARTIFACTS;
				break;
			case CN_Constants.BASCOM_TRANSFER_DECK_TO_HAND:
				returnValue = CN_Constants.INIT_TRANSFER_DECK_TO_HAND;
				break;
			case CN_Constants.BASCOM_MOVE_DISCARD_TO_TOP:
				returnValue = CN_Constants.INIT_MOVE_DISCARD_TO_TOP;
				break;
			case CN_Constants.BASCOM_TRANSFER_DISCARDS_TO_ARTIFACTS:
				returnValue = CN_Constants.INIT_TRANSFER_DISCARDS_TO_ARTIFACTS;
				break;
			case CN_Constants.BASCOM_TRANSFER_DISCARDS_TO_HAND:
				returnValue = CN_Constants.INIT_TRANSFER_DISCARDS_TO_HAND;
				break;
			case CN_Constants.BASCOM_EXECUTE_CARD:
				returnValue = CN_Constants.INIT_EXECUTE_CARD;
				break;
			case CN_Constants.BASCOM_BACK_UP_HAND:
				returnValue = CN_Constants.INIT_BACK_UP_HAND;
				break;
			case CN_Constants.BASCOM_RESTORE_BACKED_UP_HAND:
				returnValue = CN_Constants.INIT_RESTORE_BACKED_UP_HAND;
				break;
			case CN_Constants.BASCOM_FLIP_LINCHPIN:
				returnValue = CN_Constants.INIT_FLIP_LINCHPIN;
				break;
			case CN_Constants.BASCOM_TRANSFER_HAND_TO_RIPPLEPOINT:
				returnValue = CN_Constants.INIT_TRANSFER_HAND_TO_RIPPLEPOINT;
				break;
			case CN_Constants.BASCOM_DO_VORTEX:
				returnValue = CN_Constants.INIT_DO_VORTEX;
				break;
			case CN_Constants.BASCOM_PASS_HANDS:
				returnValue = CN_Constants.INIT_PASS_HANDS;
				break;
			case CN_Constants.BASCOM_NEW_ID:
				returnValue = CN_Constants.INIT_NEW_ID;
				break;
			case CN_Constants.BASCOM_SWITCH_IDS:
				returnValue = CN_Constants.INIT_SWITCH_IDS;
				break;
			case CN_Constants.BASCOM_NEW_MISSION:
				returnValue = CN_Constants.INIT_NEW_MISSION;
				break;
			case CN_Constants.BASCOM_SWITCH_MISSIONS:
				returnValue = CN_Constants.INIT_SWITCH_MISSIONS;
				break;
			case CN_Constants.BASCOM_SAVE_GAME_STATE:
				returnValue = CN_Constants.INIT_SAVE_GAME_STATE;
				break;
			case CN_Constants.BASCOM_LOAD_GAME_STATE:
				returnValue = CN_Constants.INIT_LOAD_GAME_STATE;
				break;
			case CN_Constants.BASCOM_CLEAR_GAME_STATE:
				returnValue = CN_Constants.INIT_CLEAR_GAME_STATE;
				break;
			case CN_Constants.BASCOM_RESHUFFLE_DECK:
				returnValue = CN_Constants.INIT_RESHUFFLE_DECK;
				break;
			case CN_Constants.BASCOM_RESHUFFLE_IDS:
				returnValue = CN_Constants.INIT_RESHUFFLE_IDS;
				break;
			case CN_Constants.BASCOM_RESHUFFLE_MISSIONS:
				returnValue = CN_Constants.INIT_RESHUFFLE_MISSIONS;
				break;
			case CN_Constants.BASCOM_GET_ARTIFACT_SELECTION:
				returnValue = CN_Constants.INIT_GET_ARTIFACT_SELECTION;
				break;
			case CN_Constants.BASCOM_GET_HAND_SELECTION:
				returnValue = CN_Constants.INIT_GET_HAND_SELECTION;
				break;
			case CN_Constants.BASCOM_GET_LINCHPIN_SELECTION:
				returnValue = CN_Constants.INIT_GET_LINCHPIN_SELECTION;
				break;
			case CN_Constants.BASCOM_GET_RIPPLEPOINT_SELECTION:
				returnValue = CN_Constants.INIT_GET_RIPPLEPOINT_SELECTION;
				break;
			case CN_Constants.BASCOM_GET_ID_SELECTION:
				returnValue = CN_Constants.INIT_GET_ID_SELECTION;
				break;
			case CN_Constants.BASCOM_GET_MISSION_SELECTION:
				returnValue = CN_Constants.INIT_GET_MISSION_SELECTION;
				break;
			case CN_Constants.BASCOM_GET_CARD_PLAY:
				returnValue = CN_Constants.INIT_GET_CARD_PLAY;
				break;
			case CN_Constants.BASCOM_GET_DIRECTION_SELECTION:
				returnValue = CN_Constants.INIT_GET_DIRECTION_SELECTION;
				break;
			case CN_Constants.BASCOM_GET_DECK_SELECTION:
				returnValue = CN_Constants.INIT_GET_DECK_SELECTION;
				break;
			case CN_Constants.BASCOM_GET_DISCARDS_SELECTION:
				returnValue = CN_Constants.INIT_GET_DISCARDS_SELECTION;
				break;
			case CN_Constants.BASCOM_END_TURN:
				returnValue = CN_Constants.INIT_END_TURN;
				break;
			case CN_Constants.BASCOM_SKIP_QUEUE_ITEMS:
				returnValue = CN_Constants.INIT_SKIP_QUEUE_ITEMS;
				break;
			case CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL:
				returnValue = CN_Constants.INIT_SPECIAL_LOAD_PARAMETER_LITERAL;
				break;
			case CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_HASH:
				returnValue = CN_Constants.INIT_SPECIAL_LOAD_PARAMETER_HASH;
				break;
			case CN_Constants.BASCOM_SPECIAL_REMOVE_PARAMETER:
				returnValue = CN_Constants.INIT_SPECIAL_REMOVE_PARAMETER;
				break;
			default:
				returnValue = "__UnrecognizedBascom";
		}

		return returnValue;
	}



	public void debugPrintBascomQueue()
	{
		System.out.print("\n");
		System.out.println("BASCOM QUEUE:");
		for(CN_BascomSpecifier specifier : bascomQueue)
		{
			debugPrintBasicCommandString(specifier);
		}
		System.out.print("\n");
	}



}
