package cnpc.engine;

import java.util.*;

public class CN_GameState
{
	/*
	 * This class must record:
	 *
	 * All player information, including ID, Mission, Hand, and Artifacts
	 * Deck and Discards information
	 * All timeline information
	 *
	 * Card Directory will be identical to normal card directory except with strings.
	 * Other things (deck, hands, etc.) will use cardNumbers
	 *
	 */

	public final Stack<Integer> deck;
	public final Stack<Integer> discards;
	public final Stack<Integer> idCards;
	public final Stack<Integer> idDiscards;
	public final Stack<Integer> missionCards;
	public final Stack<Integer> missionDiscards;

	public final Hashtable<Integer, String> playCardDirectory;
	public final Hashtable<Integer, String> timelineCardDirectory;
	public final Hashtable<Integer, String> idCardDirectory;
	public final Hashtable<Integer, String> missionCardDirectory;

	public final PlayerData[] players;
	public final TimelineCardData[] timeline;

	public final int currentPlayer;
	public final int previousPlayer;
	public final int victoriousPlayer;
	
	public final int victoryType;

	public final int sourceOfMostRecentBascom;

	public final int unpatchedParadoxes;
	public final int terminalYear;

	public final CN_Constants.InputState currentInputState;

	public final String gameInputHash;
	public final int patchInQuestion;
	public final int ifCounter;

	public final String inputStateMessage;
	public final String aiMessage;

	public final ArrayList<BascomSpecifier> bascomQueue;

	public final Hashtable<String, Integer> bascomPlayCardParams;
	public final Hashtable<String, Integer> bascomPlayerParams;
	public final Hashtable<String, String> bascomStringParams;

	public final ArrayList<CN_Constants.InputConstraint> inputConstraints;

	public final boolean savedGameStateExists;


	public void debugPrint()
	{
		System.out.print("DECK\n");
		debugPrintStack(deck, playCardDirectory);
		System.out.print("\nDISCARDS\n");
		debugPrintStack(discards, playCardDirectory);
		System.out.print("\nID CARDS\n");
		debugPrintStack(idCards, idCardDirectory);
		System.out.print("\nID DISCARDS\n");
		debugPrintStack(idDiscards, idCardDirectory);
		System.out.print("\nMISSION CARDS\n");
		debugPrintStack(missionCards, missionCardDirectory);
		System.out.print("\nMISSION DISCARDS\n");
		debugPrintStack(missionDiscards, missionCardDirectory);

		System.out.print("\nPLAYERS:\n");
		for(PlayerData player : players)
		{
			player.debugPrint(playCardDirectory, idCardDirectory, missionCardDirectory);
			System.out.print("\n");
		}

		System.out.print("\nTIMELINE:\n");
		for(TimelineCardData timelineCard : timeline)
		{
			timelineCard.debugPrint(playCardDirectory, timelineCardDirectory);
			System.out.print("\n");
		}

	}

	private static void debugPrintStack(Stack<Integer> stack, Hashtable<Integer, String> directory)
	{
		Integer currentValue;
		for(int i = 0; i < stack.size(); i++)
		{
			currentValue = stack.get(i);
			System.out.print(currentValue.toString() + "  -  " + directory.get(currentValue) + "\n");
		}
	}

	private static void debugPrintList(ArrayList<Integer> list, Hashtable<Integer, String> directory)
	{
		Integer currentValue;
		for(int i = 0; i < list.size(); i++)
		{
			currentValue = list.get(i);
			System.out.print(currentValue.toString() + "  -  " + directory.get(currentValue) + "\n");
		}
	}










	public CN_GameState(Stack<Integer> deck_in, Stack<Integer> discards_in, Stack<Integer> idCards_in, Stack<Integer> idDiscards_in, Stack<Integer> missionCards_in, Stack<Integer> missionDiscards_in, Hashtable<Integer, String> playCardDirectory_in, Hashtable<Integer, String> timelineCardDirectory_in, Hashtable<Integer, String> idCardDirectory_in, Hashtable<Integer, String> missionCardDirectory_in, PlayerData[] players_in, TimelineCardData[] timeline_in, int currentPlayer_in, int previousPlayer_in, int victoriousPlayer_in, int victoryType_in, int sourceOfMostRecentBascom_in, int unpatchedParadoxes_in, int terminalYear_in, CN_Constants.InputState currentInputState_in, String gameInputHash_in, int patchInQuestion_in, int ifCounter_in, String inputStateMessage_in, String aiMessage_in, ArrayList<BascomSpecifier> bascomQueue_in, Hashtable<String, Integer> bascomPlayCardParams_in, Hashtable<String, Integer> bascomPlayerParams_in, Hashtable<String, String> bascomStringParams_in, ArrayList<CN_Constants.InputConstraint> inputConstraints_in, boolean savedGameStateExists_in)
	{
		deck = deck_in;
		discards = discards_in;
		idCards = idCards_in;
		idDiscards = idDiscards_in;
		missionCards = missionCards_in;
		missionDiscards = missionDiscards_in;

		playCardDirectory = playCardDirectory_in;
		timelineCardDirectory = timelineCardDirectory_in;
		idCardDirectory = idCardDirectory_in;
		missionCardDirectory = missionCardDirectory_in;

		players = players_in;
		timeline = timeline_in;

		currentPlayer = currentPlayer_in;
		previousPlayer = previousPlayer_in;
		victoriousPlayer = victoriousPlayer_in;
		
		victoryType = victoryType_in;

		sourceOfMostRecentBascom = sourceOfMostRecentBascom_in;

		unpatchedParadoxes = unpatchedParadoxes_in;
		terminalYear = terminalYear_in;

		currentInputState = currentInputState_in;

		gameInputHash = gameInputHash_in;
		patchInQuestion = patchInQuestion_in;
		ifCounter = ifCounter_in;

		inputStateMessage = inputStateMessage_in;
		aiMessage = aiMessage_in;

		bascomQueue = bascomQueue_in;

		bascomPlayCardParams = bascomPlayCardParams_in;
		bascomPlayerParams = bascomPlayerParams_in;
		bascomStringParams = bascomStringParams_in;

		inputConstraints = inputConstraints_in;

		savedGameStateExists = savedGameStateExists_in;
	}


	public static class TimelineCardData
	{
		public final int cardNumber;
		public final boolean isFlipped;
		public final int patchedBy;
		public final boolean isTerminal;
		public final boolean isLegal;
		public final boolean isUnstable;
		public final int year;

		public TimelineCardData(int cardNumber_in, boolean isFlipped_in, int patchedBy_in, boolean isTerminal_in, boolean isLegal_in, boolean isUnstable_in, int year_in)
		{
			cardNumber = cardNumber_in;
			isFlipped = isFlipped_in;
			patchedBy = patchedBy_in;
			isTerminal = isTerminal_in;
			isLegal = isLegal_in;
			isUnstable = isUnstable_in;
			year = year_in;
		}


		public void debugPrint(Hashtable<Integer, String> playCardDirectory_in, Hashtable<Integer, String> timelineCardDirectory_in)
		{
			System.out.print("CardNumber: " + new Integer(cardNumber).toString() + " " + timelineCardDirectory_in.get(cardNumber) + "\n");
			System.out.print("isFlipped: " + new Boolean(isFlipped).toString() + "\n");
			System.out.print("patchedBy: " + new Integer(patchedBy).toString() + " " + playCardDirectory_in.get(patchedBy) + "\n");
		}
	}



	public static class PlayerData
	{
		public final int playerIndex;

		public final int id;
		public final int mission;

		public final int nextPlayer;
		public final int prevPlayer;

		public final ArrayList<Integer> hand;
		public final ArrayList<Integer> artifacts;

		public final Stack<ArrayList<Integer>> backedUpHands;


		public PlayerData(int playerIndex_in, int id_in, int mission_in, int nextPlayer_in, int prevPlayer_in, ArrayList<Integer> hand_in, ArrayList<Integer> artifacts_in, Stack<ArrayList<Integer>> backedUpHands_in)
		{
			playerIndex = playerIndex_in;
			id = id_in;
			mission = mission_in;
			nextPlayer = nextPlayer_in;
			prevPlayer = prevPlayer_in;
			hand = hand_in;
			artifacts = artifacts_in;
			backedUpHands = backedUpHands_in;
		}

		public void debugPrint(Hashtable<Integer, String> playCardDirectory_in, Hashtable<Integer, String> idCardDirectory_in, Hashtable<Integer, String> missionCardDirectory_in)
		{
			System.out.print("Player Index: " + new Integer(playerIndex).toString() + "\n");
			System.out.print("id: " + new Integer(id).toString() + " " + idCardDirectory_in.get(id) + "\n");
			System.out.print("mission: " + new Integer(mission).toString() + " " + missionCardDirectory_in.get(mission) + "\n");
			System.out.print("nextPlayer: " + new Integer(nextPlayer).toString() + "\n");
			System.out.print("prevPlayer: " + new Integer(prevPlayer).toString() + "\n");
			System.out.print("Hand:\n");
			debugPrintList(hand, playCardDirectory_in);
			System.out.print("Artifacts:\n");
			debugPrintList(artifacts, playCardDirectory_in);
		}

	}


	public static class BascomSpecifier
	{
		public final int commandIndex;
		public final int source;

		public final ArrayList<Integer> playerParams;
		public final ArrayList<Integer> playCardParams;
		public final ArrayList<String> stringParams;

		public BascomSpecifier(int commandIndex_in, int source_in)
		{
			commandIndex = commandIndex_in;
			source = source_in;

			playerParams = new ArrayList<Integer>();
			playCardParams = new ArrayList<Integer>();
			stringParams = new ArrayList<String>();
		}


	}



}
