package cnpc.engine;

//TODO: make sure identity's are all unique
//TODO: make sure only one timeline card of a particular identity can exist
//TODO: clean up, use constants for paths and stuff
//TODO: schema validation
import java.util.*;

import javax.xml.parsers.*;
import org.w3c.dom.*;
import java.io.*;

class CN_Initializer
{
	private String pathToData = null;

	int timelineCardIndexer;
	int playCardIndexer;
	int idCardIndexer;
	int missionCardIndexer;
	Hashtable<String, CN_Linchpin> tempLinchpinTable;

	public CN_Initializer()
	{
		resetIndexers();
		tempLinchpinTable = new Hashtable<String, CN_Linchpin>();
	}

	public void setVariant(String variant)
	{
		pathToData = CN_Constants.PATH_TO_DATA + variant + CN_Constants.PATH_TO_ENGINE_DATA;
	}


	public void resetIndexers()
	{
		timelineCardIndexer = 1;
		playCardIndexer = 1;
		idCardIndexer = 1;
		missionCardIndexer = 1;
	}





	public CN_CardStack<CN_MissionCard> loadMissions() throws CN_FatalException
	{
		CN_CardStack<CN_MissionCard> loadedMissions = new CN_CardStack<CN_MissionCard>();

		DocumentBuilderFactory factory;
		DocumentBuilder builder;
		Document document;
		NodeList list;
		NodeList childList;
		Node currentNode;
		Node childNode;
		int amountInDeck;

		try
		{
			factory = DocumentBuilderFactory.newInstance();
			builder = factory.newDocumentBuilder();
			document = builder.parse(new File(pathToData + CN_Constants.MISSION_CARDS_PATH));
		}
		catch(Exception e)
		{
			throw new CN_FatalException("CN_Initializer, loadMissions(): Exception in XML file loading and parsing; see cause for details!", e);
		}

		list = document.getElementsByTagName("MissionCard");
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			amountInDeck = getAmountInDeck(currentNode);
			for(int j = 0; j < amountInDeck; j++)
			{
				loadedMissions.push(createMissionCard(loadMissionCard(currentNode)));
			}
		}

		return loadedMissions;
	}








	private CN_MissionCard createMissionCard(LoadedMissionCard card) throws CN_FatalException
	{
		CN_MissionCard returnValue = new CN_MissionCard(missionCardIndexer, card.identity, card.victoryOptionTargets, card.victoryOptionTargetTypes, card.victoryOptionRequiredAmounts);
		CN_Constants.missionCardDirectory.put(missionCardIndexer, returnValue);
		missionCardIndexer++;
		return returnValue;
	}




	private LoadedMissionCard loadMissionCard(Node missionCardNode) throws CN_FatalException
	{
		NodeList list = missionCardNode.getChildNodes();
		Node currentNode;

		String newIdentity = null;
		ArrayList<ArrayList<String>> newVictoryOptionTargets = new ArrayList<ArrayList<String>>();
		ArrayList<ArrayList<CN_MissionCard.TargetType>> newVictoryOptionTargetTypes = new ArrayList<ArrayList<CN_MissionCard.TargetType>>();
		ArrayList<ArrayList<Integer>> newVictoryOptionRequiredAmounts = new ArrayList<ArrayList<Integer>>();

		int newAmountInDeck = 0;

		MissionCard_LoadedVictoryOption currentVictoryOption;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("identity"))
			{
				newIdentity = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("cardname"))
			{
			}
			else if(currentNode.getNodeName().toLowerCase().equals("victoryoption"))
			{
				currentVictoryOption = missionCard_loadVictoryOption(currentNode);
				newVictoryOptionTargets.add(currentVictoryOption.targets);
				newVictoryOptionTargetTypes.add(currentVictoryOption.targetTypes);
				newVictoryOptionRequiredAmounts.add(currentVictoryOption.requiredAmounts);
			}
			else if(currentNode.getNodeName().toLowerCase().equals("amountindeck"))
			{
				newAmountInDeck = Integer.parseInt(currentNode.getTextContent());
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new CN_FatalException("CN_Initializer, loadMissionCard(): Invalid node name!  Name: " + currentNode.getNodeName());
			}
		}

		return new LoadedMissionCard(newIdentity, newVictoryOptionTargets, newVictoryOptionTargetTypes, newVictoryOptionRequiredAmounts, newAmountInDeck);
	}




	private MissionCard_LoadedVictoryOption missionCard_loadVictoryOption(Node victoryOptionNode) throws CN_FatalException
	{
		ArrayList<String> targets = new ArrayList<String>();
		ArrayList<CN_MissionCard.TargetType> targetTypes = new ArrayList<CN_MissionCard.TargetType>();
		ArrayList<Integer> requiredAmounts = new ArrayList<Integer>();

		NodeList list = victoryOptionNode.getChildNodes();
		Node currentNode;
		MissionCard_LoadedPrereqTriple currentPrereq;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("prereq"))
			{
				currentPrereq = missionCard_loadPrereq(currentNode);
				targets.add(currentPrereq.target);
				targetTypes.add(currentPrereq.targetType);
				requiredAmounts.add(currentPrereq.requiredAmount);
			}
		}

		return new MissionCard_LoadedVictoryOption(targets, targetTypes, requiredAmounts);
	}




	private MissionCard_LoadedPrereqTriple missionCard_loadPrereq(Node prereqNode) throws CN_FatalException
	{
		String newTarget = null;
		String newTargetType = null;
		String newRequiredAmount = null;

		NodeList list = prereqNode.getChildNodes();
		Node currentNode;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("target"))
			{
				newTarget = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("targettype"))
			{
				newTargetType = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("requiredamount"))
			{
				newRequiredAmount = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new CN_FatalException("CN_Initializer, missionCard_loadPrereq(): Invalid node name!");
			}
		}

		if(!newTargetType.toLowerCase().equals(CN_Constants.EXACT_TARGET_TYPE.toLowerCase()))
		{
			Enumeration<CN_PlayCard> allCards = CN_Constants.playCardDirectory.getEnumeration();

			boolean validated = false;

			CN_PlayCard currentCard;
			while(allCards.hasMoreElements())
			{
				currentCard = allCards.nextElement();
				if(currentCard.cardType == CN_PlayCard.PlayCardType.ARTIFACT)
				{
					if(currentCard.authClass.equals(newTarget))
					{
						validated = true;
					}
				}
			}
			if(!validated)
			{
				throw new CN_FatalException("CN_Initializer, missionCard_loadPrereq(): artifact with authClass " + newTarget + " does not exist!");
			}
		}
		else
		{
			CN_PlayCard validatingPlayCard = CN_Constants.playCardDirectory.getAnInstance(newTarget);
			if(validatingPlayCard == null)
			{
				throw new CN_FatalException("CN_Initializer, missionCard_loadPrereq(): newTarget artifact " + newTarget + " does not exist!");
			}
			else if(validatingPlayCard.cardType != CN_PlayCard.PlayCardType.ARTIFACT)
			{
				throw new CN_FatalException("CN_Initializer, missionCard_loadPrereq(): PatchedBy " + newTarget + " is not an artifact!");
			}
		}

		return new MissionCard_LoadedPrereqTriple(newTarget, newTargetType, newRequiredAmount);
	}








	private class LoadedMissionCard
	{
		public final String identity;
		public final ArrayList<ArrayList<String>> victoryOptionTargets;
		public final ArrayList<ArrayList<CN_MissionCard.TargetType>> victoryOptionTargetTypes;
		public final ArrayList<ArrayList<Integer>> victoryOptionRequiredAmounts;

		public int amountInDeck;

		public LoadedMissionCard(String identity_in, ArrayList<ArrayList<String>> victoryOptionTargets_in, ArrayList<ArrayList<CN_MissionCard.TargetType>> victoryOptionTargetTypes_in, ArrayList<ArrayList<Integer>> victoryOptionRequiredAmounts_in, int amountInDeck_in)
		{
			identity = identity_in;
			victoryOptionTargets = victoryOptionTargets_in;
			victoryOptionTargetTypes = victoryOptionTargetTypes_in;
			victoryOptionRequiredAmounts = victoryOptionRequiredAmounts_in;
			amountInDeck = amountInDeck_in;
		}
	}


	private class MissionCard_LoadedVictoryOption
	{
		public final ArrayList<String> targets;
		public final ArrayList<CN_MissionCard.TargetType> targetTypes;
		public final ArrayList<Integer> requiredAmounts;

		public MissionCard_LoadedVictoryOption(ArrayList<String> targets_in, ArrayList<CN_MissionCard.TargetType> targetTypes_in ,ArrayList<Integer> requiredAmounts_in)
		{
			targets = targets_in;
			targetTypes = targetTypes_in;
			requiredAmounts = requiredAmounts_in;
		}
	}


	private class MissionCard_LoadedPrereqTriple
	{
		public final String target;
		public final CN_MissionCard.TargetType targetType;
		public final int requiredAmount;

		public MissionCard_LoadedPrereqTriple(String target_in, String targetTypeString_in, String requiredAmountString_in) throws CN_FatalException, NumberFormatException
		{
			target = target_in;
			targetType = getTargetTypeFromString(targetTypeString_in);
			requiredAmount = Integer.parseInt(requiredAmountString_in);
		}

		private CN_MissionCard.TargetType getTargetTypeFromString(String targetTypeString) throws CN_FatalException
		{
			if(targetTypeString.toLowerCase().equals("exact"))
			{
				return CN_MissionCard.TargetType.EXACT;
			}
			else if(targetTypeString.toLowerCase().equals("mostauthentic"))
			{
				return CN_MissionCard.TargetType.MOSTAUTHENTIC;
			}
			else if(targetTypeString.toLowerCase().equals("notmostauthentic"))
			{
				return CN_MissionCard.TargetType.NOTMOSTAUTHENTIC;
			}
			else
			{
				throw new CN_FatalException("CN_Initializer, getTargetTypeFromString(): invalid target type string!");
			}
		}

	}










	public CN_CardStack<CN_IdCard> loadIds() throws CN_FatalException
	{
		CN_CardStack<CN_IdCard> loadedIds = new CN_CardStack<CN_IdCard>();

		DocumentBuilderFactory factory;
		DocumentBuilder builder;
		Document document;
		NodeList list;
		NodeList childList;
		Node currentNode;
		Node childNode;
		int amountInDeck;

		try
		{
			factory = DocumentBuilderFactory.newInstance();
			builder = factory.newDocumentBuilder();
			document = builder.parse(new File(pathToData + CN_Constants.ID_CARDS_PATH));
		}
		catch(Exception e)
		{
			throw new CN_FatalException("CN_Initializer, loadIds(): Exception in XML file loading and parsing; see cause for details!", e);
		}

		list = document.getElementsByTagName("IdCard");
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			amountInDeck = getAmountInDeck(currentNode);
			for(int j = 0; j < amountInDeck; j++)
			{
				loadedIds.push(createIdCard(loadIdCard(currentNode)));
			}
		}

		return loadedIds;
	}




	private CN_IdCard createIdCard(LoadedIdCard card) throws CN_FatalException
	{
		CN_IdCard returnValue = new CN_IdCard(idCardIndexer, card.identity, card.victoryOptions);
		CN_Constants.idCardDirectory.put(idCardIndexer, returnValue);
		idCardIndexer++;
		return returnValue;
	}





	private LoadedIdCard loadIdCard(Node idCardNode) throws CN_FatalException
	{
		NodeList list = idCardNode.getChildNodes();
		Node currentNode;

		String newIdentity = null;
		ArrayList<ArrayList<String>> newVictoryOptions = new ArrayList<ArrayList<String>>();
		int newAmountInDeck = 0;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("identity"))
			{
				newIdentity = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("cardname"))
			{
			}
			else if(currentNode.getNodeName().toLowerCase().equals("victoryoption"))
			{
				newVictoryOptions.add(idCard_loadVictoryOption(currentNode));
			}
			else if(currentNode.getNodeName().toLowerCase().equals("amountindeck"))
			{
				newAmountInDeck = Integer.parseInt(currentNode.getTextContent());
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new CN_FatalException("CN_Initializer, loadIdCard(): Invalid node name!  Name: " + currentNode.getNodeName());
			}
		}

		return new LoadedIdCard(newIdentity, newVictoryOptions, newAmountInDeck);
	}



	private ArrayList<String> idCard_loadVictoryOption(Node victoryOptionNode) throws CN_FatalException
	{
		ArrayList<String> returnValue = new ArrayList<String>();

		NodeList list = victoryOptionNode.getChildNodes();
		Node currentNode;
		IdCard_LoadedPrereqPair currentPrereq;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("prereq"))
			{
				currentPrereq = idCard_loadPrereq(currentNode);
				returnValue.add(currentPrereq.target);
				returnValue.add(currentPrereq.patchedBy);
			}
		}


		return returnValue;
	}



	private IdCard_LoadedPrereqPair idCard_loadPrereq(Node prereqNode) throws CN_FatalException
	{
		String newTarget = null;
		String newPatchedBy = null;

		NodeList list = prereqNode.getChildNodes();
		Node currentNode;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("target"))
			{
				newTarget = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("patchedby"))
			{
				newPatchedBy = currentNode.getTextContent();
				if(newPatchedBy.equals(""))
				{
					newPatchedBy = null;
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new CN_FatalException("CN_Initializer, idCard_loadPrereq(): Invalid node name!");
			}
		}

		if(CN_Constants.timelineCardDirectory.getAnInstance(newTarget) == null)
		{
			throw new CN_FatalException("CN_Initializer, idCard_loadPrereq(): Target ripplepoint " + newTarget + " does not exist!");
		}

		if(newPatchedBy != null)
		{
			CN_PlayCard validatingPlayCard = CN_Constants.playCardDirectory.getAnInstance(newPatchedBy);
			if(validatingPlayCard == null)
			{
				throw new CN_FatalException("CN_Initializer, idCard_loadPrereq(): newPatchedBy patch " + newPatchedBy + " does not exist!");
			}
			else if(validatingPlayCard.cardType != CN_PlayCard.PlayCardType.PATCH)
			{
				throw new CN_FatalException("CN_Initializer, idCard_loadPrereq(): PatchedBy " + newPatchedBy + " is not a patch!");
			}
		}

		return new IdCard_LoadedPrereqPair(newTarget, newPatchedBy);

	}



	private class LoadedIdCard
	{
		public final String identity;
		public final ArrayList<ArrayList<String>> victoryOptions;
		public int amountInDeck;

		public LoadedIdCard(String identity_in, ArrayList<ArrayList<String>> victoryOptions_in, int amountInDeck_in)
		{
			identity = identity_in;
			victoryOptions = victoryOptions_in;
			amountInDeck = amountInDeck_in;
		}
	}



	private class IdCard_LoadedPrereqPair
	{
		public final String target;
		public final String patchedBy;

		public IdCard_LoadedPrereqPair(String target_in, String patchedBy_in)
		{
			target = target_in;
			patchedBy = patchedBy_in;
		}
	}








	public CN_CardStack<CN_PlayCard> loadDeck() throws CN_FatalException
	{
		CN_CardStack<CN_PlayCard> loadedDeck = new CN_CardStack<CN_PlayCard>();

		DocumentBuilderFactory factory;
		DocumentBuilder builder;
		Document document;
		NodeList list;
		NodeList childList;
		Node currentNode;
		Node childNode;
		int amountInDeck;

		try
		{
			factory = DocumentBuilderFactory.newInstance();
			builder = factory.newDocumentBuilder();
			document = builder.parse(new File(pathToData + CN_Constants.PLAY_CARDS_PATH));
		}
		catch(Exception e)
		{
			throw new CN_FatalException("CN_Initializer, loadDeck(): Exception in XML file loading and parsing; see cause for details!", e);
		}

		list = document.getElementsByTagName("PlayCard");
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			amountInDeck = getAmountInDeck(currentNode);
			for(int j = 0; j < amountInDeck; j++)
			{
				loadedDeck.push(createPlayCard(loadPlayCard(currentNode)));
			}
		}

		return loadedDeck;
	}





	private int getAmountInDeck(Node cardNode) throws NumberFormatException
	{
		int returnValue = 0;

		NodeList list = cardNode.getChildNodes();
		Node currentNode;

		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("amountindeck"))
			{
				returnValue = Integer.parseInt(currentNode.getTextContent());
			}
		}
		return returnValue;
	}



	private LoadedPlayCard loadPlayCard(Node playCardNode) throws CN_FatalException
	{
		LoadedPlayCard returnValue = new LoadedPlayCard();

		NodeList list = playCardNode.getChildNodes();
		Node currentNode;

		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("identity"))
			{
				returnValue.identity = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("cardname"))
			{
			}
			else if(currentNode.getNodeName().toLowerCase().equals("cardtype"))
			{
				returnValue.cardType = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("amountindeck"))
			{
				returnValue.amountInDeck = Integer.parseInt(currentNode.getTextContent());
			}
			else if(currentNode.getNodeName().toLowerCase().equals("authenticity"))
			{
				returnValue.authenticity = Integer.parseInt(currentNode.getTextContent());
			}
			else if(currentNode.getNodeName().toLowerCase().equals("authclass"))
			{
				returnValue.authClass = currentNode.getTextContent();
				if(returnValue.authClass.equals(""))
				{
					returnValue.authClass = null;
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("year"))
			{
				try
				{
					returnValue.year = Integer.parseInt(currentNode.getTextContent());
					if(returnValue.year >= CN_Constants.NON_TERMINAL_YEAR)
					{
						throw new CN_FatalException("CN_Initializer, loadPlayCard(): year must be less than " + String.valueOf(CN_Constants.NON_TERMINAL_YEAR) + "!");
					}
					if(returnValue.year <= CN_Constants.EARLY_YEAR)
					{
						throw new CN_FatalException("CN_Initializer, loadPlayCard(): year must be greater than " + String.valueOf(CN_Constants.EARLY_YEAR) + "!");
					}
				}
				catch(NumberFormatException e)
				{
					if(currentNode.getTextContent().toLowerCase().equals(CN_Constants.NON_TERMINAL_YEAR_STRING.toLowerCase()))
					{
						returnValue.year = CN_Constants.NON_TERMINAL_YEAR;
					}
					else if(currentNode.getTextContent().toLowerCase().equals(CN_Constants.EARLY_YEAR_STRING.toLowerCase()))
					{
						returnValue.year = CN_Constants.EARLY_YEAR;
					}
					else
					{
						throw new CN_FatalException("CN_Initializer, loadPlayCard(): Value for Year of \"" + currentNode.getTextContent() + "\" was not understood!");
					}
				}

			}
			else if(currentNode.getNodeName().toLowerCase().equals("unplayableconditions"))
			{
				NodeList subNodeList = currentNode.getChildNodes();
				Node subCurrentNode;
				for(int j = 0; subNodeList.item(j) != null; j++)
				{
					subCurrentNode = subNodeList.item(j);
					if(subCurrentNode.getNodeName().toLowerCase().equals("unplayablecondition"))
					{
						returnValue.unplayableConditions.add(subCurrentNode.getTextContent());
					}
					else if(subCurrentNode.getNodeName().toLowerCase().equals("#text"))
					{
					}
					else
					{
						throw new CN_FatalException("CN_Initializer, loadPlayCard(): Invalid node name within UnplayableConditions!  Name: " + subCurrentNode.getNodeName());
					}
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("inverterunplayablecondition"))
			{
				NodeList subNodeList = currentNode.getChildNodes();
				Node subCurrentNode;
				for(int j = 0; subNodeList.item(j) != null; j++)
				{
					subCurrentNode = subNodeList.item(j);
					if(subCurrentNode.getNodeName().toLowerCase().equals("conditionpart")) //todo: this seems like a hack?
					{
						returnValue.inverterUnplayableCondition.add(loadUnplayableConditionPart(subCurrentNode));
					}
					else if(subCurrentNode.getNodeName().toLowerCase().equals("#text"))
					{
					}
					else
					{
						throw new CN_FatalException("CN_Initializer, loadPlayCard(): Invalid node name within InverterUnplayableCondition!  Name: " + subCurrentNode.getNodeName());
					}
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("commandsequence"))
			{
				NodeList subNodeList = currentNode.getChildNodes();
				Node subCurrentNode;
				for(int j = 0; subNodeList.item(j) != null; j++)
				{
					subCurrentNode = subNodeList.item(j);
					if(!subCurrentNode.getNodeName().toLowerCase().equals("#text"))
					{
						returnValue.commandSequence.add(loadCommand(subCurrentNode));
					}
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new CN_FatalException("CN_Initializer, loadPlayCard(): Invalid node name!  Name: " + currentNode.getNodeName());
			}

		}
		return returnValue;
	}



	private UnplayableConditionPart loadUnplayableConditionPart(Node unplayableConditionPartNode) throws CN_FatalException
	{
		UnplayableConditionPart returnValue = new UnplayableConditionPart();

		NodeList list = unplayableConditionPartNode.getChildNodes();
		Node currentNode;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("linchpin"))
			{
				returnValue.linchpin = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("unplayableifflipped"))
			{
				if(currentNode.getTextContent().toLowerCase().equals("true"))
				{
					returnValue.unplayableIfFlipped = true;
				}
				else if(currentNode.getTextContent().toLowerCase().equals("false"))
				{
					returnValue.unplayableIfFlipped = false;
				}
				else
				{
					throw new CN_FatalException("CN_Initializer, loadUnplayableConditionPart(): Invalid UnplayableIfFlipped value!  Must be true or false.  Value: " + currentNode.getTextContent());
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new CN_FatalException("CN_Initializer, loadUnplayableConditionPart(): Invalid node name!  Name: " + currentNode.getNodeName());
			}
		}

		return returnValue;
	}






	private Command loadCommand(Node commandNode) throws CN_FatalException
	{
		Command returnValue = new Command();

		if(commandNode.getNodeName().toLowerCase().equals("specialcommand"))
		{
			returnValue.isSpecial = true;
			NodeList list = commandNode.getChildNodes();
			Node currentNode;
			for(int i = 0; list.item(i) != null; i++)
			{
				currentNode = list.item(i);
				if(currentNode.getNodeName().toLowerCase().equals("bascomid"))
				{
					returnValue.bascomId = getBascomId(currentNode.getTextContent());
				}
				else if(currentNode.getNodeName().toLowerCase().equals("param1"))
				{
					returnValue.param1 = currentNode.getTextContent();
				}
				else if(currentNode.getNodeName().toLowerCase().equals("param2"))
				{
					returnValue.param2 = currentNode.getTextContent();
				}
				else if(currentNode.getNodeName().toLowerCase().equals("param3"))
				{
					returnValue.param3 = currentNode.getTextContent();
				}
			}
		}
		else if(commandNode.getNodeName().toLowerCase().equals("command"))
		{
			returnValue.bascomId = getBascomId(commandNode.getTextContent());
			returnValue.isSpecial = false;
		}
		else
		{
			throw new CN_FatalException("CN_Initializer, loadCommand(): invalid name for Command Node!");
		}


		return returnValue;
	}




	private CN_PlayCard createPlayCard(LoadedPlayCard card) throws CN_FatalException
	{
		Command command;
		ArrayList<String> bascomSpecialStrings;
		ArrayList<CN_PlayCard.StoredBascom> bascomList = new ArrayList<CN_PlayCard.StoredBascom>();
		for(int i = 0; i < card.commandSequence.size(); i++)
		{
			bascomSpecialStrings = null;
			command = card.commandSequence.get(i);
			if(command.isSpecial)
			{
				bascomSpecialStrings = new ArrayList<String>();
				if(command.param1 != null)
				{
					bascomSpecialStrings.add(command.param1);
				}
				if(command.param2 != null)
				{
					bascomSpecialStrings.add(command.param2);
				}
				if(command.param3 != null)
				{
					bascomSpecialStrings.add(command.param3);
				}
			}
			bascomList.add(new CN_PlayCard.StoredBascom(command.bascomId, bascomSpecialStrings));
		}

		String unplayableConditionString;
		ArrayList<CN_PlayCard.UnplayableCondition> unplayableConditions = new ArrayList<CN_PlayCard.UnplayableCondition>();
		for(int i = 0; i < card.unplayableConditions.size(); i++)
		{
			unplayableConditionString = card.unplayableConditions.get(i);
			if(unplayableConditionString.equals(CN_Constants.UNPLAYABLE_INIT_PATCH_STANDARD))
			{
				unplayableConditions.add(CN_PlayCard.UnplayableCondition.PATCH_STANDARD);
			}
			else if(unplayableConditionString.equals(CN_Constants.UNPLAYABLE_INIT_INVERTER))
			{
				unplayableConditions.add(CN_PlayCard.UnplayableCondition.INVERTER);
			}
			else if(unplayableConditionString.equals(CN_Constants.UNPLAYABLE_INIT_DISCARDS_EMPTY))
			{
				unplayableConditions.add(CN_PlayCard.UnplayableCondition.DISCARDS_EMPTY);
			}
			else if(unplayableConditionString.equals(CN_Constants.UNPLAYABLE_INIT_NO_NON_TIMEWARP_DISCARDS_PLAYABLE))
			{
				unplayableConditions.add(CN_PlayCard.UnplayableCondition.NO_NON_TIMEWARP_DISCARDS_PLAYABLE);
			}
			else if(unplayableConditionString.equals(CN_Constants.UNPLAYABLE_INIT_ARTIFACTS_NONE))
			{
				unplayableConditions.add(CN_PlayCard.UnplayableCondition.ARTIFACTS_NONE);
			}
			else if(unplayableConditionString.equals(CN_Constants.UNPLAYABLE_INIT_ARTIFACTS_NOT_CURRENT_PLAYER))
			{
				unplayableConditions.add(CN_PlayCard.UnplayableCondition.ARTIFACTS_NOT_CURRENT_PLAYER);
			}
			else if(unplayableConditionString.equals(CN_Constants.UNPLAYABLE_INIT_ARTIFACTS_NOT_OPPONENT))
			{
				unplayableConditions.add(CN_PlayCard.UnplayableCondition.ARTIFACTS_NOT_OPPONENT);
			}
			else
			{
				throw new CN_FatalException("CN_Initializer, createPlayCard(): string \"" + unplayableConditionString + "\" is an invalid unplayable condition string!");
			}
		}

		UnplayableConditionPart currentPart;
		CN_PlayCard.InverterUnplayableConditionPart newPart;
		ArrayList<CN_PlayCard.InverterUnplayableConditionPart> inverterUnplayableCondition = new ArrayList<CN_PlayCard.InverterUnplayableConditionPart>();
		for(int i = 0; i < card.inverterUnplayableCondition.size(); i++)
		{
			currentPart = card.inverterUnplayableCondition.get(i);
			newPart = new CN_PlayCard.InverterUnplayableConditionPart(currentPart.linchpin, currentPart.unplayableIfFlipped);
			inverterUnplayableCondition.add(newPart);
		}



		CN_PlayCard returnValue;
		if(CN_PlayCard.getPlayCardTypeFromString(card.cardType) == CN_PlayCard.PlayCardType.ARTIFACT)
		{
			if(card.authenticity <= 0)
			{
				throw new CN_FatalException("CN_Initializer, createPlayCard: card is an Artifact, must have an Authenticity specified, and it must be greater than 0!");
			}
			else if(card.authClass == null)
			{
				throw new CN_FatalException("CN_Initializer, createPlayCard: card is an Artifact, must have an AuthClass specified!");
			}
			returnValue = new CN_PlayCard(playCardIndexer, card.identity, CN_PlayCard.getPlayCardTypeFromString(card.cardType), card.authenticity, card.authClass, bascomList, unplayableConditions, inverterUnplayableCondition, card.year);
		}
		else
		{
			returnValue = new CN_PlayCard(playCardIndexer, card.identity, CN_PlayCard.getPlayCardTypeFromString(card.cardType), bascomList, unplayableConditions, inverterUnplayableCondition, card.year);
		}
		CN_Constants.playCardDirectory.put(playCardIndexer, returnValue);
		playCardIndexer++;

		return returnValue;
	}




	private class LoadedPlayCard
	{
		public String identity;
		public String cardType;
		public int authenticity;
		public String authClass;
		public int year;
		public int amountInDeck;
		public final ArrayList<Command> commandSequence;
		public final ArrayList<String> unplayableConditions;
		public final ArrayList<UnplayableConditionPart> inverterUnplayableCondition;

		public LoadedPlayCard()
		{
			commandSequence = new ArrayList<Command>();
			unplayableConditions = new ArrayList<String>();
			inverterUnplayableCondition = new ArrayList<UnplayableConditionPart>();
			authenticity = CN_Constants.NONEXISTENT_AUTHENTICITY;
			authClass = null;
		}
	}


	private class Command
	{
		public int bascomId;

		public boolean isSpecial;
		public String param1;
		public String param2;
		public String param3;
	}


	private class UnplayableConditionPart
	{
		public String linchpin;
		public boolean unplayableIfFlipped;
	}







	private int getBascomId(String bascomId) throws CN_FatalException
	{
		int returnValue;

		try
		{
			returnValue = Integer.parseInt(bascomId);
		}
		catch(NumberFormatException e)
		{
			if(bascomId.toLowerCase().equals(CN_Constants.INIT_IDENT_CONDITIONAL.toLowerCase()))
				returnValue = CN_Constants.BASCOM_IDENT_CONDITIONAL;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_ELSE.toLowerCase()))
				returnValue = CN_Constants.BASCOM_ELSE;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_END_IF.toLowerCase()))
				returnValue = CN_Constants.BASCOM_END_IF;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_TRANSFER_ARTIFACT.toLowerCase()))
				returnValue = CN_Constants.BASCOM_TRANSFER_ARTIFACT;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_PLAY_ARTIFACT.toLowerCase()))
				returnValue = CN_Constants.BASCOM_PLAY_ARTIFACT;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_DISCARD_ARTIFACT.toLowerCase()))
				returnValue = CN_Constants.BASCOM_DISCARD_ARTIFACT;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_DISCARD_CARD.toLowerCase()))
				returnValue = CN_Constants.BASCOM_DISCARD_CARD;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_DRAW.toLowerCase()))
				returnValue = CN_Constants.BASCOM_DRAW;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_TRANSFER_DECK_TO_DISCARDS.toLowerCase()))
				returnValue = CN_Constants.BASCOM_TRANSFER_DECK_TO_DISCARDS;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_TRANSFER_DECK_TO_ARTIFACTS.toLowerCase()))
				returnValue = CN_Constants.BASCOM_TRANSFER_DECK_TO_ARTIFACTS;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_TRANSFER_DECK_TO_HAND.toLowerCase()))
				returnValue = CN_Constants.BASCOM_TRANSFER_DECK_TO_HAND;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_MOVE_DISCARD_TO_TOP.toLowerCase()))
				returnValue = CN_Constants.BASCOM_MOVE_DISCARD_TO_TOP;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_TRANSFER_DISCARDS_TO_ARTIFACTS.toLowerCase()))
				returnValue = CN_Constants.BASCOM_TRANSFER_DISCARDS_TO_ARTIFACTS;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_TRANSFER_DISCARDS_TO_HAND.toLowerCase()))
				returnValue = CN_Constants.BASCOM_TRANSFER_DISCARDS_TO_HAND;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_EXECUTE_CARD.toLowerCase()))
				returnValue = CN_Constants.BASCOM_EXECUTE_CARD;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_BACK_UP_HAND.toLowerCase()))
				returnValue = CN_Constants.BASCOM_BACK_UP_HAND;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_RESTORE_BACKED_UP_HAND.toLowerCase()))
				returnValue = CN_Constants.BASCOM_RESTORE_BACKED_UP_HAND;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_FLIP_LINCHPIN.toLowerCase()))
				returnValue = CN_Constants.BASCOM_FLIP_LINCHPIN;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_TRANSFER_HAND_TO_RIPPLEPOINT.toLowerCase()))
				returnValue = CN_Constants.BASCOM_TRANSFER_HAND_TO_RIPPLEPOINT;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_NEW_ID.toLowerCase()))
				returnValue = CN_Constants.BASCOM_NEW_ID;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_DO_VORTEX.toLowerCase()))
				returnValue = CN_Constants.BASCOM_DO_VORTEX;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_PASS_HANDS.toLowerCase()))
				returnValue = CN_Constants.BASCOM_PASS_HANDS;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_SWITCH_IDS.toLowerCase()))
				returnValue = CN_Constants.BASCOM_SWITCH_IDS;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_NEW_MISSION.toLowerCase()))
				returnValue = CN_Constants.BASCOM_NEW_MISSION;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_SWITCH_MISSIONS.toLowerCase()))
				returnValue = CN_Constants.BASCOM_SWITCH_MISSIONS;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_SAVE_GAME_STATE.toLowerCase()))
				returnValue = CN_Constants.BASCOM_SAVE_GAME_STATE;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_LOAD_GAME_STATE.toLowerCase()))
				returnValue = CN_Constants.BASCOM_LOAD_GAME_STATE;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_CLEAR_GAME_STATE.toLowerCase()))
				returnValue = CN_Constants.BASCOM_CLEAR_GAME_STATE;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_RESHUFFLE_DECK.toLowerCase()))
				returnValue = CN_Constants.BASCOM_RESHUFFLE_DECK;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_RESHUFFLE_IDS.toLowerCase()))
				returnValue = CN_Constants.BASCOM_RESHUFFLE_IDS;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_RESHUFFLE_MISSIONS.toLowerCase()))
				returnValue = CN_Constants.BASCOM_RESHUFFLE_MISSIONS;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_GET_ARTIFACT_SELECTION.toLowerCase()))
				returnValue = CN_Constants.BASCOM_GET_ARTIFACT_SELECTION;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_GET_HAND_SELECTION.toLowerCase()))
				returnValue = CN_Constants.BASCOM_GET_HAND_SELECTION;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_GET_LINCHPIN_SELECTION.toLowerCase()))
				returnValue = CN_Constants.BASCOM_GET_LINCHPIN_SELECTION;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_GET_RIPPLEPOINT_SELECTION.toLowerCase()))
				returnValue = CN_Constants.BASCOM_GET_RIPPLEPOINT_SELECTION;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_GET_ID_SELECTION.toLowerCase()))
				returnValue = CN_Constants.BASCOM_GET_ID_SELECTION;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_GET_MISSION_SELECTION.toLowerCase()))
				returnValue = CN_Constants.BASCOM_GET_MISSION_SELECTION;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_GET_DIRECTION_SELECTION.toLowerCase()))
				returnValue = CN_Constants.BASCOM_GET_DIRECTION_SELECTION;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_GET_DECK_SELECTION.toLowerCase()))
				returnValue = CN_Constants.BASCOM_GET_DECK_SELECTION;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_GET_DISCARDS_SELECTION.toLowerCase()))
				returnValue = CN_Constants.BASCOM_GET_DISCARDS_SELECTION;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_GET_CARD_PLAY.toLowerCase()))
				returnValue = CN_Constants.BASCOM_GET_CARD_PLAY;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_END_TURN.toLowerCase()))
				returnValue = CN_Constants.BASCOM_END_TURN;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_SKIP_QUEUE_ITEMS.toLowerCase()))
				returnValue = CN_Constants.BASCOM_SKIP_QUEUE_ITEMS;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_DRAW_REAL.toLowerCase()))
				returnValue = CN_Constants.BASCOM_DRAW_REAL;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_SPECIAL_LOAD_PARAMETER_LITERAL.toLowerCase()))
				returnValue = CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_LITERAL;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_SPECIAL_LOAD_PARAMETER_HASH.toLowerCase()))
				returnValue = CN_Constants.BASCOM_SPECIAL_LOAD_PARAMETER_HASH;
			else if(bascomId.toLowerCase().equals(CN_Constants.INIT_SPECIAL_REMOVE_PARAMETER.toLowerCase()))
				returnValue = CN_Constants.BASCOM_SPECIAL_REMOVE_PARAMETER;
			else
				throw new CN_FatalException("CN_Initializer, getBascomId: unrecognized string bascom id \""+ bascomId +"\"!");
		}

		return returnValue;

	}









	public CN_Timeline loadTimeline() throws CN_FatalException //todo: better exception handling
	{
		ArrayList<LoadedTimelineCard> timeline = new ArrayList<LoadedTimelineCard>();

		DocumentBuilderFactory factory;
		DocumentBuilder builder;
		Document document;
		NodeList list;
		NodeList childList;
		Node currentNode;
		Node childNode;

		try
		{
			factory = DocumentBuilderFactory.newInstance();
			builder = factory.newDocumentBuilder();
			document = builder.parse(new File(pathToData + CN_Constants.TIMELINE_CARDS_PATH));
		}
		catch(Exception e)
		{
			throw new CN_FatalException("CN_Initializer, loadTimeline(): Exception in XML file loading and parsing; see cause for details!", e);
		}

		list = document.getElementsByTagName("TimelineCard");
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			timeline.add(loadTimelineCard(list.item(i)));
		}

		return constructTimeline(timeline);

	}










	private CN_Timeline constructTimeline(ArrayList<LoadedTimelineCard> loadedTimeline) throws CN_FatalException
	{
		CN_Timeline returnValue;
		CN_TimelineCard[] timeline;

		ArrayList<CN_TimelineCard> tempTimeline = new ArrayList<CN_TimelineCard>();

		CN_TimelineCard newCard = null;
		CN_Ripplepoint currentRipplepoint;

		//create all linchpins first, so they can be linked with ripplepoints later
		for(LoadedTimelineCard card : loadedTimeline)
		{
			if(!card.isRipple)
			{
				newCard = createTimelineCard(card);
				tempLinchpinTable.put(newCard.identity, (CN_Linchpin)newCard);
				tempTimeline.add(newCard);
			}
		}


		for(LoadedTimelineCard card : loadedTimeline)
		{
			if(card.isRipple)
			{
				newCard = createTimelineCard(card);
				currentRipplepoint = (CN_Ripplepoint)newCard;
				for(PatchingOption option : card.patchingOptions)
				{
					addPatchingOption(currentRipplepoint, option);
				}
				for(ForceParadox forceParadox : card.flipStateOverrides)
				{
					currentRipplepoint.addForceParadox(tempLinchpinTable.get(forceParadox.linchpinIdentity), forceParadox.flipStateIsFlipped);
				}
				currentRipplepoint.lockPatchingOptions();
				tempTimeline.add(newCard);
			}
		}

		tempTimeline = sortByYear(tempTimeline);

		timeline = new CN_TimelineCard[tempTimeline.size()];
		for(int i = 0; i < timeline.length; i++)
		{
			timeline[i] = tempTimeline.get(i);
		}

		returnValue = new CN_Timeline(timeline);
		return returnValue;
	}



	private ArrayList<CN_TimelineCard> sortByYear(ArrayList<CN_TimelineCard> loadedTimeline)
	{
		ArrayList<CN_TimelineCard> returnValue = new ArrayList<CN_TimelineCard>();

		CN_TimelineCard currentLowest;
		int currentLowestIndex;
		while(!loadedTimeline.isEmpty())
		{
			currentLowest = loadedTimeline.get(0);
			currentLowestIndex = 0;
			for(int i = 0; i < loadedTimeline.size(); i++)
			{
				if(loadedTimeline.get(i).year < currentLowest.year)
				{
					currentLowest = loadedTimeline.get(i);
					currentLowestIndex = i;
				}
			}
			returnValue.add(currentLowest);
			loadedTimeline.remove(currentLowestIndex);
		}

		return returnValue;
	}



	private void addPatchingOption(CN_Ripplepoint card, PatchingOption option) throws CN_FatalException
	{
		ArrayList<CN_Linchpin> linchpinPrereqList = new ArrayList<CN_Linchpin>();
		ArrayList<CN_TimelineCard.FlipState> flipStatePrereqList = new ArrayList<CN_TimelineCard.FlipState>();

		CN_Linchpin[] linchpinPrereqs;
		CN_TimelineCard.FlipState[] flipStatePrereqs;

		for(String linchpin : option.prereqLinchpins)
		{
			if(tempLinchpinTable.get(linchpin) != null)
				linchpinPrereqList.add(tempLinchpinTable.get(linchpin));
			else
				throw new CN_FatalException("CN_Initializer, addPatchingOption(): Specified linchpin does not exist!");
		}

		for(String flipState : option.prereqFlipStates)
		{
			if(flipState.toLowerCase().equals("unflipped"))
			{
				flipStatePrereqList.add(CN_TimelineCard.FlipState.UNFLIPPED);
			}
			else if(flipState.toLowerCase().equals("flipped"))
			{
				flipStatePrereqList.add(CN_TimelineCard.FlipState.FLIPPED);
			}
			else
			{
				throw new CN_FatalException("CN_Initializer, addPatchingOption(): Invalid flip state!");
			}
		}


		linchpinPrereqs = new CN_Linchpin[linchpinPrereqList.size()];
		for(int i = 0; i < linchpinPrereqs.length; i++)
		{
			linchpinPrereqs[i] = linchpinPrereqList.get(i);
		}

		flipStatePrereqs = new CN_TimelineCard.FlipState[flipStatePrereqList.size()];
		for(int i = 0; i < flipStatePrereqs.length; i++)
		{
			flipStatePrereqs[i] = flipStatePrereqList.get(i);
		}

		card.addPatchingOption(option.patchIdentity, linchpinPrereqs, flipStatePrereqs);
	}




	private CN_TimelineCard createTimelineCard(LoadedTimelineCard card) throws CN_FatalException
	{
		CN_TimelineCard returnValue;

		if(card.isRipple)
		{
			returnValue = new CN_Ripplepoint(timelineCardIndexer, card.identity, card.cardName, card.year);
		}
		else
		{
			returnValue = new CN_Linchpin(timelineCardIndexer, card.identity, card.cardName, card.year);
		}
		CN_Constants.timelineCardDirectory.put(timelineCardIndexer, returnValue);
		timelineCardIndexer++;

		return returnValue;
	}






	private LoadedTimelineCard loadTimelineCard(Node timelineCardNode) throws CN_FatalException
	{
		LoadedTimelineCard returnValue = new LoadedTimelineCard();

		NodeList list = timelineCardNode.getChildNodes();
		Node currentNode;

		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("identity"))
			{
				returnValue.identity = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("cardname"))
			{
				returnValue.cardName = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("cardtype"))
			{
				if(currentNode.getTextContent().toLowerCase().equals("linchpin"))
				{
					returnValue.isRipple = false;
				}
				else if(currentNode.getTextContent().toLowerCase().equals("ripplepoint"))
				{
					returnValue.isRipple = true;
				}
				else
					throw new CN_FatalException("CN_Initializer, loadTimelineCard(): Invalid Card Type!");
			}
			else if(currentNode.getNodeName().toLowerCase().equals("year"))
			{
				try
				{
					returnValue.year = Integer.parseInt(currentNode.getTextContent());
				}
				catch(NumberFormatException e)
				{
					throw new CN_FatalException("CN_Initializer, loadTimelineCard(): Year is not a number!", e);
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("flipstateoverrides"))
			{
				NodeList subList = currentNode.getChildNodes();
				Node subCurrentNode;
				for(int j = 0; subList.item(j) != null; j++)
				{
					subCurrentNode = subList.item(j);
					if(subCurrentNode.getNodeName().toLowerCase().equals("forceparadox"))
					{
						returnValue.flipStateOverrides.add(loadForceParadox(subCurrentNode));
					}
					else if(subCurrentNode.getNodeName().toLowerCase().equals("#text"))
					{
					}
					else
					{
						throw new CN_FatalException("CN_Initializer, loadTimelineCard(): Invalid node name within FlipStateOverrides!  Name: " + subCurrentNode.getNodeName());
					}
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("patchingoption"))
			{
				returnValue.patchingOptions.add(loadPatchingOption(currentNode));
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new CN_FatalException("CN_Initializer, loadTimelineCard(): Invalid node name!  Name: " + currentNode.getNodeName());
			}
		}


		if(!returnValue.isRipple)
		{
			returnValue.patchingOptions.clear();
		}

		return returnValue;
	}





	private PatchingOption loadPatchingOption(Node patchingOptionNode)
	{
		PatchingOption returnValue = new PatchingOption();

		NodeList list = patchingOptionNode.getChildNodes();
		Node currentNode;

		NodeList subNodeList;
		Node subCurrentNode;

		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("patchidentity"))
			{
				returnValue.patchIdentity = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("prereq"))
			{
				subNodeList = currentNode.getChildNodes();
				for(int j = 0; subNodeList.item(j) != null; j++)
				{
					subCurrentNode = subNodeList.item(j);
					if(subCurrentNode.getNodeName().toLowerCase().equals("linchpinidentity"))
					{
						returnValue.prereqLinchpins.add(subCurrentNode.getTextContent());
					}
					else if(subCurrentNode.getNodeName().toLowerCase().equals("flipstate"))
					{
						returnValue.prereqFlipStates.add(subCurrentNode.getTextContent());
					}
				}
			}

		}

		return returnValue;
	}




	private ForceParadox loadForceParadox(Node forceParadoxNode) throws CN_FatalException
	{
		ForceParadox returnValue = new ForceParadox();

		NodeList list = forceParadoxNode.getChildNodes();
		Node currentNode;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("linchpinidentity"))
			{
				returnValue.linchpinIdentity = currentNode.getTextContent();
			}
			else if(currentNode.getNodeName().toLowerCase().equals("flipstate"))
			{
				String flipStateString = currentNode.getTextContent();
				if(flipStateString.toLowerCase().equals("flipped"))
				{
					returnValue.flipStateIsFlipped = true;
				}
				else if(flipStateString.toLowerCase().equals("unflipped"))
				{
					returnValue.flipStateIsFlipped = false;
				}
				else
				{
					throw new CN_FatalException("CN_Initializer, loadForceParadox(): FlipState \"" + flipStateString + "\" is an invalid flip state!  Must be flipped or unflipped!");
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new CN_FatalException("CN_Initializer, loadForceParadox(): Name \"" + currentNode.getNodeName() + "\" is an invalid node name!");
			}
		}
		return returnValue;
	}




	private class LoadedTimelineCard
	{
		public String identity;
		public String cardName;
		public int year;
		public final ArrayList<ForceParadox> flipStateOverrides;
		public boolean isRipple;
		public final ArrayList<PatchingOption> patchingOptions;

		public LoadedTimelineCard()
		{
			patchingOptions = new ArrayList<PatchingOption>();
			flipStateOverrides = new ArrayList<ForceParadox>();
		}

		public void debugPrint()
		{
			println("Identity: " + identity);
			println("Card Name: " + cardName);
			println("Patching Options: ");
			for(PatchingOption option : patchingOptions)
			{
				option.debugPrint();
			}
			println("\n");
		}
	}

	private class PatchingOption
	{
		public String patchIdentity;
		public final ArrayList<String> prereqLinchpins;
		public final ArrayList<String> prereqFlipStates;

		public PatchingOption()
		{
			prereqLinchpins = new ArrayList<String>();
			prereqFlipStates = new ArrayList<String>();
		}

		public void debugPrint()
		{
			println("  Patch Identity: " + patchIdentity);
			for(int i = 0; i < prereqLinchpins.size(); i++)
			{
				println("  Linchpin Prereq: " + prereqLinchpins.get(i));
				println("  Flip State Prereq: " + prereqFlipStates.get(i));
			}
			println("\n");
		}

	}


	private class ForceParadox
	{
		public String linchpinIdentity;
		public boolean flipStateIsFlipped;
	}





	private void println(String printString)
	{
		System.out.println(printString);
	}






}
