package cnpc.engine;

import java.util.*;

class CN_PlayCard extends CN_GameCard
{
	public enum PlayCardType {ARTIFACT, INVERTER, PATCH, ACTION, TIMEWARP};
	public enum UnplayableCondition {PATCH_STANDARD, INVERTER, DISCARDS_EMPTY, NO_NON_TIMEWARP_DISCARDS_PLAYABLE, ARTIFACTS_NONE, ARTIFACTS_NOT_CURRENT_PLAYER, ARTIFACTS_NOT_OPPONENT};

	public PlayCardType cardType;

	public final int year;

	public final int authenticity;
	public final String authClass;

	private final ArrayList<StoredBascom> bascomList;
	private final ArrayList<UnplayableCondition> unplayableConditions;
	private final ArrayList<InverterUnplayableConditionPart> inverterUnplayableCondition;


	public ArrayList<UnplayableCondition> getUnplayableConditions()
	{
		ArrayList<UnplayableCondition> returnValue = new ArrayList<UnplayableCondition>();
		for(int i = 0; i < unplayableConditions.size(); i++)
		{
			returnValue.add(unplayableConditions.get(i));
		}
		return returnValue;
	}


	public ArrayList<InverterUnplayableConditionPart> getInverterUnplayableCondition()
	{
		ArrayList<InverterUnplayableConditionPart> returnValue = new ArrayList<InverterUnplayableConditionPart>();
		for(int i = 0; i < inverterUnplayableCondition.size(); i++)
		{
			returnValue.add(inverterUnplayableCondition.get(i));
		}
		return returnValue;
	}



	public static class InverterUnplayableConditionPart
	{
		public final String linchpin;
		public final boolean unplayableIfFlipped;

		public InverterUnplayableConditionPart(String linchpin_in, boolean unplayableIfFlipped_in)
		{
			linchpin = linchpin_in;
			unplayableIfFlipped = unplayableIfFlipped_in;
		}
	}



	public static class StoredBascom
	{
		public final int bascomId;

		//the order of elements in the following ArrayList most certainly DOES matter.  Check Bascom specifications.

		private final ArrayList<String> bascomSpecialStrings;

		public int specialStringsSize()
		{
			return bascomSpecialStrings.size();
		}

		public String getSpecialString(int index)
		{
			return bascomSpecialStrings.get(index);
		}



		public StoredBascom(int bascomId_in, ArrayList<String> bascomSpecialStrings_in)
		{
			bascomId = bascomId_in;
			bascomSpecialStrings = bascomSpecialStrings_in;
		}



		public void debugPrint()
		{
			System.out.print("\n");
			System.out.print("\nBascom Id: " + bascomId);
			if(bascomSpecialStrings != null)
			{
				for(String string : bascomSpecialStrings)
				{
					System.out.print("\nParameter: " + string);
				}
			}
		}

	}


	public StoredBascom getBascom(int index)
	{
		return bascomList.get(index);
	}

	public int bascomSize()
	{
		return bascomList.size();
	}



	public boolean containsUnplayableCondition(UnplayableCondition condition)
	{
		return unplayableConditions.contains(condition);
	}



	public CN_PlayCard(int cardNumber_in, String identity_in, PlayCardType cardType_in, ArrayList<StoredBascom> bascomList_in, ArrayList<UnplayableCondition> unplayableConditions_in, ArrayList<InverterUnplayableConditionPart> inverterUnplayableCondition_in, int year_in) throws CN_FatalException
	{
		this(cardNumber_in, identity_in, cardType_in, CN_Constants.NONEXISTENT_AUTHENTICITY, null, bascomList_in, unplayableConditions_in, inverterUnplayableCondition_in, year_in);
	}



	public CN_PlayCard(int cardNumber_in, String identity_in, PlayCardType cardType_in, int authenticity_in, String authClass_in, ArrayList<StoredBascom> bascomList_in, ArrayList<UnplayableCondition> unplayableConditions_in, ArrayList<InverterUnplayableConditionPart> inverterUnplayableCondition_in, int year_in) throws CN_FatalException
	{
		super(cardNumber_in, identity_in, ""); //todo: eliminate card name

		bascomList = new ArrayList<StoredBascom>();
		unplayableConditions = new ArrayList<UnplayableCondition>();
		inverterUnplayableCondition = new ArrayList<InverterUnplayableConditionPart>();
		cardType = cardType_in;

		authenticity = authenticity_in;
		authClass = authClass_in;

		year = year_in;

		for(int i = 0; i < bascomList_in.size(); i++)
		{
			bascomList.add(bascomList_in.get(i));
		}

		UnplayableCondition currentUnplayableCondition;
		for(int i = 0; i < unplayableConditions_in.size(); i++)
		{
			currentUnplayableCondition = unplayableConditions_in.get(i);
			if((currentUnplayableCondition == UnplayableCondition.PATCH_STANDARD)&&(cardType != PlayCardType.PATCH))
			{
				throw new CN_FatalException("CN_PlayCard, constructor: UnplayableCondition \"PATCH_STANDARD\" can only be applied to a Patch!");
			}
			else if((currentUnplayableCondition == UnplayableCondition.INVERTER)&&(cardType != PlayCardType.INVERTER))
			{
				throw new CN_FatalException("CN_PlayCard, constructor: UnplayableCondition \"INVERTER_STANDARD\" can only be applied to an Inverter!");
			}
			unplayableConditions.add(currentUnplayableCondition);
		}

		InverterUnplayableConditionPart currentUnplayableConditionPart;
		for(int i = 0; i < inverterUnplayableCondition_in.size(); i++)
		{
			currentUnplayableConditionPart = inverterUnplayableCondition_in.get(i);
			inverterUnplayableCondition.add(currentUnplayableConditionPart);
		}


	}



	public static PlayCardType getPlayCardTypeFromString(String playCardTypeString) throws CN_FatalException
	{
		if(playCardTypeString.toLowerCase().equals("artifact"))
		{
			return PlayCardType.ARTIFACT;
		}
		else if(playCardTypeString.toLowerCase().equals("inverter"))
		{
			return PlayCardType.INVERTER;
		}
		else if(playCardTypeString.toLowerCase().equals("patch"))
		{
			return PlayCardType.PATCH;
		}
		else if(playCardTypeString.toLowerCase().equals("action"))
		{
			return PlayCardType.ACTION;
		}
		else if(playCardTypeString.toLowerCase().equals("timewarp"))
		{
			return PlayCardType.TIMEWARP;
		}
		else
		{
			throw new CN_FatalException("CN_PlayCard, constructor: invalid card type string!");
		}
	}





	public void debugPrint()
	{
		System.out.print("\n\n");
		System.out.print("\n" + cardNumber);
		System.out.print("\n" + identity);
		System.out.print("\n" + cardType);
		for(StoredBascom bascom : bascomList)
		{
			bascom.debugPrint();
		}
	}


}
