package cnpc.engine;

import java.util.*;

class CN_Player
{
	final int playerIndex;

	private CN_IdCard id;
	private CN_MissionCard mission;

	public CN_Player nextPlayer;
	public CN_Player prevPlayer;

	private final ArrayList<CN_PlayCard> hand;
	private final ArrayList<CN_PlayCard> artifacts;

	private final Stack<ArrayList<CN_PlayCard>> backedUpHands;

	public CN_Player(int playerIndex_in)
	{
		playerIndex = playerIndex_in;
		hand = new ArrayList<CN_PlayCard>();
		artifacts = new ArrayList<CN_PlayCard>();
		backedUpHands = new Stack<ArrayList<CN_PlayCard>>();
	}

	public boolean removeArtifact(CN_PlayCard artifact)
	{
		if(artifacts.contains(artifact))
		{
			artifacts.remove(artifact);
			return true;
		}
		else
		{
			return false;
		}
	}

	public void addArtifact(CN_PlayCard artifact)
	{
		artifacts.add(artifact);
	}

	public boolean isInArtifacts(CN_PlayCard card)
	{
		for(CN_PlayCard artifact : artifacts)
		{
			if(artifact.cardNumber == card.cardNumber)
			{
				return true;
			}
		}
		return false;
	}

	public boolean hasArtifacts()
	{
		return !artifacts.isEmpty();
	}

	public ArrayList<CN_PlayCard> getListOfArtifacts()
	{
		ArrayList<CN_PlayCard> returnValue = new ArrayList<CN_PlayCard>();
		for(CN_PlayCard artifact : artifacts)
		{
			returnValue.add(artifact);
		}
		return returnValue;
	}

	public ArrayList<CN_PlayCard> getListOfHandCards()
	{
		ArrayList<CN_PlayCard> returnValue = new ArrayList<CN_PlayCard>();
		for(CN_PlayCard handCard : hand)
		{
			returnValue.add(handCard);
		}
		return returnValue;
	}

	public ArrayList<CN_PlayCard> getListOfRealHandCards()
	{
		ArrayList<CN_PlayCard> returnValue;
		if(!backedUpHands.empty())
		{
			returnValue = new ArrayList<CN_PlayCard>();
			ArrayList<CN_PlayCard> realHand = backedUpHands.get(0); //todo: test this, make sure 0 is bottom of stack
			for(CN_PlayCard handCard : realHand)
			{
				returnValue.add(handCard);
			}
		}
		else
		{
			returnValue = getListOfHandCards();
		}
		return returnValue;
	}



	public void addListOfArtifacts(ArrayList<CN_PlayCard> artifactList)
	{
		CN_PlayCard currentArtifact;
		for(int i = 0; i < artifactList.size(); i++)
		{
			currentArtifact = artifactList.get(i);
			addArtifact(currentArtifact);
		}
	}



	public void addListOfHandCards(ArrayList<CN_PlayCard> cardList)
	{
		CN_PlayCard currentCard;
		for(int i = 0; i < cardList.size(); i++)
		{
			currentCard = cardList.get(i);
			addHandCard(currentCard);
		}
	}

	public void addListOfRealHandCards(ArrayList<CN_PlayCard> cardList)
	{
		CN_PlayCard currentCard;
		for(int i = 0; i < cardList.size(); i++)
		{
			currentCard = cardList.get(i);
			addRealHandCard(currentCard);
		}
	}



	public CN_IdCard setId(CN_IdCard newId)
	{
		CN_IdCard returnValue = id;
		id = newId;
//id.debugPrint();
		return returnValue;
	}

	public CN_MissionCard setMission(CN_MissionCard newMission)
	{
		CN_MissionCard returnValue = mission;
		mission = newMission;
		return returnValue;
	}

	public CN_IdCard getId()
	{
		return id;
	}

	public CN_MissionCard getMission()
	{
		return mission;
	}


	public boolean removeHandCard(CN_PlayCard card)
	{
		if(hand.contains(card))
		{
			hand.remove(card);
			return true;
		}
		else
		{
			return false;
		}
	}

	public boolean removeRealHandCard(CN_PlayCard card)
	{
		ArrayList<CN_PlayCard> realHand;
		if(!backedUpHands.empty())
		{
			realHand = backedUpHands.get(0);
		}
		else
		{
			realHand = hand;
		}

		if(realHand.contains(card))
		{
			realHand.remove(card);
			return true;
		}
		else
		{
			return false;
		}
	}




	public void addHandCard(CN_PlayCard card)
	{
		hand.add(card);
	}

	public void addRealHandCard(CN_PlayCard card)
	{
		ArrayList<CN_PlayCard> realHand;
		if(!backedUpHands.empty())
		{
			realHand = backedUpHands.get(0);
		}
		else
		{
			realHand = hand;
		}
		realHand.add(card);
	}


	public boolean isInHand(CN_PlayCard card)
	{
		for(CN_PlayCard handCard : hand)
		{
			if(handCard.cardNumber == card.cardNumber)
			{
				return true;
			}
		}
		return false;
	}

	public boolean isInBackedUpHands(CN_PlayCard card)
	{
		for(ArrayList<CN_PlayCard> list : backedUpHands)
		{
			if(isInList(card, list))
			{
				return true;
			}
		}
		return false;
	}

	private boolean isInList(CN_PlayCard card, ArrayList<CN_PlayCard> list)
	{
		for(CN_PlayCard listCard : list)
		{
			if(listCard.cardNumber == card.cardNumber)
			{
				return true;
			}
		}
		return false;
	}




	public void backUpHand()
	{
		ArrayList<CN_PlayCard> oldHand = new ArrayList<CN_PlayCard>();
		for(int i = 0; i < hand.size(); i++)
		{
			oldHand.add(hand.get(i));
		}
		backedUpHands.push(oldHand);
		hand.clear();
	}



	public ArrayList<CN_PlayCard> restoreBackedUpHand() throws CN_FatalException
	{
		if(backedUpHands.empty())
		{
			throw new CN_FatalException("CN_Player, restoreBackedUpHand(): There are no backed up hands to restore!");
		}

		ArrayList<CN_PlayCard> returnValue = new ArrayList<CN_PlayCard>();
		ArrayList<CN_PlayCard> newHand = new ArrayList<CN_PlayCard>();
		for(int i = 0; i < hand.size(); i++)
		{
			returnValue.add(hand.get(i));
		}
		hand.clear();
		newHand = backedUpHands.pop();
		for(int i = 0; i < newHand.size(); i++)
		{
			hand.add(newHand.get(i));
		}
		return returnValue;
	}




	public int testForVictory(CN_Timeline timeline, ArrayList<CN_PlayCard> artifactsFromGetList, ArrayList<Integer> highestAuthsInPlay, boolean isPlayersTurn) throws CN_FatalException
	{
		if(!isPlayersTurn)
		{
			return CN_Constants.VICTORY_TYPE_NULL;
		}

		if(id.testForVictory(timeline))
		{
			return CN_Constants.VICTORY_TYPE_ID;
		}
		else if(mission.testForVictory(artifactsFromGetList, highestAuthsInPlay))
		{
			return CN_Constants.VICTORY_TYPE_MISSION;
		}
		else if(hand.size() >= CN_Constants.WINNING_HAND_SIZE)
		{
			return CN_Constants.VICTORY_TYPE_CARDS;
		}
		return CN_Constants.VICTORY_TYPE_NULL;
	}
















	public boolean removeArtifact(String artifact)
	{
		CN_PlayCard card = findArtifactById(artifact);
		if(card != null)
		{
			artifacts.remove(card);
			return true;
		}
		else
		{
			return false;
		}
	}

	private CN_PlayCard findArtifactById(String artifactId)
	{
		for(CN_PlayCard card : artifacts)
		{
			if(card.identity.equals(artifactId))
				return card;
		}
		return null;
	}



	public CN_GameState.PlayerData toPlayerGs()
	{
		return new CN_GameState.PlayerData(playerIndex, id.cardNumber, mission.cardNumber, nextPlayer.playerIndex, prevPlayer.playerIndex, returnGsList(hand), returnGsList(artifacts), getGsBackedUpHands());
	}


	private Stack<ArrayList<Integer>> getGsBackedUpHands()
	{
		Stack<ArrayList<Integer>> returnValue = new Stack<ArrayList<Integer>>();
		Stack<ArrayList<CN_PlayCard>> intermediateStack = new Stack<ArrayList<CN_PlayCard>>();

		while(!backedUpHands.empty())
		{
			intermediateStack.push(backedUpHands.pop());
		}

		ArrayList<CN_PlayCard> currentBackedUpHand;
		while(!intermediateStack.empty())
		{
			currentBackedUpHand = intermediateStack.pop();
			backedUpHands.push(currentBackedUpHand);
			returnValue.push(returnGsList(currentBackedUpHand));
		}

		return returnValue;
	}


	public void loadGsBackedUpHands(Stack<ArrayList<Integer>> gsBackedUpHands) throws CN_FatalException
	{
		Stack<ArrayList<Integer>> intermediateStack = new Stack<ArrayList<Integer>>();

		while(!gsBackedUpHands.empty())
		{
			intermediateStack.push(gsBackedUpHands.pop());
		}

		ArrayList<Integer> currentGsBackedUpHand;
		while(!intermediateStack.empty())
		{
			currentGsBackedUpHand = intermediateStack.pop();
			gsBackedUpHands.push(currentGsBackedUpHand);
			backedUpHands.push(fromGsList(currentGsBackedUpHand));
		}
	}


	private ArrayList<CN_PlayCard> fromGsList(ArrayList<Integer> gsList) throws CN_FatalException
	{
		ArrayList<CN_PlayCard> returnValue = new ArrayList<CN_PlayCard>();

		Integer currentInteger;
		for(int i = 0; i < gsList.size(); i++)
		{
			currentInteger = gsList.get(i);
			returnValue.add(CN_Constants.playCardDirectory.getById(currentInteger));
		}

		return returnValue;
	}


	private ArrayList<Integer> returnGsList(ArrayList<CN_PlayCard> origList)
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		CN_PlayCard currentCard;
		for(int i = 0; i < origList.size(); i++)
		{
			currentCard = origList.get(i);
			returnValue.add(currentCard.cardNumber);
		}

		return returnValue;
	}



	public ArrayList<CN_PlayCard> clearHand()
	{
		ArrayList<CN_PlayCard> returnValue;
		returnValue = getListOfHandCards();
		hand.clear();
		return returnValue;
	}


	public ArrayList<CN_PlayCard> clearAllHands() throws CN_FatalException
	{
		ArrayList<CN_PlayCard> returnValue;
		returnValue = getListOfHandCards();

		for(int i = 0; i < backedUpHands.size(); i++)
		{
			for(int j = 0; j < backedUpHands.get(i).size(); j++)
			{
				returnValue.add(backedUpHands.get(i).get(j));
			}
			backedUpHands.get(i).clear();
		}
		backedUpHands.clear();
		hand.clear();

		return returnValue;
	}


	public ArrayList<CN_PlayCard> clearArtifacts()
	{
		ArrayList<CN_PlayCard> returnValue;
		returnValue = getListOfArtifacts();
		artifacts.clear();
		return returnValue;
	}

}
