package cnpc.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class UI_AboutDialog extends JDialog
{
	private final SpringLayout layout = new SpringLayout();
	private final Container content = getContentPane();

	private final Point location = new Point(100, 100);

	private final int WEST_EAST_BUFFER = 30;
	private final int NORTH_BUFFER = 10;
	private final int SOUTH_BUFFER = 10;

	private final int TITLE_VERSION_SEPARATION = 2;
	private final int VERSION_DATE_SEPARATION = 2;
	private final int DATE_DEDICATION_SEPARATION = 20;
	private final int DEDICATION_CHRONONAUTS_COPYRIGHT_SEPARATION = 20;
	private final int CHRONONAUTS_COPYRIGHT_LANGUAGE_SEPARATION = 20;
	private final int LANGUAGE_COPYRIGHT_SEPARATION = 2;
	private final int COPYRIGHT_RIGHTS_SEPARATION = 2;
	private final int RIGHTS_BUTTON_SEPARATION = 5;

	private final JLabel titleLabel = new JLabel(UI_Constants.GAME_NAME);
	private final JLabel versionLabel = new JLabel(UI_Constants.VERSION_LABEL);
	private final JLabel dateLabel = new JLabel(UI_Constants.GAME_DATE);
	private final JLabel dedicationLabel = new JLabel("");
	private final JLabel chrononautsCopyrightLabel = new JLabel("Chrononauts copyright held by Andrew Looney of Looney Labs");
	private final JLabel languageLabel = new JLabel("Source Code in Java 1.5.0");
	private final JLabel copyrightLabel = new JLabel("Copyright 2006 by Michael Barbieri.  All rights reserved.");
	private final JLabel rightsLabel = new JLabel("");
	private final JButton okButton = new JButton("OK");


	public UI_AboutDialog(Dialog owner)
	{
		super(owner, true);
		initDialog();
	}

	public UI_AboutDialog(Frame owner)
	{
		super(owner, true);
		initDialog();
	}

	private void initDialog()
	{
		setLayout(layout);

		setTitle("About " + UI_Constants.GAME_NAME);
		setResizable(false);
		setLocation(location);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);

		int dialogWidth = WEST_EAST_BUFFER + getWidestComponent().getPreferredSize().width + WEST_EAST_BUFFER;
		int dialogHeight = UI_Constants.TITLE_BAR_HEIGHT + NORTH_BUFFER + titleLabel.getPreferredSize().height + TITLE_VERSION_SEPARATION + versionLabel.getPreferredSize().height + VERSION_DATE_SEPARATION + dateLabel.getPreferredSize().height + DATE_DEDICATION_SEPARATION + dedicationLabel.getPreferredSize().height + DEDICATION_CHRONONAUTS_COPYRIGHT_SEPARATION + chrononautsCopyrightLabel.getPreferredSize().height + CHRONONAUTS_COPYRIGHT_LANGUAGE_SEPARATION + languageLabel.getPreferredSize().height + LANGUAGE_COPYRIGHT_SEPARATION + copyrightLabel.getPreferredSize().height + COPYRIGHT_RIGHTS_SEPARATION + rightsLabel.getPreferredSize().height + RIGHTS_BUTTON_SEPARATION + okButton.getPreferredSize().height + SOUTH_BUFFER;
		setSize(dialogWidth, dialogHeight);

		layout.putConstraint(SpringLayout.WEST, titleLabel, getCenteredStartLocation(titleLabel), SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, titleLabel, NORTH_BUFFER, SpringLayout.NORTH, content);

		layout.putConstraint(SpringLayout.WEST, versionLabel, getCenteredStartLocation(versionLabel), SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, versionLabel, TITLE_VERSION_SEPARATION, SpringLayout.SOUTH, titleLabel);

		layout.putConstraint(SpringLayout.WEST, dateLabel, getCenteredStartLocation(dateLabel), SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, dateLabel, VERSION_DATE_SEPARATION, SpringLayout.SOUTH, versionLabel);

		layout.putConstraint(SpringLayout.WEST, dedicationLabel, getCenteredStartLocation(dedicationLabel), SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, dedicationLabel, DATE_DEDICATION_SEPARATION, SpringLayout.SOUTH, dateLabel);

		layout.putConstraint(SpringLayout.WEST, chrononautsCopyrightLabel, getCenteredStartLocation(chrononautsCopyrightLabel), SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, chrononautsCopyrightLabel, DEDICATION_CHRONONAUTS_COPYRIGHT_SEPARATION, SpringLayout.SOUTH, dedicationLabel);

		layout.putConstraint(SpringLayout.WEST, languageLabel, getCenteredStartLocation(languageLabel), SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, languageLabel, CHRONONAUTS_COPYRIGHT_LANGUAGE_SEPARATION, SpringLayout.SOUTH, chrononautsCopyrightLabel);

		layout.putConstraint(SpringLayout.WEST, copyrightLabel, getCenteredStartLocation(copyrightLabel), SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, copyrightLabel, LANGUAGE_COPYRIGHT_SEPARATION, SpringLayout.SOUTH, languageLabel);

		layout.putConstraint(SpringLayout.WEST, rightsLabel, getCenteredStartLocation(rightsLabel), SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, rightsLabel, COPYRIGHT_RIGHTS_SEPARATION, SpringLayout.SOUTH, copyrightLabel);

		layout.putConstraint(SpringLayout.WEST, okButton, getCenteredStartLocation(okButton), SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, okButton, RIGHTS_BUTTON_SEPARATION, SpringLayout.SOUTH, rightsLabel);

		add(titleLabel);
		add(versionLabel);
		add(dateLabel);
		add(dedicationLabel);
		add(chrononautsCopyrightLabel);
		add(languageLabel);
		add(copyrightLabel);
		add(rightsLabel);
		add(okButton);

		okButton.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					dispose();
				}
			}
		);

		setVisible(true);
	}

	private int getCenteredStartLocation(JComponent component)
	{
		return (getSize().width / 2) - (component.getPreferredSize().width / 2);
	}

	private JComponent getWidestComponent()
	{
		JComponent returnValue = titleLabel;
		if(versionLabel.getPreferredSize().width > returnValue.getPreferredSize().width)
		{
			returnValue = versionLabel;
		}
		if(dateLabel.getPreferredSize().width > returnValue.getPreferredSize().width)
		{
			returnValue = dateLabel;
		}
		if(dedicationLabel.getPreferredSize().width > returnValue.getPreferredSize().width)
		{
			returnValue = dedicationLabel;
		}
		if(chrononautsCopyrightLabel.getPreferredSize().width > returnValue.getPreferredSize().width)
		{
			returnValue = chrononautsCopyrightLabel;
		}
		if(languageLabel.getPreferredSize().width > returnValue.getPreferredSize().width)
		{
			returnValue = languageLabel;
		}
		if(copyrightLabel.getPreferredSize().width > returnValue.getPreferredSize().width)
		{
			returnValue = copyrightLabel;
		}
		if(rightsLabel.getPreferredSize().width > returnValue.getPreferredSize().width)
		{
			returnValue = rightsLabel;
		}
		if(okButton.getPreferredSize().width > returnValue.getPreferredSize().width)
		{
			returnValue = okButton;
		}
		return returnValue;
	}
}
