package cnpc.ui;

import java.awt.event.*;
import javax.swing.*;

class UI_CardPanel extends JLayeredPane
{
	private final UI_GameCard representedCard;
	protected final JLabel imageLabel;
	protected final JLabel xLabel;
	protected final JLabel unstableLabel;

	private boolean isFlipped;


	public UI_CardPanel()
	{
		this(null, false);
	}


	public UI_CardPanel(UI_GameCard representedCard_in, boolean isFlipped_in)
	{
		this(representedCard_in, isFlipped_in, false, false);
	}


	public UI_CardPanel(UI_GameCard representedCard_in, boolean isFlipped_in, boolean isXedOut, boolean isUnstable)
	{
		super();

		SpringLayout panelLayout;

		representedCard = representedCard_in;
		isFlipped = isFlipped_in;

		panelLayout = new SpringLayout();
		setLayout(panelLayout);
		setPreferredSize(UI_Constants.CARD_DIMENSION);

		xLabel = new JLabel(UI_Globals.xImage);
		unstableLabel = new JLabel(UI_Globals.unstableImage);

		if(representedCard != null)
		{
			if(!isFlipped)
			{
				imageLabel = new JLabel(representedCard.frontImage);
			}
			else
			{
				imageLabel = new JLabel(representedCard.backImage);
			}
		}
		else
		{
			imageLabel = new JLabel(UI_Globals.blankImage);
		}
		addStandardMouseListener();
		setLayer(imageLabel, 0);

		panelLayout.putConstraint(SpringLayout.WEST, imageLabel, 0, SpringLayout.WEST, this);
		panelLayout.putConstraint(SpringLayout.NORTH, imageLabel, 0, SpringLayout.NORTH, this);

		add(imageLabel);

		if(isXedOut)
		{
			setLayer(xLabel, 1);
			panelLayout.putConstraint(SpringLayout.WEST, xLabel, 0, SpringLayout.WEST, this);
			panelLayout.putConstraint(SpringLayout.NORTH, xLabel, 0, SpringLayout.NORTH, this);

			add(xLabel);
		}
		else if(isUnstable)
		{
			setLayer(unstableLabel, 1);
			panelLayout.putConstraint(SpringLayout.WEST, unstableLabel, 0, SpringLayout.WEST, this);
			panelLayout.putConstraint(SpringLayout.NORTH, unstableLabel, 0, SpringLayout.NORTH, this);

			add(unstableLabel);
		}
	}


	public void addStandardMouseListener()
	{
		addMouseListener
		(
			new MouseAdapter()
			{
				public void mouseClicked(MouseEvent e)
				{
					if(e.getButton() == MouseEvent.BUTTON3)
					{
						new UI_BigImageFrame(representedCard, isFlipped);
					}
				}
			}
		);
	}



	public void setToFrontImage()
	{
		isFlipped = false;
		if(representedCard != null)
		{
			imageLabel.setIcon(representedCard.frontImage);
		}
	}


	public void setToBackImage()
	{
		isFlipped = true;
		if(representedCard != null)
		{
			imageLabel.setIcon(representedCard.backImage);
		}
	}


	public UI_GameCard getRepresentedCard()
	{
		return representedCard;
	}

}
