package cnpc.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class UI_EndOfUniverseDialog extends JDialog
{
	private final SpringLayout layout = new SpringLayout();
	private final Container content = getContentPane();
	
	private final UI_MainFrame owner;
	
	private static UI_EndOfUniverseDialog me = null;


	public static void showDialog(UI_MainFrame frame, String playerName)
	{
		new UI_EndOfUniverseDialog(frame, playerName);
	}
	
	public static void disposeDialog()
	{
		if(me == null) return;

		me.dispose();
		me = null;
	}


	private UI_EndOfUniverseDialog(UI_MainFrame owner_in, String playerName)
	{
		me = this;
		owner = owner_in;

		content.setLayout(layout);

		setTitle(UI_Constants.END_OF_UNIVERSE_DIALOG_TITLE);
		setResizable(false);
		setLocation(UI_Constants.END_OF_UNIVERSE_DIALOG_LOCATION);

		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener
		(
			new WindowAdapter()
			{
				public void windowClosing(WindowEvent e)
				{
					owner.performDisposal();
					dispose();
				}
			}
		);
		
		layoutDialog(playerName);

		setVisible(true);
	}
	
	
	
	private void layoutDialog(String playerName)
	{
		content.setBackground(Color.BLACK);
		
		JLabel imageLabel = new JLabel(UI_Globals.endOfUniverseImage);
		JLabel destroyedLabel = new JLabel(playerName + " Destroyed");
		//JLabel destroyedLabel = new JLabel("You Destroyed");
		JLabel theUniverseLabel = new JLabel("the Universe");
				
		destroyedLabel.setFont(UI_Constants.END_OF_UNIVERSE_FONT);
		destroyedLabel.setForeground(UI_Constants.END_OF_UNIVERSE_COLOR);
		destroyedLabel.setPreferredSize(new Dimension(destroyedLabel.getPreferredSize().width + 5, destroyedLabel.getPreferredSize().height));

		theUniverseLabel.setFont(UI_Constants.END_OF_UNIVERSE_FONT);
		theUniverseLabel.setForeground(UI_Constants.END_OF_UNIVERSE_COLOR);
		theUniverseLabel.setPreferredSize(new Dimension(theUniverseLabel.getPreferredSize().width + 5, theUniverseLabel.getPreferredSize().height));

		JLabel widest = getWidestJLabel(imageLabel, destroyedLabel, theUniverseLabel);

		int width = UI_Constants.END_OF_UNIVERSE_DIALOG_WEST_BUFFER + widest.getPreferredSize().width + UI_Constants.END_OF_UNIVERSE_DIALOG_EAST_BUFFER;
		int height = UI_Constants.TITLE_BAR_HEIGHT + imageLabel.getPreferredSize().height;
		setSize(width, height);
		
		int imageWestBuffer = getCenteredLabelWestBuffer(imageLabel, width);
		int destroyedWestBuffer = getCenteredLabelWestBuffer(destroyedLabel, width);
		int theUniverseWestBuffer = getCenteredLabelWestBuffer(theUniverseLabel, width);

		layout.putConstraint(SpringLayout.WEST, imageLabel, imageWestBuffer, SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, imageLabel, 0, SpringLayout.NORTH, content);

		layout.putConstraint(SpringLayout.WEST, destroyedLabel, destroyedWestBuffer, SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, destroyedLabel, UI_Constants.END_OF_UNIVERSE_DIALOG_DESTROYED_NORTH_BUFFER, SpringLayout.NORTH, content);
		
		layout.putConstraint(SpringLayout.WEST, theUniverseLabel, theUniverseWestBuffer, SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, theUniverseLabel, UI_Constants.END_OF_UNIVERSE_DIALOG_SEPARATION, SpringLayout.SOUTH, destroyedLabel);

		content.add(destroyedLabel);
		content.add(theUniverseLabel);
		content.add(imageLabel);
	}

	
	private int getCenteredLabelWestBuffer(JLabel label, int dialogWidth)
	{
		return (dialogWidth - label.getPreferredSize().width) / 2;
	}
	
	

	private JLabel getWidestJLabel(JLabel first, JLabel second, JLabel third)
	{
		JLabel[] jLabels = new JLabel[3];
		jLabels[0] = first;
		jLabels[1] = second;
		jLabels[2] = third;
		return getWidestJLabel(jLabels);
	}
	
	private JLabel getWidestJLabel(JLabel[] jLabels)
	{
		if(jLabels.length == 0)
			return new JLabel();
		
		JLabel currentWidest = jLabels[0];
		for(JLabel jLabel : jLabels)
		{
			if(jLabel.getPreferredSize().width > currentWidest.getPreferredSize().width)
				currentWidest = jLabel;
		}
		
		return currentWidest;
	}
	
}
