package cnpc.ui;

//TODO: schema validation
import cnpc.*;
import cnpc.engine.*;

import javax.xml.parsers.*;
import org.w3c.dom.*;
import java.io.*;

import javax.swing.*;

import java.util.*;


class UI_Initializer
{

	//returns the blank image
	public void initializeUI(CN_GameState firstGameState, String variant) throws UI_FatalException
	{
		String dataPath = UI_Constants.PATH_TO_DATA + variant + UI_Constants.PATH_TO_UI_DATA;

		CNPC_ProgressFrame.instance().setProcessLabel("Loading User Interface...");

		CNPC_ProgressFrame.instance().setProgress(0, "Loading Play Card Images...");
		ArrayList<UI_GameCard> playCards = loadUIGameCards(dataPath + UI_Constants.PLAY_CARDS_PATH, variant, UI_Constants.PLAYCARDDEFAULTNAMEBACK, firstGameState.playCardDirectory, UI_Constants.playCardDirectory);

		CNPC_ProgressFrame.instance().setProgress(25, "Loading Timeline Card Images...");
		ArrayList<UI_GameCard> timelineCards = loadUIGameCards(dataPath + UI_Constants.TIMELINE_CARDS_PATH, variant, firstGameState.timelineCardDirectory, UI_Constants.timelineCardDirectory);

		CNPC_ProgressFrame.instance().setProgress(50, "Loading ID Card Images...");
		ArrayList<UI_GameCard> idCards = loadUIGameCards(dataPath + UI_Constants.ID_CARDS_PATH, variant, UI_Constants.IDCARDDEFAULTNAMEBACK, firstGameState.idCardDirectory, UI_Constants.idCardDirectory);

		CNPC_ProgressFrame.instance().setProgress(75, "Loading Mission Card Images...");
		ArrayList<UI_GameCard> missionCards = loadUIGameCards(dataPath + UI_Constants.MISSION_CARDS_PATH, variant, UI_Constants.MISSIONCARDDEFAULTNAMEBACK, firstGameState.missionCardDirectory, UI_Constants.missionCardDirectory);

		CNPC_ProgressFrame.instance().setProgress(100, "");
	}



	private ArrayList<UI_GameCard> loadUIGameCards(String xmlPath, String variant, Hashtable<Integer, String> directoryGs, Hashtable<Integer, UI_GameCard> directoryUI) throws UI_FatalException
	{
		return loadUIGameCards(xmlPath, variant, "", directoryGs, directoryUI);
	}

	private ArrayList<UI_GameCard> loadUIGameCards(String xmlPath, String variant, String defaultCardNameBack, Hashtable<Integer, String> directoryGs, Hashtable<Integer, UI_GameCard> directoryUI) throws UI_FatalException
	{
		ArrayList<UI_GameCard> returnValue = new ArrayList<UI_GameCard>();

		DocumentBuilderFactory factory;
		DocumentBuilder builder;
		Document document;
		NodeList list;
		NodeList childList;
		Node currentNode;
		Node childNode;

		ArrayList<Integer> cardNumbersForCard;
		String currentIdentity;
		UI_GameCard currentGameCard;

		try
		{
			factory = DocumentBuilderFactory.newInstance();
			builder = factory.newDocumentBuilder();
			document = builder.parse(new File(xmlPath));
		}
		catch(Exception e)
		{
			throw new UI_FatalException("UI_Initializer, loadUIGameCards(): Exception in XML file loading and parsing; see cause for details!", e);
		}

		list = document.getElementsByTagName("UI_GameCard");
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			cardNumbersForCard = getKeysFromIdentity(loadIdentity(currentNode), directoryGs);
			for(Integer currentInt : cardNumbersForCard)
			{
				currentGameCard = loadGameCard(currentNode, defaultCardNameBack, currentInt, variant);
				returnValue.add(currentGameCard);
				directoryUI.put(currentGameCard.cardNumber, currentGameCard);
			}
		}
		return returnValue;
	}


	private ArrayList<Integer> getKeysFromIdentity(String identity, Hashtable<Integer, String> directoryGs)
	{
		ArrayList<Integer> returnValue = new ArrayList<Integer>();

		Set<Integer> keys;
		keys = directoryGs.keySet();
		Iterator<Integer> iter = keys.iterator();

		Integer currentKey;
		while(iter.hasNext())
		{
			currentKey = iter.next();
			if(directoryGs.get(currentKey).equals(identity))
			{
				returnValue.add(currentKey);
			}
		}

		return returnValue;
	}



	private String loadIdentity(Node gameCardNode) throws UI_FatalException
	{
		NodeList list = gameCardNode.getChildNodes();
		Node currentNode;
		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("identity"))
			{
				return currentNode.getTextContent();
			}

		}
		throw new UI_FatalException("UI_Initializer, loadIdentity(): Node contained no identity!");
	}



	private UI_GameCard loadGameCard(Node gameCardNode, String defaultCardNameBack, int cardNumber, String variant) throws UI_FatalException
	{
		NodeList list = gameCardNode.getChildNodes();
		Node currentNode;

		String newIdentity = null;
		String newCardNameFront = null;
		String newCardNameBack = null;
		String newFrontImagePath = null;
		String newBackImagePath = null;
		String newFrontImagePathLarge = null;
		String newBackImagePathLarge = null;

		for(int i = 0; list.item(i) != null; i++)
		{
			currentNode = list.item(i);
			if(currentNode.getNodeName().toLowerCase().equals("identity"))
			{
				newIdentity = currentNode.getTextContent();
				if(newIdentity.equals(""))
				{
					newIdentity = null;
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("cardname"))
			{
				newCardNameFront = currentNode.getTextContent();
				if(newCardNameFront.equals(""))
				{
					newCardNameFront = null;
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("cardnamefront"))
			{
				newCardNameFront = currentNode.getTextContent();
				if(newCardNameFront.equals(""))
				{
					newCardNameFront = null;
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("cardnameback"))
			{
				newCardNameBack = currentNode.getTextContent();
				if(newCardNameBack.equals(""))
				{
					newCardNameBack = null;
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("frontimagepath"))
			{
				newFrontImagePath = currentNode.getTextContent();
				if(newFrontImagePath.equals(""))
				{
					newFrontImagePath = null;
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("backimagepath"))
			{
				newBackImagePath = currentNode.getTextContent();
				if(newBackImagePath.equals(""))
				{
					newBackImagePath = null;
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("frontimagepath_large"))
			{
				newFrontImagePathLarge = currentNode.getTextContent();
				if(newFrontImagePathLarge.equals(""))
				{
					newFrontImagePathLarge = null;
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("backimagepath_large"))
			{
				newBackImagePathLarge = currentNode.getTextContent();
				if(newBackImagePathLarge.equals(""))
				{
					newBackImagePathLarge = null;
				}
			}
			else if(currentNode.getNodeName().toLowerCase().equals("#text"))
			{
			}
			else
			{
				throw new UI_FatalException("UI_Initializer, loadGameCard(): Invalid node name!  Name:" + currentNode.getNodeName());
			}
		}

		if(newIdentity == null)
		{
			throw new UI_FatalException("UI_Initializer, loadGameCard(): Node contained no identity!");
		}
		if(newCardNameFront == null)
		{
			throw new UI_FatalException("UI_Initializer, loadGameCard(): Node contained no card name!");
		}

		if(newCardNameBack == null)
		{
			newCardNameBack = defaultCardNameBack;
		}

		if(newFrontImagePath == null)
		{
			throw new UI_FatalException("UI_Initializer, loadGameCard(): Node contained no front image path!");
		}
		if(newBackImagePath == null)
		{
			throw new UI_FatalException("UI_Initializer, loadGameCard(): Node contained no back image path!");
		}
		if(newFrontImagePathLarge == null)
		{
			throw new UI_FatalException("UI_Initializer, loadGameCard(): Node contained no large front image path!");
		}
		if(newBackImagePathLarge == null)
		{
			throw new UI_FatalException("UI_Initializer, loadGameCard(): Node contained no large back image path!");
		}

		String path = UI_Constants.PATH_TO_DATA + variant + UI_Constants.PATH_TO_UI_DATA;
		newFrontImagePath = path + newFrontImagePath;
		newBackImagePath = path + newBackImagePath;
		newFrontImagePathLarge = path + newFrontImagePathLarge;
		newBackImagePathLarge = path + newBackImagePathLarge;

		return new UI_GameCard(cardNumber, newIdentity, newCardNameFront, newCardNameBack, new ImageIcon(newFrontImagePath), new ImageIcon(newBackImagePath), new ImageIcon(newFrontImagePathLarge), new ImageIcon(newBackImagePathLarge));
	}






}