package cnpc.ui;

import cnpc.*;
import cnpc.engine.*;

import javax.swing.UIManager;
import javax.swing.JOptionPane;

public class UI_Messenger implements CNPC_UiController
{
	private UI_MainFrame mainFrame;

	private CN_InputMessage message;
	private boolean newGame; //set if user wanted a new game in the middle of a game

	public UI_Messenger()
	{
	}

	public void initialize(CN_GameState initialState, String variant, String[] playerNames, String[] playerPersonalities)
	{
		String[] personalityNames = new String[playerPersonalities.length];
		for(int i = 0; i < playerPersonalities.length; i++)
		{
			if(playerPersonalities[i] == null)
				personalityNames[i] = "Human";
			else
				personalityNames[i] = playerPersonalities[i];
		}

		newGame = false;
		try
		{
			mainFrame = new UI_MainFrame(this, initialState, variant, playerNames, personalityNames);
		}
		catch(UI_FatalException e)
		{
			handleFatalException(e);
		}

	}

	public void renderGameState(CN_GameState state, boolean computerPlayer)
	{
		try
		{
			mainFrame.renderGameState(state, computerPlayer);
		}
		catch(UI_FatalException e)
		{
			handleFatalException(e);
		}
	}

	public CN_InputMessage getInput()
	{
		CN_InputMessage returnValue = message;
		message = null;
		return returnValue;
	}

	public void handleInputMessageException(CN_InputMessageException e)
	{
	}

	public int getMaxSupportedPlayers()
	{
		return 4;
	}

	public void setLookAndFeel()
	{
		try
		{
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		}
		catch(Exception e)
		{
		}
	}

	public int getTitleBarHeight()
	{
		return UI_Constants.TITLE_BAR_HEIGHT;
	}

	public void endOfUniverse(CN_GameState gameState, int playerNumber)
	{
		try
		{
			mainFrame.renderGameState(gameState, false, true);
		}
		catch(UI_FatalException e)
		{
			handleFatalException(e);
		}

		String playerName = mainFrame.getPlayerName(playerNumber);
		UI_EndOfUniverseDialog.showDialog(mainFrame, playerName);
	}

	public void victory(CN_GameState gameState, int playerNumber)
	{
		try
		{
			mainFrame.renderGameState(gameState, false, true);
		}
		catch(UI_FatalException e)
		{
			handleFatalException(e);
		}

		String playerName = mainFrame.getPlayerName(playerNumber);
		UI_VictoryDialog.showDialog(mainFrame, playerName, gameState.victoryType);
	}

	public boolean getEndGameFinished()
	{
		if(mainFrame == null)
			return true;

		if(mainFrame.getDisposed())
		{
			mainFrame = null;
			return true;
		}

		return false;
	}


	public boolean newGameQuery()
	{
		int selectedOption = JOptionPane.showConfirmDialog(null, "Do you want to play another game?", UI_Constants.GAME_NAME, JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
		if(selectedOption == JOptionPane.YES_OPTION)
			return true;
		else
			return false;
	}

	public boolean getUserSetNewGame()
	{
		return newGame;
	}

	public void kill()
	{
		if(mainFrame != null)
		{
			mainFrame.performDisposal();
			mainFrame = null;
		}
	}





	public void setNewGame()
	{
		newGame = true;
		mainFrame.performDisposal();
		mainFrame = null;
	}

	public void sendPlayCardClick(UI_GameCard card)
	{
		message = new CN_InputMessage(CN_InputMessage.MessageType.PLAY_CARD_CLICK, card.cardNumber, null);
	}

	public void sendTimelineCardClick(UI_GameCard card)
	{
		message = new CN_InputMessage(CN_InputMessage.MessageType.TIMELINE_CARD_CLICK, card.cardNumber, null);
	}

	public void sendIdCardSelection(UI_GameCard card)
	{
		message = new CN_InputMessage(CN_InputMessage.MessageType.ID_CARD_SELECTION, card.cardNumber, null);
	}

	public void sendMissionCardSelection(UI_GameCard card)
	{
		message = new CN_InputMessage(CN_InputMessage.MessageType.MISSION_CARD_SELECTION, card.cardNumber, null);
	}

	public void sendDiscardClick()
	{
		message = new CN_InputMessage(CN_InputMessage.MessageType.DISCARD_CLICK, null, null);
	}

	public void sendDiscardTwoClick()
	{
		message = new CN_InputMessage(CN_InputMessage.MessageType.DISCARD_TWO_CLICK, null, null);
	}

	public void sendDirectionSelection(boolean isLeft)
	{
		if(isLeft)
		{
			message = new CN_InputMessage(CN_InputMessage.MessageType.DIRECTION_SELECTION, null, CN_InputMessage.Direction.LEFT);
		}
		else
		{
			message = new CN_InputMessage(CN_InputMessage.MessageType.DIRECTION_SELECTION, null, CN_InputMessage.Direction.RIGHT);
		}
	}

	public void sendCancel()
	{
		message = new CN_InputMessage(CN_InputMessage.MessageType.CANCEL, null, null);
	}

	public void sendUndoTurn()
	{
		message = new CN_InputMessage(CN_InputMessage.MessageType.UNDO_TURN, null, null);
	}

	public void sendFinishTurn()
	{
		message = new CN_InputMessage(CN_InputMessage.MessageType.FINISH_TURN, null, null);
	}

	public void sendBeginTurn()
	{
		message = new CN_InputMessage(CN_InputMessage.MessageType.BEGIN_TURN, null, null);
	}



	private void handleFatalException(UI_FatalException e)
	{
		kill();
		CNPC_ExceptionFrame.showFrame(e);
	}

}
