package cnpc.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

import javax.swing.plaf.basic.*;

class UI_PlayerPane extends JLayeredPane
{
	public enum Orientation {NORTH, EAST, SOUTH, WEST};
	private Orientation orientation;
	private int currentLayer;


	public UI_PlayerPane(Orientation orientation_in) throws UI_FatalException
	{
		super();
		currentLayer = UI_Constants.STARTINGLAYER;
		orientation = orientation_in;
		setBorder(new BasicBorders.FieldBorder(Color.LIGHT_GRAY, Color.LIGHT_GRAY, Color.LIGHT_GRAY, Color.LIGHT_GRAY));

		switch(orientation)
		{
			case NORTH:
				setPreferredSize(new Dimension(UI_Constants.NORTHX, UI_Constants.NORTHY));
				break;
			case EAST:
				setPreferredSize(new Dimension(UI_Constants.EASTX, UI_Constants.EASTY));
				break;
			case SOUTH:
				setPreferredSize(new Dimension(UI_Constants.SOUTHX, UI_Constants.SOUTHY));
				break;
			case WEST:
				setPreferredSize(new Dimension(UI_Constants.WESTX, UI_Constants.WESTY));
				break;
			default:
				throw new UI_FatalException("UI_PlayerPane, constructor: invalid orientation!");

		}
	}


	private void placeCard(final UI_CardPanel cardPanel, int xcoord, int ycoord)
	{
		cardPanel.setBounds(xcoord, ycoord, cardPanel.getPreferredSize().width, cardPanel.getPreferredSize().height);
		cardPanel.addMouseListener
		(
			new MouseAdapter()
			{
				public void mousePressed(MouseEvent e)
				{
					if(e.getButton() == MouseEvent.BUTTON1)
					{
						setLayer(cardPanel, getLayer(cardPanel) + 20);
					}
				}
				public void mouseReleased(MouseEvent e)
				{
					if(e.getButton() == MouseEvent.BUTTON1)
					{
						setLayer(cardPanel, getLayer(cardPanel) - 20);
					}
				}
			}
		);

		add(cardPanel);
		setLayer(cardPanel, currentLayer);
		currentLayer++;
	}


	public void renderPane(ArrayList<UI_CardPanel> hand, ArrayList<UI_CardPanel> artifacts) throws UI_FatalException
	{
		removeAll();
		repaint();

		switch(orientation)
		{
			case NORTH:
				for(int i = 0; i < hand.size(); i++)
				{
					placeCard(hand.get(i), 30 * i, 0);
				}
				for(int i = 0; i < artifacts.size(); i++)
				{
					placeCard(artifacts.get(i), 30 * i, 150);
				}
				break;
			case EAST:
				for(int i = 0; i < hand.size(); i++)
				{
					placeCard(hand.get(i), 100, 30 * i);
				}
				for(int i = 0; i < artifacts.size(); i++)
				{
					placeCard(artifacts.get(i), 0, 30 * i);
				}
				break;
			case SOUTH:
				for(int i = 0; i < hand.size(); i++)
				{
					placeCard(hand.get(i), 30 * i, 150);
				}
				for(int i = 0; i < artifacts.size(); i++)
				{
					placeCard(artifacts.get(i), 30 * i, 0);
				}
				break;
			case WEST:
				for(int i = 0; i < hand.size(); i++)
				{
					placeCard(hand.get(i), 0, 30 * i);
				}
				for(int i = 0; i < artifacts.size(); i++)
				{
					placeCard(artifacts.get(i), 100, 30 * i);
				}
				break;
			default:
				throw new UI_FatalException("UI_PlayerPane, renderPane(): invalid orientation!");
		}
	}
}
