package cnpc.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

class UI_SearchFrame extends JFrame
{
	private final SpringLayout layout;
	private final Container content;

	private final Stack<UI_CardPanel> drawStack;
	private final Stack<UI_CardPanel> placeStack;

	private JButton acceptButton;

	private UI_CardPanel currentDrawPanel;
	private UI_CardPanel currentPlacePanel;


	public UI_SearchFrame(String title, Stack<UI_CardPanel> cardStack, JButton acceptButton_in)
	{
		this(title, cardStack);

		setSize(UI_Constants.SEARCH_FRAME_X_DIMENSION, UI_Constants.SEARCH_FRAME_Y_DIMENSION_BUTTON);

		WindowFocusListener[] windowFocusListeners = getWindowFocusListeners();
		for(WindowFocusListener currentListener : windowFocusListeners)
		{
			removeWindowFocusListener(currentListener);
		}

		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

		acceptButton = acceptButton_in;

		updatePanels();
	}


	public UI_SearchFrame(String title, Stack<UI_CardPanel> cardStack)
	{
		layout = new SpringLayout();

		content = getContentPane();
		content.setLayout(layout);

		setTitle(title);
		setResizable(false);
		setLocation(UI_Constants.SEARCH_FRAME_LOCATION);

		setSize(UI_Constants.SEARCH_FRAME_X_DIMENSION, UI_Constants.SEARCH_FRAME_Y_DIMENSION_NO_BUTTON);

		acceptButton = null;

		drawStack = new Stack<UI_CardPanel>();
		placeStack = new Stack<UI_CardPanel>();

		Stack<UI_CardPanel> intermStack = new Stack<UI_CardPanel>();
		while(!cardStack.empty())
		{
			intermStack.push(cardStack.pop());
		}
		while(!intermStack.empty())
		{
			cardStack.push(intermStack.peek());
			drawStack.push(intermStack.pop());
		}

		addWindowFocusListener
		(
			new WindowAdapter()
			{
				public void windowLostFocus(WindowEvent e)
				{
					dispose();
				}
			}
		);


		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

		setVisible(true);

		updatePanels();

	}






	private void updatePanels()
	{
		content.removeAll();

		MouseListener[] mouseListeners;

		if(!drawStack.empty())
		{
			currentDrawPanel = drawStack.peek();
			currentDrawPanel.setToBackImage();
			mouseListeners = currentDrawPanel.getMouseListeners();
			for(MouseListener listener : mouseListeners)
			{
				currentDrawPanel.removeMouseListener(listener);
			}
			currentDrawPanel.addMouseListener
			(
				new MouseAdapter()
				{
					public void mouseClicked(MouseEvent e)
					{
						if((e.getButton() == MouseEvent.BUTTON1)&&(e.getClickCount() % 2 == 0))
						{
							placeStack.add(drawStack.pop());
							updatePanels();
						}
					}
				}
			);
		}
		else
		{
			currentDrawPanel = new UI_CardPanel();
			currentDrawPanel.addMouseListener
			(
				new MouseAdapter()
				{
					public void mouseClicked(MouseEvent e)
					{
						if((e.getButton() == MouseEvent.BUTTON1)&&(e.getClickCount() % 2 == 0))
						{
							while(!placeStack.empty())
							{
								drawStack.add(placeStack.pop());
							}
							updatePanels();
						}
					}
				}
			);
		}

		if(!placeStack.empty())
		{
			currentPlacePanel = placeStack.peek();
			currentPlacePanel.setToFrontImage();
			mouseListeners = currentPlacePanel.getMouseListeners();
			for(MouseListener listener : mouseListeners)
			{
				currentPlacePanel.removeMouseListener(listener);
			}
			currentPlacePanel.addMouseListener
			(
				new MouseAdapter()
				{
					public void mouseClicked(MouseEvent e)
					{
						if((e.getButton() == MouseEvent.BUTTON1)&&(e.getClickCount() % 2 == 0))
						{
							drawStack.add(placeStack.pop());
							updatePanels();
						}
					}
				}
			);
		}
		else
		{
			currentPlacePanel = new UI_CardPanel();
		}

		currentDrawPanel.addStandardMouseListener();
		currentPlacePanel.addStandardMouseListener();

		if(acceptButton != null)
		{
			layout.putConstraint(SpringLayout.WEST, acceptButton, UI_Constants.SEARCH_FRAME_BUTTON_X, SpringLayout.WEST, content);
			layout.putConstraint(SpringLayout.NORTH, acceptButton, UI_Constants.SEARCH_FRAME_BUTTON_SEPARATION, SpringLayout.SOUTH, currentDrawPanel);

			content.add(acceptButton);

			if(placeStack.empty())
			{
				acceptButton.setEnabled(false);
			}
			else
			{
				acceptButton.setEnabled(true);
			}
		}

		layout.putConstraint(SpringLayout.WEST, currentDrawPanel, UI_Constants.SEARCH_FRAME_STARTING_X, SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, currentDrawPanel, UI_Constants.SEARCH_FRAME_STARTING_Y, SpringLayout.NORTH, content);
		layout.putConstraint(SpringLayout.WEST, currentPlacePanel, UI_Constants.SEARCH_FRAME_CARD_SEPARATION, SpringLayout.EAST, currentDrawPanel);
		layout.putConstraint(SpringLayout.NORTH, currentPlacePanel, UI_Constants.SEARCH_FRAME_STARTING_Y, SpringLayout.NORTH, content);

		content.add(currentDrawPanel);
		content.add(currentPlacePanel);

		validate();
	}



	public UI_GameCard getSelectedCard()
	{
		return currentPlacePanel.getRepresentedCard();
	}
}
