package cnpc.ui;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class UI_SelectorFrame extends JFrame
{
	private final SpringLayout layout;
	private final Container content;


	public UI_SelectorFrame(String title, ArrayList<JButton> buttons) throws UI_FatalException
	{
		if(buttons.size() < 1)
		{
			throw new UI_FatalException("The selector frame must have at least one button!");
		}

		layout = new SpringLayout();

		content = getContentPane();
		content.setLayout(layout);

		setTitle(title);
		setResizable(false);
		setLocation(UI_Constants.SELECTOR_FRAME_LOCATION);

		JButton currentButton;
		JButton previousButton = null;
		for(int i = 0; i < buttons.size(); i++)
		{
			currentButton = buttons.get(i);
			if(i == 0)
			{
				layout.putConstraint(SpringLayout.WEST, currentButton, UI_Constants.SELECTOR_FRAME_STARTING_X, SpringLayout.WEST, content);
				layout.putConstraint(SpringLayout.NORTH, currentButton, UI_Constants.SELECTOR_FRAME_STARTING_Y, SpringLayout.NORTH, content);
			}
			else
			{
				layout.putConstraint(SpringLayout.WEST, currentButton, UI_Constants.SELECTOR_FRAME_STARTING_X, SpringLayout.WEST, content);
				layout.putConstraint(SpringLayout.NORTH, currentButton, UI_Constants.SELECTOR_FRAME_Y_SEPARATION, SpringLayout.SOUTH, previousButton);
			}
			previousButton = currentButton;
		}

		Dimension biggestDimension;
		int highestPreferredX = 0;
		int highestPreferredY = 0;
		for(int i = 0; i < buttons.size(); i++)
		{
			currentButton = buttons.get(i);
			if(currentButton.getPreferredSize().width > highestPreferredX)
			{
				highestPreferredX = currentButton.getPreferredSize().width;
			}
			if(currentButton.getPreferredSize().height > highestPreferredY)
			{
				highestPreferredY = currentButton.getPreferredSize().height;
			}
		}

		biggestDimension = new Dimension(highestPreferredX, highestPreferredY);
		for(int i = 0; i < buttons.size(); i++)
		{
			currentButton = buttons.get(i);
			currentButton.setPreferredSize(biggestDimension);
			content.add(currentButton);
		}

		setSize(UI_Constants.SELECTOR_FRAME_STARTING_X + biggestDimension.width + UI_Constants.SELECTOR_FRAME_ENDING_X, UI_Constants.TITLE_BAR_HEIGHT + UI_Constants.SELECTOR_FRAME_STARTING_Y + (buttons.size() * biggestDimension.height) + ((buttons.size() - 1) * UI_Constants.SELECTOR_FRAME_Y_SEPARATION) + UI_Constants.SELECTOR_FRAME_ENDING_Y);

		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

		setVisible(true);
	}

}
