package cnpc.ui;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class UI_TimelineFrame extends JFrame
{
	SpringLayout layout;
	Container content;

	private final ArrayList<JComponent> disposableComponents;


	public UI_TimelineFrame() throws UI_FatalException
	{
		layout = new SpringLayout();

		content = getContentPane();
		content.setLayout(layout);

		setTitle("Timeline");
		setSize(UI_Constants.TIMELINE_FRAME_DIMENSION);
		setResizable(false);
		setLocation(UI_Constants.TIMELINE_START_LOCATION);

		disposableComponents = new ArrayList<JComponent>();

    }



	public void placeTimeline(UI_TimelineCardPanel[] timeline) throws UI_FatalException
	{
		while(!disposableComponents.isEmpty())
		{
			content.remove(disposableComponents.get(0));
			disposableComponents.remove(0);
		}

		if(timeline.length > UI_Constants.MAX_PER_ROW * UI_Constants.MAX_PER_COL)
		{
			throw new UI_FatalException("UI_TimelineFrame, initTimeline: The UI can support only 32 cards in the Timeline!");
		}

		int currentRow = 1;
		int currentColumn = 0;
		UI_CardPanel currentCard;
		UI_CardPanel cardAbove;
		UI_CardPanel cardLeft;
		for(int i = 0; i < timeline.length; i++)
		{
			currentColumn += 1;
			if(currentColumn > UI_Constants.MAX_PER_ROW)
			{
				currentRow += 1;
				if(currentRow > UI_Constants.MAX_PER_COL)
				{
					throw new UI_FatalException("UI_TimelineFrame, initTimeline: Too many rows!");
				}
				currentColumn = 1;
			}

			currentCard = timeline[i];


			if((currentColumn <= 1)&&(currentRow <= 1))
			{
				layout.putConstraint(SpringLayout.WEST, currentCard, UI_Constants.STARTING_X, SpringLayout.WEST, content);
				layout.putConstraint(SpringLayout.NORTH, currentCard, UI_Constants.STARTING_Y, SpringLayout.NORTH, content);
			}
			else if(currentRow <= 1)
			{
				cardLeft = timeline[i - 1];
				layout.putConstraint(SpringLayout.WEST, currentCard, UI_Constants.SEPARATION_X, SpringLayout.EAST, cardLeft);
				layout.putConstraint(SpringLayout.NORTH, currentCard, UI_Constants.STARTING_Y, SpringLayout.NORTH, content);
			}
			else if(currentColumn <= 1)
			{
				cardAbove = timeline[i - UI_Constants.MAX_PER_ROW];
				layout.putConstraint(SpringLayout.WEST, currentCard, UI_Constants.STARTING_X, SpringLayout.WEST, content);
				layout.putConstraint(SpringLayout.NORTH, currentCard, UI_Constants.SEPARATION_Y, SpringLayout.SOUTH, cardAbove);
			}
			else
			{
				cardAbove = timeline[i - UI_Constants.MAX_PER_ROW];
				cardLeft = timeline[i - 1];
				layout.putConstraint(SpringLayout.WEST, currentCard, UI_Constants.SEPARATION_X, SpringLayout.EAST, cardLeft);
				layout.putConstraint(SpringLayout.NORTH, currentCard, UI_Constants.SEPARATION_Y, SpringLayout.SOUTH, cardAbove);
			}

			content.add(currentCard);
			disposableComponents.add(currentCard);
		}
		validate();
	}

}
