package cnpc.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


class UI_VictoryDialog extends JDialog
{
	private final SpringLayout layout = new SpringLayout();
	private final Container content = getContentPane();

	private final JLabel victoryLabel = new JLabel();

	private final UI_MainFrame owner;

	private static UI_VictoryDialog me = null;


	public static void showDialog(UI_MainFrame frame, String playerName, int victoryType)
	{
		new UI_VictoryDialog(frame, playerName, victoryType);
	}
	
	public static void disposeDialog()
	{
		if(me == null) return;
		
		me.dispose();
		me = null;
	}



	private UI_VictoryDialog(UI_MainFrame owner_in, String playerName, int victoryType)
	{
		me = this;
		owner = owner_in;

		content.setLayout(layout);

		setLocation(UI_Constants.VICTORY_DIALOG_LOCATION);
		
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener
		(
			new WindowAdapter()
			{
				public void windowClosing(WindowEvent e)
				{
					owner.performDisposal();
					dispose();
				}
			}
		);

		setTitle(UI_Constants.GAME_NAME);
		
		String victoryLabelText = playerName + " is victorious!";
		switch(victoryType)
		{
			case UI_Constants.VICTORY_TYPE_ID:
				victoryLabelText = victoryLabelText + "  Won by ID.";
				break;
			case UI_Constants.VICTORY_TYPE_MISSION:
				victoryLabelText = victoryLabelText + "  Won by Mission.";
				break;
			case UI_Constants.VICTORY_TYPE_CARDS:
				victoryLabelText = victoryLabelText + "  Won by cards in hand.";
				break;
			default:
				break;
		}

		victoryLabel.setText(victoryLabelText);

		layoutDialog();

		setVisible(true);
	}

	private void layoutDialog()
	{
		victoryLabel.setFont(UI_Constants.VICTORY_FONT);
		victoryLabel.setForeground(UI_Constants.VICTORY_COLOR);

		layout.putConstraint(SpringLayout.WEST, victoryLabel, UI_Constants.VICTORY_DIALOG_WEST_BUFFER, SpringLayout.WEST, content);
		layout.putConstraint(SpringLayout.NORTH, victoryLabel, UI_Constants.VICTORY_DIALOG_NORTH_BUFFER, SpringLayout.NORTH, content);
		content.add(victoryLabel);

		int width = UI_Constants.VICTORY_DIALOG_WEST_BUFFER + victoryLabel.getPreferredSize().width + UI_Constants.VICTORY_DIALOG_EAST_BUFFER;
		int height = UI_Constants.TITLE_BAR_HEIGHT + UI_Constants.VICTORY_DIALOG_NORTH_BUFFER + victoryLabel.getPreferredSize().height + UI_Constants.VICTORY_DIALOG_SOUTH_BUFFER;
		setSize(width, height);
		setResizable(false);
	}
}
